@@ -1,5 +1,5 @@
 /*
- *	$Id$
+ *	$Id: libscsi.h,v 1.4.4.4 1999/06/17 18:33:13 th Exp $
  *
  *	libscsi.h -- library header for routines to handle the changer
  *			support for chio based systems
@@ -19,13 +19,18 @@
 /*
  * This function gets the actual cleaning state of the drive 
  */
-int get_clean_state(char *tape);
+int get_clean_state P((char *tape));
 
+/*
+ * This function gets the next empty slot from the changer
+ * (From this slot the tape is loaded ...)
+ */
+int GetCurrentSlot P((int fd, int drive));
 
 /*
  * Eject the actual tape from the tapedrive
  */
-void eject_tape(char *tape);
+void eject_tape P((char *tape, int type));
 
 
 /* 
@@ -36,7 +41,7 @@ int isempty P((int fd, int slot));
 /*
  * find the first empty slot 
  */
-int find_empty P((int fd));
+int find_empty P((int fd, int start, int count));
 
 /*
  * returns one if there is a tape loaded in the drive 
