@@ -1,5 +1,5 @@
 /*
- *	$Id$
+ *	$Id: scsi-hpux.c,v 1.4.4.9 2000/10/24 23:49:39 martinea Exp $
  *
  *	scsi-chio.c -- library routines to handle the changer
  *			support for chio based systems
@@ -11,11 +11,11 @@
  *      Michael C. Povel 03.06.98 added dummy for eject_tape
  */
 
-#include "config.h"
 #include "amanda.h"
 
-#if defined(HAVE_HPUX_SCSI)
+#if defined(HAVE_HPUX_SCSI_CHIO)
 # include <sys/scsi.h>
+# include <sys/mtio.h>  /* for eject_tape ioctls */
 
 char *moddesc = "@(#)" __FILE__
 		": HP/UX SCSI changer support routines @(#)";
@@ -26,6 +26,17 @@ char *moddesc = "@(#)" __FILE__
 static struct element_addresses changer_info;
 static int changer_info_init = 0;
 
+
+/* Get the number of the first free slot
+ * return > 0 number of empty slot
+ * return = 0 no slot free
+ * return < 0 error
+ */
+int GetCurrentSlot(int fd)
+{
+
+}
+
 static int get_changer_info(fd)
 {
 int rc = 0;
@@ -80,12 +91,11 @@ int get_clean_state(char *dev)
 
 
 void eject_tape(char *tape)
-/* This function ejects the tape from the drive */
+     /* This function ejects the tape from the drive */
 {
-#if 0
 /*
   This code works for Linux .... 
-  maybe someone can do something like this under HPUX
+  This code works for HPUX too, see 'man 7 mt'
 */
     int mtfd;
     struct mtop mt_com;
@@ -103,7 +113,6 @@ void eject_tape(char *tape)
 */
     }
     close(mtfd);
-#endif
 }
 
 /* 
@@ -234,7 +243,7 @@ int rc;
 		get_pname(), rc, strerror(errno));
 	return(-2);
     }
-    return (0);
+    return (rc);
 }
 
 int get_slot_count(int fd)
@@ -266,4 +275,40 @@ int get_drive_count(int fd)
     return changer_info.num_data_transfers;
 }
 
+/* This function should ask the drive if it is ready */
+int Tape_Ready(char *tapedev, char *changerdev, int changerfd, int wait)
+{
+  FILE *out=NULL;
+  int cnt=0;
+
+  if (strcmp(tapedev, changerdev) == 0)
+    {
+      sleep(wait);
+      return(0);
+    }
+
+  while ((cnt<wait) && (NULL==(out=fopen(tapedev,"w+")))){
+    cnt++;
+    sleep(1);
+  }
+  if (out != NULL)
+    fclose(out);
+  return 0;
+}
+
+int OpenDevice(char * tapedev)
+{
+  int DeviceFD;
+
+  DeviceFD = open(tapedev, O_RDWR);
+  return(DeviceFD);
+}
+
+int CloseDevice(int DeviceFD)
+{
+  int ret;
+
+  ret = close(DeviceFD);
+  return(ret);
+}
 #endif
