@@ -25,7 +25,7 @@
  *			   University of Maryland at College Park
  */
 /*
- * $Id$
+ * $Id: amandad.c,v 1.32.2.4 1999/09/10 23:27:08 jrj Exp $
  *
  * handle client-host side of Amanda network communications, including
  * security checks, execution of the proper service, and acking the
@@ -73,40 +73,6 @@ void sendnak P((pkt_t *hdr, pkt_t *msg, char *str));
 void setup_rep P((pkt_t *hdr, pkt_t *msg));
 char *strlower P((char *str));
 
-void sigchild_jump P((int sig));
-void sigchild_flag P((int sig));
-
-
-/*
- * We trap SIGCHLD to break out of our select loop.
- */
-
-jmp_buf sigjmp;
-int got_sigchild = 0;
-
-void sigchild_jump(sig) 
-int sig;
-{ 
-    got_sigchild = 1;
-    longjmp(sigjmp, 1);
-}
-
-void sigchild_flag(sig) 
-int sig;
-{ 
-    got_sigchild = 1;
-}
-
-/* XXX these moved out of main because of the setjmp.  restructure! */
-struct service_s *servp;
-fd_set insock;
-pkt_t in_msg, out_msg, dup_msg;
-char *cmd = NULL, *base = NULL;
-char *pwname, **vp;
-struct passwd *pwptr;
-int retry_count, rc, reqlen;
-int req_pipe[2], rep_pipe[2];
-
 int main(argc, argv)
 int argc;
 char **argv;
@@ -117,6 +83,21 @@ char **argv;
     unsigned long malloc_hist_1, malloc_size_1;
     unsigned long malloc_hist_2, malloc_size_2;
 
+    /* in_msg: The first incoming request.
+       dup_msg: Any other incoming message.
+       out_msg: Standard, i.e. non-repeated, ACK and REP.
+       rej_msg: Any other outgoing message.
+     */
+    pkt_t in_msg, out_msg, rej_msg, dup_msg;
+    char *cmd = NULL, *base = NULL;
+    char **vp;
+    int retry_count, rc, reqlen;
+    int req_pipe[2], rep_pipe[2];
+    int dglen = 0;
+
+    struct service_s *servp;
+    fd_set insock;
+
     for(fd = 3; fd < FD_SETSIZE; fd++) {
 	/*
 	 * Make sure nobody spoofs us with a lot of extra open files
@@ -127,41 +108,40 @@ char **argv;
 	close(fd);
     }
 
+    safe_cd();
+
     set_pname("amandad");
 
     malloc_size_1 = malloc_inuse(&malloc_hist_1);
 
     erroutput_type = (ERR_INTERACTIVE|ERR_SYSLOG);
 
-    pwname = CLIENT_LOGIN;
-    pwptr = getpwnam(pwname);
-
 #ifdef FORCE_USERID
 
     /* we'd rather not run as root */
     if(geteuid() == 0) {
 #ifdef KRB4_SECURITY
-        if(pwptr == NULL)
-	    error("error [cannot find user %s in passwd file]\n", pwname);
+        if(client_uid == (uid_t) -1) {
+	    error("error [cannot find user %s in passwd file]\n", CLIENT_LOGIN);
+	}
+
         /*
 	 * if we're using kerberos security, we'll need to be root in
 	 * order to get at the machine's srvtab entry, so we hang on to
 	 * some root privledges for now.  We give them up entirely later.
 	 */
-	setegid(pwptr->pw_gid);
-	seteuid(pwptr->pw_uid);
+	setegid(client_gid);
+	seteuid(client_uid);
 #else
-	initgroups(pwname, pwptr->pw_gid);
-	setgid(pwptr->pw_gid);
-	setuid(pwptr->pw_uid);
+	initgroups(CLIENT_LOGIN, client_gid);
+	setgid(client_gid);
+	setuid(client_uid);
 #endif  /* KRB4_SECURITY */
     }
 #endif	/* FORCE_USERID */
 
     /* initialize */
 
-    chdir("/");		/* someplace not world writable */
-    umask(077);
     dbopen();
     {
 	extern int db_fd;
@@ -182,6 +162,12 @@ char **argv;
     dgram_zero(&out_msg.dgram);
     dgram_socket(&out_msg.dgram, 0);
 
+    dgram_zero(&rej_msg.dgram);
+    dgram_socket(&rej_msg.dgram, 0);
+
+    dgram_zero(&rej_msg.dgram);
+    dgram_socket(&rej_msg.dgram, 0);
+
     /* set up input and response pipes */
 
     if(pipe(req_pipe) == -1 || pipe(rep_pipe) == -1)
@@ -214,7 +200,7 @@ char **argv;
     dbprintf(("got packet:\n--------\n%s--------\n\n", in_msg.dgram.cur));
 
     parse_pkt_header(&in_msg);
-    if(in_msg.type != P_REQ && in_msg.type != P_NAK) {
+    if(in_msg.type != P_REQ && in_msg.type != P_NAK && in_msg.type != P_ACK) {
 	/* XXX */
 	dbprintf(("this is a %s packet, nak'ing it\n", 
 		  in_msg.type == P_BOGUS? "bogus" : "unexpected"));
@@ -224,7 +210,7 @@ char **argv;
 		in_msg.type == P_REP? "rep ": "",
 		"packet", NULL);
 	}
-	sendnak(&in_msg, &out_msg, parse_errmsg);
+	sendnak(&in_msg, &rej_msg, parse_errmsg);
 	dbclose();
 	return 1;
     }
@@ -241,7 +227,7 @@ char **argv;
 
     if(servp->name == NULL) {
 	errstr = newstralloc2(errstr, "unknown service: ", in_msg.service);
-	sendnak(&in_msg, &out_msg, errstr);
+	sendnak(&in_msg, &rej_msg, errstr);
 	dbclose();
 	return 1;
     }
@@ -255,7 +241,7 @@ char **argv;
 			      "service ", base, " unavailable",
 			      NULL);
 	amfree(base);
-	sendnak(&in_msg, &out_msg, errstr);
+	sendnak(&in_msg, &rej_msg, errstr);
 	dbclose();
 	return 1;
     }
@@ -299,12 +285,12 @@ char **argv;
      */
 
     if(geteuid() == 0) {
-	pwname = CLIENT_LOGIN;
-	if((pwptr = getpwnam(pwname)) == NULL)
-	    error("error [cannot find user %s in passwd file]\n", pwname);
-	initgroups(pwname, pwptr->pw_gid);
-	setgid(pwptr->pw_gid);
-	setuid(pwptr->pw_uid);
+	if(client_uid == (uid_t) -1) {
+	    error("error [cannot find user %s in passwd file]\n", CLIENT_LOGIN);
+	}
+	initgroups(CLIENT_LOGIN, client_gid);
+	setgid(client_gid);
+	setuid(client_uid);
     }
 
 #endif  /* KRB4_SECURITY && FORCE_USERID */
@@ -313,10 +299,8 @@ char **argv;
 
     /* spawn child to handle request */
 
-    signal(SIGCHLD, sigchild_flag);
-
     switch(fork()) {
-    case -1: error("could not fork service: %s", strerror(errno));
+    case -1: error("could not fork for %s: %s", cmd, strerror(errno));
 
     default:		/* parent */
 
@@ -348,22 +332,36 @@ char **argv;
 	/* run service */
 
 	execle(cmd, cmd, NULL, safe_env());
-	error("could not exec service: %s", strerror(errno));
+	error("could not exec %s: %s", cmd, strerror(errno));
     }
     amfree(cmd);
 
-    if(!setjmp(sigjmp))
-	signal(SIGCHLD, sigchild_jump);
-    else /* got jump, turn it off now */
-	signal(SIGCHLD, sigchild_flag);
+    setup_rep(&in_msg, &out_msg);
+#ifdef KRB4_SECURITY
+    add_mutual_authenticator(&out_msg.dgram);
+#endif
 
-    while(!got_sigchild) {
+    while(1) {
 
 	FD_ZERO(&insock);
 	FD_SET(0, &insock);
-	if(select(1, (SELECT_ARG_TYPE *)&insock, NULL, NULL, NULL) < 0)
+	FD_SET(rep_pipe[0], &insock);
+
+	if(select(rep_pipe[0]+1, (SELECT_ARG_TYPE *)&insock, NULL, NULL, NULL) < 0)
 	    error("select failed: %s", strerror(errno));
 
+	if(FD_ISSET(rep_pipe[0],&insock)) {
+	    if((rc = read(rep_pipe[0], out_msg.dgram.cur+dglen,
+			  MAX_DGRAM-out_msg.dgram.len)) <= 0) {
+		if (rc < 0)
+		    error("reading response pipe: %s", strerror(errno));
+		break;
+	    }
+	    dglen += rc;
+	}
+	if(!FD_ISSET(0,&insock))
+	    continue;
+
 	if((n = dgram_recv(&dup_msg.dgram, RECV_TIMEOUT, &dup_msg.peer)) <= 0) {
 	    char *s;
 
@@ -385,35 +383,36 @@ char **argv;
 	 *
 	 * It should suffice to ACK whenever the sender is identical.
 	 */
+	dbprintf(("%s: got packet:\n----\n%s----\n\n", argv[0],
+		  dup_msg.dgram.data));
+	parse_pkt_header(&dup_msg);
 	if(dup_msg.peer.sin_addr.s_addr == in_msg.peer.sin_addr.s_addr &&
 	   dup_msg.peer.sin_port == in_msg.peer.sin_port) {
-	    dbprintf(("%s: received dup packet, ACKing it\n", argv[0]));
-	    sendack(&in_msg, &out_msg);
+	    if(dup_msg.type == P_REQ) {
+		dbprintf(("%s: received dup P_REQ packet, ACKing it\n", argv[0]));
+		sendack(&in_msg, &rej_msg);
+	    }
+	    else {
+		dbprintf(("%s: It's not a P_REQ, ignoring it\n", argv[0]));
+	    }
 	}
 	else {
 	    dbprintf(("%s: received other packet, NAKing it\n", argv[0]));
+	    dbprintf(("  addr: peer %s dup %s, port: peer %d dup %d\n",
+		      inet_ntoa(in_msg.peer.sin_addr),
+		      inet_ntoa(dup_msg.peer.sin_addr),
+		      (int)ntohs(in_msg.peer.sin_port),
+		      (int)ntohs(dup_msg.peer.sin_port)));
 	    /* XXX dup_msg filled in? */
-	    sendnak(&dup_msg, &out_msg, "amandad busy");
+	    sendnak(&dup_msg, &rej_msg, "amandad busy");
 	}
 
     }
 
     /* XXX reap child?  log if non-zero status?  don't respond if non zero? */
-
     /* setup header for out_msg */
 
-    setup_rep(&in_msg, &out_msg);
-#ifdef KRB4_SECURITY
-    add_mutual_authenticator(&out_msg.dgram);
-#endif
-
-    /* read response packet from pipe: up to MAX_DGRAM bytes are sent */
-
-    if((rc = read(rep_pipe[0], out_msg.dgram.cur,
-                  MAX_DGRAM-out_msg.dgram.len)) < 0)
-        error("reading response pipe: %s", strerror(errno));
-
-    out_msg.dgram.len += rc;
+    out_msg.dgram.len += dglen;
     out_msg.dgram.data[out_msg.dgram.len] = '\0';
     aclose(rep_pipe[0]);
 
@@ -446,21 +445,25 @@ send_response:
 
 	    continue;
 	}
-	dbprintf(("%s: got ack:\n----\n%s----\n\n", argv[0],
+	dbprintf(("%s: got packet:\n----\n%s----\n\n", argv[0],
 		  dup_msg.dgram.data));
 	parse_pkt_header(&dup_msg);
 
-	if(dup_msg.type == P_ACK && 
-	   dup_msg.peer.sin_addr.s_addr == in_msg.peer.sin_addr.s_addr &&
-	   dup_msg.peer.sin_port == in_msg.peer.sin_port)
-	    break;
+	
+	if(dup_msg.peer.sin_addr.s_addr == in_msg.peer.sin_addr.s_addr &&
+	   dup_msg.peer.sin_port == in_msg.peer.sin_port) {
+	    if(dup_msg.type == P_ACK)
+		break;
+	    else
+		dbprintf(("%s: It's not an ack\n", argv[0]));
+	}
 	else {
 	    dbprintf(("%s: weird, it's not a proper ack\n", argv[0]));
-	    dbprintf(("  addr: peer %lX dup %lX, port: peer %lX dup %lX\n",
-		      (unsigned long)in_msg.peer.sin_addr.s_addr,
-		      (unsigned long)dup_msg.peer.sin_addr.s_addr,
-		      (unsigned long)in_msg.peer.sin_port,
-		      (unsigned long)dup_msg.peer.sin_port));
+	    dbprintf(("  addr: peer %s dup %s, port: peer %d dup %d\n",
+		      inet_ntoa(in_msg.peer.sin_addr),
+		      inet_ntoa(dup_msg.peer.sin_addr),
+		      (int)ntohs(in_msg.peer.sin_port),
+		      (int)ntohs(dup_msg.peer.sin_port)));
 	}		
     }
     /* XXX log if retry count exceeded */
@@ -538,6 +541,6 @@ char *str;
 {
     char *s;
     for(s=str; *s; s++)
-	if(isupper(*s)) *s = tolower(*s);
+	if(isupper((int)*s)) *s = tolower(*s);
     return str;
 }
