@@ -24,14 +24,14 @@
  * file named AUTHORS, in the root directory of this distribution.
  */
 /* 
- * $Id$
+ * $Id: calcsize.c,v 1.24.2.3 2000/09/24 01:52:34 martinea Exp $
  *
  * traverse directory tree to get backup size estimates
  */
 #include "amanda.h"
 #include "statfs.h"
 
-#define ROUND(n,x)	(x + (x % n) ? (n - x % n) : 0)
+#define ROUND(n,x)	((x) + (n) - 1 - (((x) + (n) - 1) % (n)))
 
 /*
 static unsigned long round_function(n, x)
@@ -135,7 +135,7 @@ char **argv;
     return 0;
 #else
     int i;
-    char *dirname, *amname;
+    char *dirname=NULL, *amname=NULL;
     int fd;
     unsigned long malloc_hist_1, malloc_size_1;
     unsigned long malloc_hist_2, malloc_size_2;
@@ -150,6 +150,10 @@ char **argv;
 	close(fd);
     }
 
+    set_pname("calcsize");
+
+    safe_cd();
+
     malloc_size_1 = malloc_inuse(&malloc_hist_1);
 
 #if 0
@@ -160,7 +164,7 @@ char **argv;
 
     /* need at least program, amname, and directory name */
 
-    if(argc < 2) {
+    if(argc < 3) {
 #ifdef BUILTIN_EXCLUDE_SUPPORT
       usage:
 #endif
@@ -241,12 +245,18 @@ char **argv;
 
     /* the amanda name can be different from the directory name */
 
-    amname = *argv;
-    argc--, argv++;
+    if (argc > 0) {
+	amname = *argv;
+	argc--, argv++;
+    } else
+	error("missing <name>");
 
     /* the toplevel directory name to search from */
-    dirname = *argv;
-    argc--, argv++;
+    if (argc > 0) {
+	dirname = *argv;
+	argc--, argv++;
+    } else
+	error("missing <dir>");
 
     /* the dump levels to calculate sizes for */
 
@@ -312,13 +322,13 @@ char *parent_dir;
     DIR *d;
     struct dirent *f;
     struct stat finfo;
-    char *dirname, *newdir;
-    char *newname = NULL;
+    char *dirname, *newname = NULL;
+    char *newbase = NULL;
     dev_t parent_dev = 0;
     int i;
     int l;
 
-    if(parent_dir && chdir(parent_dir) != -1 && stat(".", &finfo) != -1)
+    if(parent_dir && stat(parent_dir, &finfo) != -1)
 	parent_dev = finfo.st_dev;
 
     push_name(parent_dir);
@@ -333,55 +343,59 @@ char *parent_dir;
 	    continue;
 #endif
 
-	if(chdir(dirname) == -1 || (d = opendir(".")) == NULL) {
+	if((d = opendir(dirname)) == NULL) {
 	    perror(dirname);
 	    continue;
 	}
 
 	l = strlen(dirname);
 	if(l > 0 && dirname[l - 1] != '/') {
-	    newname = newstralloc2(newname, dirname, "/");
+	    newbase = newstralloc2(newbase, dirname, "/");
 	} else {
-	    newname = newstralloc(newname, dirname);
+	    newbase = newstralloc(newbase, dirname);
 	}
 
 	while((f = readdir(d)) != NULL) {
-	    if(f->d_name[0] == '.' && f->d_name[1] == '\0')
-		continue;
-	    if(f->d_name[0] == '.' && f->d_name[1] == '.' &&
-	       f->d_name[2] == '\0')
+	    if(is_dot_or_dotdot(f->d_name)) {
 		continue;
+	    }
 
-	    if(lstat(f->d_name, &finfo) == -1) {
+	    newname = newstralloc2(newname, newbase, f->d_name);
+	    if(lstat(newname, &finfo) == -1) {
 		fprintf(stderr, "%s/%s: %s\n",
 			dirname, f->d_name, strerror(errno));
 		continue;
 	    }
 
-	    if(finfo.st_dev != parent_dev)
+	    if(finfo.st_dev != parent_dev) {
 		continue;
+	    }
 
 	    if((finfo.st_mode & S_IFMT) == S_IFDIR) {
-		newdir = stralloc2(newname, f->d_name);
-		push_name(newdir);
-		amfree(newdir);
+		push_name(newname);
 	    }
 
 	    for(i = 0; i < ndumps; i++) {
-		if(finfo.st_ctime >= dumpdate[i])
-		    if (
+		if(finfo.st_ctime >= dumpdate[i]) {
+		    int exclude = 0;
+		    int is_symlink = 0;
+
 #ifdef BUILTIN_EXCLUDE_SUPPORT
-			!check_exclude(f->d_name) &&
+		    exclude = check_exclude(f->d_name);
 #endif
-			/* regular files */
-			((finfo.st_mode & S_IFMT) == S_IFREG)
-			  /* directories */
-			  || ((finfo.st_mode & S_IFMT) == S_IFDIR)
 #ifdef S_IFLNK
-			  /* symlinks */
-			  || ((finfo.st_mode & S_IFMT) == S_IFLNK)
+		    is_symlink = ((finfo.st_mode & S_IFMT) == S_IFLNK);
 #endif
-			) add_file(i, &finfo);
+		    if (! exclude &&
+			  /* regular files */
+			((finfo.st_mode & S_IFMT) == S_IFREG
+			  /* directories */
+			  || (finfo.st_mode & S_IFMT) == S_IFDIR
+			  /* symbolic links */
+			  || is_symlink)) {
+			add_file(i, &finfo);
+		    }
+		}
 	    }
 	}
 
@@ -392,6 +406,7 @@ char *parent_dir;
 	    perror(dirname);
 #endif
     }
+    amfree(newbase);
     amfree(newname);
 }
 
@@ -442,7 +457,8 @@ int level;
 struct stat *sp;
 {
     /* keep the size in kbytes, rounded up, plus a 1k header block */
-    dumpstats[level].total_size += (ST_BLOCKS(*sp) + 1)/2 + 1;
+    if((sp->st_mode & S_IFMT) == S_IFREG || (sp->st_mode & S_IFMT) == S_IFDIR)
+    	dumpstats[level].total_size += (ST_BLOCKS(*sp) + 1)/2 + 1;
 }
 
 long final_size_dump(level, topdir)
@@ -451,11 +467,15 @@ char *topdir;
 {
     generic_fs_stats_t stats;
     int mapsize;
+    char *s;
 
     /* calculate the map sizes */
 
-    if(chdir(topdir) == -1 || get_fs_stats(".", &stats) == -1)
-	error("statfs %s: %s", topdir, strerror(errno));
+    s = stralloc2(topdir, "/.");
+    if(get_fs_stats(s, &stats) == -1) {
+	error("statfs %s: %s", s, strerror(errno));
+    }
+    amfree(s);
 
     mapsize = (stats.files + 7) / 8;	/* in bytes */
     mapsize = (mapsize + 1023) / 1024;  /* in kbytes */
@@ -505,7 +525,8 @@ int level;
 struct stat *sp;
 {
     /* just add up the block counts */
-    dumpstats[level].total_size += ST_BLOCKS(*sp);
+    if((sp->st_mode & S_IFMT) == S_IFREG || (sp->st_mode & S_IFMT) == S_IFDIR)
+    	dumpstats[level].total_size += ST_BLOCKS(*sp);
 }
 
 long final_size_unknown(level, topdir)
