@@ -24,7 +24,7 @@
  * file named AUTHORS, in the root directory of this distribution.
  */
 /*
- * $Id$
+ * $Id: rundump.c,v 1.23.2.3 1999/09/11 01:14:05 jrj Exp $
  *
  * runs DUMP program as root
  */
@@ -54,6 +54,7 @@ char **argv;
 #ifndef ERRMSG
     char *dump_program;
     int i;
+    char *e;
 #endif /* ERRMSG */
     int fd;
 
@@ -67,6 +68,8 @@ char **argv;
 	close(fd);
     }
 
+    safe_cd();
+
     set_pname("rundump");
 
     dbopen();
@@ -81,23 +84,21 @@ char **argv;
 
 #else								/* } { */
 
-    /* we should be invoked by CLIENT_LOGIN */
-    {
-	struct passwd *pwptr;
-	char *pwname = CLIENT_LOGIN;
-	if((pwptr = getpwnam(pwname)) == NULL)
-	    error("error [cannot find user %s in passwd file]\n", pwname);
+    if(client_uid == (uid_t) -1) {
+	error("error [cannot find user %s in passwd file]\n", CLIENT_LOGIN);
+    }
 
 #ifdef FORCE_USERID
-	if (getuid() != pwptr->pw_uid)
-	    error("error [must be invoked by %s]\n", pwname);
+    if (getuid() != client_uid) {
+	error("error [must be invoked by %s]\n", CLIENT_LOGIN);
+    }
 
-	if (geteuid() != 0)
-	    error("error [must be setuid root]\n");
+    if (geteuid() != 0) {
+	error("error [must be setuid root]\n");
+    }
 #endif	/* FORCE_USERID */
 
-	setuid(0);
-    }
+    setuid(0);
 
 #ifdef XFSDUMP
 
@@ -126,15 +127,15 @@ char **argv;
 #if defined(DUMP)
         dump_program = DUMP;
 #else
-#if defined(XFSDUMP)
+# if defined(XFSDUMP)
         dump_program = XFSDUMP;
-#else
-#if defined(VXDUMP)
+# else
+#  if defined(VXDUMP)
 	dump_program = VXDUMP;
-#else
+#  else
         dump_program = "dump";
-#endif
-#endif
+#  endif
+# endif
 #endif
 
     dbprintf(("running: %s: ",dump_program));
@@ -144,11 +145,11 @@ char **argv;
 
     execve(dump_program, argv, safe_env());
 
-    dbprintf(("failed (%s)\n", strerror(errno)));
+    e = strerror(errno);
+    dbprintf(("execve of %s failed (%s)\n", dump_program, e));
     dbclose();
 
-    fprintf(stderr, "rundump: could not exec %s: %s\n",
-	    dump_program, strerror(errno));
+    fprintf(stderr, "rundump: could not exec %s: %s\n", dump_program, e);
     return 1;
 #endif								/* } */
 }
