@@ -24,7 +24,7 @@
  * file named AUTHORS, in the root directory of this distribution.
  */
 /* 
- * $Id$
+ * $Id: sendbackup-dump.c,v 1.65.2.6 2001/03/15 02:17:34 jrjackson Exp $
  *
  * send backup data using BSD dump
  */
@@ -82,10 +82,10 @@ static regex_t re_table[] = {
   { DMP_SIZE, "DUMP: [0-9][0-9]* KB actual output", 1024},
 		/* HPUX dump */
 
-  { DMP_SIZE, "vxdump: vxdump: [0-9][0-9]* tape blocks", 1024},
+  { DMP_SIZE, "vxdump: [0-9][0-9]* tape blocks", 1024},
 		/* HPUX 10.20 and above vxdump */
 
-  { DMP_SIZE, "vxdump: vxdump: [0-9][0-9]* blocks", 1024},
+  { DMP_SIZE, "vxdump: [0-9][0-9]* blocks", 1024},
 		/* UnixWare vxdump */
 
   { DMP_SIZE, "   VXDUMP: [0-9][0-9]* blocks",                            512},
@@ -105,13 +105,14 @@ static regex_t re_table[] = {
   /* XXX add more ERROR entries here by scanning dump sources? */
 
   /* any blank or non-strange DUMP: lines are marked as normal */
-  { DMP_NORMAL, "^  DUMP:" },
+  { DMP_NORMAL, "^ *DUMP:" },
   { DMP_NORMAL, "^dump:" },					/* OSF/1 */
   { DMP_NORMAL, "^vdump:" },					/* OSF/1 */
-  { DMP_NORMAL, "^  vxdump:" },					/* HPUX10 */
+  { DMP_NORMAL, "^ *vxdump:" },					/* HPUX10 */
+  { DMP_NORMAL, "^ *vxfs *vxdump:" },				/* Solaris */
   { DMP_NORMAL, "^xfsdump:" },					/* IRIX xfs */
-  { DMP_NORMAL, "^  VXDUMP:" },                                 /* Sinix */
-  { DMP_NORMAL, "^  UFSDUMP:" },                                /* Sinix */
+  { DMP_NORMAL, "^ *VXDUMP:" },                                 /* Sinix */
+  { DMP_NORMAL, "^ *UFSDUMP:" },                                /* Sinix */
 
 #ifdef VDUMP	/* this is for OSF/1 3.2's vdump for advfs */
   { DMP_NORMAL, "^The -s option is ignored"},			/* OSF/1 */
@@ -132,10 +133,10 @@ static regex_t re_table[] = {
 };
 
 static void start_backup(host, disk, level, dumpdate, dataf, mesgf, indexf)
-char *host;
-char *disk;
-int level, dataf, mesgf, indexf;
-char *dumpdate;
+    char *host;
+    char *disk;
+    int level, dataf, mesgf, indexf;
+    char *dumpdate;
 {
     int dumpin, dumpout;
     char *dumpkeys = NULL;
@@ -143,7 +144,6 @@ char *dumpdate;
     char *cmd = NULL;
     char *indexcmd = NULL;
     char level_str[NUM_STR_SIZE];
-    char *fstype = NULL;
 
     ap_snprintf(level_str, sizeof(level_str), "%d", level);
 
@@ -152,15 +152,26 @@ char *dumpdate;
 
     NAUGHTY_BITS;
 
-    if(compress)
-	comppid = pipespawn(COMPRESS_PATH, &dumpout, dataf, mesgf,
-			    COMPRESS_PATH,
+    if(compress) {
+	char *compopt = skip_argument;
+
 #if defined(COMPRESS_BEST_OPT) && defined(COMPRESS_FAST_OPT)
-			    compress == COMPR_BEST? 
-			        COMPRESS_BEST_OPT : COMPRESS_FAST_OPT,
+	if(compress == COMPR_BEST) {
+	    compopt = COMPRESS_BEST_OPT;
+	} else {
+	    compopt = COMPRESS_FAST_OPT;
+	}
 #endif
-			    (char *)0);
-    else {
+	comppid = pipespawn(COMPRESS_PATH, STDIN_PIPE,
+			    &dumpout, &dataf, &mesgf,
+			    COMPRESS_PATH, compopt, NULL);
+	dbprintf(("%s-gnutar: pid %ld: %s",
+		  get_pname(), (long)comppid, COMPRESS_PATH));
+	if(compopt != skip_argument) {
+	    dbprintf((" %s", compopt));
+	}
+	dbprintf(("\n"));
+    } else {
 	dumpout = dataf;
 	comppid = -1;
     }
@@ -178,8 +189,7 @@ char *dumpdate;
     /* normal dump */
 #ifdef XFSDUMP						/* { */
 #ifdef DUMP						/* { */
-    fstype = amname_to_fstype(device);
-    if (strcmp(fstype, "xfs") == 0)
+    if (strcmp(amname_to_fstype(disk), "xfs") == 0)
 #else							/* } { */
     if (1)
 #endif							/* } */
@@ -187,9 +197,6 @@ char *dumpdate;
         char *progname = cmd = newvstralloc(cmd, libexecdir, "/", "rundump",
 					    versionsuffix(), NULL);
 	program->backup_name  = XFSDUMP;
-#ifndef XFSRESTORE
-#define XFSRESTORE "xfsrestore"
-#endif
 	program->restore_name = XFSRESTORE;
 
 	indexcmd = vstralloc(XFSRESTORE,
@@ -205,25 +212,21 @@ char *dumpdate;
 	start_index(createindex, dumpout, mesgf, indexf, indexcmd);
 
 	dumpkeys = stralloc(level_str);
-	if (no_record)
-	{
-	    dumppid = pipespawn(progname, &dumpin, dumpout, mesgf,
-				"xfsdump", "-J", "-F", "-l", dumpkeys, "-",
-				device, (char *)0);
-	}
-	else
-	{
-	    dumppid = pipespawn(progname, &dumpin, dumpout, mesgf,
-				"xfsdump", "-F", "-l", dumpkeys, "-",
-				device, (char *)0);
-	}
+	dumppid = pipespawn(progname, STDIN_PIPE,
+			    &dumpin, &dumpout, &mesgf,
+			    "xfsdump",
+			    no_record ? "-J" : skip_argument,
+			    "-F",
+			    "-l", dumpkeys,
+			    "-",
+			    device,
+			    NULL);
     }
     else
 #endif							/* } */
 #ifdef VXDUMP						/* { */
 #ifdef DUMP
-    fstype = amname_to_fstype(device);
-    if (strcmp(fstype, "vxfs") == 0)
+    if (strcmp(amname_to_fstype(disk), "vxfs") == 0)
 #else
     if (1)
 #endif
@@ -235,12 +238,13 @@ char *dumpdate;
 	char *progname = cmd = newvstralloc(cmd, VXDUMP, NULL);
 #endif
 	program->backup_name  = VXDUMP;
-#ifndef VXRESTORE
-#define VXRESTORE "vxrestore"
-#endif
 	program->restore_name = VXRESTORE;
 
-	dumpkeys = vstralloc(level_str, no_record ? "" : "u", "s", "f", NULL);
+	dumpkeys = vstralloc(level_str,
+			     no_record ? "" : "u",
+			     "s",
+			     "f",
+			     NULL);
 
 	indexcmd = vstralloc(VXRESTORE,
 			     " -tvf", " -",
@@ -252,9 +256,14 @@ char *dumpdate;
 
 	start_index(createindex, dumpout, mesgf, indexf, indexcmd);
 
-	dumppid = pipespawn(progname, &dumpin, dumpout, mesgf, 
-			    "vxdump", dumpkeys, "1048576", "-", device,
-			    (char *)0);
+	dumppid = pipespawn(progname, STDIN_PIPE,
+			    &dumpin, &dumpout, &mesgf, 
+			    "vxdump",
+			    dumpkeys,
+			    "1048576",
+			    "-",
+			    device,
+			    NULL);
     }
     else
 #endif							/* } */
@@ -268,15 +277,15 @@ char *dumpdate;
     {
         char *progname = cmd = newvstralloc(cmd, libexecdir, "/", "rundump",
 					    versionsuffix(), NULL);
-	amfree(device);
-	device = amname_to_dirname(disk);
+	device = newstralloc(device, amname_to_dirname(disk));
 	program->backup_name  = VDUMP;
-#ifndef VRESTORE
-#define VRESTORE "vrestore"
-#endif
 	program->restore_name = VRESTORE;
 
-	dumpkeys = vstralloc(level_str, no_record ? "" : "u", "b", "f", NULL);
+	dumpkeys = vstralloc(level_str,
+			     no_record ? "" : "u",
+			     "b",
+			     "f",
+			     NULL);
 
 	indexcmd = vstralloc(VRESTORE,
 			     " -tvf", " -",
@@ -288,11 +297,14 @@ char *dumpdate;
 
 	start_index(createindex, dumpout, mesgf, indexf, indexcmd);
 
-	dumppid = pipespawn(cmd, &dumpin, dumpout, mesgf, 
-			    "vdump", dumpkeys,
+	dumppid = pipespawn(cmd, STDIN_PIPE,
+			    &dumpin, &dumpout, &mesgf, 
+			    "vdump",
+			    dumpkeys,
 			    "60",
-			    "-", device,
-			    (char *)0);
+			    "-",
+			    device,
+			    NULL);
     }
     else
 #endif							/* } */
@@ -302,11 +314,14 @@ char *dumpdate;
 #define RESTORE "restore"
 #endif
 
+	dumpkeys = vstralloc(level_str,
+			     no_record ? "" : "u",
+			     "s",
 #ifdef HAVE_HONOR_NODUMP
-	dumpkeys = vstralloc(level_str, no_record ? "" : "u", "s", "h", "f", NULL);
-#else
-	dumpkeys = vstralloc(level_str, no_record ? "" : "u", "s", "f", NULL);
+			     "h",
 #endif
+			     "f",
+			     NULL);
 
 	indexcmd = vstralloc(RESTORE,
 			     " -tvf", " -",
@@ -319,18 +334,25 @@ char *dumpdate;
 
 	start_index(createindex, dumpout, mesgf, indexf, indexcmd);
 
-	dumppid = pipespawn(cmd, &dumpin, dumpout, mesgf, 
-			    "dump", dumpkeys,
+	dumppid = pipespawn(cmd, STDIN_PIPE,
+			    &dumpin, &dumpout, &mesgf, 
+			    "dump",
+			    dumpkeys,
 			    "1048576",
 #ifdef HAVE_HONOR_NODUMP
 			    "0",
 #endif
-			    "-", device,
-			    (char *)0);
+			    "-",
+			    device,
+			    NULL);
     }
 #else							/* } { */
     /* AIX backup program */
-    dumpkeys = vstralloc("-", level_str, no_record ? "" : "u", "f", NULL);
+    dumpkeys = vstralloc("-",
+			 level_str,
+			 no_record ? "" : "u",
+			 "f",
+			 NULL);
 
     indexcmd = vstralloc(RESTORE,
 			 " -B",
@@ -343,13 +365,17 @@ char *dumpdate;
 
     start_index(createindex, dumpout, mesgf, indexf, indexcmd);
 
-    dumppid = pipespawn(cmd, &dumpin, dumpout, mesgf, 
-			"backup", dumpkeys, "-", device, (char *)0);
+    dumppid = pipespawn(cmd, STDIN_PIPE,
+			&dumpin, &dumpout, &mesgf, 
+			"backup",
+			dumpkeys,
+			"-",
+			device,
+			NULL);
 #endif							/* } */
 
     amfree(dumpkeys);
     amfree(device);
-    amfree(fstype);
     amfree(cmd);
     amfree(indexcmd);
 
@@ -359,7 +385,8 @@ char *dumpdate;
     aclose(dumpout);
     aclose(dataf);
     aclose(mesgf);
-    aclose(indexf);
+    if (createindex)
+	aclose(indexf);
 }
 
 static void end_backup(status)
