@@ -24,18 +24,18 @@
  * file named AUTHORS, in the root directory of this distribution.
  */
 /* 
- * $Id$
+ * $Id: sendbackup.c,v 1.44.2.11 2001/03/15 02:17:34 jrjackson Exp $
  *
  * common code for the sendbackup-* programs.
  */
 
+#include "amanda.h"
 #include "sendbackup.h"
+#include "pipespawn.h"
 #include "stream.h"
 #include "arglist.h"
-#include "../tape-src/tapeio.h"
-#include "amanda.h"
-
-#define DATABUF_SIZE	TAPE_BLOCK_BYTES
+#include "getfsent.h"
+#include "version.h"
 
 #define TIMEOUT 30
 
@@ -69,7 +69,7 @@ backup_program_t *program = NULL;
 
 /* local functions */
 int main P((int argc, char **argv));
-void parse_options P((char *str));
+void parse_options P((char *str, char *disk));
 char *optionstr P((void));
 char *childstr P((int pid));
 int check_status P((int pid, amwait_t w));
@@ -79,8 +79,9 @@ int pipefork P((void (*func) P((void)), char *fname, int *stdinfd,
 void parse_backup_messages P((int mesgin));
 static void process_dumpline P((char *str));
 
-void parse_options(str)
+void parse_options(str, disk)
 char *str;
+char *disk;
 {
     char *e, *i, *j, *k;
     int ch;
@@ -112,15 +113,33 @@ char *str;
 		e = "";
 	    }
 #undef sc
-	    if (*e != '\0' && access(j, F_OK) != 0) {
-		/* if exclude list file does not exist, ignore it.
-		 * Should not test for R_OK, because the file may be
-		 * readable by root only! */
-		dbprintf(("%s: exclude list file \"%s\" does not exist, ignoring\n",
-			  get_pname(), j));
-	        amfree(efile);
+/* BEGIN HPS */
+	    if (*e != '\0')
+		{
+		  char *file = j;
+		  if(*file != '/')
+		  {
+			char *dirname = amname_to_dirname(disk);
+			efile = vstralloc(dirname,"/",file, NULL);
+		  }
+		  else
+			efile = stralloc(file);
+		  
+		  if(access(efile, F_OK) != 0) {
+			/* if exclude list file does not exist, ignore it.
+			 * Should not test for R_OK, because the file may be
+			 * readable by root only! */
+			dbprintf(("%s: exclude list file \"%s\" does not exist, ignoring\n",
+				  get_pname(), efile));
+			amfree(efile);
+		  }
+		  estr = NULL;
 	    } else
-	        efile = newvstralloc(efile, "--exclude", e, "=", j, NULL);
+/* END HPS */
+	    {
+		  estr = stralloc(j);
+		  amfree(efile);
+	    }
 	    k[0] = ';';
 	}
     }
@@ -200,21 +219,22 @@ char **argv;
 		close(fd);
     }
 
+    safe_cd();
+
     set_pname("sendbackup");
 
     malloc_size_1 = malloc_inuse(&malloc_hist_1);
 
-    chdir("/tmp");
     interactive = (argc > 1 && strcmp(argv[1],"-t") == 0);
     erroutput_type = (ERR_INTERACTIVE|ERR_SYSLOG);
-    umask(0007);
     dbopen();
+    dbprintf(("%s: version %s\n", argv[0], version()));
 
     if(interactive) {
 	/*
 	 * In interactive (debug) mode, the backup data is sent to
-	 * /dev/null, the program does not fork and none of the network
-	 * connections back to driver programs on the tape host are set up.
+        * /dev/null and none of the network connections back to driver
+        * programs on the tape host are set up.
 	 * Index service is run and goes to stdout.
 	 */
 	fprintf(stderr, "%s: running in interactive test mode\n", get_pname());
@@ -259,7 +279,7 @@ char **argv;
 	}
     }
 
-    dbprintf(("%s: got input request: %s\n", argv[0], line));
+    dbprintf(("%s: got input request: %s\n", get_pname(), line));
 
     s = line;
     ch = *s++;
@@ -318,8 +338,11 @@ char **argv;
     }
     options = s - 1;
 
-    dbprintf(("  parsed request as: program `%s' disk `%s' lev %d since %s opt `%s'\n",
-	      prog, disk, level, dumpdate, options));
+    dbprintf(("  parsed request as: program `%s'\n", prog));
+    dbprintf(("                     disk `%s'\n", disk));
+    dbprintf(("                     lev %d\n", level));
+    dbprintf(("                     since %s\n", dumpdate));
+    dbprintf(("                     opt `%s'\n", options));
 
     {
       int i;
@@ -329,48 +352,47 @@ char **argv;
       if (programs[i]) {
 	program = programs[i];
       } else {
-	dbprintf(("ERROR [%s: unknown program %s]\n", argv[0], prog));
-	error("ERROR [%s: unknown program %s]\n", argv[0], prog);
+	error("ERROR [%s: unknown program %s]", get_pname(), prog);
       }
     }
 
-    parse_options(options);
+    parse_options(options, disk);
 
 #ifdef KRB4_SECURITY
     if(krb4_auth) {
 	if(read(KEY_PIPE, session_key, sizeof session_key) 
 	   != sizeof session_key) {
-	  dbprintf(("ERROR [%s: could not read session key]\n", argv[0]));
-	  error("ERROR [%s: could not read session key]\n", argv[0]);
+	  error("ERROR [%s: could not read session key]", get_pname());
 	}
     }
 #endif
 
     if(!interactive) {
-      data_socket = stream_server(&data_port, DATABUF_SIZE*2, DATABUF_SIZE*2);
+      data_socket = stream_server(&data_port, DATABUF_SIZE*2, DEFAULT_SIZE);
+      if(data_socket < 0) {
+	error("ERROR [%s: could not create data socket: %s]",
+	      get_pname(), strerror(errno));
+      }
       mesg_socket = stream_server(&mesg_port, DEFAULT_SIZE, DEFAULT_SIZE);
+      if(mesg_socket < 0) {
+	error("ERROR [%s: could not create mesg socket: %s]",
+	      get_pname(), strerror(errno));
+      }
     }
     if (!interactive && createindex) {
       index_socket = stream_server(&index_port, DEFAULT_SIZE, DEFAULT_SIZE);
+      if(index_socket < 0) {
+	error("ERROR [%s: could not create index socket: %s]",
+	      get_pname(), strerror(errno));
+      }
     } else {
       index_port = -1;
     }
 
-    if(!interactive) {
-      switch(fork()) {
-      case -1:	/* fork error */
-	s = strerror(errno);
-        dbprintf(("ERROR [%s: could not fork: %s]\n", argv[0], s));
-        error("ERROR [%s: could not fork: %s]\n", argv[0], s);
-      default:	/* parent */
-        printf("CONNECT DATA %d MESG %d INDEX %d\n",
-	       data_port, mesg_port, index_port);
-        printf("OPTIONS %s\n", optionstr());
-        exit(0);
-      case 0:	/* child, keep going */
-        break;
-      }
-    }
+    printf("CONNECT DATA %d MESG %d INDEX %d\n",
+	   data_port, mesg_port, index_port);
+    printf("OPTIONS %s\n", optionstr());
+    freopen("/dev/null","w",stdout);
 
     if (createindex)
       dbprintf(("  waiting for connect on %d, then %d, then %d\n",
@@ -381,21 +403,18 @@ char **argv;
 
     if(interactive) {
       if((dataf = open("/dev/null", O_RDWR)) < 0) {
-	s = strerror(errno);
-	dbprintf(("ERROR [%s: open of /dev/null for debug data stream: %s]\n",
-		  argv[0], s));
-	error("ERROR [%s: open of /dev/null for debug data stream: %s]\n",
-		  argv[0], s);
+	error("ERROR [%s: open of /dev/null for debug data stream: %s]",
+	      get_pname(), strerror(errno));
       }
       mesgf = 2;
     } else {
       dataf = stream_accept(data_socket, TIMEOUT, DEFAULT_SIZE, DEFAULT_SIZE);
       if(dataf == -1) {
-        dbprintf(("%s: timeout on data port %d\n", argv[0], data_port));
+        dbprintf(("%s: timeout on data port %d\n", get_pname(), data_port));
       }
       mesgf = stream_accept(mesg_socket, TIMEOUT, DEFAULT_SIZE, DEFAULT_SIZE);
       if(mesgf == -1) {
-        dbprintf(("%s: timeout on mesg port %d\n", argv[0], mesg_port));
+        dbprintf(("%s: timeout on mesg port %d\n", get_pname(), mesg_port));
       }
     }
     if(interactive) {
@@ -404,7 +423,7 @@ char **argv;
       indexf = stream_accept(index_socket,
 			     TIMEOUT, DEFAULT_SIZE, DEFAULT_SIZE);
       if (indexf == -1) {
-	dbprintf(("%s: timeout on index port %d\n", argv[0], index_port));
+	dbprintf(("%s: timeout on index port %d\n", get_pname(), index_port));
       }
     }
 
@@ -432,7 +451,7 @@ char **argv;
 	    exit(1);
         }
 
-        dbprintf(("%s: kerberos handshakes succeeded!\n", argv[0]));
+        dbprintf(("%s: kerberos handshakes succeeded!\n", get_pname()));
       }
     }
 #endif
@@ -447,9 +466,7 @@ char **argv;
     }
 
     if(pipe(mesgpipe) == -1) {
-      s = strerror(errno);
-      dbprintf(("error [opening mesg pipe: %s]\n", s));
-      error("error [opening mesg pipe: %s]", s);
+      error("error [opening mesg pipe: %s]", strerror(errno));
     }
 
     program->start_backup(host, disk, level, dumpdate, dataf, mesgpipe[1],
@@ -503,31 +520,68 @@ amwait_t w;
 {
     char *thiserr = NULL;
     char *str;
-    int ret, sig;
+    int ret, sig, rc;
     char number[NUM_STR_SIZE];
 
-    if(WIFSIGNALED(w))	ret = 0, sig = WTERMSIG(w);
-    else sig = 0, ret = WEXITSTATUS(w);
-
     str = childstr(pid);
 
+    if(WIFSIGNALED(w)) {
+	ret = 0;
+	rc = sig = WTERMSIG(w);
+    } else {
+	sig = 0;
+	rc = ret = WEXITSTATUS(w);
+    }
+
+    if(pid == indexpid) {
+	/*
+	 * Treat an index failure (other than signal) as a "STRANGE"
+	 * rather than an error so the dump goes ahead and gets processed
+	 * but the failure is noted.
+	 */
+	if(ret != 0) {
+	    fprintf(stderr, "? %s returned %d\n", str, ret);
+	    rc = 0;
+	}
+    }
+
 #ifndef HAVE_GZIP
-    if(pid == comppid) 	/* compress returns 2 sometimes; it's ok */
-	if(ret == 2) return 0;
+    if(pid == comppid) {
+	/*
+	 * compress returns 2 sometimes, but it is ok.
+	 */
+	if(ret == 2) {
+	    rc = 0;
+	}
+    }
 #endif
 
 #ifdef DUMP_RETURNS_1
-    if(pid == dumppid && tarpid == -1)
-        /* Ultrix dump returns 1 sometimes; it's ok too */
-        if(ret == 1) return 0;
+    if(pid == dumppid && tarpid == -1) {
+        /*
+	 * Ultrix dump returns 1 sometimes, but it is ok.
+	 */
+        if(ret == 1) {
+	    rc = 0;
+	}
+    }
 #endif
 
 #ifdef IGNORE_TAR_ERRORS
-    if(pid == tarpid)
-	/* tar bitches about active filesystems, but I don't care */
-        if(ret == 2) return 0;
+    if(pid == tarpid) {
+	/*
+	 * tar bitches about active filesystems, but we do not care.
+	 */
+        if(ret == 2) {
+	    rc = 0;
+	}
+    }
 #endif
 
+    if(rc == 0) {
+	return 0;				/* normal exit */
+    }
+
     if(ret == 0) {
 	ap_snprintf(number, sizeof(number), "%d", sig);
 	thiserr = vstralloc(str, " got signal ", number, NULL);
@@ -573,87 +627,6 @@ void write_tapeheader()
     fprintf(stderr, "%s: info end\n", get_pname());
 }
 
-#ifdef STDC_HEADERS
-int pipespawn(char *prog, int *stdinfd, int stdoutfd, int stderrfd, ...)
-#else
-int pipespawn(prog, stdinfd, stdoutfd, stderrfd, va_alist)
-char *prog;
-int *stdinfd;
-int stdoutfd, stderrfd;
-va_dcl
-#endif
-{
-    va_list ap;
-#define MAX_PIPESPAWN_ARGS	32
-    char *argv[MAX_PIPESPAWN_ARGS+1];
-    int pid, i, inpipe[2];
-    char *e;
-
-    dbprintf(("%s: spawning \"%s\" in pipeline\n", get_pname(), prog));
-    dbprintf(("%s: argument list:", get_pname()));
-    arglist_start(ap, stderrfd);		/* setup argv */
-    for(i = 0; i < MAX_PIPESPAWN_ARGS; i++) {
-	char *arg = arglist_val(ap, char *);
-	if (arg == NULL)
-	    break;
-	dbprintf((" \"%s\"", arg));
-    }
-    arglist_end(ap);
-    dbprintf(("\n"));
-
-    if(pipe(inpipe) == -1) {
-      e = strerror(errno);
-      dbprintf(("error [open pipe to %s: %s]\n", prog, e));
-      error("error [open pipe to %s: %s]", prog, e);
-    }
-
-    switch(pid = fork()) {
-    case -1:
-      e = strerror(errno);
-      dbprintf(("error [fork %s: %s]\n", prog, e));
-      error("error [fork %s: %s]", prog, e);
-    default:	/* parent process */
-	aclose(inpipe[0]);	/* close input side of pipe */
-	*stdinfd = inpipe[1];
-	break;
-    case 0:		/* child process */
-	aclose(inpipe[1]);	/* close output side of pipe */
-
-	if(dup2(inpipe[0], 0) == -1) {
-	  e = strerror(errno);
-	  dbprintf(("error [spawn %s: dup2 in: %s]\n", prog, e));
-	  error("error [spawn %s: dup2 in: %s]", prog, e);
-	}
-	if(dup2(stdoutfd, 1) == -1) {
-	  e = strerror(errno);
-	  dbprintf(("error [spawn %s: dup2 out: %s]\n", prog, e));
-	  error("error [spawn %s: dup2 out: %s]", prog, e);
-	}
-	if(dup2(stderrfd, 2) == -1) {
-	  e = strerror(errno);
-	  dbprintf(("error [spawn %s: dup2 err: %s]\n", prog, e));
-	  error("error [spawn %s: dup2 err: %s]", prog, e);
-	}
-
-	arglist_start(ap, stderrfd);		/* setup argv */
-	for(i = 0; i < MAX_PIPESPAWN_ARGS; i++) {
-            if ((argv[i] = arglist_val(ap, char *)) == NULL) {
-		break;
-	    }
-	}
-	argv[i] = NULL;
-	arglist_end(ap);
-
-	execve(prog, argv, safe_env());
-	e = strerror(errno);
-	dbprintf(("error [exec %s: %s]\n", prog, e));
-	error("error [exec %s: %s]", prog, e);
-	/* NOTREACHED */
-    }
-    return pid;
-}
-
-
 int pipefork(func, fname, stdinfd, stdoutfd, stderrfd)
 void (*func) P((void));
 char *fname;
@@ -661,21 +634,16 @@ int *stdinfd;
 int stdoutfd, stderrfd;
 {
     int pid, inpipe[2];
-    char *e;
 
     dbprintf(("%s: forking function %s in pipeline\n", get_pname(), fname));
 
     if(pipe(inpipe) == -1) {
-      e = strerror(errno);
-      dbprintf(("error [open pipe to %s: %s]\n", fname, e));
-      error("error [open pipe to %s: %s]", fname, e);
+	error("error [open pipe to %s: %s]", fname, strerror(errno));
     }
 
     switch(pid = fork()) {
     case -1:
-      e = strerror(errno);
-      dbprintf(("error [fork %s: %s]\n", fname, e));
-      error("error [fork %s: %s]", fname, e);
+	error("error [fork %s: %s]", fname, strerror(errno));
     default:	/* parent process */
 	aclose(inpipe[0]);	/* close input side of pipe */
 	*stdinfd = inpipe[1];
@@ -684,19 +652,13 @@ int stdoutfd, stderrfd;
 	aclose(inpipe[1]);	/* close output side of pipe */
 
 	if(dup2(inpipe[0], 0) == -1) {
-	  e = strerror(errno);
-	  dbprintf(("error [fork %s: dup2 in: %s]\n", fname, e));
-	  error("error [fork %s: dup2 in: %s]", fname, e);
+	    error("error [dup2 0 %s: dup2 in: %s]", fname, strerror(errno));
 	}
 	if(dup2(stdoutfd, 1) == -1) {
-	  e = strerror(errno);
-	  dbprintf(("error [fork %s: dup2 out: %s]\n", fname, e));
-	  error("error [fork %s: dup2 out: %s]", fname, e);
+	    error("error [dup2 1 %s: dup2 out: %s]", fname, strerror(errno));
 	}
 	if(dup2(stderrfd, 2) == -1) {
-	  e = strerror(errno);
-	  dbprintf(("error [fork %s: dup2 err: %s]\n", fname, e));
-	  error("error [fork %s: dup2 err: %s]", fname, e);
+	    error("error [dup2 2 %s: dup2 err: %s]", fname, strerror(errno));
 	}
 
 	func();
@@ -721,26 +683,17 @@ int mesgin;
     }
 
     if(errno) {
-	char *s = strerror(errno);
-
-	dbprintf(("error [read mesg pipe: %s]\n", s));
-	error("error [read mesg pipe: %s]", s);
-	/* NOTREACHED */
+	error("error [read mesg pipe: %s]", strerror(errno));
     }
 
     while((wpid = wait(&retstat)) != -1) {
-	/* we know that it exited, so we don't have to check WIFEXITED */
-	if((WIFSIGNALED(retstat) || WEXITSTATUS(retstat)) &&
-	   check_status(wpid, retstat))
-	    goterror = 1;
+	if(check_status(wpid, retstat)) goterror = 1;
     }
 
-    if(goterror) {
-      dbprintf(("error [%s]\n", errorstr ? errorstr : "(empty errorstr?)"));
-      error("error [%s]", errorstr ? errorstr : "(empty errorstr?)");
+    if(errorstr) {
+	error("error [%s]", errorstr);
     } else if(dump_size == -1) {
-      dbprintf(("error [no backup size line]\n"));
-      error("error [no backup size line]");
+	error("error [no backup size line]");
     }
 
     program->end_backup(goterror);
@@ -863,29 +816,23 @@ char *cmd;
   struct sigaction act, oact;
   int pipefd[2];
   FILE *pipe_fp;
-  char *e;
+  int exitcode;
 
   if (!createindex)
     return;
 
   if (pipe(pipefd) != 0) {
-    e = strerror(errno);
-    dbprintf(("creating index pipe: %s\n", e));
-    error("creating index pipe: %s", e);
+    error("creating index pipe: %s", strerror(errno));
   }
 
   switch(indexpid = fork()) {
   case -1:
-    e = strerror(errno);
-    dbprintf(("forking index tee process: %s\n", e));
-    error("forking index tee process: %s", e);
+    error("forking index tee process: %s", strerror(errno));
 
   default:
     aclose(pipefd[0]);
     if (dup2(pipefd[1], input) == -1) {
-      e = strerror(errno);
-      dbprintf(("dup'ping index tee output: %s", e));
-      error("dup'ping index tee output: %s", e);
+      error("dup'ping index tee output: %s", strerror(errno));
     }
     aclose(pipefd[1]);
     return;
@@ -918,15 +865,11 @@ char *cmd;
   sigemptyset(&act.sa_mask);
   act.sa_flags = 0;
   if (sigaction(SIGPIPE, &act, &oact) != 0) {
-    e = strerror(errno);
-    dbprintf(("couldn't set index SIGPIPE handler [%s]\n", e));
-    error("couldn't set index SIGPIPE handler [%s]", e);
+    error("couldn't set index SIGPIPE handler [%s]", strerror(errno));
   }
 
   if ((pipe_fp = popen(cmd, "w")) == NULL) {
-    e = strerror(errno);
-    dbprintf(("couldn't start index creator [%s]\n", e));
-    error("couldn't start index creator [%s]", e);
+    error("couldn't start index creator [%s]", strerror(errno));
   }
 
   dbprintf(("%s: started index creator: \"%s\"\n", get_pname(), cmd));
@@ -941,9 +884,7 @@ char *cmd;
       continue;
 
     if (bytes_read < 0) {
-      e = strerror(errno);
-      dbprintf(("index tee cannot read [%s]\n", e));
-      error("index tee cannot read [%s]", e);
+      error("index tee cannot read [%s]", strerror(errno));
     }
 
     if (bytes_read == 0)
@@ -958,13 +899,8 @@ char *cmd;
 	  /* the signal handler may have assigned to index_finished
 	   * just as we waited for write() to complete. */
 	  if (!index_finished) {
-	      e = strerror(errno);
-	      dbprintf(("index tee cannot write to index creator [%s]\n", e));
-#if 0
-	      /* only write debugging info for write error to index
-	       * pipe. */
-	      error("index tee cannot write to index creator [%s]", e);
-#endif
+	      dbprintf(("index tee cannot write to index creator [%s]\n",
+			strerror(errno)));
 	      index_finished = 1;
 	}
       } else {
@@ -982,9 +918,7 @@ char *cmd;
       if ((just_written < 0) && (errno == EINTR))
 	continue;
       if (just_written < 0) {
-	e = strerror(errno);
-	dbprintf(("index tee cannot write [%s]\n", e));
-	error("index tee cannot write [%s]", e);
+	error("index tee cannot write [%s]", strerror(errno));
       } else {
 	bytes_written += just_written;
 	ptr += just_written;
@@ -996,21 +930,20 @@ char *cmd;
 
   /* finished */
   /* check the exit code of the pipe and moan if not 0 */
-  if ((pipefd[0] = pclose(pipe_fp)) != 0) {
-    e = strerror(errno);
-    dbprintf(("index pipe returned %d [%s]\n", pipefd[0], e));
-    error("index pipe returned %d [%s]", pipefd[0], e);
+  if ((exitcode = pclose(pipe_fp)) != 0) {
+    dbprintf(("%s: index pipe returned %d\n", get_pname(), exitcode));
+  } else {
+    dbprintf(("%s: index created successfully\n", get_pname()));
   }
   pipe_fp = NULL;
 
-  dbprintf(("%s: index created successfully\n", get_pname()));
-  exit(0);
+  exit(exitcode);
 }
 
-extern backup_program_t dump_program, backup_program;
+extern backup_program_t dump_program, gnutar_program;
 
 backup_program_t *programs[] = {
-  &dump_program, &backup_program, NULL
+  &dump_program, &gnutar_program, NULL
 };
 
 #ifdef KRB4_SECURITY
