@@ -24,14 +24,15 @@
  * file named AUTHORS, in the root directory of this distribution.
  */
 /* 
- * $Id$
+ * $Id: sendbackup.h,v 1.11.4.1 2000/05/27 21:20:32 vectro Exp $
  *
  * a few common decls for the sendbackup-* sources
  */
 #include "amanda.h"
+#include "pipespawn.h"
 
 void write_tapeheader P((void));
-int pipespawn P((char *prog, int *stdinfd, int stdoutfd, int stderrfd, ...));
+
 void start_index P((int createindex, int input, int mesg, 
 		    int index, char *cmd));
 
@@ -76,3 +77,6 @@ extern backup_program_t *programs[], *program;
 
 #define COMPR_FAST 1
 #define COMPR_BEST 2
+
+extern char *efile;
+extern char *estr;
