@@ -24,13 +24,15 @@
  * file named AUTHORS, in the root directory of this distribution.
  */
 /* 
- * $Id$
+ * $Id: sendsize.c,v 1.97.2.14 2001/03/15 02:17:34 jrjackson Exp $
  *
  * send estimated backup sizes using dump
  */
 
 #include "amanda.h"
+#include "pipespawn.h"
 #include "amandates.h"
+#include "util.h"
 #include "getfsent.h"
 #include "version.h"
 
@@ -125,13 +127,13 @@ char **argv;
 	close(fd);
     }
 
+    safe_cd();
+
     set_pname("sendsize");
 
     malloc_size_1 = malloc_inuse(&malloc_hist_1);
 
-    chdir("/tmp");
     erroutput_type = (ERR_INTERACTIVE|ERR_SYSLOG);
-    umask(0007);
     dbopen();
     dbprintf(("%s: version %s\n", argv[0], version()));
 
@@ -218,7 +220,7 @@ char **argv;
 	s[-1] = '\0';
 
 	spindle = 0;				/* default spindle */
-	amfree(exclude);				/* default is no exclude */
+	amfree(exclude);			/* default is no exclude */
 
 	skip_whitespace(s, ch);			/* find the spindle */
 	if(ch != '\0') {
@@ -360,7 +362,8 @@ disk_estimates_t *est;
       case 0:
 	break;
       case -1:
-        error("calc_estimates: fork returned: %s", strerror(errno));
+        error("%s: calc_estimates: fork returned: %s",
+	      get_pname(), strerror(errno));
       default:
 	return;
       }
@@ -432,7 +435,7 @@ disk_estimates_t *est;
 
     switch(calcpid = fork()) {
     case -1:
-        error("%s: fork returned: %s", cmd, strerror(errno));
+        error("%s: %s: fork returned: %s", get_pname(), cmd, strerror(errno));
     default:
         break;
     case 0:
@@ -458,7 +461,7 @@ disk_estimates_t *est;
 /* local functions */
 void dump_calc_estimates P((disk_estimates_t *est));
 long getsize_dump P((char *disk, int level));
-long getsize_smbtar P((char *disk, int level));
+long getsize_smbtar P((char *disk, int level, char* exclude));
 long getsize_gnutar P((char *disk, int level,
 		       char *exclude, time_t dumpsince));
 long handle_dumpline P((char *str));
@@ -499,7 +502,7 @@ disk_estimates_t *est;
 	if(est->est[level].needestimate) {
 	    dbprintf(("%s: getting size via smbclient for %s level %d\n",
 		      get_pname(), est->amname, level));
-	    size = getsize_smbtar(est->amname, level);
+	    size = getsize_smbtar(est->amname, level, est->exclude);
 
 	    amflock(1, "size");
 
@@ -547,42 +550,56 @@ typedef struct regex_s {
 } regex_t;
 
 regex_t re_size[] = {
+#ifdef DUMP
     {"  DUMP: estimated -*[0-9][0-9]* tape blocks", 1024},
     {"  DUMP: [Ee]stimated [0-9][0-9]* blocks", 512},
     {"  DUMP: [Ee]stimated [0-9][0-9]* bytes", 1},	       /* Ultrix 4.4 */
     {" UFSDUMP: estimated [0-9][0-9]* blocks", 512},           /* NEC EWS-UX */
-    {"vdump: Dumping [0-9][0-9]* bytes, ", 1},		      /* OSF/1 vdump */
     {"dump: Estimate: [0-9][0-9]* tape blocks", 1024},		    /* OSF/1 */
-    {"vxdump: estimated [0-9][0-9]* blocks", 512},          /* HPUX's vxdump */
     {"backup: There are an estimated [0-9][0-9]* tape blocks.",1024}, /* AIX */
     {"backup: estimated [0-9][0-9]* 1k blocks", 1024},		      /* AIX */
     {"backup: estimated [0-9][0-9]* tape blocks", 1024},	      /* AIX */
     {"backup: [0-9][0-9]* tape blocks on [0-9][0-9]* tape(s)",1024},  /* AIX */
     {"backup: [0-9][0-9]* 1k blocks on [0-9][0-9]* volume(s)",1024},  /* AIX */
-    {"[0-9][0-9]* blocks, [0-9][0-9]*.[0-9][0-9]* volumes", 1024},
-                                                          /* DU 3.2g dump -E */
     {"dump: Estimate: [0-9][0-9]* blocks being output to pipe",1024},
                                                               /* DU 4.0 dump */
     {"dump: Dumping [0-9][0-9]* bytes, ", 1},                /* DU 4.0 vdump */
     {"DUMP: estimated [0-9][0-9]* KB output", 1024},                 /* HPUX */
-    {"xfsdump: estimated dump size: [0-9][0-9]* bytes", 1},  /* Irix 6.2 xfs */
     {"  UFSDUMP: estimated [0-9][0-9]* blocks", 512},               /* Sinix */
-    {"  VXDUMP: estimated [0-9][0-9]* blocks", 512},                /* Sinix */
-    {"Total bytes written: [0-9][0-9]*", 1},		    /* Gnutar client */
-
-#ifdef SAMBA_CLIENT
-    {"Total bytes listed: [0-9][0-9]*", 1},		     /* Samba client */
-#endif
 
 #ifdef HAVE_DUMP_ESTIMATE
-# ifdef SAMBA_CLIENT
-    /* On DU 4.0, dump -E prints a line that matches an output line of
-       smbclient.  So, even if both are enabled by configure, dump
-       estimates will be disabled here.  */
-#  undef HAVE_DUMP_ESTIMATE
-# else
-    {"[0-9][0-9]* blocks", 1024},			  /* DU 4.0 dump  -E */
-# endif
+    {"[0-9][0-9]* blocks, [0-9][0-9]*.[0-9][0-9]* volumes", 1024},
+                                                          /* DU 3.2g dump -E */
+    {"^[0-9][0-9]* blocks$", 1024},			  /* DU 4.0 dump  -E */
+    {"^[0-9][0-9]*$", 1},			       /* Solaris ufsdump -S */
+#endif
+#endif
+
+#ifdef VDUMP
+    {"vdump: Dumping [0-9][0-9]* bytes, ", 1},		      /* OSF/1 vdump */
+#endif
+    
+#ifdef VXDUMP
+    {"vxdump: estimated [0-9][0-9]* blocks", 512},          /* HPUX's vxdump */
+    {"  VXDUMP: estimated [0-9][0-9]* blocks", 512},                /* Sinix */
+#endif
+
+#ifdef XFSDUMP
+    {"xfsdump: estimated dump size: [0-9][0-9]* bytes", 1},  /* Irix 6.2 xfs */
+#endif
+    
+#ifdef GNUTAR
+    {"Total bytes written: [0-9][0-9]*", 1},		    /* Gnutar client */
+#endif
+
+#ifdef SAMBA_CLIENT
+#if SAMBA_VERSION >= 2
+#define SAMBA_DEBUG_LEVEL "0"
+    {"Total number of bytes: [0-9][0-9]*", 1},			 /* Samba du */
+#else
+#define SAMBA_DEBUG_LEVEL "3"
+    {"Total bytes listed: [0-9][0-9]*", 1},			/* Samba dir */
+#endif
 #endif
 
     { NULL, 0 }
@@ -590,10 +607,10 @@ regex_t re_size[] = {
 
 
 long getsize_dump(disk, level)
-char *disk;
-int level;
+    char *disk;
+    int level;
 {
-    int pipefd[2], nullfd, killctl[2];
+    int pipefd[2], nullfd, stdoutfd, killctl[2];
     pid_t dumppid;
     long size;
     FILE *dumpout;
@@ -614,7 +631,7 @@ int level;
     cmd = vstralloc(libexecdir, "/rundump", versionsuffix(), NULL);
     rundump_cmd = stralloc(cmd);
 
-    nullfd = open("/dev/null", O_RDWR);
+    stdoutfd = nullfd = open("/dev/null", O_RDWR);
     pipefd[0] = pipefd[1] = killctl[0] = killctl[1] = -1;
     pipe(pipefd);
 
@@ -689,13 +706,17 @@ int level;
 # else							/* } { */
 	dumpkeys = vstralloc(level_str,
 #  ifdef HAVE_DUMP_ESTIMATE				/* { */
-			     "E",
+			     HAVE_DUMP_ESTIMATE,
 #  endif						/* } */
 #  ifdef HAVE_HONOR_NODUMP				/* { */
 			     "h",
 #  endif						/* } */
 			     "s", "f", NULL);
 
+#  ifdef HAVE_DUMP_ESTIMATE
+	stdoutfd = pipefd[1];
+#  endif
+
 #  ifdef HAVE_HONOR_NODUMP				/* { */
 	dbprintf(("%s: running \"%s%s %s 0 1048576 - %s\"\n",
 		  get_pname(), cmd, name, dumpkeys, device));
@@ -717,6 +738,7 @@ int level;
 
     switch(dumppid = fork()) {
     case -1:
+	dbprintf(("cannot fork for killpgrp: %s\n", strerror(errno)));
 	amfree(dumpkeys);
 	amfree(cmd);
 	amfree(rundump_cmd);
@@ -748,7 +770,8 @@ int level;
 		close(killctl[1]);
 		close(nullfd);
 		execle(killpgrp_cmd, killpgrp_cmd, (char *)0, safe_env());
-		dbprintf(("cannot execute %s\n", killpgrp_cmd));
+		dbprintf(("cannot execute %s: %s\n", killpgrp_cmd,
+		    strerror(errno)));
 		exit(-1);
 	    }
 
@@ -758,7 +781,7 @@ int level;
 	}
 
 	dup2(nullfd, 0);
-	dup2(nullfd, 1);
+	dup2(stdoutfd, 1);
 	dup2(pipefd[1], 2);
 	aclose(pipefd[0]);
 	if (killctl[0] != -1)
@@ -808,11 +831,8 @@ int level;
 # endif
 #endif
 	{
-	  dbprintf(("%s: exec %s failed or no dump program available",
-		    get_pname(), cmd));
-	  error("%s: exec %s failed or no dump program available",
-		get_pname(), cmd);
-	  exit(1);
+	    error("%s: exec %s failed or no dump program available: %s",
+		  get_pname(), cmd, strerror(errno));
 	}
     }
 
@@ -894,89 +914,93 @@ int level;
 }
 
 #ifdef SAMBA_CLIENT
-long getsize_smbtar(disk, level)
-char *disk;
-int level;
+long getsize_smbtar(disk, level, exclude_spec)
+    char *disk;
+    int level;
+    char *exclude_spec;
 {
-    int pipefd[2], nullfd, dumppid;
+    int pipefd = -1, nullfd = -1, passwdfd = -1;
+    int dumppid;
     long size;
     FILE *dumpout;
-    char *tarkeys, *sharename, *pass, *domain = NULL;
+    char *tarkeys, *sharename, *user_and_password = NULL, *domain = NULL;
+    int lpass;
+    char *pwtext;
     char *line;
 
-    if ((pass = findpass(disk, &domain)) == NULL) {
-	error("[sendsize : error in smbtar diskline, unable to find password]");
-    }
-    if ((sharename = makesharename(disk, 0)) == NULL) {
-	memset(pass, '\0', strlen(pass));
-	amfree(pass);
+    if ((user_and_password = findpass(disk, &domain)) == NULL) {
 	if(domain) {
 	    memset(domain, '\0', strlen(domain));
 	    amfree(domain);
 	}
-	error("[sendsize : can't make share name of %s]", disk);
+	error("%s-smbtar: cannot find password for %s", get_pname(), disk);
+    }
+    lpass = strlen(user_and_password);
+    if ((pwtext = strchr(user_and_password, '%')) == NULL) {
+	memset(user_and_password, '\0', lpass);
+	amfree(user_and_password);
+	if(domain) {
+	    memset(domain, '\0', strlen(domain));
+	    amfree(domain);
+	}
+	error("%s-smbtar: password field not \'user%%pass\' for %s",
+	      get_pname(), disk);
+    }
+    *pwtext++ = '\0';
+    if ((sharename = makesharename(disk, 0)) == NULL) {
+	memset(user_and_password, '\0', lpass);
+	amfree(user_and_password);
+	if(domain) {
+	    memset(domain, '\0', strlen(domain));
+	    amfree(domain);
+	}
+	error("%s-smbtar: cannot make share name of %s", get_pname(), disk);
     }
     nullfd = open("/dev/null", O_RDWR);
-    pipe(pipefd);
+
+#if SAMBA_VERSION >= 2
+    if (level == 0)
+	tarkeys = "archive 0;recurse;du";
+    else
+	tarkeys = "archive 1;recurse;du";
+#else
     if (level == 0)
 	tarkeys = "archive 0;recurse;dir";
     else
 	tarkeys = "archive 1;recurse;dir";
+#endif
 
-    dbprintf(("%s: running \"%s \'%s\' %s -d 3 -U %s -E%s%s -c \'%s\'\"\n",
-	      get_pname(), SAMBA_CLIENT, sharename, "XXXXX", SAMBA_USER,
-	      domain ? " -W " : "", domain ? domain : "",
-	      tarkeys));
-
-    switch(dumppid = fork()) {
-    case -1:
-	memset(pass, '\0', strlen(pass));
-	amfree(pass);
-	if(domain) {
-	    memset(domain, '\0', strlen(domain));
-	    amfree(domain);
-	}
-	amfree(sharename);
-	return -1;
-    default:
-	amfree(sharename);
-	break; 
-    case 0:   /* child process */
-	dup2(nullfd, 0);
-	dup2(nullfd, 1);
-	dup2(pipefd[1], 2);
-	aclose(pipefd[0]);
-
-	execle(SAMBA_CLIENT, "smbclient", sharename, pass,
-	       "-d", "3",
-	       "-U", SAMBA_USER,
-	       "-E",
-	       domain ? "-W" : "-c",
-	       domain ? domain : tarkeys,
-	       domain ? "-c" : (char *)0,
-	       domain ? tarkeys : (char *)0,
-	       (char *)0,
-	       safe_env());
-	/* should not get here */
-	memset(pass, '\0', strlen(pass));
-	amfree(pass);
-	if(domain) {
-	    memset(domain, '\0', strlen(domain));
-	    amfree(domain);
-	}
-	amfree(sharename);
-	exit(1);
-	/* NOTREACHED */
-    }
-    memset(pass, '\0', strlen(pass));
-    amfree(pass);
+    dumppid = pipespawn(SAMBA_CLIENT, STDERR_PIPE|PASSWD_PIPE,
+	      &nullfd, &nullfd, &pipefd, 
+	      "PASSWD_FD", &passwdfd,
+	      "smbclient",
+	      sharename,
+	      "-d", SAMBA_DEBUG_LEVEL,
+	      "-U", user_and_password,				/* user only */
+	      "-E",
+	      domain ? "-W" : skip_argument,
+	      domain ? domain : skip_argument,
+	      "-c", tarkeys,
+	      NULL);
     if(domain) {
 	memset(domain, '\0', strlen(domain));
 	amfree(domain);
     }
+    aclose(nullfd);
+    if(fullwrite(passwdfd, pwtext, strlen(pwtext)) < 0) {
+	int save_errno = errno;
+
+	memset(user_and_password, '\0', lpass);
+	amfree(user_and_password);
+	aclose(passwdfd);
+	error("%s-smbtar: password write failed: %s",
+	      get_pname(), strerror(save_errno));
+    }
+    memset(user_and_password, '\0', lpass);
+    amfree(user_and_password);
+    aclose(passwdfd);
     amfree(sharename);
-    aclose(pipefd[1]);
-    dumpout = fdopen(pipefd[0],"r");
+    dumpout = fdopen(pipefd,"r");
 
     for(size = -1; (line = agets(dumpout)) != NULL; free(line)) {
 	dbprintf(("%s\n",line));
@@ -1001,8 +1025,8 @@ int level;
 
     wait(NULL);
 
-    aclose(nullfd);
     afclose(dumpout);
+    pipefd = -1;
 
     return size;
 }
@@ -1015,141 +1039,109 @@ int level;
 char *exclude_spec;
 time_t dumpsince;
 {
-    int pipefd[2], nullfd, dumppid;
-    long size;
-    FILE *dumpout;
+    int pipefd = -1, nullfd = -1, dumppid;
+    long size = -1;
+    FILE *dumpout = NULL;
     char *incrname = NULL;
+    char *basename = NULL;
     char *dirname = NULL;
+    char *inputname = NULL;
+    FILE *in = NULL;
+    FILE *out = NULL;
     char *exclude_arg = NULL;
+    char *efile = NULL, *estr = NULL;
     char *line = NULL;
     char *cmd = NULL;
-    char *cmd_line;
     char dumptimestr[80];
     struct tm *gmtm;
 
 #ifdef GNUTAR_LISTED_INCREMENTAL_DIR
     {
-      char *basename = NULL;
-      char number[NUM_STR_SIZE];
-      char *s;
-      int ch;
+	char number[NUM_STR_SIZE];
+	char *s;
+	int ch;
+	int baselevel;
 
-      basename = vstralloc(GNUTAR_LISTED_INCREMENTAL_DIR,
-			   "/",
-			   host,
-			   disk,
-			   NULL);
-      /*
-       * The loop starts at the first character of the host name,
-       * not the '/'.
-       */
-      s = basename + sizeof(GNUTAR_LISTED_INCREMENTAL_DIR);
-      while((ch = *s++) != '\0') {
-	if(ch == '/' || isspace(ch)) s[-1] = '_';
-      }
-
-      ap_snprintf(number, sizeof(number), "%d", level);
-      incrname = vstralloc(basename, "_", number, ".new", NULL);
-      unlink(incrname);
-      umask(0007);
-
-      if (level == 0) {
-	FILE *out;
-
-notincremental:
-
-	out = fopen(incrname, "w");
-	if (out == NULL) {
-	  dbprintf(("error [opening %s: %s]\n", incrname, strerror(errno)));
-	  amfree(incrname);
-	  amfree(basename);
-	  amfree(dirname);
-	  return -1;
+	basename = vstralloc(GNUTAR_LISTED_INCREMENTAL_DIR,
+			     "/",
+			     host,
+			     disk,
+			     NULL);
+	/*
+	 * The loop starts at the first character of the host name,
+	 * not the '/'.
+	 */
+	s = basename + sizeof(GNUTAR_LISTED_INCREMENTAL_DIR);
+	while((ch = *s++) != '\0') {
+	    if(ch == '/' || isspace(ch)) s[-1] = '_';
 	}
 
-	if (fclose(out) == EOF) {
-	  dbprintf(("error [closing %s: %s]\n", incrname, strerror(errno)));
-	  out = NULL;
-	  amfree(incrname);
-	  amfree(basename);
-	  amfree(dirname);
-	  return -1;
-	}
-	out = NULL;
+	ap_snprintf(number, sizeof(number), "%d", level);
+	incrname = vstralloc(basename, "_", number, ".new", NULL);
+	unlink(incrname);
 
-      } else {
-	FILE *in = NULL, *out;
-	char *inputname = NULL;
-	char buf[BUFSIZ];
-	int baselevel = level;
+	/*
+	 * Open the listed incremental file from the previous level.  Search
+	 * backward until one is found.  If none are found (which will also
+	 * be true for a level 0), arrange to read from /dev/null.
+	 */
+	baselevel = level;
+	while (in == NULL) {
+	    if (--baselevel >= 0) {
+		ap_snprintf(number, sizeof(number), "%d", baselevel);
+		inputname = newvstralloc(inputname,
+					 basename, "_", number, NULL);
+	    } else {
+		inputname = newstralloc(inputname, "/dev/null");
+	    }
+	    if ((in = fopen(inputname, "r")) == NULL) {
+		int save_errno = errno;
 
-	while (in == NULL && --baselevel >= 0) {
-	  ap_snprintf(number, sizeof(number), "%d", baselevel);
-	  inputname = newvstralloc(inputname, basename, "_", number, NULL);
-	  in = fopen(inputname, "r");
+		dbprintf(("%s-gnutar: error opening %s: %s\n",
+			  get_pname(),
+			  inputname,
+			  strerror(save_errno)));
+		if (baselevel < 0) {
+		    goto common_exit;
+		}
+	    }
 	}
 
-	if (in == NULL) {
-	  amfree(inputname);
-	  goto notincremental;
+	/*
+	 * Copy the previous listed incremental file to the new one.
+	 */
+	if ((out = fopen(incrname, "w")) == NULL) {
+	    dbprintf(("opening %s: %s\n", incrname, strerror(errno)));
+	    goto common_exit;
 	}
 
-	out = fopen(incrname, "w");
-	if (out == NULL) {
-	  dbprintf(("error [opening %s: %s]\n", incrname, strerror(errno)));
-	  amfree(incrname);
-	  amfree(basename);
-	  amfree(inputname);
-	  amfree(dirname);
-	  return -1;
+	for (; (line = agets(in)) != NULL; free(line)) {
+	    if (fputs(line, out) == EOF || putc('\n', out) == EOF) {
+		dbprintf(("writing to %s: %s\n", incrname, strerror(errno)));
+		goto common_exit;
+	    }
 	}
-
-	while(fgets(buf, sizeof(buf), in) != NULL)
-	  if (fputs(buf, out) == EOF) {
-	    dbprintf(("error [writing to %s: %s]\n", incrname,
-		      strerror(errno)));
-	    amfree(incrname);
-	    amfree(basename);
-	    amfree(inputname);
-	    amfree(dirname);
-	    return -1;
-	  }
+	amfree(line);
 
 	if (ferror(in)) {
-	  dbprintf(("error [reading from %s: %s]\n", inputname,
-		    strerror(errno)));
-	  amfree(incrname);
-	  amfree(basename);
-	  amfree(inputname);
-	  amfree(dirname);
-	  return -1;
+	    dbprintf(("reading from %s: %s\n", inputname, strerror(errno)));
+	    goto common_exit;
 	}
-
 	if (fclose(in) == EOF) {
-	  dbprintf(("error [closing %s: %s]\n", inputname, strerror(errno)));
-	  in = NULL;
-	  amfree(incrname);
-	  amfree(basename);
-	  amfree(inputname);
-	  amfree(dirname);
-	  return -1;
+	    dbprintf(("closing %s: %s\n", inputname, strerror(errno)));
+	    in = NULL;
+	    goto common_exit;
 	}
 	in = NULL;
-
 	if (fclose(out) == EOF) {
-	  dbprintf(("error [closing %s: %s]\n", incrname, strerror(errno)));
-	  out = NULL;
-	  amfree(incrname);
-	  amfree(basename);
-	  amfree(inputname);
-	  amfree(dirname);
-	  return -1;
+	    dbprintf(("closing %s: %s\n", incrname, strerror(errno)));
+	    out = NULL;
+	    goto common_exit;
 	}
 	out = NULL;
 
 	amfree(inputname);
-      }
-      amfree(basename);
+	amfree(basename);
     }
 #endif
 
@@ -1165,101 +1157,74 @@ notincremental:
 
     if (exclude_spec == NULL) {
 	amfree(exclude_arg);
+	estr = NULL;
+	efile = NULL;
 	/* do nothing */
 #define sc "--exclude-list="
     } else if (strncmp(exclude_spec, sc, sizeof(sc)-1)==0) {
 	char *file = exclude_spec + sizeof(sc)-1;
+/* BEGIN HPS */
+	if(*file != '/')
+	  file = vstralloc(dirname,"/",file, NULL);
+/* END HPS */
+	estr = NULL;
 	if (access(file, F_OK) == 0)
-	    exclude_arg = newstralloc2(exclude_arg, "--exclude-from=", file);
+	    efile = newstralloc(efile, file);
 	else {
 	    dbprintf(("%s: missing exclude list file \"%s\" discarded\n",
 		      get_pname(), file));
-	    amfree(exclude_arg);
+	    amfree(efile);
 	}
 #undef sc
 #define sc "--exclude-file="
     } else if (strncmp(exclude_spec, sc, sizeof(sc)-1)==0) {
-	exclude_arg = newstralloc2(exclude_arg, "--exclude=",
-				   exclude_spec+sizeof(sc)-1);
+	efile = NULL;
+	estr = newstralloc(estr, exclude_spec+sizeof(sc)-1);
 #undef sc
     } else {
-	dbprintf(("error [exclude_spec is neither --exclude-list nor --exclude-file: %s]\n", exclude_spec));
-	error("error: exclude_spec is neither --exclude-list nor --exclude-file: %s]", exclude_spec);
+	error("exclude_spec is neither --exclude-list nor --exclude-file: %s",
+	      exclude_spec);
     }
 
-    cmd_line = vstralloc(cmd,
-			 " --create",
-			 " --directory ", dirname,
-#ifdef GNUTAR_LISTED_INCREMENTAL_DIR
-			 " --listed-incremental ", incrname,
-#else
-			 " --incremental",
-			 " --newer ", dumptimestr,
-#endif
-			 " --sparse",
-			 " --one-file-system",
-#ifdef ENABLE_GNUTAR_ATIME_PRESERVE
-			 " --atime-preserve",
-#endif
-			 " --ignore-failed-read",
-			 " --totals",
-			 " --file", " /dev/null",
-			 " ", exclude_arg ? exclude_arg : ".",
-			 exclude_arg ? " ." : "",
-			 NULL);
-
-    dbprintf(("%s: running \"%s\"\n", get_pname(), cmd_line));
-
     nullfd = open("/dev/null", O_RDWR);
-    pipe(pipefd);
-
-    switch(dumppid = fork()) {
-    case -1:
-      amfree(cmd);
-      amfree(dirname);
-      return -1;
-    default:
-      break;
-    case 0:
-      dup2(nullfd, 0);
-      dup2(nullfd, 1);
-      dup2(pipefd[1], 2);
-      aclose(pipefd[0]);
-
-      execle(cmd,
+    dumppid = pipespawn(cmd, STDERR_PIPE, &nullfd, &nullfd, &pipefd,
 #ifdef GNUTAR
-	     GNUTAR,
+	      GNUTAR,
 #else
-	     "tar",
+	      "tar",
 #endif
-	     "--create",
-	     "--directory", dirname,
+	      "--create",
+	      "--file", "/dev/null",
+	      "--directory", dirname,
+	      "--one-file-system",
 #ifdef GNUTAR_LISTED_INCREMENTAL_DIR
-	     "--listed-incremental", incrname,
+	      "--listed-incremental", incrname,
 #else
-	     "--incremental", "--newer", dumptimestr,
+	      "--incremental",
+	      "--newer", dumptimestr,
 #endif
-	     "--sparse",
-	     "--one-file-system",
 #ifdef ENABLE_GNUTAR_ATIME_PRESERVE
-	     "--atime-preserve",
+	      /* --atime-preserve causes gnutar to call
+	       * utime() after reading files in order to
+	       * adjust their atime.  However, utime()
+	       * updates the file's ctime, so incremental
+	       * dumps will think the file has changed. */
+	      "--atime-preserve",
 #endif
-	     "--ignore-failed-read",
-	     "--totals",
-	     "--file", "/dev/null",
-	     exclude_arg ? exclude_arg : ".",
-	     exclude_arg ? "." : (char *)0,
-	     (char *)0,
-	     safe_env());
-
-      exit(1);
-      break;
-    }
-    amfree(exclude_arg);
+	      "--sparse",
+	      "--ignore-failed-read",
+	      "--totals",
+	      efile ? "--exclude-from" : skip_argument,
+	      efile ? efile : skip_argument,
+	      estr ? "--exclude" : skip_argument,
+	      estr ? estr : skip_argument,
+	      ".",
+	      NULL);
+    amfree(efile);
+    amfree(estr);
     amfree(cmd);
 
-    aclose(pipefd[1]);
-    dumpout = fdopen(pipefd[0],"r");
+    dumpout = fdopen(pipefd,"r");
 
     for(size = -1; (line = agets(dumpout)) != NULL; free(line)) {
 	dbprintf(("%s\n",line));
@@ -1282,14 +1247,22 @@ notincremental:
 
     kill(-dumppid, SIGTERM);
 
-    unlink(incrname);
-    amfree(incrname);
-    amfree(dirname);
-
     wait(NULL);
 
+common_exit:
+
+    if (incrname) {
+	unlink(incrname);
+    }
+    amfree(incrname);
+    amfree(basename);
+    amfree(dirname);
+    amfree(inputname);
+
     aclose(nullfd);
     afclose(dumpout);
+    afclose(in);
+    afclose(out);
 
     return size;
 }
