@@ -24,7 +24,7 @@
  * file named AUTHORS, in the root directory of this distribution.
  */
 /*
- * $Id$
+ * $Id: amanda.h,v 1.66.2.7 2000/09/23 15:33:58 martinea Exp $
  *
  * the central header file included by all amanda sources
  */
@@ -246,7 +246,7 @@
  *
  *  malloc_enter(char *) -- stack trace for malloc reports
  *  malloc_leave(char *) -- stack trace for malloc reports
- *  malloc_leave(void *) -- mark an area as never to be free-d
+ *  malloc_mark(void *) -- mark an area as never to be free-d
  *  malloc_chain_check(void) -- check the malloc area now
  *  malloc_dump(int fd) -- report the malloc contents to a file descriptor
  *  malloc_list(int fd, ulong a, ulong b) -- report memory activated since
@@ -361,6 +361,7 @@ extern void debug_close P((void));
 extern void debug_printf P((char *format, ...))
     __attribute__ ((format (printf, 1, 2)));
 extern int  debug_fd P((void));
+extern FILE *  debug_fp P((void));
 #else
 #   define dbopen()
 #   define dbclose()
@@ -420,8 +421,8 @@ extern int    onerror         P((void (*errf)(void)));
 #if defined(USE_DBMALLOC)
 extern void  *debug_alloc           P((char *c, int l, int size));
 extern void  *debug_newalloc        P((char *c, int l, void *old, int size));
-extern char  *debug_stralloc        P((char *c, int l, char *str));
-extern char  *debug_newstralloc     P((char *c, int l, char *oldstr, char *newstr));
+extern char  *debug_stralloc        P((char *c, int l, const char *str));
+extern char  *debug_newstralloc     P((char *c, int l, char *oldstr, const char *newstr));
 extern char  *dbmalloc_caller_loc   P((char *file, int line));
 extern int   debug_alloc_push	    P((char *file, int line));
 extern void  debug_alloc_pop	    P((void));
@@ -464,17 +465,17 @@ extern void  debug_alloc_pop	    P((void));
 #define vstralloc debug_alloc_push(__FILE__,__LINE__)?0:debug_vstralloc
 #define newvstralloc debug_alloc_push(__FILE__,__LINE__)?0:debug_newvstralloc
 
-extern char  *debug_vstralloc       P((char *str, ...));
-extern char  *debug_newvstralloc    P((char *oldstr, char *newstr, ...));
+extern char  *debug_vstralloc       P((const char *str, ...));
+extern char  *debug_newvstralloc    P((char *oldstr, const char *newstr, ...));
 
 #else
 
 extern void  *alloc           P((int size));
 extern void  *newalloc        P((void *old, int size));
-extern char  *stralloc        P((char *str));
-extern char  *newstralloc     P((char *oldstr, char *newstr));
-extern char  *vstralloc       P((char *str, ...));
-extern char  *newvstralloc    P((char *oldstr, char *newstr, ...));
+extern char  *stralloc        P((const char *str));
+extern char  *newstralloc     P((char *oldstr, const char *newstr));
+extern char  *vstralloc       P((const char *str, ...));
+extern char  *newvstralloc    P((char *oldstr, const char *newstr, ...));
 
 #define debug_alloc_push(s,l)
 #define debug_alloc_pop()
@@ -484,9 +485,14 @@ extern char  *newvstralloc    P((char *oldstr, char *newstr, ...));
 #define	newstralloc2(p,s1,s2) newvstralloc((p),(s1),(s2),NULL)
 
 extern void  *sbuf_man        P((void *bufs, void *ptr));
+extern uid_t  client_uid;
+extern gid_t  client_gid;
+extern void   safe_cd	      P((void));
+extern void   save_core	      P((void));
 extern char **safe_env        P((void));
 extern char  *validate_regexp P((char *regex));
 extern char  *validate_glob   P((char *glob));
+extern char  *clean_regex     P((char *regex));
 extern int    match           P((char *regex, char *str));
 extern int    match_glob      P((char *glob, char *str));
 extern char  *glob_to_regex   P((char *glob));
@@ -500,6 +506,8 @@ extern char  *dbmalloc_areads P((char *c, int l, int fd));
 extern char  *agets	      P((FILE *file));
 extern char  *areads	      P((int fd));
 #endif
+extern ssize_t  areads_dataready  P((int fd));
+extern void     areads_relbuf     P((int fd));
 
 /*
  * amfree(ptr) -- if allocated, release space and set ptr to NULL.
@@ -538,23 +546,31 @@ extern char  *areads	      P((int fd));
  *  }
  *
  * Since aclose() sets the argument to -1, this will loop forever.
+ * Just copy fd to a temp variable and use that with aclose().
+ *
+ * Aclose() interacts with areads() to inform it to release any buffer
+ * it has outstanding on the file descriptor.
  */
 
 #define aclose(fd) do {							\
-    assert(fd >= 0);							\
-    close(fd);								\
+    if((fd) >= 0) {							\
+	close(fd);							\
+	areads_relbuf(fd);						\
+    }									\
     (fd) = -1;								\
 } while(0)
 
 #define afclose(f) do {							\
-    assert(f != NULL);							\
-    fclose(f);								\
+    if((f) != NULL) {							\
+	fclose(f);							\
+    }									\
     (f) = NULL;								\
 } while(0)
 
 #define apclose(p) do {							\
-    assert(p != NULL);							\
-    pclose(p);								\
+    if((p) != NULL) {							\
+	pclose(p);							\
+    }									\
     (p) = NULL;								\
 } while(0)
 
@@ -679,6 +695,11 @@ extern char  *areads	      P((int fd));
     if(fp) *fp = '\0';							\
 } while(0)
 
+#define is_dot_or_dotdot(s)						\
+    ((s)[0] == '.'							\
+     && ((s)[1] == '\0'							\
+	 || ((s)[1] == '.' && (s)[2] == '\0')))
+
 /* from amflock.c */
 extern int    amflock   P((int fd, char *resource));
 extern int    amroflock P((int fd, char *resource));
@@ -711,16 +732,24 @@ extern int accept P((int s, struct sockaddr *addr, int *addrlen));
 extern double atof P((const char *ptr));
 #endif
 
-#ifndef HAVE_BCOPY_DECL
+#ifndef HAVE_BCOPY
+# define bcopy(from,to,n) ((void)memmove((to), (from), (n)))
+#else
+# ifndef HAVE_BCOPY_DECL
 extern void bcopy P((const void *s1, void *s2, size_t n));
+# endif
 #endif
 
 #ifndef HAVE_BIND_DECL
 extern int bind P((int s, const struct sockaddr *name, int namelen));
 #endif
 
-#ifndef HAVE_BZERO_DECL
+#ifndef HAVE_BZERO
+#define bzero(s,n) ((void)memset((s),0,(n)))
+#else
+# ifndef HAVE_BZERO_DECL
 extern void bzero P((void *s, size_t n));
+# endif
 #endif
 
 #ifndef HAVE_CLOSELOG_DECL
