@@ -24,7 +24,7 @@
  * file named AUTHORS, in the root directory of this distribution.
  */
 /*
- * $Id$
+ * $Id: amflock.c,v 1.17.4.6 1999/11/02 21:52:08 oliva Exp $
  *
  * file locking routines, put here to hide the system dependant stuff
  * from the rest of the code
@@ -48,12 +48,7 @@
 #include "amanda.h"
 
 #if defined(USE_POSIX_FCNTL)
-   static struct flock lock = {
-	F_UNLCK,	/* Lock type, will be set below. */
-	SEEK_SET,	/* Offset below starts at beginning, */
-	0,		/* thus lock region starts at byte 0 */
-	0		/* and goes to EOF. */
-   };			/* Don't need other field(s). */
+     static struct flock lock; /* zero-initialized */
 #endif
 
 #if !defined(USE_POSIX_FCNTL) && defined(USE_FLOCK)
@@ -277,7 +272,7 @@ int op;    /* true to lock; false to unlock */
 
 	mypid = (long)getpid();
 
-	lockfile = vstralloc("/tmp", "/am", res, ".lock", NULL);
+	lockfile = vstralloc(AMANDA_TMPDIR, "/am", res, ".lock", NULL);
 
 	if (!op) {
 		/* unlock the resource */
@@ -291,7 +286,7 @@ int op;    /* true to lock; false to unlock */
 	/* lock the resource */
 
 	ap_snprintf(pid_str, sizeof(pid_str), "%ld", mypid);
-	tlockfile = vstralloc("/tmp", "am", res, ".", pid_str, NULL);
+	tlockfile = vstralloc(AMANDA_TMPDIR, "am", res, ".", pid_str, NULL);
 
 	(void)create_lock(tlockfile, mypid);
 
@@ -329,6 +324,7 @@ char *resource;
 
 #ifdef USE_POSIX_FCNTL
 	lock.l_type = F_RDLCK;
+	lock.l_whence = SEEK_SET;
 	r = fcntl(fd, F_SETLKW, &lock);
 #else
 	r = amflock(fd, resource);
@@ -348,6 +344,7 @@ char *resource;
 
 #ifdef USE_POSIX_FCNTL
 	lock.l_type = F_WRLCK;
+	lock.l_whence = SEEK_SET;
 	r = fcntl(fd, F_SETLKW, &lock);
 #else
 #ifdef USE_FLOCK
@@ -379,6 +376,7 @@ char *resource;
 
 #ifdef USE_POSIX_FCNTL
 	lock.l_type = F_UNLCK;
+	lock.l_whence = SEEK_SET;
 	r = fcntl(fd, F_SETLK, &lock);
 #else
 #ifdef USE_FLOCK
@@ -428,26 +426,43 @@ main()
     }
 
     unlink(filen);
-    if ((lockfd = open(filen, O_RDONLY|O_CREAT|O_EXCL, 0600)) == -1)
-	return (-1);
+    if ((lockfd = open(filen, O_RDONLY|O_CREAT|O_EXCL, 0600)) == -1) {
+	perror (filen);
+	exit(10);
+    }
 
-    if (amroflock(lockfd, resn) != 0)
+    if (amroflock(lockfd, resn) != 0) {
+	perror ("amroflock");
 	exit(1);
-    if (amfunlock(lockfd, resn) != 0)
+    }
+    if (amfunlock(lockfd, resn) != 0) {
+	perror ("amfunlock/2");
 	exit(2);
+    }
 
-    aclose(lockfd);
+    /*
+     * Do not use aclose() here.  During configure we do not have
+     * areads_relbuf() available and it makes configure think all
+     * the tests have failed.
+     */
+    close(lockfd);
 
     unlink(filen);
-    if ((lockfd = open(filen, O_WRONLY|O_CREAT|O_EXCL, 0600)) == -1)
-	return (-1);
+    if ((lockfd = open(filen, O_WRONLY|O_CREAT|O_EXCL, 0600)) == -1) {
+	perror (filen);
+	exit(20);
+    }
 
-    if (amflock(lockfd, resn) != 0)
+    if (amflock(lockfd, resn) != 0) {
+	perror ("amflock");
 	exit(3);
-    if (amfunlock(lockfd, resn) != 0)
+    }
+    if (amfunlock(lockfd, resn) != 0) {
+	perror ("amfunlock/4");
 	exit(4);
+    }
 
-    aclose(lockfd);
+    close(lockfd);
 
     exit(0);
 }
