@@ -25,7 +25,7 @@
  *			   University of Maryland at College Park
  */
 /*
- * $Id$
+ * $Id: dgram.h,v 1.6.2.2 1999/09/01 17:58:39 jrj Exp $
  *
  * interface for datagram module
  */
@@ -34,7 +34,7 @@
 
 #include "amanda.h"
 
-#define MAX_DGRAM	8192
+#define MAX_DGRAM      (64*1024)
 
 typedef struct dgram_s {
     char *cur;
@@ -49,9 +49,8 @@ int dgram_send P((char *hostname, int port, dgram_t *dgram));
 int dgram_send_addr P((struct sockaddr_in addr, dgram_t *dgram));
 int dgram_recv P((dgram_t *dgram, int timeout, struct sockaddr_in *fromaddr));
 void dgram_zero P((dgram_t *dgram));
-void dgram_cat P((dgram_t *dgram, char *str));
+void dgram_cat P((dgram_t *dgram, const char *str));
 void dgram_eatline P((dgram_t *dgram));
-int bind_reserved P((int sock, struct sockaddr_in *addrp));
 
 #if defined(USE_DBMALLOC)
 extern dgram_t  *dbmalloc_dgram_alloc  P((char *c, int l));
