@@ -25,7 +25,7 @@
  *			   University of Maryland at College Park
  */
 /*
- * $Id$
+ * $Id: error.c,v 1.8.4.1 2001/01/05 02:29:08 jrjackson Exp $
  *
  * error handling common to Amanda programs
  */
@@ -99,6 +99,11 @@ arglist_function(void error, char *, format)
 	fflush(stderr);
     }
 
+    if(debug_fp() != NULL) {
+	dbprintf(("%s\n", linebuf));
+	dbclose();
+    }
+
     /* traverse function list, calling in reverse order */
 
     for(i=MAXFUNCS-1; i >= 0; i--) {
