@@ -24,7 +24,7 @@
  * file named AUTHORS, in the root directory of this distribution.
  */
 /*
- * $Id$
+ * $Id: fileheader.c,v 1.11.4.1 1998/12/07 23:53:54 martinea Exp $
  *
  */
 
@@ -258,6 +258,14 @@ int buflen;
 			    sizeof(file->cont_filename), bp);
 	    }
 #undef SC
+#define SC "PARTIAL="
+	    else if(strncmp(line,SC,strlen(SC)) == 0) {
+		s = line + strlen(SC);
+		if(strncmp(s,"yes",3)==0 || strncmp(s,"YES",3)==0)
+		    file->is_partial=1;
+		ch = *s++;
+	    }
+#undef SC
 #define SC "To restore, position tape at start of file and run:"
 	    else if(strncmp(line,SC,strlen(SC)) == 0) {
 	    }
@@ -361,6 +369,9 @@ int buflen;
 					    file->cont_filename, "\n", NULL);
 		        strncat(buffer,line,buflen-strlen(buffer));
 		      }
+		      if(file->is_partial != 0) {
+			strncat(buffer,"PARTIAL=YES\n",buflen-strlen(buffer));
+		      }
 		      strncat(buffer,
 			"To restore, position tape at start of file and run:\n",
 			buflen-strlen(buffer));
