@@ -24,7 +24,7 @@
  * file named AUTHORS, in the root directory of this distribution.
  */
 /*
- * $Id$
+ * $Id: match.c,v 1.10.4.1 2000/09/23 15:33:58 martinea Exp $
  *
  * functions for checking and matching regular expressions
  */
@@ -50,6 +50,22 @@ char *regex;
     return NULL;
 }
 
+char *clean_regex(regex)
+char *regex;
+{
+    char *result;
+    int i, j;
+    result = malloc(2*strlen(regex)+1);
+
+    for(i=0,j=0;i<strlen(regex);i++) {
+	if(!isalnum(regex[i]))
+	    result[j++]='\\';
+	result[j++]=regex[i];
+    }
+    result[j++] = '\0';
+    return result;
+}
+
 int match(regex, str)
 char *regex, *str;
 {
