@@ -24,17 +24,40 @@
  * file named AUTHORS, in the root directory of this distribution.
  */
 /*
- * $Id$
+ * $Id: security.c,v 1.17.2.6 2000/12/04 22:44:59 jrjackson Exp $
  *
  * wrapper file for kerberos security
  */
 
 #include "amanda.h"
 
-#if defined(TEST)
+/*
+ * Change the following from #undef to #define to cause detailed logging
+ * of the security steps, e.g. into /tmp/amanda/amandad*debug.
+ */
+#undef SHOW_SECURITY_DETAIL
+
+/*
+ * If we don't have the new-style wait access functions, use our own,
+ * compatible with old-style BSD systems at least.  Note that we don't
+ * care about the case w_stopval == WSTOPPED since we don't ask to see
+ * stopped processes, so should never get them from wait.
+ */
+#ifndef WEXITSTATUS
+#   define WEXITSTATUS(r)       (((union wait *) &(r))->w_retcode)
+#   define WTERMSIG(r)          (((union wait *) &(r))->w_termsig)
+
+#   undef  WIFSIGNALED
+#   define WIFSIGNALED(r)       (((union wait *) &(r))->w_termsig != 0)
+#endif
+
+#if defined(TEST)						/* { */
+#define SHOW_SECURITY_DETAIL
 #undef dbprintf
 #define dbprintf(p)	printf p
+#endif								/* } */
 
+#if defined(SHOW_SECURITY_DETAIL)				/* { */
 void show_stat_info(a, b)
     char *a, *b;
 {
@@ -69,11 +92,11 @@ void show_stat_info(a, b)
     amfree(owner);
     amfree(group);
 }
-#endif
+#endif								/* } */
 
-#ifdef KRB4_SECURITY
+#ifdef KRB4_SECURITY						/* { */
 #include "krb4-security.c"
-#endif
+#endif								/* } */
 
 int bsd_security_ok P((struct sockaddr_in *addr,
 		       char *str, unsigned long cksum, char **errstr));
@@ -93,21 +116,21 @@ char *str;
 unsigned long cksum;
 char **errstr;
 {
-#ifdef KRB4_SECURITY
+#ifdef KRB4_SECURITY						/* { */
     if(krb4_auth)
 	return krb4_security_ok(addr, str, cksum, errstr);
     else
-#endif
+#endif								/* } */
 	return bsd_security_ok(addr, str, cksum, errstr);
 }
 
-#ifdef BSD_SECURITY
+#ifdef BSD_SECURITY						/* { */
 
 int bsd_security_ok(addr, str, cksum, errstr)
-struct sockaddr_in *addr;
-char *str;
-unsigned long cksum;
-char **errstr;
+     struct sockaddr_in *addr;
+     char *str;
+     unsigned long cksum;
+     char **errstr;
 {
     char *remotehost = NULL, *remoteuser = NULL, *localuser = NULL;
     char *bad_bsd = NULL;
@@ -116,15 +139,21 @@ char **errstr;
     int myuid, i, j;
     char *s, *fp;
     int ch;
-#ifdef USE_AMANDAHOSTS
+    char number[NUM_STR_SIZE];
+#ifdef USE_AMANDAHOSTS						/* { */
     FILE *fPerm;
     char *pbuf = NULL;
     char *ptmp;
     int pbuf_len;
     int amandahostsauth = 0;
-#else
+#else								/* } { */
+    FILE *fError;
     int saved_stderr;
-#endif
+    int fd[2];
+    amwait_t wait_exitcode;
+    int exitcode;
+    pid_t pid, ruserok_pid;
+#endif								/* } */
 
     *errstr = NULL;
 
@@ -185,22 +214,20 @@ char **errstr;
 	     if ( strcmp(hp->h_aliases[j],inet_ntoa(addr->sin_addr)) == 0)
 	         break;                          /* name is good, keep it */
         }
-    }
-    if( !hp->h_addr_list[i] && !hp->h_aliases[j] ) {
-	*errstr = vstralloc("[",
-			    "ip address ", inet_ntoa(addr->sin_addr),
-			    " is not in the ip list for ", remotehost,
-			    "]",
-			    NULL);
-	amfree(remotehost);
-	return 0;
+	if( !hp->h_aliases[j] ) {
+	    *errstr = vstralloc("[",
+			        "ip address ", inet_ntoa(addr->sin_addr),
+			        " is not in the ip list for ", remotehost,
+			        "]",
+			        NULL);
+	    amfree(remotehost);
+	    return 0;
+	}
     }
 
     /* next, make sure the remote port is a "reserved" one */
 
     if(ntohs(addr->sin_port) >= IPPORT_RESERVED) {
-	char number[NUM_STR_SIZE];
-
 	ap_snprintf(number, sizeof(number), "%d", ntohs(addr->sin_port));
 	*errstr = vstralloc("[",
 			    "host ", remotehost, ": ",
@@ -248,18 +275,20 @@ char **errstr;
     /* lookup our local user name */
 
     myuid = getuid();
-    if((pwptr = getpwuid(myuid)) == NULL)
+    if((pwptr = getpwuid(myuid)) == NULL) {
         error("error [getpwuid(%d) fails]", myuid);
+    }
 
     localuser = stralloc(pwptr->pw_name);
 
     dbprintf(("bsd security: remote host %s user %s local user %s\n",
 	      remotehost, remoteuser, localuser));
 
+#ifndef USE_AMANDAHOSTS						/* { */
     /*
-     * note that some versions of ruserok (eg SunOS 3.2) look in
+     * Note that some versions of ruserok (eg SunOS 3.2) look in
      * "./.rhosts" rather than "~localuser/.rhosts", so we have to
-     * chdir ourselves.  Sigh.
+     * change directories ourselves.  Sigh.
      *
      * And, believe it or not, some ruserok()'s try an initgroup just
      * for the hell of it.  Since we probably aren't root at this point
@@ -267,13 +296,24 @@ char **errstr;
      * into our stderr output even though the initgroup failure is not a
      * problem and is expected.  Thanks a lot.  Not.
      */
-    chdir(pwptr->pw_dir);       /* pamper braindead ruserok's */
-#ifndef USE_AMANDAHOSTS
-    saved_stderr = dup(2);
-    close(2);			/*  " */
+    if (pipe(fd) != 0) {
+	error("error [pipe() fails]");
+    }
+    if ((ruserok_pid = fork ()) < 0) {
+	error("error [fork() fails]");
+    } else if (ruserok_pid == 0) {
+	close(fd[0]);
+	fError = fdopen(fd[1], "w");
+	/* pamper braindead ruserok's */
+	if(chdir(pwptr->pw_dir) != 0) {
+	    fprintf(fError, "[chdir(%s) failed: %s]",
+		    pwptr->pw_dir, strerror(errno));
+	    fclose(fError);
+	    exit(1);
+	}
 
-#if defined(TEST)
-    {
+#if defined(SHOW_SECURITY_DETAIL)				/* { */
+	{
 	char *dir = stralloc(pwptr->pw_dir);
 
 	dbprintf(("calling ruserok(%s, %d, %s, %s)\n",
@@ -286,57 +326,107 @@ char **errstr;
 	}
 	show_stat_info(dir, "/.rhosts");
 	amfree(dir);
-    }
-#endif
+	}
+#endif								/* } */
+
+	saved_stderr = dup(2);
+	close(2);
+	(void)open("/dev/null", 2);
+
+	if(ruserok(remotehost, myuid == 0, remoteuser, localuser) == -1) {
+	    char *fmt;
+
+	    dup2(saved_stderr,2);
+	    close(saved_stderr);
+	    fmt = "[access as %s not allowed from %s@%s] ruserok failed\n";
+	    fprintf(fError, fmt, localuser, remoteuser, remotehost);
+	    fclose(fError);
+	    fmt = stralloc2("check failed: ", fmt);
+	    dbprintf((fmt, localuser, remoteuser, remotehost));
+	    exit(1);
+	}
 
-    if(ruserok(remotehost, myuid == 0, remoteuser, localuser) == -1) {
 	dup2(saved_stderr,2);
 	close(saved_stderr);
+	dbprintf(("bsd security check to %s from %s@%s passed\n",
+		  localuser, remoteuser, remotehost));
+	exit(0);
+    }
+    close(fd[1]);
+    fError = fdopen(fd[0], "r");
+
+    while((pid = wait(&wait_exitcode)) == (pid_t)-1 && errno == EINTR) {}
+    if (pid == (pid_t)-1) {
 	*errstr = vstralloc("[",
 			    "access as ", localuser, " not allowed",
 			    " from ", remoteuser, "@", remotehost,
-			    "]", NULL);
-	dbprintf(("check failed: %s\n", *errstr));
-	amfree(remotehost);
-	amfree(localuser);
-	amfree(remoteuser);
-	return 0;
+			    "] wait failed: ",
+			    strerror(errno),
+			    NULL);
+	exitcode = 0;
+    } else if (pid != ruserok_pid) {
+	ap_snprintf(number, sizeof(number), "%ld", (long)pid);
+	*errstr = vstralloc("[",
+			    "access as ", localuser, " not allowed",
+			    " from ", remoteuser, "@", remotehost,
+			    "] wait got pid ",
+			    number,
+			    NULL);
+	exitcode = 0;
+    } else if (WIFSIGNALED(wait_exitcode)) {
+	ap_snprintf(number, sizeof(number), "%d", WTERMSIG(wait_exitcode));
+	*errstr = vstralloc("[",
+			    "access as ", localuser, " not allowed",
+			    " from ", remoteuser, "@", remotehost,
+			    "] got signal ", number,
+			    NULL);
+	exitcode = 0;
+    } else {
+	exitcode = WEXITSTATUS(wait_exitcode);
     }
-
-    dup2(saved_stderr,2);
-    close(saved_stderr);
-    chdir("/");		/* now go someplace where I can't drop core :-) */
-    dbprintf(("bsd security check passed\n"));
-    amfree(remotehost);
-    amfree(localuser);
-    amfree(remoteuser);
-    return 1;
-#else
-    /* We already chdired to ~amandauser */
-
-#if defined(TEST)
+    if(exitcode) {
+	if((*errstr = agets(fError)) == NULL) {
+	    *errstr = vstralloc("[",
+			        "access as ", localuser, " not allowed",
+			        " from ", remoteuser, "@", remotehost,
+			        "] could not get result",
+			        NULL);
+	}
+    }
+    fclose(fError);
+    return *errstr == NULL;
+#else								/* } { */
+#if defined(SHOW_SECURITY_DETAIL)				/* { */
     show_stat_info(pwptr->pw_dir, "/.amandahosts");
-#endif
+#endif								/* } */
 
-    if((fPerm = fopen(".amandahosts", "r")) == NULL) {
-#if defined(TEST)
-	dbprintf(("fopen failed: %s\n", strerror(errno)));
-#endif
+    ptmp = stralloc2(pwptr->pw_dir, "/.amandahosts");
+    if((fPerm = fopen(ptmp, "r")) == NULL) {
+	/*
+	 * Put an explanation in the amandad.debug log that will help a
+	 * system administrator fix the problem, but don't send a clue
+	 * back to the other end to tell them what to fix in order to
+	 * be able to hack our system.
+         */
+	dbprintf(("fopen of %s failed: %s\n", ptmp, strerror(errno)));
 	*errstr = vstralloc("[",
 			    "access as ", localuser, " not allowed",
 			    " from ", remoteuser, "@", remotehost,
-			    "]", NULL);
-	dbprintf(("check failed: %s\n", *errstr));
+			    "] open of ",
+			    ptmp,
+			    " failed", NULL);
+	amfree(ptmp);
 	amfree(remotehost);
 	amfree(localuser);
 	amfree(remoteuser);
 	return 0;
     }
+    amfree(ptmp);
 
     for(; (pbuf = agets(fPerm)) != NULL; free(pbuf)) {
-#if defined(TEST)
+#if defined(SHOW_SECURITY_DETAIL)				/* { */
 	dbprintf(("processing line: <%s>\n", pbuf));
-#endif
+#endif								/* } */
 	pbuf_len = strlen(pbuf);
 	s = pbuf;
 	ch = *s++;
@@ -360,7 +450,7 @@ char **errstr;
 	    skip_non_whitespace(s, ch);
 	    s[-1] = '\0';			/* terminate remoteuser field */
 	}
-#if defined(TEST)
+#if defined(SHOW_SECURITY_DETAIL)				/* { */
 	dbprintf(("comparing %s with\n", pbuf));
 	dbprintf(("          %s (%s)\n",
 		  remotehost,
@@ -369,8 +459,9 @@ char **errstr;
 	dbprintf(("          %s (%s)\n",
 		  remoteuser,
 		  (strcasecmp(ptmp, remoteuser) == 0) ? "match" : "no match"));
-#endif
-	if(strcasecmp(pbuf, remotehost) == 0 && strcasecmp(ptmp, remoteuser) == 0) {
+#endif								/* } */
+	if(strcasecmp(pbuf, remotehost) == 0
+	   && strcasecmp(ptmp, remoteuser) == 0) {
 	    amandahostsauth = 1;
 	    break;
 	}
@@ -379,8 +470,7 @@ char **errstr;
     afclose(fPerm);
     amfree(pbuf);
 
-    if( amandahostsauth ) {
-	chdir("/");      /* now go someplace where I can't drop core :-) */
+    if(amandahostsauth) {
 	dbprintf(("amandahosts security check passed\n"));
 	amfree(remotehost);
 	amfree(localuser);
@@ -391,7 +481,7 @@ char **errstr;
     *errstr = vstralloc("[",
 			"access as ", localuser, " not allowed",
 			" from ", remoteuser, "@", remotehost,
-			"]", NULL);
+			"] amandahostsauth failed", NULL);
     dbprintf(("check failed: %s\n", *errstr));
 
     amfree(remotehost);
@@ -399,10 +489,10 @@ char **errstr;
     amfree(remoteuser);
     return 0;
 
-#endif
+#endif								/* } */
 }
 
-#else	/* ! BSD_SECURITY */
+#else	/* ! BSD_SECURITY */					/* } { */
 
 int bsd_security_ok(addr, str, cksum, errstr)
 struct sockaddr_in *addr;
@@ -410,18 +500,18 @@ char *str;
 unsigned long cksum;
 char **errstr;
 {
-#if defined(TEST)
+#if defined(SHOW_SECURITY_DETAIL)				/* { */
     dbprintf(("You configured Amanda using --without-bsd-security, so it\n"));
     dbprintf(("will let anyone on the Internet connect and do dumps of\n"));
     dbprintf(("your system unless you have some other kind of protection,\n"));
     dbprintf(("such as a firewall or TCP wrappers.\n"));
-#endif
+#endif								/* } */
     return 1;
 }
 
-#endif /* ! BSD_SECURITY */
+#endif /* ! BSD_SECURITY */					/* } */
 
-#if defined(TEST)
+#if defined(TEST)						/* { */
 
 int
 main (argc, argv)
@@ -462,4 +552,4 @@ main (argc, argv)
     return r;
 }
 
-#endif
+#endif								/* } */
