@@ -25,7 +25,7 @@
  *			   University of Maryland at College Park
  */
 /*
- * $Id$
+ * $Id: stream.h,v 1.4.2.2 2001/02/28 02:11:22 jrjackson Exp $
  *
  * interface to stream module
  */
@@ -34,10 +34,16 @@
 
 #include "amanda.h"
 
+/* Note: This must be kept in sync with the DATABUF_SIZE defined in
+ * client-src/sendbackup-krb4.c, or kerberos encryption won't work...
+ *	- Chris Ross (cross@uu.net)  4-Jun-1998
+ */
+#define DATABUF_SIZE TAPE_BLOCK_BYTES
 #define DEFAULT_SIZE -1
 
 int stream_server P((int *port, int sendsize, int recvsize));
 int stream_accept P((int sock, int timeout, int sendsize, int recvsize));
-int stream_client P((char *hostname, int port, int sendsize, int recvsize));
+int stream_client P((char *hostname, int port, int sendsize, int recvsize,
+    int *localport));
 
 #endif
