@@ -1,6 +1,6 @@
 /*
  * Amanda, The Advanced Maryland Automatic Network Disk Archiver
- * Copyright (c) 1991-1998 University of Maryland at College Park
+ * Copyright (c) 1991-1998, 2000 University of Maryland at College Park
  * All Rights Reserved.
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
@@ -24,7 +24,7 @@
  * file named AUTHORS, in the root directory of this distribution.
  */
 /*
- * $Id$
+ * $Id: amrecover.c,v 1.29.4.8 2001/02/28 02:11:23 jrjackson Exp $
  *
  * an interactive program for recovering backed-up files
  */
@@ -38,6 +38,7 @@
 #ifdef HAVE_NETINET_IP_H
 #include <netinet/ip.h>
 #endif
+#include "stream.h"
 #include "amrecover.h"
 #include "getfsent.h"
 #include "dgram.h"
@@ -45,6 +46,7 @@
 #if defined(KRB4_SECURITY)
 #include "krb4-security.h"
 #endif
+#include "util.h"
 
 #ifdef HAVE_LIBREADLINE
 #  ifdef HAVE_READLINE_READLINE_H
@@ -64,8 +66,6 @@
 #  endif
 #endif
 
-char *errstr = NULL;
-
 extern int process_line P((char *line));
 int guess_disk P((char *cwd, int cwd_len, char **dn_guess, char **mpt_guess));
 
@@ -121,18 +121,18 @@ int get_line ()
 	    if(server_line) {
 		fputs(server_line, stderr);	/* show the last line read */
 		fputc('\n', stderr);
-		errno = save_errno;
 	    }
-	    if(errno != 0) {
-		fprintf(stderr, "%s: ", get_pname());
-		errno = save_errno;
-		perror("Error reading line from server");
+	    if(save_errno != 0) {
+		fprintf(stderr, "%s: Error reading line from server: %s\n",
+				get_pname(),
+				strerror(save_errno));
 	    } else {
 		fprintf(stderr, "%s: Unexpected server end of file\n",
 			get_pname());
 	    }
 	    amfree(line);
 	    amfree(server_line);
+	    errno = save_errno;
 	    return -1;
 	}
 	if(line) {
@@ -298,8 +298,8 @@ char *s;
 /* do this by looking for the longest mount point which matches the
    current directory */
 int guess_disk (cwd, cwd_len, dn_guess, mpt_guess)
-char *cwd, **dn_guess, **mpt_guess;
-int cwd_len;
+     char *cwd, **dn_guess, **mpt_guess;
+     int cwd_len;
 {
     int longest_match = 0;
     int current_length;
@@ -328,7 +328,14 @@ int cwd_len;
 	    longest_match = current_length;
 	    amfree(*mpt_guess);
 	    *mpt_guess = stralloc(fsent.mntdir);
-	    fsname = newstralloc(fsname, fsent.fsname+strlen(DEV_PREFIX));
+	    if(strncmp(fsent.fsname,DEV_PREFIX,(strlen(DEV_PREFIX))))
+	    {
+	        fsname = newstralloc(fsname, fsent.fsname);
+            }
+	    else
+	    {
+	        fsname = newstralloc(fsname,fsent.fsname+strlen(DEV_PREFIX));
+	    }
 	    local_disk = is_local_fstype(&fsent);
 	}
     }
@@ -389,10 +396,8 @@ int main(argc, argv)
 int argc;
 char **argv;
 {
-    struct sockaddr_in server;
-    struct sockaddr_in myname;
+    int my_port;
     struct servent *sp;
-    struct hostent *hp;
     int i;
     time_t timer;
     char *lineread = NULL;
@@ -415,8 +420,10 @@ char **argv;
     }
 
     set_pname("amrecover");
+    dbopen();
 
     if (geteuid() != 0) {
+	erroutput_type |= ERR_SYSLOG;
 	error("amrecover must be run by root");
     }
 
@@ -430,10 +437,8 @@ char **argv;
     server_name = newstralloc(server_name, DEFAULT_SERVER);
 #ifdef DEFAULT_TAPE_SERVER
     tape_server_name = newstralloc(tape_server_name, DEFAULT_TAPE_SERVER);
-    tape_device_name = newstralloc(tape_device_name, DEFAULT_TAPE_DEVICE);
 #else
     amfree(tape_server_name);
-    amfree(tape_device_name);
 #endif
     if (argc > 1 && argv[1][0] != '-')
     {
@@ -447,9 +452,7 @@ char **argv;
 	new_argv = (char **) malloc ((argc + 1 + 1) * sizeof (*new_argv));
 	if (new_argv == NULL)
 	{
-	    (void)fprintf(stderr, "%s: no memory for argument list\n",
-			  get_pname());
-	    exit(1);
+	    error("no memory for argument list");
 	}
 	new_argv[0] = argv[0];
 	new_argv[1] = "-C";
@@ -493,8 +496,6 @@ char **argv;
 	exit(1);
     }
 
-    dbopen();
-
     amfree(disk_name);
     amfree(mount_point);
     amfree(disk_path);
@@ -506,10 +507,7 @@ char **argv;
     act.sa_flags = 0;
     if (sigaction(SIGINT, &act, &oact) != 0)
     {
-	perror("amrecover: Error setting signal handler");
-	dbprintf(("Error setting signal handler\n"));
-	dbclose();
-	exit(1);
+	error("error setting signal handler: %s", strerror(errno));
     }
 
     service_name = stralloc2("amandaidx", SERVICE_SUFFIX);
@@ -518,54 +516,20 @@ char **argv;
 	   version(), server_name);  
     if ((sp = getservbyname(service_name, "tcp")) == NULL)
     {
-	perror("amrecover: amandaidx/tcp unknown protocol");
-	dbprintf(("%s/tcp unknown protocol\n", service_name));
-	dbclose();
-	exit(1);
+	error("%s/tcp unknown protocol", service_name);
     }
-    if ((hp = gethostbyname(server_name)) == NULL)
+    server_socket = stream_client(server_name,
+				  ntohs(sp->s_port),
+				  DEFAULT_SIZE,
+				  DEFAULT_SIZE,
+				  &my_port);
+    if (server_socket < 0)
     {
-	(void)fprintf(stderr, "%s: %s is an unknown host\n",
-		      get_pname(), server_name);
-	dbprintf(("%s is an unknown host\n", server_name));
-	dbclose();
-	exit(1);
+	error("cannot connect to %s: %s", server_name, strerror(errno));
     }
-    memset((char *)&server, 0, sizeof(server));
-    memcpy((char *)&server.sin_addr, hp->h_addr, hp->h_length);
-    server.sin_family = hp->h_addrtype;
-    server.sin_port = sp->s_port;
-    if ((server_socket = socket(AF_INET, SOCK_STREAM, 0)) == -1)
-
+    if (my_port >= IPPORT_RESERVED)
     {
-	perror("amrecover: Error creating socket");
-	dbprintf(("Error creating socket\n"));
-	dbclose();
-	exit(1);
-    }
-
-    /*
-     * Bind our end of the socket to a privileged port.
-     */
-    if ((hp = gethostbyname(localhost)) == NULL)
-    {
-	(void)fprintf(stderr, "%s: %s is an unknown host\n",
-		      get_pname(), localhost);
-	dbprintf(("%s is an unknown host\n", localhost));
-	dbclose();
-	exit(1);
-    }
-    memset((char *)&myname, 0, sizeof(myname));
-    memcpy((char *)&myname.sin_addr, hp->h_addr, hp->h_length);
-    myname.sin_family = hp->h_addrtype;
-    if (bind_reserved(server_socket, &myname) != 0)
-    {
-	int save_errno = errno;
-
-	perror("amrecover: Error binding socket");
-	dbprintf(("Error binding socket: %s\n", strerror(save_errno)));
-	dbclose();
-	exit(1);
+	error("did not get a reserved port: %d", my_port);
     }
 
 #if 0
@@ -578,15 +542,6 @@ char **argv;
     seteuid(getuid());
 #endif
 
-    if (connect(server_socket, (struct sockaddr *)&server, sizeof(server))
-	== -1)
-    {
-	perror("amrecover: Error connecting to server");
-	dbprintf(("Error connecting to server\n"));
-	dbclose();
-	exit(1);
-    }
-
     /* get server's banner */
     if (grab_reply(1) == -1)
 	exit(1);
