@@ -1,6 +1,6 @@
 /*
  * Amanda, The Advanced Maryland Automatic Network Disk Archiver
- * Copyright (c) 1991-1998 University of Maryland at College Park
+ * Copyright (c) 1991-1998, 2000 University of Maryland at College Park
  * All Rights Reserved.
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
@@ -24,7 +24,7 @@
  * file named AUTHORS, in the root directory of this distribution.
  */
 /*
- * $Id$
+ * $Id: amrecover.h,v 1.8.4.2 2000/01/21 05:08:34 oliva Exp $
  *
  * data structures and declarations for amrecover
  */
@@ -70,7 +70,10 @@ extern void set_disk P((char *dsk, char *mtpt));
 extern void set_host P((char *host));
 extern int set_date P((char *date));
 extern void set_directory P((char *dir));
+extern void set_tape P((char *tape));
 extern void show_directory P((void));
+extern void set_mode P((int mode));
+extern void show_mode P((void));
 
 extern void list_disk_history P((void));
 extern void list_directory P((void));
@@ -90,3 +93,8 @@ extern void delete_regex P((char *regex));
 extern void delete_file P((char *path, char *regex));
 
 extern void extract_files P((void));
+
+#ifdef SAMBA_CLIENT
+#define SAMBA_SMBCLIENT 0
+#define SAMBA_TAR       1
+#endif
