@@ -1,6 +1,6 @@
 /*
  * Amanda, The Advanced Maryland Automatic Network Disk Archiver
- * Copyright (c) 1991-1998 University of Maryland at College Park
+ * Copyright (c) 1991-1998, 2000 University of Maryland at College Park
  * All Rights Reserved.
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
@@ -24,7 +24,7 @@
  * file named AUTHORS, in the root directory of this distribution.
  */
 /*
- * $Id$
+ * $Id: extract_list.c,v 1.43.2.16 2001/03/21 20:55:21 jrjackson Exp $
  *
  * implements the "extract" command in amrecover
  */
@@ -34,6 +34,7 @@
 #include "amrecover.h"
 #include "fileheader.h"
 #include "dgram.h"
+#include "stream.h"
 #ifdef SAMBA_CLIENT
 #include "findpass.h"
 #endif
@@ -41,6 +42,7 @@
 #if defined(KRB4_SECURITY)
 #include "krb4-security.h"
 #endif
+#include "util.h"
 
 typedef struct EXTRACT_LIST_ITEM
 {
@@ -71,6 +73,9 @@ pid_t extract_restore_child_pid = -1;
 
 static EXTRACT_LIST *extract_list = NULL;
 
+#ifdef SAMBA_CLIENT
+unsigned short samba_extract_method = SAMBA_TAR;
+#endif /* SAMBA_CLIENT */
 
 #define READ_TIMEOUT	30*60
 
@@ -80,7 +85,7 @@ int datafd;
 char *buffer;
 int buflen;
 {
-    int maxfd, nfound, size;
+    int maxfd, nfound, size = 0;
     fd_set readset, selectset;
     struct timeval timeout;
     char *dataptr;
@@ -131,7 +136,8 @@ int buflen;
 	    case 0:
 		spaceleft -= size;
 		dataptr += size;
-		fprintf(stderr,"EOF, check amidxtaped.debug file.\n");
+		fprintf(stderr,"EOF, check amidxtaped.debug file on %s.\n",
+			       tape_server_name);
 		break;
 	    default:
 		spaceleft -= size;
@@ -431,9 +437,9 @@ char *regex;
     char *cmd = NULL;
     char *err = NULL;
     int  i;
-    char *dir, *dir_undo, dir_undo_ch;
+    char *dir, *dir_undo, dir_undo_ch = '\0';
     char *ditem_path = NULL;
-    char *l;
+    char *l = NULL;
     int  added;
     char *s, *fp;
     int ch;
@@ -453,8 +459,11 @@ char *regex;
     /* convert path (assumed in cwd) to one on disk */
     if (strcmp(disk_path, "/") == 0)
 	path_on_disk = stralloc2("/", regex);
-    else
-	path_on_disk = vstralloc(disk_path, "/", regex, NULL);
+    else {
+	char *clean_disk_path = clean_regex(disk_path);
+	path_on_disk = vstralloc(clean_disk_path, "/", regex, NULL);
+	amfree(clean_disk_path);
+    }
 
     path_on_disk_slash = stralloc2(path_on_disk, "/");
 
@@ -692,12 +701,12 @@ char *regex;
     char *cmd = NULL;
     char *err = NULL;
     int  i;
-    char *date, *date_undo, date_undo_ch;
-    char *tape, *tape_undo, tape_undo_ch;
-    char *dir, *dir_undo, dir_undo_ch;
+    char *date, *date_undo, date_undo_ch = '\0';
+    char *tape, *tape_undo, tape_undo_ch = '\0';
+    char *dir, *dir_undo, dir_undo_ch = '\0';
     int  level;
     char *ditem_path = NULL;
-    char *l;
+    char *l = NULL;
     int  deleted;
     char *s;
     int ch;
@@ -716,8 +725,11 @@ char *regex;
     /* convert path (assumed in cwd) to one on disk */
     if (strcmp(disk_path, "/") == 0)
 	path_on_disk = stralloc2("/", regex);
-    else
-	path_on_disk = vstralloc(disk_path, "/", regex, NULL);
+    else {
+	char *clean_disk_path = clean_regex(disk_path);
+	path_on_disk = vstralloc(clean_disk_path, "/", regex, NULL);
+	amfree(clean_disk_path);
+    }
 
     path_on_disk_slash = stralloc2(path_on_disk, "/");
 
@@ -1017,10 +1029,8 @@ char *cmd;
    Return tape server socket on success, -1 on error. */
 static int extract_files_setup P((void))
 {
-    struct sockaddr_in tape_server;
-    struct sockaddr_in myname;
     struct servent *sp;
-    struct hostent *hp;
+    int my_port;
     int tape_server_socket;
     char *disk_regex = NULL;
     char *service_name = NULL;
@@ -1037,48 +1047,25 @@ static int extract_files_setup P((void))
 	return -1;
     }
     amfree(service_name);
-    if ((hp = gethostbyname(tape_server_name)) == NULL)
-    {
-	printf("%s is an unknown host\n", tape_server_name);
-	return -1;
-    }
-    memset((char *)&tape_server, 0, sizeof(tape_server));
-    memcpy((char *)&tape_server.sin_addr, hp->h_addr, hp->h_length);
-    tape_server.sin_family = hp->h_addrtype;
-    tape_server.sin_port = sp->s_port;
-
-    /* contact the tape server */
-    if ((tape_server_socket = socket(AF_INET, SOCK_STREAM, 0)) == -1)
-    {
-	perror("Error creating socket");
-	exit(1);
-    }
-    if ((hp = gethostbyname(localhost)) == NULL)
-    {
-	(void)fprintf(stderr, "%s: %s is an unknown host\n",
-		      get_pname(), localhost);
-	dbprintf(("%s is an unknown host\n", localhost));
-	dbclose();
-	exit(1);
-    }
-    memset((char *)&myname, 0, sizeof(myname));
-    memcpy((char *)&myname.sin_addr, hp->h_addr, hp->h_length);
-    myname.sin_family = hp->h_addrtype;
     seteuid(0);					/* it either works ... */
     setegid(0);
-    if (bind_reserved(tape_server_socket, &myname) != 0)
+    tape_server_socket = stream_client(tape_server_name,
+				       ntohs(sp->s_port),
+				       DEFAULT_SIZE,
+				       DATABUF_SIZE*2,
+				       &my_port);
+    if (tape_server_socket < 0)
     {
-	perror("amrecover: Error binding socket");
-	exit(2);
+	printf("cannot connect to %s: %s\n", tape_server_name, strerror(errno));
+	return -1;
+    }
+    if (my_port >= IPPORT_RESERVED) {
+	aclose(tape_server_socket);
+	printf("did not get a reserved port: %d\n", my_port);
+	return -1;
     }
     setegid(getgid());
     seteuid(getuid());				/* put it back */
-    if (connect(tape_server_socket, (struct sockaddr *)&tape_server,
-		sizeof(struct sockaddr_in)) == -1)
-    {
-	perror("Error connecting to tape server");
-	exit(2);
-    }
 
     /* do the security thing */
 #if defined(KRB4_SECURITY)
@@ -1187,7 +1174,8 @@ static void extract_files_child(in_fd, elist)
     int in_fd;
     EXTRACT_LIST *elist;
 {
-    int no_initial_params;
+    int save_errno;
+    int extra_params = 0;
     int i,j=0;
     char **restore_args = NULL;
     int files_off_tape;
@@ -1227,6 +1215,7 @@ static void extract_files_child(in_fd, elist)
 	if (strcmp(file.program, GNUTAR) == 0)
 	    dumptype = IS_GNUTAR;
 #endif
+
 	if (dumptype == IS_UNKNOWN) {
 	    len_program = strlen(file.program);
 	    if(len_program >= 3 &&
@@ -1235,35 +1224,46 @@ static void extract_files_child(in_fd, elist)
 	}
 
 #ifdef SAMBA_CLIENT
-	if (dumptype == IS_UNKNOWN && strcmp(file.program, SAMBA_CLIENT) == 0)
-	    dumptype = IS_SAMBA;
+	if (dumptype == IS_UNKNOWN && strcmp(file.program, SAMBA_CLIENT) ==0) {
+	    if (samba_extract_method == SAMBA_TAR)
+	      dumptype = IS_TAR;
+	    else
+	      dumptype = IS_SAMBA;
+	}
 #endif
     }
 
     /* form the arguments to restore */
+    files_off_tape = length_of_tape_list(elist);
     switch (dumptype) {
     case IS_SAMBA:
 #ifdef SAMBA_CLIENT
-    	no_initial_params = 10;
+    	extra_params = 10;
     	break;
 #endif
     case IS_TAR:
     case IS_GNUTAR:
-        no_initial_params = 3;
+        extra_params = 3;
         break;
     case IS_UNKNOWN:
     case IS_DUMP:
 #ifdef AIX_BACKUP
-        no_initial_params = 2;
+        extra_params = 2;
 #else
-        no_initial_params = 4;
+#if defined(XFSDUMP)
+	if (strcmp(file.program, XFSDUMP) == 0) {
+            extra_params = 4 + files_off_tape;
+	} else
+#endif
+	{
+        extra_params = 4;
+	}
 #endif
     	break;
     }
 
-    files_off_tape = length_of_tape_list(elist);
     if ((restore_args
-	 = (char **)malloc((no_initial_params + files_off_tape + 1)
+	 = (char **)malloc((extra_params + files_off_tape + 1)
 			   * sizeof(char *)))
 	== NULL) 
     {
@@ -1278,16 +1278,15 @@ static void extract_files_child(in_fd, elist)
     	if (smbpass) {
             restore_args[j++] = stralloc(file.disk);
 	    passwd_field=j;
-    	    restore_args[j++] = smbpass;
     	    restore_args[j++] = stralloc("-U");
-    	    restore_args[j++] = stralloc(SAMBA_USER);
+    	    restore_args[j++] = smbpass;
     	    if (domain) {
             	restore_args[j++] = stralloc("-W");
     	    	restore_args[j++] = stralloc(domain);
    	    } else
-		no_initial_params -= 2;
+		extra_params -= 2;
     	} else
-	    no_initial_params -= 6;
+	    extra_params -= 6;
     	restore_args[j++] = stralloc("-d0");
     	restore_args[j++] = stralloc("-Tx");
 	restore_args[j++] = stralloc("-");	/* data on stdin */
@@ -1305,9 +1304,17 @@ static void extract_files_child(in_fd, elist)
 #ifdef AIX_BACKUP
         restore_args[j++] = stralloc("-xB");
 #else
+#if defined(XFSDUMP)
+	if (strcmp(file.program, XFSDUMP) == 0) {
+            restore_args[j++] = stralloc("-v");
+            restore_args[j++] = stralloc("silent");
+	} else
+#endif
+	{
         restore_args[j++] = stralloc("xbf");
         restore_args[j++] = stralloc("2");	/* read in units of 1K */
         restore_args[j++] = stralloc("-");	/* data on stdin */
+	}
 #endif
     }
   
@@ -1317,43 +1324,82 @@ static void extract_files_child(in_fd, elist)
     	case IS_TAR:
     	case IS_GNUTAR:
     	case IS_SAMBA:
-	    restore_args[j+i] = stralloc2(".", fn->path);
+	    restore_args[j++] = stralloc2(".", fn->path);
 	    break;
 	case IS_UNKNOWN:
 	case IS_DUMP:
-	    restore_args[j+i] = stralloc(fn->path);
+#if defined(XFSDUMP)
+	    if (strcmp(file.program, XFSDUMP) == 0) {
+		/*
+		 * xfsrestore needs a -s option before each file to be
+		 * restored, and also wants them to be relative paths.
+		 */
+		restore_args[j++] = stralloc("-s");
+		restore_args[j++] = stralloc(fn->path + 1);
+	    } else
+#endif
+	    {
+	    restore_args[j++] = stralloc(fn->path);
+	    }
   	}
     }
-    j+=i;
+#if defined(XFSDUMP)
+    if (strcmp(file.program, XFSDUMP) == 0) {
+	restore_args[j++] = stralloc("-");
+	restore_args[j++] = stralloc(".");
+    }
+#endif
     restore_args[j] = NULL;
-  
+
     switch (dumptype) {
     case IS_SAMBA:
 #ifdef SAMBA_CLIENT
     	cmd = stralloc(SAMBA_CLIENT);
     	break;
+#else
+	/* fall through to ... */
 #endif
     case IS_TAR:
     case IS_GNUTAR:
 #ifndef GNUTAR
 	fprintf(stderr, "warning: GNUTAR program not available.\n");
-	cmd = "tar";
+	cmd = stralloc("tar");
 #else
   	cmd = stralloc(GNUTAR);
 #endif
     	break;
     case IS_UNKNOWN:
     case IS_DUMP:
-#ifndef RESTORE
-	fprintf(stderr, "RESTORE program not available.\n");
-	cmd = "restore";
-#else
-    	cmd = stralloc(RESTORE);
+	cmd = NULL;
+#if defined(DUMP)
+	if (strcmp(file.program, DUMP) == 0) {
+    	    cmd = stralloc(RESTORE);
+	}
 #endif
+#if defined(VDUMP)
+	if (strcmp(file.program, VDUMP) == 0) {
+    	    cmd = stralloc(VRESTORE);
+	}
+#endif
+#if defined(VXDUMP)
+	if (strcmp(file.program, VXDUMP) == 0) {
+    	    cmd = stralloc(VXRESTORE);
+	}
+#endif
+#if defined(XFSDUMP)
+	if (strcmp(file.program, XFSDUMP) == 0) {
+    	    cmd = stralloc(XFSRESTORE);
+	}
+#endif
+	if (cmd == NULL) {
+	    fprintf(stderr, "warning: restore program for %s not available.\n",
+		    file.program);
+	    cmd = stralloc("restore");
+	}
     }
     if (cmd) {
         dbprintf(("Exec'ing %s with arguments:\n", cmd));
-	for (i = 0; i < no_initial_params + files_off_tape; i++) {
+	for (i = 0; i < j; i++) {
 	    if( i == passwd_field)
 		dbprintf(("\tXXXXX\n"));
 	    else
@@ -1361,10 +1407,12 @@ static void extract_files_child(in_fd, elist)
 	}
         (void)execv(cmd, restore_args);
 	/* only get here if exec failed */
-	for (i = 0; i < no_initial_params + files_off_tape; i++) {
+	save_errno = errno;
+	for (i = 0; i < j; i++) {
   	    amfree(restore_args[i]);
   	}
   	amfree(restore_args);
+	errno = save_errno;
         perror("amrecover couldn't exec");
         fprintf(stderr, " problem executing %s\n", cmd);
 	amfree(cmd);
@@ -1424,6 +1472,12 @@ void extract_files P((void))
 	tape_device_name = newstralloc(tape_device_name, l+4);
     }
 
+    if (strcmp(tape_device_name, "/dev/null") == 0)
+    {
+	printf("amrecover: warning: using %s as the tape device will not work\n",
+	       tape_device_name);
+    }
+
     first=1;
     for (elist = first_tape_list(); elist != NULL; elist = next_tape_list(elist))
 	if(elist->tape[0]!='/') {
@@ -1453,6 +1507,10 @@ void extract_files P((void))
     printf("\n");
     getcwd(buf, sizeof(buf));
     printf("Restoring files into directory %s\n", buf);
+#ifdef SAMBA_CLIENT
+    if (samba_extract_method == SAMBA_SMBCLIENT)
+      printf("(unless it is a Samba backup, that will go through to the SMB server)\n");
+#endif
     if (!okay_to_continue())
 	return;
     printf("\n");
@@ -1518,7 +1576,8 @@ void extract_files P((void))
 	    fprintf(stderr,
 		    "extract_list - child returned non-zero status: %d\n",
 		    WEXITSTATUS(child_stat));
-	    exit(1);
+	    if (!okay_to_continue())
+		return;
 	}
 
 	/* finish up */
