@@ -1,6 +1,6 @@
 /*
  * Amanda, The Advanced Maryland Automatic Network Disk Archiver
- * Copyright (c) 1991-1998 University of Maryland at College Park
+ * Copyright (c) 1991-1998, 2000 University of Maryland at College Park
  * All Rights Reserved.
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
@@ -24,7 +24,7 @@
  * file named AUTHORS, in the root directory of this distribution.
  */
 /*
- * $Id$
+ * $Id: help.c,v 1.4.4.2 2000/01/21 05:08:38 oliva Exp $
  *
  * implements the "help" command in amrecover
  */
@@ -50,11 +50,13 @@ void help_list P((void))
     printf("lcd directory     - change cwd on local file system\n");
     printf("ls                - list directory on virtual file system\n");
     printf("lpwd              - show cwd on local file system\n");
+    printf("mode              - show the method used to extract SMB shares\n");
     printf("pwd               - show cwd on virtual file system\n");
     printf("quit\n");
     printf("setdate {YYYY-MM-DD|--MM-DD|---DD} - set date of look\n");
     printf("setdisk diskname [mountpoint] - select disk on dump host\n");
     printf("sethost host      - select dump host\n");
-
+    printf("settape [host:][device|default] - select tape server and/or device\n");
+    printf("setmode smb|tar   - select the method used to extract SMB shares\n");
     printf("\n");
 }
