@@ -1,6 +1,6 @@
 /*
  * Amanda, The Advanced Maryland Automatic Network Disk Archiver
- * Copyright (c) 1991-1998 University of Maryland at College Park
+ * Copyright (c) 1991-1998, 2000 University of Maryland at College Park
  * All Rights Reserved.
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
@@ -24,7 +24,7 @@
  * file named AUTHORS, in the root directory of this distribution.
  */
 /*
- * $Id$
+ * $Id: set_commands.c,v 1.11.2.3 2000/01/21 05:08:41 oliva Exp $
  *
  * implements the "set" commands in amrecover
  */
@@ -32,6 +32,9 @@
 #include "amanda.h"
 #include "amrecover.h"
 
+#ifdef SAMBA_CLIENT
+extern unsigned short samba_extract_method;
+#endif /* SAMBA_CLIENT */
 
 /* sets a date, mapping given date into standard form if needed */
 int set_date(date)
@@ -334,3 +337,70 @@ void show_directory P((void))
     else
 	printf("%s%s\n", mount_point, disk_path);
 }
+
+
+/* set the tape server and device */
+void set_tape (tape)
+    char *tape;
+{
+    char *tapedev = strchr(tape, ':');
+
+    if (tapedev)
+    {
+	if (tapedev != tape) {
+	    *tapedev = '\0';
+	    tape_server_name = newstralloc(tape_server_name, tape);
+	} else
+	    amfree(tape_server_name);
+	++tapedev;
+    } else
+	tapedev = tape;
+    
+    if (tapedev[0])
+    {
+	if (strcmp(tapedev, "default") == 0)
+	    amfree(tape_device_name);
+	else
+	    tape_device_name = newstralloc(tape_device_name, tapedev);
+    }
+
+    if (tape_device_name)
+	printf ("Using tape %s", tape_device_name);
+    else
+	printf ("Using default tape");
+
+    if (tape_server_name)
+	printf (" from server %s.\n", tape_server_name);
+    else
+	printf (".\nTape server unspecified, assumed to be %s.\n",
+		server_name);
+}
+
+void set_mode (mode)
+int mode;
+{
+#ifdef SAMBA_CLIENT
+  if (mode == SAMBA_SMBCLIENT) {
+    printf ("SAMBA dumps will be extracted using smbclient\n");
+    samba_extract_method = SAMBA_SMBCLIENT;
+  } else {
+    if (mode == SAMBA_TAR) {
+      printf ("SAMBA dumps will be extracted as TAR dumps\n");
+      samba_extract_method = SAMBA_TAR;
+    }
+  }
+#endif /* SAMBA_CLIENT */
+}
+
+void show_mode (void) 
+{
+#ifdef SAMBA_CLIENT
+  printf ("SAMBA dumps are extracted ");
+
+  if (samba_extract_method == SAMBA_TAR) {
+    printf (" as TAR dumps\n");
+  } else {
+    printf ("using smbclient\n");
+  }
+#endif /* SAMBA_CLIENT */
+}
