@@ -23,7 +23,7 @@
  * Authors: the Amanda Development Team.  Its members are listed in a
  * file named AUTHORS, in the root directory of this distribution.
  */
-/* $Id$
+/* $Id: amidxtaped.c,v 1.25.2.3 1999/10/02 22:02:03 jrj Exp $
  *
  * This daemon extracts a dump image off a tape for amrecover and
  * returns it over the network. It basically, reads a number of
@@ -79,7 +79,7 @@ get_client_line()
 	    break;
 	}
 	/*
-	 * Hmmm.  We got a "line" from areads(), which means it saw
+	 * Hmmm.  We got a "line" from agets(), which means it saw
 	 * a '\n' (or EOF, etc), but there was not a '\r' before it.
 	 * Put a '\n' back in the buffer and loop for more.
 	 */
@@ -93,10 +93,6 @@ int main(argc, argv)
 int argc;
 char **argv;
 {
-#ifdef FORCE_USERID
-    char *pwname;
-    struct passwd *pwptr;
-#endif	/* FORCE_USERID */
     int amrestore_nargs;
     char **amrestore_args;
     char *buf = NULL;
@@ -123,6 +119,8 @@ char **argv;
 	close(fd);
     }
 
+    safe_cd();
+
     set_pname("amidxtaped");
 
 #ifdef FORCE_USERID
@@ -130,13 +128,13 @@ char **argv;
     /* we'd rather not run as root */
 
     if(geteuid() == 0) {
-	pwname = CLIENT_LOGIN;
-	if((pwptr = getpwnam(pwname)) == NULL)
-	    error("error [cannot find user %s in passwd file]\n", pwname);
+	if(client_uid == (uid_t) -1) {
+	    error("error [cannot find user %s in passwd file]\n", CLIENT_LOGIN);
+	}
 
-	initgroups(pwname, pwptr->pw_gid);
-	setgid(pwptr->pw_gid);
-	setuid(pwptr->pw_uid);
+	initgroups(CLIENT_LOGIN, client_gid);
+	setgid(client_gid);
+	setuid(client_uid);
     }
 
 #endif	/* FORCE_USERID */
@@ -237,7 +235,9 @@ char **argv;
 	(void)execv(amrestore_path, amrestore_args);
 
 	/* only get here if exec failed */
-	dbprintf(("Child couldn't exec amrestore\n"));
+	dbprintf(("Child could not exec %s: %s\n",
+		  amrestore_path,
+		  strerror(errno)));
 	return 1;
 	/*NOT REACHED*/
     }
