@@ -25,7 +25,7 @@
  *			   University of Maryland at College Park
  */
 /*
- * $Id$
+ * $Id: amadmin.c,v 1.49.2.14 2001/02/28 14:21:54 martinea Exp $
  *
  * controlling process for the Amanda backup system
  */
@@ -47,6 +47,12 @@ void force P((int argc, char **argv));
 void force_one P((disk_t *dp));
 void unforce P((int argc, char **argv));
 void unforce_one P((disk_t *dp));
+void force_bump P((int argc, char **argv));
+void force_bump_one P((disk_t *dp));
+void force_no_bump P((int argc, char **argv));
+void force_no_bump_one P((disk_t *dp));
+void unforce_bump P((int argc, char **argv));
+void unforce_bump_one P((disk_t *dp));
 void reuse P((int argc, char **argv));
 void noreuse P((int argc, char **argv));
 void info P((int argc, char **argv));
@@ -62,22 +68,28 @@ void bumpsize P((void));
 void diskloop P((int argc, char **argv, char *cmdname,
 		 void (*func) P((disk_t *dp))));
 char *seqdatestr P((int seq));
-static int next_level0 P((disk_t *dp, info_t *ip));
+static int next_level0 P((disk_t *dp, info_t *info));
 int bump_thresh P((int level));
-int search_logfile P((char *label, int datestamp, int datestamp_aux, char *logfile));
 void export_db P((int argc, char **argv));
 void import_db P((int argc, char **argv));
 void disklist P((int argc, char **argv));
 void disklist_one P((disk_t *dp));
 
+char *config_name = NULL;
+char *config_dir = NULL;
+
+static char *conffile = NULL;
+static char *conf_tapelist = NULL;
+
 int main(argc, argv)
 int argc;
 char **argv;
 {
-    char *confdir = NULL;
     int fd;
     unsigned long malloc_hist_1, malloc_size_1;
     unsigned long malloc_hist_2, malloc_size_2;
+    char *conf_diskfile;
+    char *conf_infofile;
 
     for(fd = 3; fd < FD_SETSIZE; fd++) {
 	/*
@@ -89,6 +101,8 @@ char **argv;
 	close(fd);
     }
 
+    safe_cd();
+
     set_pname("amadmin");
 
     malloc_size_1 = malloc_inuse(&malloc_hist_1);
@@ -101,27 +115,47 @@ char **argv;
 	for(argc=0; version_info[argc]; printf("%s",version_info[argc++]));
 	return 0;
     }
-    confdir = vstralloc(CONFIG_DIR, "/", argv[1], NULL);
-    if(chdir(confdir)) {
-	fprintf(stderr,"%s: could not find config dir %s\n", argv[0], confdir);
-	amfree(confdir);
-	usage();
+    config_name = argv[1];
+    config_dir = vstralloc(CONFIG_DIR, "/", config_name, "/", NULL);
+    conffile = stralloc2(config_dir, CONFFILE_NAME);
+    if(read_conffile(conffile)) {
+	error("could not find config file \"%s\"", conffile);
     }
-    amfree(confdir);
+    amfree(conffile);
+    conf_diskfile = getconf_str(CNF_DISKFILE);
+    if (*conf_diskfile == '/') {
+	conf_diskfile = stralloc(conf_diskfile);
+    } else {
+	conf_diskfile = stralloc2(config_dir, conf_diskfile);
+    }
+    if((diskqp = read_diskfile(conf_diskfile)) == NULL) {
+	error("could not load disklist \"%s\"", conf_diskfile);
+    }
+    amfree(conf_diskfile);
+    conf_tapelist = getconf_str(CNF_TAPELIST);
+    if (*conf_tapelist == '/') {
+	conf_tapelist = stralloc(conf_tapelist);
+    } else {
+	conf_tapelist = stralloc2(config_dir, conf_tapelist);
+    }
+    if(read_tapelist(conf_tapelist)) {
+	error("could not load tapelist \"%s\"", conf_tapelist);
+    }
+    conf_infofile = getconf_str(CNF_INFOFILE);
+    if (*conf_infofile == '/') {
+	conf_infofile = stralloc(conf_infofile);
+    } else {
+	conf_infofile = stralloc2(config_dir, conf_infofile);
+    }
+    if(open_infofile(conf_infofile)) {
+	error("could not open info db \"%s\"", conf_infofile);
+    }
+    amfree(conf_infofile);
 
-    if(read_conffile(CONFFILE_NAME))
-	error("could not find \"%s\" in this directory.\n", CONFFILE_NAME);
-
-    if((diskqp = read_diskfile(getconf_str(CNF_DISKFILE))) == NULL)
-	error("could not load \"%s\"\n", getconf_str(CNF_DISKFILE));
-
-    if(read_tapelist(getconf_str(CNF_TAPELIST)))
-	error("could not load \"%s\"\n", getconf_str(CNF_TAPELIST));
-
-    if(open_infofile(getconf_str(CNF_INFOFILE)))
-	error("could not open info db \"%s\"\n", getconf_str(CNF_INFOFILE));
-
-    if(strcmp(argv[2],"force") == 0) force(argc, argv);
+    if(strcmp(argv[2],"force-bump") == 0) force_bump(argc, argv);
+    else if(strcmp(argv[2],"force-no-bump") == 0) force_no_bump(argc, argv);
+    else if(strcmp(argv[2],"unforce-bump") == 0) unforce_bump(argc, argv);
+    else if(strcmp(argv[2],"force") == 0) force(argc, argv);
     else if(strcmp(argv[2],"unforce") == 0) unforce(argc, argv);
     else if(strcmp(argv[2],"reuse") == 0) reuse(argc, argv);
     else if(strcmp(argv[2],"no-reuse") == 0) noreuse(argc, argv);
@@ -141,6 +175,8 @@ char **argv;
     }
     close_infofile();
     clear_tapelist();
+    amfree(conf_tapelist);
+    amfree(config_dir);
 
     malloc_size_2 = malloc_inuse(&malloc_hist_2);
 
@@ -159,9 +195,15 @@ void usage P((void))
     fprintf(stderr, "    Valid <command>s are:\n");
     fprintf(stderr,"\tversion\t\t\t\t# Show version info.\n");
     fprintf(stderr,
-	    "\tforce <hostname> <disks> ...\t# Force level 0 tonight.\n");
+	    "\tforce <hostname> <disks> ...\t# Force level 0 at next run.\n");
     fprintf(stderr,
 	    "\tunforce <hostname> <disks> ...\t# Clear force command.\n");
+    fprintf(stderr,
+	    "\tforce-bump <hostname> <disks> ...\t# Force bump at next run.\n");
+    fprintf(stderr,
+	    "\tforce-no-bump <hostname> <disks> ...\t# Force no-bump at next run.\n");
+    fprintf(stderr,
+	    "\tunforce-bump <hostname> <disks> ...\t# Clear bump command.\n");
     fprintf(stderr,
 	    "\treuse <tapelabel> ...\t\t# re-use this tape.\n");
     fprintf(stderr,
@@ -184,9 +226,9 @@ void usage P((void))
 	    "\texport [<hostname> [<disks>]]\t# Export curinfo database to stdout.\n");
     fprintf(stderr,
 	    "\timport\t\t\t\t# Import curinfo database from stdin.\n");
-/*  fprintf(stderr,
-**	    "\tdisklist [<hostname> [<disks> ...]]\t# Debug disklist entries.\n");
-*/
+    fprintf(stderr,
+  	    "\tdisklist [<hostname> [<disks> ...]]\t# Show disklist entries.\n");
+
     exit(1);
 }
 
@@ -214,16 +256,49 @@ int seq;
 
 
 /* when is next level 0 due? 0 = tonight, 1 = tommorrow, etc*/
-static int next_level0(dp, ip)
+static int next_level0(dp, info)
 disk_t *dp;
-info_t *ip;
+info_t *info;
 {
     if(dp->strategy == DS_NOFULL)
 	return 1;	/* fake it */
-    else if(ip->inf[0].date < (time_t)0)
+    else if(info->inf[0].date < (time_t)0)
 	return 0;	/* new disk */
     else
-	return dp->dumpcycle - days_diff(ip->inf[0].date, today);
+	return dp->dumpcycle - days_diff(info->inf[0].date, today);
+}
+
+static void check_dumpuser()
+{
+    static int been_here = 0;
+    uid_t uid_me;
+    uid_t uid_dumpuser;
+    char *dumpuser;
+    struct passwd *pw;
+
+    if (been_here) {
+       return;
+    }
+    uid_me = getuid();
+    uid_dumpuser = uid_me;
+    dumpuser = getconf_str(CNF_DUMPUSER);
+
+    if ((pw = getpwnam(dumpuser)) == NULL) {
+	error("cannot look up dump user \"%s\"", dumpuser);
+	/* NOTREACHED */
+    }
+    uid_dumpuser = pw->pw_uid;
+    if ((pw = getpwuid(uid_me)) == NULL) {
+	error("cannot look up my own uid %ld", (long)uid_me);
+	/* NOTREACHED */
+    }
+    if (uid_me != uid_dumpuser) {
+	fprintf(stderr, "WARNING: running as user \"%s\" instead of \"%s\".\n",
+	        pw->pw_name, dumpuser);
+	fprintf(stderr, "WARNING: run \"amcheck -st %s\"\n", config_name);
+    }
+    been_here = 1;
+    return;
 }
 
 /* ----------------------------------------------- */
@@ -279,12 +354,20 @@ disk_t *dp;
 {
     char *hostname = dp->host->hostname;
     char *diskname = dp->name;
-    info_t inf;
+    info_t info;
 
-    get_info(hostname, diskname, &inf);
-    inf.command = PLANNER_FORCE;
-    if(put_info(hostname, diskname, &inf) == 0) {
-	printf("%s: %s:%s is set to a forced level 0 tonight.\n",
+#if TEXTDB
+    check_dumpuser();
+#endif
+    get_info(hostname, diskname, &info);
+    info.command |= FORCE_FULL;
+    if(info.command & FORCE_BUMP) {
+	info.command ^= FORCE_BUMP;
+	printf("%s: WARNING: %s:%s FORCE_BUMP command was cleared.\n",
+	       get_pname(), hostname, diskname);
+    }
+    if(put_info(hostname, diskname, &info) == 0) {
+	printf("%s: %s:%s is set to a forced level 0 at next run.\n",
 	       get_pname(), hostname, diskname);
     } else {
 	fprintf(stderr, "%s: %s:%s could not be forced.\n",
@@ -309,12 +392,15 @@ disk_t *dp;
 {
     char *hostname = dp->host->hostname;
     char *diskname = dp->name;
-    info_t inf;
+    info_t info;
 
-    get_info(hostname, diskname, &inf);
-    if(inf.command == PLANNER_FORCE) {
-	inf.command = NO_COMMAND;
-	if(put_info(hostname, diskname, &inf) == 0){
+    get_info(hostname, diskname, &info);
+    if(info.command & FORCE_FULL) {
+#if TEXTDB
+	check_dumpuser();
+#endif
+	info.command ^= FORCE_FULL;
+	if(put_info(hostname, diskname, &info) == 0){
 	    printf("%s: force command for %s:%s cleared.\n",
 		   get_pname(), hostname, diskname);
 	} else {
@@ -337,6 +423,129 @@ char **argv;
 }
 
 
+/* ----------------------------------------------- */
+
+
+void force_bump_one(dp)
+disk_t *dp;
+{
+    char *hostname = dp->host->hostname;
+    char *diskname = dp->name;
+    info_t info;
+
+#if TEXTDB
+    check_dumpuser();
+#endif
+    get_info(hostname, diskname, &info);
+    info.command |= FORCE_BUMP;
+    if(info.command & FORCE_NO_BUMP) {
+	info.command ^= FORCE_NO_BUMP;
+	printf("%s: WARNING: %s:%s FORCE_NO_BUMP command was cleared.\n",
+	       get_pname(), hostname, diskname);
+    }
+    if (info.command & FORCE_FULL) {
+	info.command ^= FORCE_FULL;
+	printf("%s: WARNING: %s:%s FORCE_FULL command was cleared.\n",
+	       get_pname(), hostname, diskname);
+    }
+    if(put_info(hostname, diskname, &info) == 0) {
+	printf("%s: %s:%s is set to bump at next run.\n",
+	       get_pname(), hostname, diskname);
+    } else {
+	fprintf(stderr, "%s: %s:%s could not be forced to bump.\n",
+		get_pname(), hostname, diskname);
+    }
+}
+
+
+void force_bump(argc, argv)
+int argc;
+char **argv;
+{
+    diskloop(argc, argv, "force-bump", force_bump_one);
+}
+
+
+/* ----------------------------------------------- */
+
+
+void force_no_bump_one(dp)
+disk_t *dp;
+{
+    char *hostname = dp->host->hostname;
+    char *diskname = dp->name;
+    info_t info;
+
+#if TEXTDB
+    check_dumpuser();
+#endif
+    get_info(hostname, diskname, &info);
+    info.command |= FORCE_NO_BUMP;
+    if(info.command & FORCE_BUMP) {
+	info.command ^= FORCE_BUMP;
+	printf("%s: WARNING: %s:%s FORCE_BUMP command was cleared.\n",
+	       get_pname(), hostname, diskname);
+    }
+    if(put_info(hostname, diskname, &info) == 0) {
+	printf("%s: %s:%s is set to not bump at next run.\n",
+	       get_pname(), hostname, diskname);
+    } else {
+	fprintf(stderr, "%s: %s:%s could not be force to not bump.\n",
+		get_pname(), hostname, diskname);
+    }
+}
+
+
+void force_no_bump(argc, argv)
+int argc;
+char **argv;
+{
+    diskloop(argc, argv, "force-no-bump", force_no_bump_one);
+}
+
+
+/* ----------------------------------------------- */
+
+
+void unforce_bump_one(dp)
+disk_t *dp;
+{
+    char *hostname = dp->host->hostname;
+    char *diskname = dp->name;
+    info_t info;
+
+    get_info(hostname, diskname, &info);
+    if(info.command & (FORCE_BUMP | FORCE_NO_BUMP)) {
+#if TEXTDB
+	check_dumpuser();
+#endif
+	if(info.command & FORCE_BUMP)
+	    info.command ^= FORCE_BUMP;
+	if(info.command & FORCE_NO_BUMP)
+	    info.command ^= FORCE_NO_BUMP;
+	if(put_info(hostname, diskname, &info) == 0) {
+	    printf("%s: bump command for %s:%s cleared.\n",
+		   get_pname(), hostname, diskname);
+	} else {
+	    fprintf(stderr, "%s: %s:%s bump command could not be cleared.\n",
+		    get_pname(), hostname, diskname);
+	}
+    }
+    else {
+	printf("%s: no bump command outstanding for %s:%s, unchanged.\n",
+	       get_pname(), hostname, diskname);
+    }
+}
+
+
+void unforce_bump(argc, argv)
+int argc;
+char **argv;
+{
+    diskloop(argc, argv, "unforce-bump", unforce_bump_one);
+}
+
+
 /* ----------------------------------------------- */
 
 void reuse(argc, argv)
@@ -352,6 +561,7 @@ char **argv;
 	usage();
     }
 
+    check_dumpuser();
     for(count=3; count< argc; count++) {
 	tp = lookup_tapelabel(argv[count]);
 	if ( tp == NULL) {
@@ -359,13 +569,19 @@ char **argv;
 		argv[count]);
 	    continue;
 	}
-	if( tp->reuse == 0 ) tp->reuse = 1;
-	else fprintf(stderr, "reuse: tape %s already reusable.\n",
-		     argv[count]);
+	if( tp->reuse == 0 ) {
+	    tp->reuse = 1;
+	    printf("%s: marking tape %s as reusable.\n",
+		   get_pname(), argv[count]);
+	} else {
+	    fprintf(stderr, "%s: tape %s already reusable.\n",
+		    get_pname(), argv[count]);
+	}
     }
 
-    if(write_tapelist(getconf_str(CNF_TAPELIST)))
-	error("could not write \"%s\"\n", getconf_str(CNF_TAPELIST));
+    if(write_tapelist(conf_tapelist)) {
+	error("could not write tapelist \"%s\"", conf_tapelist);
+    }
 }
 
 void noreuse(argc, argv)
@@ -381,6 +597,7 @@ char **argv;
 	usage();
     }
 
+    check_dumpuser();
     for(count=3; count< argc; count++) {
 	tp = lookup_tapelabel(argv[count]);
 	if ( tp == NULL) {
@@ -388,13 +605,19 @@ char **argv;
 		argv[count]);
 	    continue;
 	}
-	if( tp->reuse == 1 ) tp->reuse = 0;
-	else fprintf(stderr, "no-reuse: tape %s already not reusable.\n",
-		     argv[count]);
+	if( tp->reuse == 1 ) {
+	    tp->reuse = 0;
+	    printf("%s: marking tape %s as not reusable.\n",
+		   get_pname(), argv[count]);
+	} else {
+	    fprintf(stderr, "%s: tape %s already not reusable.\n",
+		    get_pname(), argv[count]);
+	}
     }
 
-    if(write_tapelist(getconf_str(CNF_TAPELIST)))
-	error("could not write \"%s\"\n", getconf_str(CNF_TAPELIST));
+    if(write_tapelist(conf_tapelist)) {
+	error("could not write tapelist \"%s\"", conf_tapelist);
+    }
 }
 
 
@@ -407,9 +630,9 @@ disk_t *dp;
 {
     char *hostname = dp->host->hostname;
     char *diskname = dp->name;
-    info_t inf;
+    info_t info;
 
-    if(get_info(hostname, diskname, &inf)) {
+    if(get_info(hostname, diskname, &info)) {
 	printf("%s: %s:%s NOT currently in database.\n",
 	       get_pname(), hostname, diskname);
 	return;
@@ -442,27 +665,32 @@ char **argv;
 void info_one(dp)
 disk_t *dp;
 {
-    info_t inf;
+    info_t info;
     int lev, lev0date;
     struct tm *tm;
     stats_t *sp;
 
-    get_info(dp->host->hostname, dp->name, &inf);
+    get_info(dp->host->hostname, dp->name, &info);
 
     printf("\nCurrent info for %s %s:\n", dp->host->hostname, dp->name);
-    if(inf.command) printf("  (Forcing to level 0 dump on next run)\n");
+    if(info.command & FORCE_FULL) 
+	printf("  (Forcing to level 0 dump at next run)\n");
+    if(info.command & FORCE_BUMP) 
+	printf("  (Forcing bump at next run)\n");
+    if(info.command & FORCE_NO_BUMP) 
+	printf("  (Forcing no-bump at next run)\n");
     printf("  Stats: dump rates (kps), Full:  %5.1f, %5.1f, %5.1f\n",
-	   inf.full.rate[0], inf.full.rate[1], inf.full.rate[2]);
+	   info.full.rate[0], info.full.rate[1], info.full.rate[2]);
     printf("                    Incremental:  %5.1f, %5.1f, %5.1f\n",
-	   inf.incr.rate[0], inf.incr.rate[1], inf.incr.rate[2]);
+	   info.incr.rate[0], info.incr.rate[1], info.incr.rate[2]);
     printf("          compressed size, Full: %5.1f%%,%5.1f%%,%5.1f%%\n",
-	   inf.full.comp[0]*100, inf.full.comp[1]*100, inf.full.comp[2]*100);
+	   info.full.comp[0]*100, info.full.comp[1]*100, info.full.comp[2]*100);
     printf("                    Incremental: %5.1f%%,%5.1f%%,%5.1f%%\n",
-	   inf.incr.comp[0]*100, inf.incr.comp[1]*100, inf.incr.comp[2]*100);
+	   info.incr.comp[0]*100, info.incr.comp[1]*100, info.incr.comp[2]*100);
 
     printf("  Dumps: lev datestmp  tape             file   origK   compK secs\n");
-    lev0date = inf.inf[0].date;
-    for(lev = 0, sp = &inf.inf[0]; lev < 9; lev++, sp++) {
+    lev0date = info.inf[0].date;
+    for(lev = 0, sp = &info.inf[0]; lev < 9; lev++, sp++) {
 	if(sp->date < (time_t)0 && sp->label[0] == '\0') continue;
 	tm = localtime(&sp->date);
 	printf("          %d  %04d%02d%02d  %-15s  %4d %7ld %7ld %4ld\n",
@@ -492,14 +720,14 @@ disk_t *dp;
 {
     host_t *hp;
     int days;
-    info_t inf;
+    info_t info;
 
     hp = dp->host;
-    if(get_info(hp->hostname, dp->name, &inf)) {
+    if(get_info(hp->hostname, dp->name, &info)) {
 	printf("new disk %s:%s ignored.\n", hp->hostname, dp->name);
     }
     else {
-	days = next_level0(dp, &inf);
+	days = next_level0(dp, &info);
 	if(days < 0) {
 	    printf("Overdue %2d day%s %s:%s\n",
 		   -days, (-days == 1) ? ": " : "s:",
@@ -562,15 +790,14 @@ void balance()
     } *sp;
     int seq, max_seq, total, balanced, runs_per_cycle, overdue, max_overdue;
     int runspercycle;
-    info_t inf;
+    info_t info;
 
     total = getconf_int(CNF_TAPECYCLE);
     max_seq = getconf_int(CNF_DUMPCYCLE)-1;	/* print at least this many */
     time(&today);
     runtapes = getconf_int(CNF_RUNTAPES);
     dumpcycle = getconf_int(CNF_DUMPCYCLE);
-    runspercycle = !getconf_seen(CNF_RUNSPERCYCLE) ? 0 : 
-		   getconf_int(CNF_RUNSPERCYCLE);
+    runspercycle = getconf_int(CNF_RUNSPERCYCLE);
     overdue = 0;
     max_overdue = 0;
 
@@ -595,11 +822,11 @@ void balance()
 	sp[seq].disks = sp[seq].origsize = sp[seq].outsize = 0;
 
     for(dp = diskqp->head; dp != NULL; dp = dp->next) {
-	if(get_info(dp->host->hostname, dp->name, &inf)) {
+	if(get_info(dp->host->hostname, dp->name, &info)) {
 	    printf("new disk %s:%s ignored.\n", dp->host->hostname, dp->name);
 	    continue;
 	}
-	seq = next_level0(dp, &inf);
+	seq = next_level0(dp, &info);
 	if(seq < 0) {
 	    overdue++;
 	    if (-seq > max_overdue)
@@ -616,12 +843,12 @@ void balance()
 	if(seq > max_seq) max_seq = seq;
 
 	sp[seq].disks++;
-	sp[seq].origsize += inf.inf[0].size;
-	sp[seq].outsize += inf.inf[0].csize;
+	sp[seq].origsize += info.inf[0].size;
+	sp[seq].outsize += info.inf[0].csize;
 
 	sp[total].disks++;
-	sp[total].origsize += inf.inf[0].size;
-	sp[total].outsize += inf.inf[0].csize;
+	sp[total].origsize += info.inf[0].size;
+	sp[total].outsize += info.inf[0].csize;
     }
 
     if(sp[total].outsize == 0) {
@@ -762,8 +989,9 @@ void bumpsize()
 	   getconf_real(CNF_BUMPMULT));
 
     printf("      Bump -> To  Threshold\n");
-    for(l = 1; l < 9; l++)
+    for(l = 1; l < 9; l++) {
 	printf("\t%d  ->  %d  %9d KB\n", l, l+1, bump_thresh(l));
+    }
     putchar('\n');
 }
 
@@ -783,23 +1011,26 @@ char **argv;
     printf("CURINFO Version %s CONF %s\n", version(), getconf_str(CNF_ORG));
 
     curtime = time(0);
-    if(gethostname(hostname, sizeof(hostname)-1) == -1)
-	error("could not determine host name: %s\n", strerror(errno));
+    if(gethostname(hostname, sizeof(hostname)-1) == -1) {
+	error("could not determine host name: %s", strerror(errno));
+    }
     hostname[sizeof(hostname)-1] = '\0';
     printf("# Generated by:\n#    host: %s\n#    date: %s",
 	   hostname, ctime(&curtime));
 
     printf("#    command:");
-    for(i = 0; i < argc; i++)
+    for(i = 0; i < argc; i++) {
 	printf(" %s", argv[i]);
+    }
 
     printf("\n# This file can be merged back in with \"amadmin import\".\n");
     printf("# Edit only with care.\n");
 
-    if(argc >= 4)
+    if(argc >= 4) {
 	diskloop(argc, argv, "export", export_one);
-    else for(dp = diskqp->head; dp != NULL; dp = dp->next)
+    } else for(dp = diskqp->head; dp != NULL; dp = dp->next) {
 	export_one(dp);
+    }
 }
 
 void export_one(dp)
@@ -815,6 +1046,8 @@ disk_t *dp;
     }
     printf("host: %s\ndisk: %s\n", dp->host->hostname, dp->name);
     printf("command: %d\n", info.command);
+    printf("last_level: %d\n",info.last_level);
+    printf("consecutive_runs: %d\n",info.consecutive_runs);
     printf("full-rate:");
     for(i=0;i<AVG_COUNT;i++) printf(" %f", info.full.rate[i]);
     printf("\nfull-comp:");
@@ -952,6 +1185,10 @@ int import_one P((void))
     char *hostname = NULL;
     char *diskname = NULL;
 
+#if TEXTDB
+    check_dumpuser();
+#endif
+
     memset(&info, 0, sizeof(info_t));
 
     for(level = 0; level < DUMP_LEVELS; level++) {
@@ -1005,10 +1242,21 @@ int import_one P((void))
     if((line = impget_line()) == NULL) goto shortfile_err;
     if(sscanf(line, "command: %d", &info.command) != 1) goto parse_err;
 
-    /* get rate: and comp: lines for full dumps */
+    /* get last_level and consecutive_runs */
 
     amfree(line);
     if((line = impget_line()) == NULL) goto shortfile_err;
+    rc = sscanf(line, "last_level: %d", &info.last_level);
+    if(rc == 1) {
+	amfree(line);
+	if((line = impget_line()) == NULL) goto shortfile_err;
+	if(sscanf(line, "consecutive_runs: %d", &info.consecutive_runs) != 1) goto parse_err;
+	amfree(line);
+	if((line = impget_line()) == NULL) goto shortfile_err;
+    }
+
+    /* get rate: and comp: lines for full dumps */
+
     rc = sscanf(line, "full-rate: %f %f %f",
 		&info.full.rate[0], &info.full.rate[1], &info.full.rate[2]);
     if(rc != 3) goto parse_err;
@@ -1208,6 +1456,9 @@ disk_t *dp;
     case DS_HANOI:
 	printf("HANOI\n");
 	break;
+    case DS_INCRONLY:
+	printf("INCRONLY\n");
+	break;
     }
 
     printf("        compress ");
