@@ -1,6 +1,6 @@
 /*
  * Amanda, The Advanced Maryland Automatic Network Disk Archiver
- * Copyright (c) 1991-1998 University of Maryland at College Park
+ * Copyright (c) 1991-2000 University of Maryland at College Park
  * All Rights Reserved.
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
@@ -24,7 +24,7 @@
  * file named AUTHORS, in the root directory of this distribution.
  */
 /*
- * $Id$
+ * $Id: amcheck.c,v 1.50.2.20 2001/03/15 02:34:57 jrjackson Exp $
  *
  * checks for common problems in server and clients
  */
@@ -54,43 +54,47 @@
 #   define WIFSIGNALED(r)        (((union wait *) &(r))->w_termsig != 0)
 #endif
 
-#define CHECK_TIMEOUT   30
 #define BUFFER_SIZE	32768
 
-static int mailout, overwrite;
+static int conf_ctimeout;
+static int overwrite;
 dgram_t *msg = NULL;
-char *confname;
+char *config_name = NULL;
+char *config_dir = NULL;
+
+static disklist_t *origqp;
 
 /* local functions */
 
 void usage P((void));
 int start_client_checks P((int fd));
-int start_server_check P((int fd));
+int start_server_check P((int fd, int do_localchk, int do_tapechk));
 int main P((int argc, char **argv));
 int scan_init P((int rc, int ns, int bk));
 int taperscan_slot P((int rc, char *slotstr, char *device));
 char *taper_scan P((void));
+int test_server_pgm P((FILE *outf, char *dir, char *pgm,
+		       int suid, uid_t dumpuid));
 
 void usage()
 {
-    error("Usage: amcheck%s [-M <username>] [-mwsc] <conf>", versionsuffix());
+    error("Usage: amcheck%s [-M <username>] [-mwsclt] <conf>", versionsuffix());
 }
 
 static unsigned long malloc_hist_1, malloc_size_1;
 static unsigned long malloc_hist_2, malloc_size_2;
 
-char *tempfname = NULL;
-
 int main(argc, argv)
 int argc;
 char **argv;
 {
     char buffer[BUFFER_SIZE], *cmd = NULL;
-    char *confdir, *version_string;
+    char *version_string;
     char *mainfname = NULL;
     char pid_str[NUM_STR_SIZE];
     int do_clientchk, clientchk_pid, client_probs;
-    int do_serverchk, serverchk_pid, server_probs;
+    int do_localchk, do_tapechk, serverchk_pid, server_probs;
+    int chk_flag;
     int opt, size, result_port, tempfd, mainfd;
     amwait_t retstat;
     pid_t pid;
@@ -99,6 +103,10 @@ char **argv;
     int fd;
     char *mailto = NULL;
     extern char *optarg;
+    int mailout;
+    char *tempfname = NULL;
+    char *conffile;
+    char *conf_diskfile;
     
     for(fd = 3; fd < FD_SETSIZE; fd++) {
 	/*
@@ -110,7 +118,10 @@ char **argv;
 	close(fd);
     }
 
+    safe_cd();
+
     set_pname("amcheck");
+    dbopen();
 
     malloc_size_1 = malloc_inuse(&malloc_hist_1);
 
@@ -132,37 +143,71 @@ char **argv;
     }
 
     mailout = overwrite = 0;
-    do_serverchk = do_clientchk = 1;
+    do_localchk = do_tapechk = do_clientchk = 0;
+    chk_flag = 0;
     server_probs = client_probs = 0;
     tempfd = mainfd = -1;
 
     /* process arguments */
 
-    while((opt = getopt(argc, argv, "M:mwsc")) != EOF) {
+    while((opt = getopt(argc, argv, "M:mwsclt")) != EOF) {
 	switch(opt) {
-	case 'm':	mailout = 1; break;
-	case 'M':	mailout = 1; mailto=optarg; break;
-	case 'w':	overwrite = 1; break;
-	case 's':	do_serverchk = 1; do_clientchk = 0; break;
-	case 'c':	do_serverchk = 0; do_clientchk = 1; break;
+	case 'M':	mailto=optarg;
+	case 'm':	
+#ifdef MAILER
+			mailout = 1;
+#else
+			printf("You can't use -%c because configure didn't find a mailer.\n",
+				opt);
+			exit(1);
+#endif
+			break;
+	case 's':	do_localchk = 1; do_tapechk = 1;
+			chk_flag = 1;
+			break;
+	case 'c':	do_clientchk = 1;
+			chk_flag = 1;
+			break;
+	case 'l':	do_localchk = 1;
+			chk_flag = 1;
+			break;
+	case 'w':	do_tapechk = 1; overwrite = 1;
+			chk_flag = 1;
+			break;
+	case 't':	do_tapechk = 1;
+			chk_flag = 1;
+			break;
 	case '?':
 	default:
 	    usage();
 	}
     }
     argc -= optind, argv += optind;
+    if(! chk_flag) {
+	do_localchk = do_clientchk = do_tapechk = 1;
+    }
 
     if(argc != 1) usage();
 
-    confname = *argv;
+    config_name = stralloc(*argv);
 
-    confdir = vstralloc(CONFIG_DIR, "/", confname, NULL);
-    if(chdir(confdir) != 0)
-	error("could not cd to confdir %s: %s", confdir, strerror(errno));
-    amfree(confdir);
-
-    if(read_conffile(CONFFILE_NAME))
-	error("could not read amanda config file");
+    config_dir = vstralloc(CONFIG_DIR, "/", config_name, "/", NULL);
+    conffile = stralloc2(config_dir, CONFFILE_NAME);
+    if(read_conffile(conffile)) {
+	error("could not find config file \"%s\"", conffile);
+    }
+    amfree(conffile);
+    conf_ctimeout = getconf_int(CNF_CTIMEOUT);
+    conf_diskfile = getconf_str(CNF_DISKFILE);
+    if (*conf_diskfile == '/') {
+	conf_diskfile = stralloc(conf_diskfile);
+    } else {
+	conf_diskfile = stralloc2(config_dir, conf_diskfile);
+    }
+    if((origqp = read_diskfile(conf_diskfile)) == NULL) {
+	error("could not load disklist %s", conf_diskfile);
+    }
+    amfree(conf_diskfile);
 
     /*
      * If both server and client side checks are being done, the server
@@ -172,16 +217,18 @@ char **argv;
      * If the output is to be mailed, the main output is also a disk file,
      * otherwise it is stdout.
      */
-    if(do_clientchk && do_serverchk) {
+    if(do_clientchk && (do_localchk || do_tapechk)) {
 	/* we need the temp file */
-	tempfname = vstralloc("/tmp/amcheck.temp.", pid_str, NULL);
+	tempfname = vstralloc(AMANDA_TMPDIR, "/amcheck.temp.", pid_str, NULL);
+	malloc_mark(tempfname);
 	if((tempfd = open(tempfname, O_RDWR|O_CREAT|O_TRUNC, 0600)) == -1)
 	    error("could not open %s: %s", tempfname, strerror(errno));
     }
 
     if(mailout) {
 	/* the main fd is a file too */
-	mainfname = vstralloc("/tmp/amcheck.main.", pid_str, NULL);
+	mainfname = vstralloc(AMANDA_TMPDIR, "/amcheck.main.", pid_str, NULL);
+	malloc_mark(mainfname);
 	if((mainfd = open(mainfname, O_RDWR|O_CREAT|O_TRUNC, 0600)) == -1)
 	    error("could not open %s: %s", mainfname, strerror(errno));
     }
@@ -191,18 +238,19 @@ char **argv;
 
     /* start server side checks */
 
-    if(do_serverchk)
-	serverchk_pid = start_server_check(mainfd);
-    else
+    if(do_localchk || do_tapechk) {
+	serverchk_pid = start_server_check(mainfd, do_localchk, do_tapechk);
+    } else {
 	serverchk_pid = 0;
+    }
 
     /* start client side checks */
 
     if(do_clientchk) {
-	clientchk_pid = start_client_checks(do_serverchk? tempfd : mainfd);
-    }
-    else
+	clientchk_pid = start_client_checks((do_localchk || do_tapechk) ? tempfd : mainfd);
+    } else {
 	clientchk_pid = 0;
+    }
 
     /* wait for child processes and note any problems */
 
@@ -210,24 +258,21 @@ char **argv;
 	if((pid = wait(&retstat)) == -1) {
 	    if(errno == EINTR) continue;
 	    else break;
-	}
-	else if(pid == clientchk_pid) {
+	} else if(pid == clientchk_pid) {
 	    client_probs = WIFSIGNALED(retstat) || WEXITSTATUS(retstat);
 	    clientchk_pid = 0;
-	}
-	else if(pid == serverchk_pid) {
+	} else if(pid == serverchk_pid) {
 	    server_probs = WIFSIGNALED(retstat) || WEXITSTATUS(retstat);
 	    serverchk_pid = 0;
-	}
-	else {
+	} else {
 	    char number[NUM_STR_SIZE];
 	    char *wait_msg = NULL;
 
 	    ap_snprintf(number, sizeof(number), "%ld", (long)pid);
 	    wait_msg = vstralloc("parent: reaped bogus pid ", number, "\n",
 				 NULL);
-	    for(l = 0, n = strlen(buffer); l < n; l += s) {
-		if((s = write(mainfd, buffer + l, n - l)) < 0) {
+	    for(l = 0, n = strlen(wait_msg); l < n; l += s) {
+		if((s = write(mainfd, wait_msg + l, n - l)) < 0) {
 		    error("write main file: %s", strerror(errno));
 		}
 	    }
@@ -238,7 +283,7 @@ char **argv;
 
     /* copy temp output to main output and write tagline */
 
-    if(do_clientchk && do_serverchk) {
+    if(do_clientchk && (do_localchk || do_tapechk)) {
 	if(lseek(tempfd, 0, 0) == -1)
 	    error("seek temp file: %s", strerror(errno));
 
@@ -254,7 +299,6 @@ char **argv;
 	aclose(tempfd);
 	unlink(tempfname);
     }
-    amfree(tempfname);
 
     version_string = vstralloc("\n",
 			       "(brought to you by Amanda ", version(), ")\n",
@@ -265,6 +309,8 @@ char **argv;
 	}
     }
     amfree(version_string);
+    amfree(config_dir);
+    amfree(config_name);
 
     malloc_size_2 = malloc_inuse(&malloc_hist_2);
 
@@ -273,6 +319,7 @@ char **argv;
     }
 
     /* send mail if requested, but only if there were problems */
+#ifdef MAILER
 
     if((server_probs || client_probs) && mailout) {
 	fflush(stdout);
@@ -291,15 +338,18 @@ char **argv;
 	    error("mail command failed: %s", cmd);
 	amfree(cmd);
     }
-    if(mailout)
+    if(mailout) {
 	unlink(mainfname);
+    }
+#endif
+    dbclose();
     return (server_probs || client_probs);
 }
 
 /* --------------------------------------------------- */
 
 int nslots, backwards, found, got_match, tapedays;
-extern char *datestamp;
+char *datestamp;
 char *first_match_label = NULL, *first_match = NULL, *found_device = NULL;
 char *label;
 char *searchlabel, *labelstr;
@@ -388,7 +438,7 @@ char *taper_scan()
     found = 0;
     got_match = 0;
 
-    changer_scan(scan_init, taperscan_slot);
+    changer_find(scan_init, taperscan_slot, searchlabel);
 
     if(found == 2)
 	searchlabel = first_match_label;
@@ -414,17 +464,53 @@ char *taper_scan()
     return found ? found_device : NULL;
 }
 
-int start_server_check(fd)
-int fd;
+int test_server_pgm(outf, dir, pgm, suid, dumpuid)
+FILE *outf;
+char *dir;
+char *pgm;
+int suid;
+uid_t dumpuid;
+{
+    struct stat statbuf;
+    int pgmbad = 0;
+
+    pgm = vstralloc(dir, "/", pgm, versionsuffix(), NULL);
+    if(stat(pgm, &statbuf) == -1) {
+	fprintf(outf, "ERROR: program %s: does not exist\n",
+		pgm);
+	pgmbad = 1;
+    } else if (!S_ISREG(statbuf.st_mode)) {
+	fprintf(outf, "ERROR: program %s: not a file\n",
+		pgm);
+	pgmbad = 1;
+    } else if (access(pgm, X_OK) == -1) {
+	fprintf(outf, "ERROR: program %s: not executable\n",
+		pgm);
+	pgmbad = 1;
+    } else if (suid \
+	       && dumpuid != 0
+	       && (statbuf.st_uid != 0 || (statbuf.st_mode & 04000) == 0)) {
+	fprintf(outf, "WARNING: program %s: not setuid-root\n",
+		pgm);
+    }
+    amfree(pgm);
+    return pgmbad;
+}
+
+int start_server_check(fd, do_localchk, do_tapechk)
+    int fd;
 {
     char *errstr, *tapename;
     generic_fs_stats_t fs;
     tape_t *tp;
     FILE *outf;
     holdingdisk_t *hdp;
-    int pid, tapebad, disklow, logbad;
-    int inparallel;
-    int testtape = 1;
+    int pid;
+    int tapebad = 0, disklow = 0, logbad = 0;
+    int userbad = 0, infobad = 0, indexbad = 0, pgmbad = 0;
+    int testtape = do_tapechk;
+    uid_t uid_me = getuid();
+    uid_t uid_dumpuser = uid_me;
 
     switch(pid = fork()) {
     case -1: error("could not fork server check: %s", strerror(errno));
@@ -439,7 +525,6 @@ int fd;
     set_pname("amcheck-server");
 
     amfree(msg);
-    amfree(tempfname);
 
     startclock();
 
@@ -450,93 +535,201 @@ int fd;
     fprintf(outf, "Amanda Tape Server Host Check\n");
     fprintf(outf, "-----------------------------\n");
 
-    /* check that the tapelist file is writable if it already exists */
-
+    /*
+     * Make sure we are running as the dump user.
+     */
     {
-	char *confdir;
-	char *tapefile;
+	char *dumpuser = getconf_str(CNF_DUMPUSER);
+	struct passwd *pw;
+
+	if ((pw = getpwnam(dumpuser)) == NULL) {
+	    fprintf(outf, "ERROR: cannot look up dump user \"%s\"\n", dumpuser);
+	    userbad = 1;
+	} else {
+	    uid_dumpuser = pw->pw_uid;
+	}
+	if ((pw = getpwuid(uid_me)) == NULL) {
+	    fprintf(outf, "ERROR: cannot look up my own uid (%ld)\n",
+		    (long)uid_me);
+	    userbad = 1;
+	}
+	if (uid_me != uid_dumpuser) {
+	    fprintf(outf, "ERROR: running as user \"%s\" instead of \"%s\"\n",
+		    pw->pw_name, dumpuser);
+	    userbad = 1;
+	}
+    }
+
+    /*
+     * Look up the programs used on the server side.
+     */
+    if(do_localchk) {
+	if(access(libexecdir, X_OK) == -1) {
+	    fprintf(outf, "ERROR: program dir %s: not accessible\n",
+		    libexecdir);
+	    pgmbad = 1;
+	} else {
+	    pgmbad = pgmbad \
+		     || test_server_pgm(outf, libexecdir, "planner",
+					1, uid_dumpuser);
+	    pgmbad = pgmbad \
+		     || test_server_pgm(outf, libexecdir, "dumper",
+					1, uid_dumpuser);
+	    pgmbad = pgmbad \
+		     || test_server_pgm(outf, libexecdir, "driver",
+					0, uid_dumpuser);
+	    pgmbad = pgmbad \
+		     || test_server_pgm(outf, libexecdir, "taper",
+					0, uid_dumpuser);
+	    pgmbad = pgmbad \
+		     || test_server_pgm(outf, libexecdir, "amtrmidx",
+					0, uid_dumpuser);
+	    pgmbad = pgmbad \
+		     || test_server_pgm(outf, libexecdir, "amlogroll",
+					0, uid_dumpuser);
+	}
+	if(access(sbindir, X_OK) == -1) {
+	    fprintf(outf, "ERROR: program dir %s: not accessible\n",
+		    sbindir);
+	    pgmbad = 1;
+	} else {
+	    pgmbad = pgmbad \
+		     || test_server_pgm(outf, sbindir, "amgetconf",
+					0, uid_dumpuser);
+	    pgmbad = pgmbad \
+		     || test_server_pgm(outf, sbindir, "amcheck",
+					1, uid_dumpuser);
+	    pgmbad = pgmbad \
+		     || test_server_pgm(outf, sbindir, "amdump",
+					0, uid_dumpuser);
+	    pgmbad = pgmbad \
+		     || test_server_pgm(outf, sbindir, "amreport",
+					0, uid_dumpuser);
+	}
+    }
+
+    /*
+     * Check that the directory for the tapelist file is writable, as well
+     * as the tapelist file itself (if it already exists).  Also, check for
+     * a "hold" file (just because it is convenient to do it here) and warn
+     * if tapedev is set to "/dev/null".
+     */
+
+    if(do_localchk || do_tapechk) {
 	char *conf_tapelist;
+	char *tapefile;
+	char *tape_dir;
+	char *lastslash;
+	char *holdfile;
 
 	conf_tapelist=getconf_str(CNF_TAPELIST);
-	confdir = vstralloc(CONFIG_DIR, "/", confname, NULL);
-	tapefile = vstralloc(confdir, "/", conf_tapelist, NULL);
-	if(access(confdir, W_OK) == -1)
-	    fprintf(outf, "ERROR: %s is unwritable: %s\n",
-		    confdir, strerror(errno));
-
-	if(access(tapefile, F_OK) == 0 && access(tapefile, W_OK) != 0)
-	    fprintf(outf, "ERROR: %s is not writable\n", tapefile);
-
-	amfree(confdir);
+	if (*conf_tapelist == '/') {
+	    tapefile = stralloc(conf_tapelist);
+	} else {
+	    tapefile = stralloc2(config_dir, conf_tapelist);
+	}
+	/*
+	 * XXX There Really Ought to be some error-checking here... dhw
+	 */
+	tape_dir = stralloc(tapefile);
+	if ((lastslash = strrchr((const char *)tape_dir, '/')) != NULL) {
+	    *lastslash = '\0';
+	/*
+	 * else whine Really Loudly about a path with no slashes??!?
+	 */
+	}
+	if(access(tape_dir, W_OK) == -1) {
+	    fprintf(outf, "ERROR: tapelist dir %s: not writable\n", tape_dir);
+	    tapebad = 1;
+	} else if(access(tapefile, F_OK) == 0 && access(tapefile, W_OK) != 0) {
+	    fprintf(outf, "ERROR: tape list %s: not writable\n", tapefile);
+	    tapebad = 1;
+	} else if(read_tapelist(tapefile)) {
+	    fprintf(outf, "ERROR: tape list %s: parse error\n", tapefile);
+	    tapebad = 1;
+	}
+	holdfile = vstralloc(config_dir, "/", "hold", NULL);
+	if(access(holdfile, F_OK) != -1) {
+	    fprintf(outf, "WARNING: hold file %s exists\n", holdfile);
+	}
 	amfree(tapefile);
-
-	if(read_tapelist(conf_tapelist))
-	    fprintf(outf, "ERROR: parse error in %s", conf_tapelist);
+	amfree(tape_dir);
+	amfree(holdfile);
+	tapename = getconf_str(CNF_TAPEDEV);
+	if (strcmp(tapename, "/dev/null") == 0) {
+	    fprintf(outf,
+		    "WARNING: tapedev is %s, dumps will be thrown away\n",
+		    tapename);
+	    testtape = 0;
+	    do_tapechk = 0;
+	}
     }
 
     /* check available disk space */
 
-    disklow = 0;
-
-    inparallel = getconf_int(CNF_INPARALLEL);
-
-    for(hdp = holdingdisks; hdp != NULL; hdp = hdp->next) {
-	if(get_fs_stats(hdp->diskdir, &fs) == -1) {
-	    fprintf(outf, "ERROR: statfs %s: %s\n",
-		    hdp->diskdir, strerror(errno));
-	    disklow = 1;
-	}
-	else if(access(hdp->diskdir, W_OK) == -1) {
-	    fprintf(outf, "ERROR: %s is unwritable: %s\n",
-		    hdp->diskdir, strerror(errno));
-	    disklow = 1;
-	}
-	else if(fs.avail == -1) {
-	    fprintf(outf,
-	       "WARNING: avail disk space unknown for %s. %ld KB requested.\n",
-		    hdp->diskdir, hdp->disksize);
-	    disklow = 1;
-	}
-	else if(hdp->disksize > 0) {
-	    if(fs.avail < hdp->disksize) {
-		fprintf(outf,
-			"WARNING: %s: only %ld KB free (%ld KB requested).\n",
-			hdp->diskdir, fs.avail, hdp->disksize);
+    if(do_localchk) {
+	for(hdp = holdingdisks; hdp != NULL; hdp = hdp->next) {
+	    if(get_fs_stats(hdp->diskdir, &fs) == -1) {
+		fprintf(outf, "ERROR: holding disk %s: statfs: %s\n",
+			hdp->diskdir, strerror(errno));
 		disklow = 1;
 	    }
-	    else
-		fprintf(outf,
-			"%s: %ld KB disk space available, that's plenty.\n",
-			hdp->diskdir, fs.avail);
-	}
-	else {
-	    if(fs.avail < -hdp->disksize) {
-		fprintf(outf,
-			"WARNING: %s: only %ld KB free, using nothing.\n",
-			hdp->diskdir, fs.avail);
+	    else if(access(hdp->diskdir, W_OK) == -1) {
+		fprintf(outf, "ERROR: holding disk %s: not writable: %s\n",
+			hdp->diskdir, strerror(errno));
 		disklow = 1;
 	    }
-	    else
+	    else if(fs.avail == -1) {
 		fprintf(outf,
-			"%s: %ld KB disk space available, using %ld KB.\n",
-			hdp->diskdir, fs.avail, fs.avail + hdp->disksize);
+			"WARNING: holding disk %s: available space unknown (%ld KB requested)\n",
+			hdp->diskdir, (long)hdp->disksize);
+		disklow = 1;
+	    }
+	    else if(hdp->disksize > 0) {
+		if(fs.avail < hdp->disksize) {
+		    fprintf(outf,
+			    "WARNING: holding disk %s: only %ld KB free (%ld KB requested)\n",
+			    hdp->diskdir, (long)fs.avail, (long)hdp->disksize);
+		    disklow = 1;
+		}
+		else
+		    fprintf(outf,
+			    "Holding disk %s: %ld KB disk space available, that's plenty\n",
+			    hdp->diskdir, fs.avail);
+	    }
+	    else {
+		if(fs.avail < -hdp->disksize) {
+		    fprintf(outf,
+			    "WARNING: holding disk %s: only %ld KB free, using nothing\n",
+			    hdp->diskdir, fs.avail);
+		    disklow = 1;
+		}
+		else
+		    fprintf(outf,
+			    "Holding disk %s: %ld KB disk space available, using %ld KB\n",
+			    hdp->diskdir, fs.avail, fs.avail + hdp->disksize);
+	    }
 	}
     }
 
     /* check that the log file is writable if it already exists */
 
-    {
-	char *logdir;
+    if(do_localchk) {
+	char *conf_logdir;
 	char *logfile;
+	char *olddir;
+	struct stat stat_old;
 
-	logbad = 0;
+	conf_logdir = getconf_str(CNF_LOGDIR);
+	if (*conf_logdir == '/') {
+	    conf_logdir = stralloc(conf_logdir);
+	} else {
+	    conf_logdir = stralloc2(config_dir, conf_logdir);
+	}
+	logfile = vstralloc(conf_logdir, "/log", NULL);
 
-	logdir = getconf_str(CNF_LOGDIR);
-	logfile = vstralloc(logdir, "/log", NULL);
-
-	if(access(logdir, W_OK) == -1) {
-	    fprintf(outf, "ERROR: %s is unwritable: %s\n",
-		logdir, strerror(errno));
+	if(access(conf_logdir, W_OK) == -1) {
+	    fprintf(outf, "ERROR: log dir %s: not writable\n", conf_logdir);
 	    logbad = 1;
 	}
 
@@ -544,51 +737,63 @@ int fd;
 	    testtape = 0;
 	    logbad = 1;
 	    if(access(logfile, W_OK) != 0)
-		fprintf(outf, "ERROR: %s is not writable\n", logfile);
+		fprintf(outf, "ERROR: log file %s: not writable\n",
+			logfile);
+	}
+
+	olddir = vstralloc(conf_logdir, "/oldlog", NULL);
+	if (stat(olddir,&stat_old) == 0) { /* oldlog exist */
+	    if(!(S_ISDIR(stat_old.st_mode))) {
+		fprintf(outf, "ERROR: oldlog directory \"%s\" is not a directory\n", olddir);
+	    }
+	    if(access(olddir, W_OK) == -1) {
+		fprintf(outf, "ERROR: oldlog dir %s: not writable\n", olddir);
+	    }
+	}
+	else if(lstat(olddir,&stat_old) == 0) {
+	    fprintf(outf, "ERROR: oldlog directory \"%s\" is not a directory\n", olddir);
 	}
 
 	if (testtape) {
-	    logfile = newvstralloc(logfile, logdir, "/amdump", NULL);
+	    logfile = newvstralloc(logfile, conf_logdir, "/amdump", NULL);
 	    if (access(logfile, F_OK) == 0) {
 		testtape = 0;
 		logbad = 1;
 	    }
 	}
-	
+
 	amfree(logfile);
     }
 
     if (testtape) {
 	/* check that the tape is a valid amanda tape */
 
-	tapebad = 0;
-
 	tapedays = getconf_int(CNF_TAPECYCLE);
 	labelstr = getconf_str(CNF_LABELSTR);
 	tapename = getconf_str(CNF_TAPEDEV);
 
 	if (!getconf_seen(CNF_TPCHANGER) && getconf_int(CNF_RUNTAPES) != 1) {
 	    fprintf(outf,
-		    "WARNING: if a tape changer is not available, runtapes must be set to 1.\n");
+		    "WARNING: if a tape changer is not available, runtapes must be set to 1\n");
 	}
 
 	if(changer_init() && (tapename = taper_scan()) == NULL) {
-	    fprintf(outf, "ERROR: %s.\n", changer_resultstr);
+	    fprintf(outf, "ERROR: %s\n", changer_resultstr);
 	    tapebad = 1;
 	} else if(access(tapename,F_OK|R_OK|W_OK) == -1) {
-	    fprintf(outf, "ERROR: %s: %s\n",tapename,strerror(errno));
+	    fprintf(outf, "ERROR: %s: %s\n", tapename, strerror(errno));
 	    tapebad = 1;
 	} else if((errstr = tape_rdlabel(tapename, &datestamp, &label)) != NULL) {
-	    fprintf(outf, "ERROR: %s: %s.\n", tapename, errstr);
+	    fprintf(outf, "ERROR: %s: %s\n", tapename, errstr);
 	    tapebad = 1;
 	} else {
 	    tp = lookup_tapelabel(label);
 	    if(tp != NULL && !reusable_tape(tp)) {
-		fprintf(outf, "ERROR: cannot overwrite active tape %s.\n", label);
+		fprintf(outf, "ERROR: cannot overwrite active tape %s\n", label);
 		tapebad = 1;
 	    }
 	    if(!match(labelstr, label)) {
-		fprintf(outf, "ERROR: label %s doesn't match labelstr \"%s\".\n",
+		fprintf(outf, "ERROR: label %s doesn't match labelstr \"%s\"\n",
 			label, labelstr);
 		tapebad = 1;
 	    }
@@ -605,38 +810,215 @@ int fd;
 	if(!tapebad && overwrite) {
 	    if((errstr = tape_writable(tapename)) != NULL) {
 		fprintf(outf,
-			"ERROR: tape %s label ok, but is not writable.\n", label);
+			"ERROR: tape %s label ok, but is not writable\n",
+			label);
 		tapebad = 1;
 	    }
-	    else fprintf(outf, "Tape %s is writable.\n", label);
+	    else fprintf(outf, "Tape %s is writable\n", label);
 	}
-	else fprintf(outf, "NOTE: skipping tape-writable test.\n");
+	else fprintf(outf, "NOTE: skipping tape-writable test\n");
 
 	if(!tapebad)
-	    fprintf(outf, "Tape %s label ok.\n", label);
+	    fprintf(outf, "Tape %s label ok\n", label);
+    } else if (do_tapechk) {
+	fprintf(outf, "WARNING: skipping tape test because amdump or amflush seem to be running\n");
+	fprintf(outf, "WARNING: if they are not, you must run amcleanup\n");
     } else {
-	fprintf(outf, "WARNING: skipping tape test because amdump or amflush seem to be running;\nWARNING: if they are not, you must run amcleanup.\n");
+	fprintf(outf, "NOTE: skipping tape checks\n");
     }
 
-    {
-	char *indexdir = getconf_str(CNF_INDEXDIR);
+    /*
+     * See if the information file and index directory for each client
+     * and disk is OK.  Since we may be seeing clients and/or disks for
+     * the first time, these are just warnings, not errors.
+     */
+    if(do_localchk) {
+	char *conf_infofile;
+	char *conf_indexdir;
+	char *hostinfodir = NULL;
+	char *hostindexdir = NULL;
+	char *diskdir = NULL;
+	char *infofile = NULL;
 	struct stat statbuf;
-	if((stat(indexdir, &statbuf) == -1)
-	   || !S_ISDIR(statbuf.st_mode)
-	   || (access(indexdir, W_OK) == -1)) {
-	    fprintf(outf, "Index dir \"%s\" doesn't exist or is not writable.\n",
-		    indexdir);
+	disk_t *dp;
+	host_t *hostp;
+	int indexdir_checked = 0;
+	int hostindexdir_checked = 0;
+	char *host;
+	char *disk;
+
+	conf_infofile = stralloc(getconf_str(CNF_INFOFILE));
+	if (*conf_infofile == '/') {
+	    conf_infofile = stralloc(conf_infofile);
+	} else {
+	    conf_infofile = stralloc2(config_dir, conf_infofile);
+	}
+	conf_indexdir = stralloc(getconf_str(CNF_INDEXDIR));
+	if (*conf_indexdir == '/') {
+	    conf_indexdir = stralloc(conf_indexdir);
+	} else {
+	    conf_indexdir = stralloc2(config_dir, conf_indexdir);
+	}
+#if TEXTDB
+	if(stat(conf_infofile, &statbuf) == -1) {
+	    fprintf(outf, "NOTE: info dir %s: does not exist\n", conf_infofile);
+	    fprintf(outf, "NOTE: it will be created on the next run\n");
+	    amfree(conf_infofile);
+	} else if (!S_ISDIR(statbuf.st_mode)) {
+	    fprintf(outf, "ERROR: info dir %s: not a directory\n", conf_infofile);
+	    amfree(conf_infofile);
+	    infobad = 1;
+	} else if (access(conf_infofile, W_OK) == -1) {
+	    fprintf(outf, "ERROR: info dir %s: not writable\n", conf_infofile);
+	    amfree(conf_infofile);
+	    infobad = 1;
+	} else {
+	    strappend(conf_infofile, "/");
+	}
+#endif
+	while(!empty(*origqp)) {
+	    hostp = origqp->head->host;
+	    host = sanitise_filename(hostp->hostname);
+#if TEXTDB
+	    if(conf_infofile) {
+		hostinfodir = newstralloc2(hostinfodir, conf_infofile, host);
+		if(stat(hostinfodir, &statbuf) == -1) {
+		    fprintf(outf, "NOTE: info dir %s: does not exist\n",
+			    hostinfodir);
+		    amfree(hostinfodir);
+		} else if (!S_ISDIR(statbuf.st_mode)) {
+		    fprintf(outf, "ERROR: info dir %s: not a directory\n",
+			    hostinfodir);
+		    amfree(hostinfodir);
+		    infobad = 1;
+		} else if (access(hostinfodir, W_OK) == -1) {
+		    fprintf(outf, "ERROR: info dir %s: not writable\n",
+			    hostinfodir);
+		    amfree(hostinfodir);
+		    infobad = 1;
+		} else {
+		    strappend(hostinfodir, "/");
+		}
+	    }
+#endif
+	    for(dp = hostp->disks; dp != NULL; dp = dp->hostnext) {
+		disk = sanitise_filename(dp->name);
+#if TEXTDB
+		if(hostinfodir) {
+		    diskdir = newstralloc2(diskdir, hostinfodir, disk);
+		    infofile = vstralloc(diskdir, "/", "info", NULL);
+		    if(stat(diskdir, &statbuf) == -1) {
+			fprintf(outf, "NOTE: info dir %s: does not exist\n",
+				diskdir);
+		    } else if (!S_ISDIR(statbuf.st_mode)) {
+			fprintf(outf, "ERROR: info dir %s: not a directory\n",
+				diskdir);
+			infobad = 1;
+		    } else if (access(diskdir, W_OK) == -1) {
+			fprintf(outf, "ERROR: info dir %s: not writable\n",
+				diskdir);
+			infobad = 1;
+		    } else if(stat(infofile, &statbuf) == -1) {
+			fprintf(outf, "WARNING: info file %s: does not exist\n",
+				infofile);
+		    } else if (!S_ISREG(statbuf.st_mode)) {
+			fprintf(outf, "ERROR: info file %s: not a file\n",
+				infofile);
+			infobad = 1;
+		    } else if (access(infofile, R_OK) == -1) {
+			fprintf(outf, "ERROR: info file %s: not readable\n",
+				infofile);
+			infobad = 1;
+		    }
+		    amfree(infofile);
+		}
+#endif
+		if(dp->index) {
+		    if(! indexdir_checked) {
+			if(stat(conf_indexdir, &statbuf) == -1) {
+			    fprintf(outf, "NOTE: index dir %s: does not exist\n",
+				    conf_indexdir);
+			    amfree(conf_indexdir);
+			} else if (!S_ISDIR(statbuf.st_mode)) {
+			    fprintf(outf, "ERROR: index dir %s: not a directory\n",
+				    conf_indexdir);
+			    amfree(conf_indexdir);
+			    indexbad = 1;
+			} else if (access(conf_indexdir, W_OK) == -1) {
+			    fprintf(outf, "ERROR: index dir %s: not writable\n",
+				    conf_indexdir);
+			    amfree(conf_indexdir);
+			    indexbad = 1;
+			} else {
+			    strappend(conf_indexdir, "/");
+			}
+			indexdir_checked = 1;
+		    }
+		    if(conf_indexdir) {
+			if(! hostindexdir_checked) {
+			    hostindexdir = stralloc2(conf_indexdir, host);
+			    if(stat(hostindexdir, &statbuf) == -1) {
+			        fprintf(outf, "NOTE: index dir %s: does not exist\n",
+				        hostindexdir);
+			        amfree(hostindexdir);
+			    } else if (!S_ISDIR(statbuf.st_mode)) {
+			        fprintf(outf, "ERROR: index dir %s: not a directory\n",
+				        hostindexdir);
+			        amfree(hostindexdir);
+			        indexbad = 1;
+			    } else if (access(hostindexdir, W_OK) == -1) {
+			        fprintf(outf, "ERROR: index dir %s: not writable\n",
+				        hostindexdir);
+			        amfree(hostindexdir);
+			        indexbad = 1;
+			    } else {
+				strappend(hostindexdir, "/");
+			    }
+			    hostindexdir_checked = 1;
+			}
+			if(hostindexdir) {
+			    diskdir = newstralloc2(diskdir, hostindexdir, disk);
+			    if(stat(diskdir, &statbuf) == -1) {
+				fprintf(outf, "NOTE: index dir %s: does not exist\n",
+					diskdir);
+			    } else if (!S_ISDIR(statbuf.st_mode)) {
+				fprintf(outf, "ERROR: index dir %s: not a directory\n",
+					diskdir);
+				indexbad = 1;
+			    } else if (access(diskdir, W_OK) == -1) {
+				fprintf(outf, "ERROR: index dir %s: is not writable\n",
+					diskdir);
+				indexbad = 1;
+			    }
+			}
+		    }
+		}
+		amfree(disk);
+		remove_disk(origqp, dp);
+	    }
+	    amfree(host);
+	    amfree(hostindexdir);
+	    hostindexdir_checked = 0;
+	}
+	amfree(diskdir);
+	amfree(hostinfodir);
+	amfree(conf_infofile);
+	amfree(conf_indexdir);
+    }
+
+    if(do_localchk) {
+	if(access(COMPRESS_PATH, X_OK) == -1) {
+	    fprintf(outf, "WARNING: %s is not executable, server-compression and indexing will not work\n",
+	            COMPRESS_PATH);
 	}
     }
 
-    if (access(COMPRESS_PATH, X_OK) == -1)
-      fprintf(outf, "%s is not executable, server-compression and indexing will not work\n",
-	      COMPRESS_PATH);
-
     amfree(datestamp);
     amfree(label);
+    amfree(config_dir);
+    amfree(config_name);
 
-    fprintf(outf, "Server check took %s seconds.\n", walltime_str(curclock()));
+    fprintf(outf, "Server check took %s seconds\n", walltime_str(curclock()));
 
     fflush(outf);
 
@@ -646,7 +1028,13 @@ int fd;
 	malloc_list(fd, malloc_hist_1, malloc_hist_2);
     }
 
-    exit(tapebad || disklow || logbad);
+    exit(userbad \
+	 || tapebad \
+	 || disklow \
+	 || logbad \
+	 || infobad \
+	 || indexbad \
+	 || pgmbad);
     /* NOTREACHED */
     return 0;
 }
@@ -661,13 +1049,13 @@ static void handle_response P((proto_t *p, pkt_t *pkt));
 int start_client_checks(fd)
 int fd;
 {
-    disklist_t *origqp;
     disk_t *dp;
     host_t *hostp;
     char *req = NULL;
     int hostcount, rc, pid;
     int amanda_port;
     struct servent *amandad;
+    int userbad = 0;
 
 #ifdef KRB4_SECURITY
     int kamanda_port;
@@ -685,13 +1073,8 @@ int fd;
 
     set_pname("amcheck-clients");
 
-    amfree(tempfname);
-
     startclock();
 
-    if((origqp = read_diskfile(getconf_str(CNF_DISKFILE))) == NULL)
-	error("could not load \"%s\"\n", getconf_str(CNF_DISKFILE));
-
     if((outf = fdopen(fd, "w")) == NULL)
 	error("fdopen %d: %s", fd, strerror(errno));
     errf = outf;
@@ -699,6 +1082,33 @@ int fd;
     fprintf(outf, "\nAmanda Backup Client Hosts Check\n");
     fprintf(outf,   "--------------------------------\n");
 
+    /*
+     * Make sure we are running as the dump user.
+     */
+    {
+	uid_t uid_me = getuid();
+	uid_t uid_dumpuser = uid_me;
+	char *dumpuser = getconf_str(CNF_DUMPUSER);
+	struct passwd *pw;
+
+	if ((pw = getpwnam(dumpuser)) == NULL) {
+	    fprintf(outf, "ERROR: cannot look up dump user \"%s\"\n", dumpuser);
+	    userbad = 1;
+	} else {
+	    uid_dumpuser = pw->pw_uid;
+	}
+	if ((pw = getpwuid(uid_me)) == NULL) {
+	    fprintf(outf, "ERROR: cannot look up my own uid %ld\n",
+		    (long)uid_me);
+	    userbad = 1;
+	}
+	if (uid_me != uid_dumpuser) {
+	    fprintf(outf, "ERROR: running as user \"%s\" instead of \"%s\"\n",
+		    pw->pw_name, dumpuser);
+	    userbad = 1;
+	}
+    }
+
 #ifdef KRB4_SECURITY
     kerberos_service_init();
 #endif
@@ -748,18 +1158,18 @@ int fd;
 #ifdef KRB4_SECURITY
 	if(hostp->disks->auth == AUTH_KRB4)
 	    rc = make_krb_request(hostp->hostname, kamanda_port, req,
-				  hostp, CHECK_TIMEOUT, handle_response);
+				  hostp, conf_ctimeout, handle_response);
 	else
 #endif
 	    rc = make_request(hostp->hostname, amanda_port, req,
-			      hostp, CHECK_TIMEOUT, handle_response);
+			      hostp, conf_ctimeout, handle_response);
 
 	req = NULL;				/* do not own this any more */
 
 	if(rc) {
 	    /* couldn't resolve hostname */
 	    fprintf(outf,
-		    "ERROR: %s: couldn't resolve hostname\n", hostp->hostname);
+		    "ERROR: %s: could not resolve hostname\n", hostp->hostname);
 	    remote_errors++;
 	}
 	check_protocol();
@@ -768,19 +1178,22 @@ int fd;
     amfree(msg);
 
     fprintf(outf,
-     "Client check: %d host%s checked in %s seconds, %d problem%s found.\n",
+     "Client check: %d host%s checked in %s seconds, %d problem%s found\n",
 	    hostcount, (hostcount == 1) ? "" : "s",
 	    walltime_str(curclock()),
 	    remote_errors, (remote_errors == 1) ? "" : "s");
     fflush(outf);
 
+    amfree(config_dir);
+    amfree(config_name);
+
     malloc_size_2 = malloc_inuse(&malloc_hist_2);
 
     if(malloc_size_1 != malloc_size_2) {
 	malloc_list(fd, malloc_hist_1, malloc_hist_2);
     }
 
-    exit(remote_errors > 0);
+    exit(userbad || remote_errors > 0);
     /* NOTREACHED */
     return 0;
 }
