@@ -24,7 +24,7 @@
  * file named AUTHORS, in the root directory of this distribution.
  */
 /*
- * $Id$
+ * $Id: amindexd.c,v 1.39.2.12 2001/02/25 21:10:26 martinea Exp $
  *
  * This is the server daemon part of the index client/server system.
  * It is assumed that this is launched from inetd instead of being
@@ -47,12 +47,13 @@
 #include "dgram.h"
 #include "version.h"
 #include "protocol.h"
-#include "version.h"
 #include "amindex.h"
 #include "disk_history.h"
 #include "list_dir.h"
 #include "logfile.h"
 #include "token.h"
+#include "find.h"
+#include "tapefile.h"
 
 #ifdef HAVE_NETINET_IN_SYSTM_H
 #include <netinet/in_systm.h>
@@ -75,9 +76,11 @@ char local_hostname[MAX_HOSTNAME_LENGTH+1];	/* me! */
 char *remote_hostname = NULL;			/* the client */
 char *dump_hostname = NULL;			/* machine we are restoring */
 char *disk_name;				/* disk we are restoring */
-char *config = NULL;				/* config we are restoring */
+char *config_name = NULL;			/* config we are restoring */
+char *config_dir = NULL;			/* config directory */
 char *target_date = NULL;
 disklist_t *disk_list;				/* all disks in cur config */
+find_result_t *output_find = NULL;
 
 static int amindexd_debug = 0;
 
@@ -135,6 +138,7 @@ char **emsg;
 	if (system(cmd)!=0) {
 	    amfree(*emsg);
 	    *emsg = vstralloc("\"", cmd, "\" failed", NULL);
+	    unlink(filename);
 	    errno = -1;
 	    amfree(filename);
 	    amfree(cmd);
@@ -184,12 +188,14 @@ char **emsg;
 	dir_slash = stralloc2(dir, "/");
     }
 
-    filename_gz=getindexfname(dump_hostname, disk_name, dump_item->date,
-			      dump_item->level);
+    filename_gz = getindexfname(dump_hostname, disk_name, dump_item->date,
+			        dump_item->level);
     if((filename = uncompress_file(filename_gz, emsg)) == NULL) {
+	amfree(filename_gz);
 	amfree(dir_slash);
 	return -1;
     }
+    amfree(filename_gz);
 
     if((fp = fopen(filename,"r"))==0) {
 	amfree(*emsg);
@@ -237,7 +243,7 @@ arglist_function1(void reply, int, n, char *, fmt)
     ap_vsnprintf(buf+4, sizeof(buf)-4, fmt, args);
     arglist_end(args);
 
-    if (printf("%s\r\n", buf) <= 0)
+    if (printf("%s\r\n", buf) < 0)
     {
 	dbprintf(("! error %d (%s) in printf\n", errno, strerror(errno)));
 	uncompress_remove = remove_files(uncompress_remove);
@@ -263,7 +269,7 @@ arglist_function1(void lreply, int, n, char *, fmt)
     ap_vsnprintf(buf+4, sizeof(buf)-4, fmt, args);
     arglist_end(args);
 
-    if (printf("%s\r\n", buf) <= 0)
+    if (printf("%s\r\n", buf) < 0)
     {
 	dbprintf(("! error %d (%s) in printf\n", errno, strerror(errno)));
 	uncompress_remove = remove_files(uncompress_remove);
@@ -290,7 +296,7 @@ arglist_function1(void fast_lreply, int, n, char *, fmt)
     ap_vsnprintf(buf+4, sizeof(buf)-4, fmt, args);
     arglist_end(args);
 
-    if (printf("%s\r\n", buf) <= 0)
+    if (printf("%s\r\n", buf) < 0)
     {
 	dbprintf(("! error %d (%s) in printf\n", errno, strerror(errno)));
 	uncompress_remove = remove_files(uncompress_remove);
@@ -307,17 +313,18 @@ int is_dump_host_valid(host)
 char *host;
 {
     struct stat dir_stat;
+    char *fn;
 
-    if (config == NULL) {
+    if (config_name == NULL) {
 	reply(501, "Must set config before setting host.");
 	return -1;
     }
 
 #if 0
     /* only let a client restore itself for now unless it is the server */
-    if (strcmp(remote_hostname, local_hostname) == 0)
+    if (strcasecmp(remote_hostname, local_hostname) == 0)
 	return 0;
-    if (strcmp(remote_hostname, host) != 0)
+    if (strcasecmp(remote_hostname, host) != 0)
     {
 	reply(501,
 	      "You don't have the necessary permissions to set dump host to %s.",
@@ -328,12 +335,14 @@ char *host;
 
     /* check that the config actually handles that host */
     /* assume in index dir already */
-    if (stat (host, &dir_stat) != 0 || !S_ISDIR(dir_stat.st_mode))
-    {
+    fn = getindexfname(host, NULL, NULL, 0);
+    if (stat (fn, &dir_stat) != 0 || !S_ISDIR(dir_stat.st_mode)) {
 	reply(501, "No index records for host: %s. Invalid?", host);
+	amfree(fn);
 	return -1;
     }
 
+    amfree(fn);
     return 0;
 }
 
@@ -341,32 +350,23 @@ char *host;
 int is_disk_valid(disk)
 char *disk;
 {
-    char *search_str;
-    char *last_slash;
+    char *fn;
     struct stat dir_stat;
 
-    if (config == NULL || dump_hostname == NULL) {
+    if (config_name == NULL || dump_hostname == NULL) {
 	reply(501, "Must set config,host before setting disk.");
 	return -1;
     }
 
-    search_str = getindexfname(dump_hostname, disk, "00000000", 0);
-    last_slash = strrchr (search_str, '/');
-    if (last_slash == NULL)
-    {
-	reply(599, "Cannot convert host/disk to index directory.");
-	return -1;
-    }
+    fn = getindexfname(dump_hostname, disk, NULL, 0);
 
-    /* check that the config actually handles that host and disk */
-    /* assume in index dir already */
-    *last_slash = '\0';
-    if (stat (search_str, &dir_stat) != 0 || !S_ISDIR(dir_stat.st_mode))
-    {
+    if (stat (fn, &dir_stat) != 0 || !S_ISDIR(dir_stat.st_mode)) {
 	reply(501, "No index records for disk: %s. Invalid?", disk);
+	amfree(fn);
 	return -1;
     }
 
+    amfree(fn);
     return 0;
 }
 
@@ -374,7 +374,11 @@ char *disk;
 int is_config_valid(config)
 char *config;
 {
-    char *conf_dir = NULL;
+    char *conffile;
+    char *conf_diskfile;
+    char *conf_tapelist;
+    char *conf_indexdir;
+    struct stat dir_stat;
 
     /* check that the config actually exists */
     if (config == NULL) {
@@ -382,37 +386,52 @@ char *config;
 	return -1;
     }
 
-    /* cd to confdir */
-    conf_dir = vstralloc(CONFIG_DIR, "/", config, NULL);
-    if (chdir(conf_dir) == -1)
-    {
-	reply(501, "Couldn't cd into %s.  Misconfiguration?", conf_dir);
-	amfree(conf_dir);
-	return -1;
-    }
-
     /* read conffile */
-    if (read_conffile(CONFFILE_NAME))
-    {
-	reply(501, "Couldn't read config file %s/%s!", conf_dir, CONFFILE_NAME);
-	amfree(conf_dir);
+    conffile = stralloc2(config_dir, CONFFILE_NAME);
+    if (read_conffile(conffile)) {
+	reply(501, "Could not read config file %s!", conffile);
+	amfree(conffile);
 	return -1;
     }
-    amfree(conf_dir);
-
-    /* read the disk file while we are here - just in case we need it */
-    if ((disk_list = read_diskfile(getconf_str(CNF_DISKFILE))) == NULL)
-    {
-	reply(501, "Couldn't read disk file");
+    amfree(conffile);
+    conf_diskfile = getconf_str(CNF_DISKFILE);
+    if (*conf_diskfile == '/') {
+	conf_diskfile = stralloc(conf_diskfile);
+    } else {
+	conf_diskfile = stralloc2(config_dir, conf_diskfile);
+    }
+    if ((disk_list = read_diskfile(conf_diskfile)) == NULL) {
+	reply(501, "Could not read disk file %s!", conf_diskfile);
+	amfree(conf_diskfile);
 	return -1;
     }
+    amfree(conf_diskfile);
+    conf_tapelist = getconf_str(CNF_TAPELIST);
+    if (*conf_tapelist == '/') {
+	conf_tapelist = stralloc(conf_tapelist);
+    } else {
+	conf_tapelist = stralloc2(config_dir, conf_tapelist);
+    }
+    if(read_tapelist(conf_tapelist)) {
+	reply(501, "Could not read tapelist file %s!", conf_tapelist);
+	amfree(conf_tapelist);
+	return -1;
+    }
+    amfree(conf_tapelist);
 
     /* okay, now look for the index directory */
-    if (chdir(getconf_str(CNF_INDEXDIR)) == -1)
-    {
-	reply(501, "Index directory %s does not exist", getconf_str(CNF_INDEXDIR));
+    conf_indexdir = getconf_str(CNF_INDEXDIR);
+    if(*conf_indexdir == '/') {
+	conf_indexdir = stralloc(conf_indexdir);
+    } else {
+	conf_indexdir = stralloc2(config_dir, conf_indexdir);
+    }
+    if (stat (conf_indexdir, &dir_stat) != 0 || !S_ISDIR(dir_stat.st_mode)) {
+	reply(501, "Index directory %s does not exist", conf_indexdir);
+	amfree(conf_indexdir);
 	return -1;
     }
+    amfree(conf_indexdir);
 
     return 0;
 }
@@ -420,128 +439,35 @@ char *config;
 
 int build_disk_table P((void))
 {
-    char *date;
-    char *host;
-    int level;
-    char *tape;
-    int file;
-    char *status;
-    char *cmd = NULL;
-    FILE *fp;
-    int first_line = 0;
-    char *s;
-    int ch;
-    char *line;
+    char date[100];
+    find_result_t *find_output;
 
-    if (config == NULL || dump_hostname == NULL || disk_name == NULL) {
+    if (config_name == NULL || dump_hostname == NULL || disk_name == NULL) {
 	reply(590, "Must set config,host,disk before building disk table");
 	return -1;
     }
 
-    {
-	char *rxdiskname, *shdiskname;
-	rxdiskname = rxquote(disk_name);
-	shdiskname = shquote(rxdiskname);
-	amfree(rxdiskname);
-	cmd = vstralloc(sbindir, "/", "amadmin", versionsuffix(),
-			" ", config,
-			" ", "find",
-			" ", dump_hostname,
-			" \\^", shdiskname, "\\$",
-			NULL);
-	amfree(shdiskname);
+    if(output_find == NULL) { /* do it the first time only */
+	output_find = find_dump(NULL,0,NULL);
+	sort_find_result("DLKHB", &output_find);
     }
 
-    dbprintf(("! %s\n",cmd));
-    if ((fp = popen(cmd, "r")) == NULL)
-    {
-	reply(599, "System error %s", strerror(errno));
-	amfree(cmd);
-	return -1;
-    }
-    amfree(cmd);
     clear_list();
-    for(; (line = agets(fp)) != NULL; free(line)) {
-	if (first_line++ == 0) {
-	    continue;
+    for(find_output = output_find; find_output != NULL; 
+	find_output = find_output->next) {
+	if(strcasecmp(dump_hostname, find_output->hostname) == 0 &&
+	   strcmp(disk_name    , find_output->diskname) == 0 &&
+	   strcmp("OK"         , find_output->status)   == 0) {
+	    ap_snprintf(date, sizeof(date), "%04d-%02d-%02d",
+			find_output->datestamp/10000,
+			(find_output->datestamp/100) %100,
+			find_output->datestamp %100);
+	    add_dump(date, find_output->level, find_output->label, 
+		     find_output->filenum);
+	    dbprintf(("- %s %d %s %d\n", date, find_output->level, 
+		     find_output->label, find_output->filenum));
 	}
-
-#define sc "No dump to list"
-	if (strncmp(line, sc, sizeof(sc)-1) == 0) {
-#undef sc
-	    reply(598, "Error: disk not found.");
-	    apclose(fp);
-	    return -1;
-	}
-	
-	s = line;
-	ch = *s++;
-
-	skip_whitespace(s, ch);			/* find the date */
-	if (ch == '\0') {
-	    continue;				/* no date field */
-	}
-	date = s - 1;
-	skip_non_whitespace(s, ch);
-	s[-1] = '\0';
-
-	skip_whitespace(s, ch);			/* find the host name */
-	if (ch == '\0') {
-	    continue;				/* no host name */
-	}
-	host = s - 1;
-	skip_non_whitespace(s, ch);
-	s[-1] = '\0';
-	if (strcmp(dump_hostname, host) != 0) {
-	    continue;				/* wrong host */
-	}
-
-	skip_whitespace(s, ch);			/* find the disk name */
-	if (ch == '\0') {
-	    continue;				/* no disk name */
-	}
-	host = s - 1;
-	skip_non_whitespace(s, ch);
-	s[-1] = '\0';
-	if (strcmp(disk_name, host) != 0) {
-	    continue;				/* wrong disk */
-	}
-
-	skip_whitespace(s, ch);			/* find the level number */
-	if (ch == '\0' || sscanf(s - 1, "%d", &level) != 1) {
-	    continue;				/* bad level */
-	}
-	skip_integer(s, ch);
-
-	skip_whitespace(s, ch);			/* find the tape label */
-	if (ch == '\0') {
-	    continue;				/* no tape field */
-	}
-	tape = s - 1;
-	skip_non_whitespace(s, ch);
-	s[-1] = '\0';
-
-	skip_whitespace(s, ch);			/* find the file number */
-	if (ch == '\0' || sscanf(s - 1, "%d", &file) != 1) {
-	    continue;				/* bad file number */
-	}
-	skip_integer(s, ch);
-
-	skip_whitespace(s, ch);			/* find the status */
-	if (ch == '\0') {
-	    continue;				/* no status field */
-	}
-	status = s - 1;
-	skip_non_whitespace(s, ch);
-	s[-1] = '\0';
-	if (strcmp("OK", status) != 0) {
-	    continue;			/* wrong status */
-	}
-
-	add_dump(date, level, tape, file);
-	dbprintf(("- %s %d %s %d\n", date, level, tape, file));
     }
-    apclose(fp);
     return 0;
 }
 
@@ -550,20 +476,20 @@ int disk_history_list P((void))
 {
     DUMP_ITEM *item;
 
-    if (config == NULL || dump_hostname == NULL || disk_name == NULL) {
+    if (config_name == NULL || dump_hostname == NULL || disk_name == NULL) {
 	reply(502, "Must set config,host,disk before listing history");
 	return -1;
     }
 
     lreply(200, " Dump history for config \"%s\" host \"%s\" disk \"%s\"",
-	  config, dump_hostname, disk_name);
+	  config_name, dump_hostname, disk_name);
 
     for (item=first_dump(); item!=NULL; item=next_dump(item))
 	lreply(201, " %s %d %s %d", item->date, item->level, item->tape,
 	       item->file);
 
     reply(200, "Dump history for config \"%s\" host \"%s\" disk \"%s\"",
-	  config, dump_hostname, disk_name);
+	  config_name, dump_hostname, disk_name);
 
     return 0;
 }
@@ -580,12 +506,12 @@ char *dir;
     FILE *fp;
     int last_level;
     char *ldir = NULL;
-    char *filename_gz;
+    char *filename_gz = NULL;
     char *filename = NULL;
     int ldir_len;
     static char *emsg = NULL;
 
-    if (config == NULL || dump_hostname == NULL || disk_name == NULL) {
+    if (config_name == NULL || dump_hostname == NULL || disk_name == NULL) {
 	reply(502, "Must set config,host,disk before asking about directories");
 	return -1;
     }
@@ -616,15 +542,17 @@ char *dir;
     /* go back till we hit a level 0 dump */
     do
     {
-	filename_gz=getindexfname(dump_hostname, disk_name,
-				  item->date, item->level);
 	amfree(filename);
+	filename_gz = getindexfname(dump_hostname, disk_name,
+				    item->date, item->level);
 	if((filename = uncompress_file(filename_gz, &emsg)) == NULL) {
 	    reply(599, "System error %s", emsg);
+	    amfree(filename_gz);
 	    amfree(emsg);
 	    amfree(ldir);
 	    return -1;
 	}
+	amfree(filename_gz);
 	dbprintf(("f %s\n", filename));
 	if ((fp = fopen(filename, "r")) == NULL) {
 	    reply(599, "System error %s", strerror(errno));
@@ -667,7 +595,7 @@ int  recursive;
 
     clear_dir_list();
 
-    if (config == NULL || dump_hostname == NULL || disk_name == NULL) {
+    if (config_name == NULL || dump_hostname == NULL || disk_name == NULL) {
 	reply(502, "Must set config,host,disk before listing a directory");
 	return -1;
     }
@@ -714,7 +642,8 @@ int  recursive;
     if(recursive)
     {
 	lreply(200, " Opaque recursive list of %s", dir);
-	for (dir_item = dir_list; dir_item != NULL; dir_item = dir_item->next)
+	for (dir_item = get_dir_list(); dir_item != NULL; 
+	     dir_item = dir_item->next)
 	    fast_lreply(201, " %s %d %-16s %s",
 			dir_item->dump->date, dir_item->dump->level,
 			dir_item->dump->tape, dir_item->path);
@@ -723,7 +652,8 @@ int  recursive;
     else
     {
 	lreply(200, " Opaque list of %s", dir);
-	for (dir_item = dir_list; dir_item != NULL; dir_item = dir_item->next)
+	for (dir_item = get_dir_list(); dir_item != NULL; 
+	     dir_item = dir_item->next)
 	    lreply(201, " %s %d %-16s %s",
 		   dir_item->dump->date, dir_item->dump->level,
 		   dir_item->dump->tape, dir_item->path);
@@ -741,7 +671,7 @@ int tapedev_is P((void))
     char *result;
 
     /* check state okay to do this */
-    if (config == NULL) {
+    if (config_name == NULL) {
 	reply(501, "Must set config before asking about tapedev.");
 	return -1;
     }
@@ -764,14 +694,14 @@ int are_dumps_compressed P((void))
     disk_t *diskp;
 
     /* check state okay to do this */
-    if (config == NULL || dump_hostname == NULL || disk_name == NULL) {
+    if (config_name == NULL || dump_hostname == NULL || disk_name == NULL) {
 	reply(501, "Must set config,host,disk name before asking about dumps.");
 	return -1;
     }
 
     /* now go through the list of disks and find which have indexes */
     for (diskp = disk_list->head; diskp != NULL; diskp = diskp->next)
-	if ((strcmp(diskp->host->hostname, dump_hostname) == 0)
+	if ((strcasecmp(diskp->host->hostname, dump_hostname) == 0)
 	    && (strcmp(diskp->name, disk_name) == 0))
 	    break;
 
@@ -794,10 +724,6 @@ int main(argc, argv)
 int argc;
 char **argv;
 {
-#ifdef FORCE_USERID
-    char *pwname;
-    struct passwd *pwptr;
-#endif	/* FORCE_USERID */
     char *line = NULL, *part = NULL;
     char *s, *fp;
     int ch;
@@ -823,6 +749,8 @@ char **argv;
 	close(fd);
     }
 
+    safe_cd();
+
     set_pname("amindexd");
 
 #ifdef FORCE_USERID
@@ -830,13 +758,13 @@ char **argv;
     /* we'd rather not run as root */
 
     if(geteuid() == 0) {
-	pwname = CLIENT_LOGIN;
-	if((pwptr = getpwnam(pwname)) == NULL)
-	    error("error [cannot find user %s in passwd file]\n", pwname);
+	if(client_uid == (uid_t) -1) {
+	    error("error [cannot find user %s in passwd file]\n", CLIENT_LOGIN);
+	}
 
-	initgroups(pwname, pwptr->pw_gid);
-	setgid(pwptr->pw_gid);
-	setuid(pwptr->pw_uid);
+	initgroups(CLIENT_LOGIN, client_gid);
+	setgid(client_gid);
+	setuid(client_uid);
     }
 
 #endif	/* FORCE_USERID */
@@ -852,14 +780,13 @@ char **argv;
 	argv++;
     }
 
-    amfree(config);
     if (argc > 0) {
-	config = newstralloc(config, *argv);
+	config_name = stralloc(*argv);
+	config_dir = vstralloc(CONFIG_DIR, "/", config_name, "/", NULL);
 	argc--;
 	argv++;
     }
 
-    umask(0007);
     dbopen();
     dbprintf(("%s: version %s\n", get_pname(), version()));
 
@@ -916,7 +843,9 @@ char **argv;
     amfree(disk_name);
     amfree(target_date);
 
-    if (config != NULL && is_config_valid(config) != -1) return 1;
+    if (config_name != NULL && is_config_valid(config_name) != -1) {
+	return 1;
+    }
 
     reply(220, "%s AMANDA index server (%s) ready.", local_hostname,
 	  version());
@@ -960,7 +889,7 @@ char **argv;
 		break;
 	    }
 	    /*
-	     * Hmmm.  We got a "line" from areads(), which means it saw
+	     * Hmmm.  We got a "line" from agets(), which means it saw
 	     * a '\n' (or EOF, etc), but there was not a '\r' before it.
 	     * Put a '\n' back in the buffer and loop for more.
 	     */
@@ -1039,11 +968,17 @@ char **argv;
 	    s[-1] = ch;
 	} else if (strcmp(cmd, "SCNF") == 0 && arg) {
 	    s[-1] = '\0';
+	    amfree(config_name);
+	    amfree(config_dir);
+	    config_name = newstralloc(config_name, arg);
+	    config_dir = vstralloc(CONFIG_DIR, "/", config_name, "/", NULL);
 	    if (is_config_valid(arg) != -1) {
-		config = newstralloc(config, arg);
 		amfree(dump_hostname);		/* invalidate any value */
 		amfree(disk_name);		/* invalidate any value */
-		reply(200, "Config set to %s.", config);
+		reply(200, "Config set to %s.", config_name);
+	    } else {
+		amfree(config_name);
+		amfree(config_dir);
 	    }
 	    s[-1] = ch;
 	} else if (strcmp(cmd, "DATE") == 0 && arg) {
@@ -1072,6 +1007,7 @@ char **argv;
     }
 
     uncompress_remove = remove_files(uncompress_remove);
+    free_find_result(&output_find);
     reply(200, "Good bye.");
     dbclose();
     return 0;
