@@ -1,6 +1,6 @@
 /*
  * Amanda, The Advanced Maryland Automatic Network Disk Archiver
- * Copyright (c) 1991-1998 University of Maryland at College Park
+ * Copyright (c) 1991-1998, 2000 University of Maryland at College Park
  * All Rights Reserved.
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
@@ -24,7 +24,7 @@
  * file named AUTHORS, in the root directory of this distribution.
  */
 /*
- * $Id$
+ * $Id: amlabel.c,v 1.18.2.15 2000/10/11 20:42:40 martinea Exp $
  *
  * write an Amanda label on a tape
  */
@@ -33,16 +33,13 @@
 #include "tapefile.h"
 #include "tapeio.h"
 #include "changer.h"
+
 #ifdef HAVE_LIBVTBLC
 #include <vtblc.h>
-
-static int vtbl_no      = -1;
-static char *datestr    = NULL;
 #endif /* HAVE_LIBVTBLC */
 
-char *pname = "amlabel";
-
-int slotcommand;
+char *config_name = NULL;
+char *config_dir = NULL;
 
 /* local functions */
 
@@ -57,19 +54,31 @@ char *argv0;
 }
 
 int main(argc, argv)
-int argc;
-char **argv;
+    int argc;
+    char **argv;
 {
-    char *confdir, *outslot = NULL;
-    char *errstr, *confname, *label, *oldlabel=NULL, *tapename = NULL;
+    char *conffile;
+    char *conf_tapelist;
+    char *outslot = NULL;
+    char *errstr = NULL, *label, *oldlabel=NULL, *tapename = NULL;
     char *labelstr, *slotstr;
-    char *olddatestamp=NULL, *tapefilename;
+    char *olddatestamp=NULL;
+    char *conf_tapelist_old;
     unsigned long malloc_hist_1, malloc_size_1;
     unsigned long malloc_hist_2, malloc_size_2;
     int fd;
+    int have_changer;
     int force, tape_ok;
     tape_t *tp;
+    int slotcommand;
+    uid_t uid_me;
+    uid_t uid_dumpuser;
+    char *dumpuser;
+    struct passwd *pw;
+
 #ifdef HAVE_LIBVTBLC
+    int vtbl_no      = -1;
+    char *datestr    = NULL;
     char *rawtapedev = NULL;
     int first_seg, last_seg;
 #endif /* HAVE_LIBVTBLC */
@@ -84,7 +93,10 @@ char **argv;
 	close(fd);
     }
 
+    safe_cd();
+
     set_pname("amlabel");
+    dbopen();
 
     malloc_size_1 = malloc_inuse(&malloc_hist_1);
 
@@ -97,7 +109,7 @@ char **argv;
     if(argc != 3+force && argc != 5+force)
 	usage(argv[0]);
 
-    confname = argv[1+force];
+    config_name = argv[1+force];
     label = argv[2+force];
 
     if(argc == 5+force) {
@@ -105,25 +117,47 @@ char **argv;
 	    usage(argv[0]);
 	slotstr = argv[4+force];
 	slotcommand = 1;
-    }
-    else {
+    } else {
 	slotstr = "current";
 	slotcommand = 0;
     }
 
-    confdir = vstralloc(CONFIG_DIR, "/", confname, NULL);
-    if(chdir(confdir) != 0)
-	error("could not cd to confdir %s: %s", confdir, strerror(errno));
+    config_dir = vstralloc(CONFIG_DIR, "/", config_name, "/", NULL);
+    conffile = stralloc2(config_dir, CONFFILE_NAME);
+    if (read_conffile(conffile)) {
+	error("could not read config file \"%s\"", conffile);
+    }
+    conf_tapelist = getconf_str(CNF_TAPELIST);
+    if (*conf_tapelist == '/') {
+	conf_tapelist = stralloc(conf_tapelist);
+    } else {
+	conf_tapelist = stralloc2(config_dir, conf_tapelist);
+    }
+    if (read_tapelist(conf_tapelist)) {
+	error("could not load tapelist \"%s\"", conf_tapelist);
+    }
 
-    if(read_conffile(CONFFILE_NAME))
-	error("could not read amanda config file");
+    uid_me = getuid();
+    uid_dumpuser = uid_me;
+    dumpuser = getconf_str(CNF_DUMPUSER);
+
+    if ((pw = getpwnam(dumpuser)) == NULL) {
+	error("cannot look up dump user \"%s\"", dumpuser);
+	/* NOTREACHED */
+    }
+    uid_dumpuser = pw->pw_uid;
+    if ((pw = getpwuid(uid_me)) == NULL) {
+	error("cannot look up my own uid %ld", (long)uid_me);
+	/* NOTREACHED */
+    }
+    if (uid_me != uid_dumpuser) {
+	error("running as user \"%s\" instead of \"%s\"",
+	      pw->pw_name, dumpuser);
+	/* NOTREACHED */
+    }
 
     labelstr = getconf_str(CNF_LABELSTR);
 
-    tapefilename = getconf_str(CNF_TAPELIST);
-    if(read_tapelist(tapefilename))
-	error("parse error in %s", tapefilename);
-
     if(!match(labelstr, label))
 	error("label %s doesn't match labelstr \"%s\"", label, labelstr);
 
@@ -132,19 +166,20 @@ char **argv;
 	    error("label %s already on a tape\n",label);
     }
 
-    if(!changer_init()) {
+    if((have_changer = changer_init()) == 0) {
 	if(slotcommand) {
 	    fprintf(stderr,
-	     "%s: no tpchanger specified in %s/%s, so slot command invalid\n",
-		    argv[0], confdir, CONFFILE_NAME);
+	     "%s: no tpchanger specified in \"%s\", so slot command invalid\n",
+		    argv[0], conffile);
 	    usage(argv[0]);
 	}
 	tapename = stralloc(getconf_str(CNF_TAPEDEV));
 #ifdef HAVE_LIBVTBLC
 	rawtapedev = stralloc(getconf_str(CNF_RAWTAPEDEV));
 #endif /* HAVE_LIBVTBLC */
-    }
-    else {
+    } else if(have_changer != 1) {
+	error("changer initialization failed: %s", strerror(errno));
+    } else {
 	if(changer_loadslot(slotstr, &outslot, &tapename)) {
 	    error("could not load slot \"%s\": %s", slotstr, changer_resultstr);
 	}
@@ -333,15 +368,56 @@ char **argv;
 	}
 #endif /* HAVE_LINUX_ZFTAPE_H */
 
+	if (tape_ok) {
+	    printf(", checking label"); fflush(stdout);
+
+	    if((errstr = tape_rdlabel(tapename, &olddatestamp, &oldlabel)) != NULL) {
+		putchar('\n');
+		if (strcmp(errstr, "not an amanda tape") != 0)
+		    error(errstr);
+		error("no label found, are you sure %s is non-rewinding?",
+		      tapename);
+	    }
+
+	    if (strcmp("X", olddatestamp) != 0 ||
+		strcmp(label, oldlabel) != 0) {
+		putchar('\n');
+		error("read label %s back, timestamp %s (expected X), what now?",
+		      oldlabel, olddatestamp);
+	    }
+
+	    /* write tape list */
+
+    	    /* XXX add cur_tape number to tape list structure */
+	    remove_tapelabel(label);
+    	    add_tapelabel(0, label);
+       	    conf_tapelist_old = stralloc2(conf_tapelist, ".amlabel");
+	    if (rename(conf_tapelist, conf_tapelist_old) != 0) {
+		if(errno != ENOENT)
+		    error("could not rename \"%s\" to \"%s\": %s",
+			  conf_tapelist, conf_tapelist_old, strerror(errno));
+	    }
+	    amfree(conf_tapelist_old);
+	    if(write_tapelist(conf_tapelist)) {
+	        error("couldn't write tapelist: %s", strerror(errno));
+	    }
+	} /* write tape list */
+
+        if(have_changer) {
+	    /* Now we try to inform the changer, about the new label */
+	    changer_label(outslot,label); 
+	}
 	printf(", done.\n");
-    }
-    else {
+    } else {
 	printf("\ntape not labeled\n");
     }
 
     amfree(outslot);
     amfree(tapename);
-    amfree(confdir);
+    amfree(conffile);
+    amfree(conf_tapelist);
+    amfree(config_dir);
+    config_name=NULL;
 
     malloc_size_2 = malloc_inuse(&malloc_hist_2);
 
@@ -349,5 +425,6 @@ char **argv;
 	malloc_list(fileno(stderr), malloc_hist_1, malloc_hist_2);
     }
 
+    dbclose();
     return 0;
 }
