@@ -24,7 +24,7 @@
  * file named AUTHORS, in the root directory of this distribution.
  */
 /*
- * $Id$
+ * $Id: amtape.c,v 1.22.2.8 2001/03/26 22:23:36 jrjackson Exp $
  *
  * tape changer interface program
  */
@@ -55,6 +55,11 @@ int show_init_all P((int rc, int ns, int bk));
 int show_init_current P((int rc, int ns, int bk));
 int show_slot P((int rc, char *slotstr, char *device));
 int taperscan_slot P((int rc, char *slotstr, char *device));
+int update_one_slot P((int rc, char *slotstr, char *device));
+void update_labeldb P((int argc, char **argv));
+
+char *config_name = NULL;
+char *config_dir = NULL;
 
 void usage()
 {
@@ -74,6 +79,7 @@ void usage()
     fprintf(stderr, "\t\tlabel <label>        find and load labeled tape\n");
     fprintf(stderr, "\t\ttaper                perform taper's scan alg.\n");
     fprintf(stderr, "\t\tdevice               show current tape device\n");
+    fprintf(stderr, "\t\tupdate               update the label matchingdatabase\n");
 
     exit(1);
 }
@@ -82,12 +88,17 @@ int main(argc, argv)
 int argc;
 char **argv;
 {
-    char *confdir;
-    char *confname;
+    char *conffile;
+    char *conf_tapelist;
     char *argv0 = argv[0];
     unsigned long malloc_hist_1, malloc_size_1;
     unsigned long malloc_hist_2, malloc_size_2;
     int fd;
+    int have_changer;
+    uid_t uid_me;
+    uid_t uid_dumpuser;
+    char *dumpuser;
+    struct passwd *pw;
 
     for(fd = 3; fd < FD_SETSIZE; fd++) {
 	/*
@@ -99,7 +110,10 @@ char **argv;
 	close(fd);
     }
 
+    safe_cd();
+
     set_pname("amtape");
+    dbopen();
 
     malloc_size_1 = malloc_inuse(&malloc_hist_1);
 
@@ -107,17 +121,48 @@ char **argv;
 
     if(argc < 3) usage();
 
-    confname = argv[1];
+    config_name = argv[1];
 
-    confdir = vstralloc(CONFIG_DIR, "/", confname, NULL);
-    if(chdir(confdir) != 0)
-	error("could not cd to confdir %s: %s", confdir, strerror(errno));
+    config_dir = vstralloc(CONFIG_DIR, "/", config_name, "/", NULL);
+    conffile = stralloc2(config_dir, CONFFILE_NAME);
+    if (read_conffile(conffile)) {
+	error("could not read config file \"%s\"", conffile);
+    }
+    conf_tapelist = getconf_str(CNF_TAPELIST);
+    if (*conf_tapelist == '/') {
+	conf_tapelist = stralloc(conf_tapelist);
+    } else {
+	conf_tapelist = stralloc2(config_dir, conf_tapelist);
+    }
+    if (read_tapelist(conf_tapelist)) {
+	error("could not load tapelist \"%s\"", conf_tapelist);
+    }
+    amfree(conf_tapelist);
 
-    if(read_conffile(CONFFILE_NAME))
-	error("could not read amanda config file");
+    uid_me = getuid();
+    uid_dumpuser = uid_me;
+    dumpuser = getconf_str(CNF_DUMPUSER);
 
-    if(!changer_init())
-	error("no tpchanger specified in %s/%s", confdir, CONFFILE_NAME);
+    if ((pw = getpwnam(dumpuser)) == NULL) {
+	error("cannot look up dump user \"%s\"", dumpuser);
+	/* NOTREACHED */
+    }
+    uid_dumpuser = pw->pw_uid;
+    if ((pw = getpwuid(uid_me)) == NULL) {
+	error("cannot look up my own uid %ld", (long)uid_me);
+	/* NOTREACHED */
+    }
+    if (uid_me != uid_dumpuser) {
+	error("running as user \"%s\" instead of \"%s\"",
+	      pw->pw_name, dumpuser);
+	/* NOTREACHED */
+    }
+
+    if((have_changer = changer_init()) == 0) {
+	error("no tpchanger specified in \"%s\"", conffile);
+    } else if (have_changer != 1) {
+	error("changer initialization failed: %s", strerror(errno));
+    }
 
     /* switch on command name */
 
@@ -131,13 +176,15 @@ char **argv;
     else if(strcmp(argv[0], "show") == 0)  show_slots(argc, argv);
     else if(strcmp(argv[0], "taper") == 0) taper_scan(argc, argv);
     else if(strcmp(argv[0], "device") == 0) show_device(argc, argv);
+    else if(strcmp(argv[0], "update") == 0) update_labeldb(argc, argv);
     else {
 	fprintf(stderr, "%s: unknown command \"%s\"\n", argv0, argv[0]);
 	usage();
     }
 
     amfree(changer_resultstr);
-    amfree(confdir);
+    amfree(conffile);
+    amfree(config_dir);
 
     malloc_size_2 = malloc_inuse(&malloc_hist_2);
 
@@ -145,6 +192,7 @@ char **argv;
 	malloc_list(fileno(stderr), malloc_hist_1, malloc_hist_2);
     }
 
+    dbclose();
     return 0;
 }
 
@@ -213,16 +261,27 @@ int argc;
 char **argv;
 {
     char *slotstr = NULL, *devicename = NULL;
+    char *errstr;
+    int is_advance;
 
     if(argc != 2)
 	usage();
 
+    is_advance = (strcmp(argv[1], "advance") == 0);
     if(changer_loadslot(argv[1], &slotstr, &devicename)) {
 	error("could not load slot %s: %s", slotstr, changer_resultstr);
     }
+    if(! is_advance && (errstr = tape_rewind(devicename)) != NULL) {
+	fprintf(stderr,
+		"%s: could not rewind %s: %s", get_pname(), devicename, errstr);
+	amfree(errstr);
+    }
 
-    fprintf(stderr, "%s: changed to slot %s on %s\n",
-	    get_pname(), slotstr, devicename);
+    fprintf(stderr, "%s: changed to slot %s", get_pname(), slotstr);
+    if(! is_advance) {
+	fprintf(stderr, " on %s", devicename);
+    }
+    fputc('\n', stderr);
     amfree(slotstr);
     amfree(devicename);
 }
@@ -231,7 +290,7 @@ char **argv;
 /* ---------------------------- */
 
 int nslots, backwards, found, got_match, tapedays;
-extern char *datestamp;
+char *datestamp;
 char *label = NULL, *first_match_label = NULL, *first_match = NULL;
 char *searchlabel, *labelstr;
 tape_t *tp;
@@ -269,6 +328,12 @@ char *device;
 	    fprintf(stderr, " (wrong tape)\n");
 	else {
 	    fprintf(stderr, " (exact label match)\n");
+	    if((errstr = tape_rewind(device)) != NULL) {
+		fprintf(stderr,
+			"%s: could not rewind %s: %s",
+			get_pname(), device, errstr);
+		amfree(errstr);
+	    }
 	    found = 1;
 	    amfree(datestamp);
 	    amfree(label);
@@ -294,7 +359,7 @@ char **argv;
 
     found = 0;
 
-    changer_scan(scan_init, loadlabel_slot);
+    changer_find(scan_init, loadlabel_slot, searchlabel);
 
     if(found)
 	fprintf(stderr, "%s: label %s is now loaded.\n",
@@ -336,6 +401,28 @@ int rc, ns, bk;
     return ret;
 }
 
+int update_one_slot(rc, slotstr, device)
+int rc;
+char *slotstr, *device;
+{
+    char *errstr;
+
+    if(rc > 1)
+	error("could not load slot %s: %s", slotstr, changer_resultstr);
+    else if(rc == 1)
+	fprintf(stderr, "slot %s: %s\n", slotstr, changer_resultstr);
+    else if((errstr = tape_rdlabel(device, &datestamp, &label)) != NULL)
+	fprintf(stderr, "slot %s: %s\n", slotstr, errstr);
+    else {
+	fprintf(stderr, "slot %s: date %-8s label %s\n",
+		slotstr, datestamp, label);
+	changer_label(slotstr,label);
+    }
+    amfree(datestamp);
+    amfree(label);
+    return 0;
+}
+
 int show_slot(rc, slotstr, device)
 int rc;
 char *slotstr, *device;
@@ -367,6 +454,16 @@ char **argv;
     changer_current(show_init_current, show_slot);
 }
 
+void update_labeldb(argc, argv)
+int argc;
+char **argv;
+{
+    if(argc != 1)
+	usage();
+
+    changer_scan(show_init_all, update_one_slot);
+}
+
 void show_slots(argc, argv)
 int argc;
 char **argv;
@@ -443,9 +540,6 @@ char **argv;
 {
     char *slotstr = NULL, *device = NULL;
 
-    if(read_tapelist(getconf_str(CNF_TAPELIST)))
-	error("could not load \"%s\"\n", getconf_str(CNF_TAPELIST));
-
     if((tp = lookup_last_reusable_tape(0)) == NULL)
 	searchlabel = NULL;
     else
@@ -460,7 +554,10 @@ char **argv;
     if(searchlabel) fprintf(stderr, "tape label %s or ", searchlabel);
     fprintf(stderr, "a new tape.\n");
 
-    changer_scan(scan_init, taperscan_slot);
+    if (searchlabel != NULL)
+      changer_find(scan_init, taperscan_slot,searchlabel);
+    else
+      changer_scan(scan_init, taperscan_slot);
 
     if(found == 2) {
 	fprintf(stderr, "%s: %s: settling for first labelstr match\n",
@@ -503,7 +600,7 @@ void show_device(argc, argv)
 int argc;
 char **argv;
 {
-    char *slot, *device;
+    char *slot = NULL, *device = NULL;
 
     if(changer_loadslot("current", &slot, &device))
 	error("Could not load current slot.\n");
