@@ -24,21 +24,29 @@
  * file named AUTHORS, in the root directory of this distribution.
  */
 /*
- * $Id$
+ * $Id: changer.h,v 1.6.4.3 1999/09/08 23:27:53 jrj Exp $
  *
  * interface routines for tape changers
  */
 #include "amanda.h"
 
+extern int changer_debug;
 extern char *changer_resultstr;
 
 int changer_init P((void));
 int changer_reset P((char **slotstr));
 int changer_clean P((char **slotstr));
 int changer_eject P((char **slotstr));
+int changer_label P((char *slotsp, char *labelstr));
 int changer_info P((int *nslotsp, char **curslotstr, int *backwards));
+int changer_query P((int *nslotsp, char **curslotstr, int *backwards,
+		     int *searchable));
+int changer_search P((char *searchlabel, char **outslotstr, char **devicename));
 int changer_loadslot P((char *inslotstr, char **outslotstr, char **devicename));
 void changer_current P((int (*user_init)(int rc, int nslots, int backwards),
 		     int (*user_slot)(int rc, char *slotstr, char *device)));
 void changer_scan P((int (*user_init)(int rc, int nslots, int backwards),
 		     int (*user_slot)(int rc, char *slotstr, char *device)));
+void changer_find P((int (*user_init)(int rc, int nslots, int backwards),
+		     int (*user_slot)(int rc, char *slotstr, char *device),
+                     char *searchlabel));
