@@ -1,6 +1,6 @@
 /*
  * Amanda, The Advanced Maryland Automatic Network Disk Archiver
- * Copyright (c) 1991-1998 University of Maryland at College Park
+ * Copyright (c) 1991-2000 University of Maryland at College Park
  * All Rights Reserved.
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
@@ -25,7 +25,7 @@
  *			   University of Maryland at College Park
  */
 /*
- * $Id$
+ * $Id: conffile.c,v 1.54.2.16 2001/01/04 21:09:15 jrjackson Exp $
  *
  * read configuration file
  */
@@ -39,6 +39,7 @@
 #include "arglist.h"
 
 #include "conffile.h"
+#include "diskfile.h"
 #include "clock.h"
 
 #ifdef HAVE_LIMITS_H
@@ -69,12 +70,13 @@ typedef enum {
     INCLUDEFILE,
     ORG, MAILTO, DUMPUSER,
     TAPECYCLE, TAPEDEV, CHNGRDEV, CHNGRFILE, LABELSTR,
-    BUMPSIZE, BUMPDAYS, BUMPMULT, ETIMEOUT,
-    TAPELIST, DISKFILE, INFOFILE, LOGDIR, LOGFILE,
+    BUMPSIZE, BUMPDAYS, BUMPMULT, ETIMEOUT, DTIMEOUT, CTIMEOUT,
+    TAPEBUFS, TAPELIST, DISKFILE, INFOFILE, LOGDIR, LOGFILE,
     DISKDIR, DISKSIZE, INDEXDIR, NETUSAGE, INPARALLEL, TIMEOUT,
     TPCHANGER, RUNTAPES,
     DEFINE, DUMPTYPE, TAPETYPE, INTERFACE,
     PRINTER, RESERVE,
+    COLUMNSPEC,
 
     /* holding disk */
     COMMENT, DIRECTORY, USE, CHUNKSIZE,
@@ -105,7 +107,7 @@ typedef enum {
     KRB4_AUTH, BSD_AUTH,
 
     /* dump strategy */
-    SKIP, STANDARD, NOFULL, NOINC, HANOI,
+    SKIP, STANDARD, NOFULL, NOINC, HANOI, INCRONLY,
 
     /* exclude list */
     LIST,
@@ -157,6 +159,7 @@ static val_t conf_diskfile;
 static val_t conf_diskdir;
 static val_t conf_tapetype;
 static val_t conf_indexdir;
+static val_t conf_columnspec;
 
 /* ints */
 static val_t conf_dumpcycle;
@@ -172,6 +175,9 @@ static val_t conf_maxdumps;
 static val_t conf_bumpsize;
 static val_t conf_bumpdays;
 static val_t conf_etimeout;
+static val_t conf_dtimeout;
+static val_t conf_ctimeout;
+static val_t conf_tapebufs;
 static val_t conf_reserve;
 
 /* reals */
@@ -196,8 +202,10 @@ static int seen_logdir, seen_bumpsize, seen_bumpmult, seen_bumpdays;
 static int seen_tapetype, seen_dumpcycle, seen_runspercycle;
 static int seen_maxcycle, seen_tapecycle;
 static int seen_disksize, seen_netusage, seen_inparallel, seen_timeout;
-static int seen_indexdir, seen_etimeout;
+static int seen_indexdir, seen_etimeout, seen_dtimeout, seen_ctimeout;
+static int seen_tapebufs;
 static int seen_reserve;
+static int seen_columnspec;
 
 static int allow_overwrites;
 static int token_pushed;
@@ -330,7 +338,11 @@ struct byname {
     /*{ "TIMEOUT", CNF_TIMEOUT, INT },*/
     { "MAXDUMPS", CNF_MAXDUMPS, INT },
     { "ETIMEOUT", CNF_ETIMEOUT, INT },
+    { "DTIMEOUT", CNF_DTIMEOUT, INT },
+    { "CTIMEOUT", CNF_CTIMEOUT, INT },
+    { "TAPEBUFS", CNF_TAPEBUFS, INT },
     { "RAWTAPEDEV", CNF_RAWTAPEDEV, STRING },
+    { "COLUMNSPEC", CNF_COLUMNSPEC, STRING },
     { "RESERVE", CNF_RESERVE, INT },
     { NULL }
 };
@@ -347,7 +359,7 @@ char *str;
     tmpstr = stralloc(str);
     s = tmpstr;
     while((ch = *s++) != '\0') {
-	if(islower(ch)) s[-1] = toupper(ch);
+	if(islower((int) ch)) s[-1] = toupper(ch);
     }
 
     for(np = byname_table; np->name != NULL; np++)
@@ -403,6 +415,9 @@ confparm_t parm;
     /*case CNF_TIMEOUT: return seen_timeout;*/
     case CNF_INDEXDIR: return seen_indexdir;
     case CNF_ETIMEOUT: return seen_etimeout;
+    case CNF_DTIMEOUT: return seen_dtimeout;
+    case CNF_CTIMEOUT: return seen_ctimeout;
+    case CNF_TAPEBUFS: return seen_tapebufs;
     case CNF_RAWTAPEDEV: return seen_rawtapedev;
     case CNF_RESERVE: return seen_reserve;
     default: return 0;
@@ -428,6 +443,9 @@ confparm_t parm;
     /*case CNF_TIMEOUT: r = conf_timeout.i; break;*/
     case CNF_MAXDUMPS: r = conf_maxdumps.i; break;
     case CNF_ETIMEOUT: r = conf_etimeout.i; break;
+    case CNF_DTIMEOUT: r = conf_dtimeout.i; break;
+    case CNF_CTIMEOUT: r = conf_ctimeout.i; break;
+    case CNF_TAPEBUFS: r = conf_tapebufs.i; break;
     case CNF_RESERVE: r = conf_reserve.i; break;
 
     default:
@@ -478,6 +496,7 @@ confparm_t parm;
     case CNF_TAPETYPE: r = conf_tapetype.s; break;
     case CNF_INDEXDIR: r = conf_indexdir.s; break;
     case CNF_RAWTAPEDEV: r = conf_rawtapedev.s; break;
+    case CNF_COLUMNSPEC: r = conf_columnspec.s; break;
 
     default:
 	error("error [unknown getconf_str parm: %d]", parm);
@@ -486,6 +505,11 @@ confparm_t parm;
     return r;
 }
 
+holdingdisk_t *getconf_holdingdisks()
+{
+    return holdingdisks;
+}
+
 dumptype_t *lookup_dumptype(str)
 char *str;
 {
@@ -543,12 +567,7 @@ static void init_defaults()
     malloc_mark(conf_org.s);
     conf_mailto.s = newstralloc(conf_mailto.s, "operators");
     malloc_mark(conf_mailto.s);
-#ifdef CLIENT_LOGIN
-    s = CLIENT_LOGIN;
-#else
-    s = "bin";
-#endif
-    conf_dumpuser.s = newstralloc(conf_dumpuser.s, s);
+    conf_dumpuser.s = newstralloc(conf_dumpuser.s, CLIENT_LOGIN);
     malloc_mark(conf_dumpuser.s);
 #ifdef DEFAULT_TAPE_DEVICE
     s = DEFAULT_TAPE_DEVICE;
@@ -592,12 +611,14 @@ static void init_defaults()
     malloc_mark(conf_tapetype.s);
     conf_indexdir.s = newstralloc(conf_indexdir.s, "/usr/adm/amanda/index");
     malloc_mark(conf_indexdir.s);
+    conf_columnspec.s = newstralloc(conf_columnspec.s, "");
+    malloc_mark(conf_columnspec.s);
 
     conf_dumpcycle.i	= 10;
-    conf_runspercycle.i	= -1;
+    conf_runspercycle.i	= 0;
     conf_tapecycle.i	= 15;
     conf_runtapes.i	= 1;
-    conf_disksize.i	= 200*1024;
+    conf_disksize.i	= 0;
     conf_netusage.i	= 300;
     conf_inparallel.i	= 10;
     conf_maxdumps.i	= 1;
@@ -606,6 +627,9 @@ static void init_defaults()
     conf_bumpdays.i	= 2;
     conf_bumpmult.r	= 1.5;
     conf_etimeout.i     = 300;
+    conf_dtimeout.i     = 1800;
+    conf_ctimeout.i     = 30;
+    conf_tapebufs.i     = 20;
     conf_reserve.i	= 100;
 
     /* defaults for internal variables */
@@ -621,8 +645,10 @@ static void init_defaults()
     seen_tapetype = seen_dumpcycle = seen_runspercycle = 0;
     seen_maxcycle = seen_tapecycle = 0;
     seen_disksize = seen_netusage = seen_inparallel = seen_timeout = 0;
-    seen_indexdir = seen_etimeout =  0;
+    seen_indexdir = seen_etimeout = seen_dtimeout = seen_ctimeout = 0;
+    seen_tapebufs = 0;
     seen_reserve = 0;
+    seen_columnspec = 0;
     line_num = got_parserror = 0;
     allow_overwrites = 0;
     token_pushed = 0;
@@ -717,7 +743,11 @@ char *filename;
     FILE *save_conf     = conf;
     char *save_confname = confname;
 
-    confname = stralloc(filename);
+    if (*filename == '/' || config_dir == NULL) {
+	confname = stralloc(filename);
+    } else {
+	confname = stralloc2(config_dir, filename);
+    }
 
     if((conf = fopen(confname, "r")) == NULL)
 	error("could not open conf file \"%s\": %s", confname, strerror(errno));
@@ -778,8 +808,12 @@ keytab_t main_keytable[] = {
     { "CHANGERDEV", CHNGRDEV },
     { "CHANGERFILE", CHNGRFILE },
     { "ETIMEOUT", ETIMEOUT },
+    { "DTIMEOUT", DTIMEOUT },
+    { "CTIMEOUT", CTIMEOUT },
+    { "TAPEBUFS", TAPEBUFS },
     { "RAWTAPEDEV", RAWTAPEDEV },
     { "RESERVE", RESERVE },
+    { "COLUMNSPEC", COLUMNSPEC },
     { NULL, IDENT }
 };
 
@@ -829,7 +863,11 @@ static int read_confline()
     case TAPETYPE:  get_simple(&conf_tapetype,  &seen_tapetype,  IDENT);  break;
     case INDEXDIR:  get_simple(&conf_indexdir,  &seen_indexdir,  STRING); break;
     case ETIMEOUT:  get_simple(&conf_etimeout,  &seen_etimeout,  INT);    break;
+    case DTIMEOUT:  get_simple(&conf_dtimeout,  &seen_dtimeout,  INT);    break;
+    case CTIMEOUT:  get_simple(&conf_ctimeout,  &seen_ctimeout,  INT);    break;
+    case TAPEBUFS:  get_simple(&conf_tapebufs,  &seen_tapebufs,  INT);    break;
     case RESERVE:   get_simple(&conf_reserve,  &seen_reserve,	 INT);    break;
+    case COLUMNSPEC:get_simple(&conf_columnspec,&seen_columnspec,STRING);break;
 
     case LOGFILE: /* XXX - historical */
 	/* truncate the filename part and pretend he said "logdir" */
@@ -871,6 +909,7 @@ static int read_confline()
 	    int i;
 
 	    i = get_number();
+	    i = (i/TAPE_BLOCK_SIZE)*TAPE_BLOCK_SIZE;
 
 	    if(!seen_disksize) {
 		conf_disksize.i = i;
@@ -950,13 +989,15 @@ static void get_holdingdisk()
 	    break;
 	case USE:
 	    get_simple((val_t *)&hdcur.disksize, &hdcur.s_size, INT);
+	    hdcur.disksize = (hdcur.disksize/TAPE_BLOCK_SIZE)*TAPE_BLOCK_SIZE;
 	    break;
 	case CHUNKSIZE:
 	    get_simple((val_t *)&hdcur.chunksize, &hdcur.s_csize, INT);
 	    if(hdcur.chunksize == 0)
 	        hdcur.chunksize =  ((INT_MAX/1024)-(2*TAPE_BLOCK_SIZE));
-	    else if(hdcur.chunksize == -1)
-	        hdcur.chunksize = -((INT_MAX/1024)-(2*TAPE_BLOCK_SIZE));
+	    else if(hdcur.chunksize < 0)
+		parserror("Negative chunksize (%ld) is no longer supported", hdcur.chunksize);
+	    hdcur.chunksize = (hdcur.chunksize/TAPE_BLOCK_SIZE)*TAPE_BLOCK_SIZE;
 	    break;
 	case RBRACE:
 	    done = 1;
@@ -983,7 +1024,7 @@ static void init_holdingdisk_defaults()
     hdcur.diskdir = stralloc(conf_diskdir.s);
     malloc_mark(hdcur.diskdir);
     hdcur.disksize = 0;
-    hdcur.chunksize = -((INT_MAX/1024)-(2*TAPE_BLOCK_SIZE));
+    hdcur.chunksize = 1024*1024*1024; /* 1 Gb */
 
     hdcur.s_comment = 0;
     hdcur.s_disk = 0;
@@ -1032,12 +1073,31 @@ keytab_t dumptype_keytable[] = {
     { NULL, IDENT }
 };
 
-static void get_dumptype()
+dumptype_t *read_dumptype(name, from, fname, linenum)
+     char *name;
+     FILE *from;
+     char *fname;
+     int *linenum;
 {
     int done;
     int save_overwrites;
     keytab_t *save_kt;
     val_t tmpval;
+    FILE *saved_conf = NULL;
+    char *saved_fname = NULL;
+
+    if (from) {
+	saved_conf = conf;
+	conf = from;
+    }
+
+    if (fname) {
+	saved_fname = confname;
+	confname = fname;
+    }
+
+    if (linenum)
+	line_num = *linenum;
 
     save_overwrites = allow_overwrites;
     allow_overwrites = 1;
@@ -1047,13 +1107,20 @@ static void get_dumptype()
 
     init_dumptype_defaults();
 
-    get_conftoken(IDENT);
-    dpcur.name = stralloc(tokenval.s);
-    malloc_mark(dpcur.name);
+    if (name) {
+	dpcur.name = name;
+    } else {
+	get_conftoken(IDENT);
+	dpcur.name = stralloc(tokenval.s);
+	malloc_mark(dpcur.name);
+    }
+
     dpcur.seen = line_num;
 
-    get_conftoken(LBRACE);
-    get_conftoken(NL);
+    if (! name) {
+	get_conftoken(LBRACE);
+	get_conftoken(NL);
+    }
 
     done = 0;
     do {
@@ -1149,7 +1216,11 @@ static void get_dumptype()
 	default:
 	    parserror("dump type parameter expected");
 	}
-	if(tok != NL && tok != END) get_conftoken(NL);
+	if(tok != NL && tok != END &&
+	   /* When a name is specified, we shouldn't consume the NL
+	      after the RBRACE.  */
+	   (tok != RBRACE || name == 0))
+	    get_conftoken(NL);
     } while(!done);
 
     /* XXX - there was a stupidity check in here for skip-incr and
@@ -1159,8 +1230,23 @@ static void get_dumptype()
 
     allow_overwrites = save_overwrites;
     keytable = save_kt;
+
+    if (linenum)
+	*linenum = line_num;
+
+    if (fname)
+	confname = saved_fname;
+
+    if (from)
+	conf = saved_conf;
+
+    return lookup_dumptype(dpcur.name);
 }
 
+static void get_dumptype()
+{
+    read_dumptype(NULL, NULL, NULL, NULL);
+}
 
 static void init_dumptype_defaults()
 {
@@ -1716,6 +1802,7 @@ keytab_t strategy_keytable[] = {
     { "NOINC", NOINC },
     { "SKIP", SKIP },
     { "STANDARD", STANDARD },
+    { "INCRONLY", INCRONLY },
     { NULL, IDENT }
 };
 
@@ -1746,6 +1833,9 @@ static void get_strategy()
     case HANOI:
 	strat = DS_HANOI;
 	break;
+    case INCRONLY:
+	strat = DS_INCRONLY;
+	break;
     default:
 	parserror("STANDARD or NOFULL expected");
 	strat = DS_STANDARD;
@@ -1853,6 +1943,7 @@ static int get_time()
 
 keytab_t numb_keytable[] = {
     { "B", MULT1 },
+    { "BPS", MULT1 },
     { "BYTE", MULT1 },
     { "BYTES", MULT1 },
     { "DAY", MULT1 },
@@ -1861,16 +1952,26 @@ keytab_t numb_keytable[] = {
     { "K", MULT1K },
     { "KB", MULT1K },
     { "KBPS", MULT1K },
+    { "KBYTE", MULT1K },
     { "KBYTES", MULT1K },
+    { "KILOBYTE", MULT1K },
     { "KILOBYTES", MULT1K },
     { "KPS", MULT1K },
     { "M", MULT1M },
     { "MB", MULT1M },
     { "MBPS", MULT1M },
+    { "MBYTE", MULT1M },
     { "MBYTES", MULT1M },
     { "MEG", MULT1M },
+    { "MEGABYTE", MULT1M },
     { "MEGABYTES", MULT1M },
+    { "G", MULT1G },
     { "GB", MULT1G },
+    { "GBPS", MULT1G },
+    { "GBYTE", MULT1G },
+    { "GBYTES", MULT1G },
+    { "GIG", MULT1G },
+    { "GIGABYTE", MULT1G },
     { "GIGABYTES", MULT1G },
     { "MPS", MULT1M },
     { "TAPE", MULT1 },
@@ -2200,6 +2301,9 @@ tok_t exp;
 
 #ifdef TEST
 
+char *config_name = NULL;
+char *config_dir = NULL;
+
 void
 dump_configuration(filename)
     char *filename;
@@ -2241,10 +2345,14 @@ dump_configuration(filename)
     /*printf("conf_timeout = %d\n", getconf_int(CNF_TIMEOUT));*/
     printf("conf_maxdumps = %d\n", getconf_int(CNF_MAXDUMPS));
     printf("conf_etimeout = %d\n", getconf_int(CNF_ETIMEOUT));
+    printf("conf_dtimeout = %d\n", getconf_int(CNF_DTIMEOUT));
+    printf("conf_ctimeout = %d\n", getconf_int(CNF_CTIMEOUT));
+    printf("conf_tapebufs = %d\n", getconf_int(CNF_TAPEBUFS));
     printf("conf_reserve  = %d\n", getconf_int(CNF_RESERVE));
 
     /*printf("conf_diskdir = \"%s\"\n", getconf_str(CNF_DISKDIR));*/
     /*printf("conf_disksize = %d\n", getconf_int(CNF_DISKSIZE));*/
+    printf("conf_columnspec = \"%s\"\n", getconf_str(CNF_COLUMNSPEC));
     printf("conf_indexdir = \"%s\"\n", getconf_str(CNF_INDEXDIR));
     printf("num_holdingdisks = %d\n", num_holdingdisks);
     for(hp = holdingdisks; hp != NULL; hp = hp->next) {
@@ -2302,6 +2410,9 @@ dump_configuration(filename)
 	case DS_HANOI:
 	    printf("HANOI");
 	    break;
+	case DS_INCRONLY:
+	    printf("INCRONLY");
+	    break;
 	}
 	putchar('\n');
 	printf("	COMPRATE %f, %f\n", dp->comprate[0], dp->comprate[1]);
@@ -2350,9 +2461,11 @@ dump_configuration(filename)
 
 int
 main(argc, argv)
-int argc;
-char *argv[];
+    int argc;
+    char *argv[];
 {
+  char *conffile;
+  char *diskfile;
   int result;
   int fd;
   unsigned long malloc_hist_1, malloc_size_1;
@@ -2373,13 +2486,39 @@ char *argv[];
   malloc_size_1 = malloc_inuse(&malloc_hist_1);
 
   startclock();
-  if (argc>1)
-    if(chdir(argv[1])) {
-      perror(argv[1]);
-      return 1;
+
+  if (argc > 1) {
+    if (argv[1][0] == '/') {
+      config_dir = stralloc(argv[1]);
+      config_name = strrchr(config_dir, '/') + 1;
+      config_name[-1] = '\0';
+      config_dir = newstralloc2(config_dir, config_dir, "/");
+    } else {
+      config_name = stralloc(argv[1]);
+      config_dir = vstralloc(CONFIG_DIR, "/", config_name, "/", NULL);
     }
-  result = read_conffile(CONFFILE_NAME);
+  } else {
+    char my_cwd[STR_SIZE];
+
+    if (getcwd(my_cwd, sizeof(my_cwd)) == NULL) {
+      error("cannot determine current working directory");
+    }
+    config_dir = stralloc2(my_cwd, "/");
+    if ((config_name = strrchr(my_cwd, '/')) != NULL) {
+      config_name = stralloc(config_name + 1);
+    }
+  }
+
+  conffile = stralloc2(config_dir, CONFFILE_NAME);
+  result = read_conffile(conffile);
+  if (result == 0) {
+    diskfile = getconf_str(CNF_DISKFILE);
+    if (diskfile != NULL && access(diskfile, R_OK) == 0) {
+      result = (read_diskfile(diskfile) == NULL);
+    }
+  }
   dump_configuration(CONFFILE_NAME);
+  amfree(conffile);
 
   malloc_size_2 = malloc_inuse(&malloc_hist_2);
 
