@@ -1,6 +1,6 @@
 /*
  * Amanda, The Advanced Maryland Automatic Network Disk Archiver
- * Copyright (c) 1991-1998 University of Maryland at College Park
+ * Copyright (c) 1991-2000 University of Maryland at College Park
  * All Rights Reserved.
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
@@ -25,7 +25,7 @@
  *			   University of Maryland at College Park
  */
 /*
- * $Id$
+ * $Id: conffile.h,v 1.24.2.8 2000/04/09 08:10:30 oliva Exp $
  *
  * interface for config file reading code
  */
@@ -44,7 +44,9 @@ typedef enum conf_e {
     CNF_MAXCYCLE, CNF_TAPECYCLE,
     CNF_DISKSIZE, CNF_NETUSAGE, CNF_INPARALLEL, CNF_TIMEOUT,
     CNF_BUMPSIZE, CNF_BUMPMULT, CNF_BUMPDAYS, CNF_TPCHANGER, CNF_RUNTAPES,
-    CNF_MAXDUMPS, CNF_ETIMEOUT, CNF_RAWTAPEDEV, CNF_PRINTER, CNF_RESERVE
+    CNF_MAXDUMPS, CNF_ETIMEOUT, CNF_DTIMEOUT, CNF_CTIMEOUT,
+    CNF_TAPEBUFS, CNF_RAWTAPEDEV, CNF_PRINTER, CNF_RESERVE,
+    CNF_COLUMNSPEC
 } confparm_t;
 
 typedef enum auth_e {
@@ -59,9 +61,9 @@ typedef struct tapetype_s {
 
     char *comment;
     char *lbl_templ;
-    unsigned long length;
-    unsigned long filemark;
-    long speed;
+    unsigned int length;
+    unsigned int filemark;
+    int speed;
 
     /* seen flags */
     int s_comment;
@@ -79,6 +81,7 @@ typedef struct tapetype_s {
 #define DS_4		4	/* ? (0 1 2 3 4 5 6 7 8 9 10 11 ...) */
 #define DS_5		5	/* ? (0 1 1 1 1 1 1 1 1 1 1 1 ...) */
 #define DS_HANOI	6	/* Tower of Hanoi (? ? ? ? ? ...) */
+#define DS_INCRONLY	7	/* Forced fulls (0 1 1 2 2 FORCE0 1 1 ...) */
 
 /* Compression types */
 #define COMP_NONE	0	/* No compression */
@@ -96,10 +99,10 @@ typedef struct dumptype_s {
     char *program;
     char *exclude;
     int exclude_list;
-    long priority;
-    long dumpcycle;
+    int priority;
+    int dumpcycle;
     int maxcycle;
-    long frequency;
+    int frequency;
     auth_t auth;
     int maxdumps;
     time_t start_t;
@@ -107,13 +110,13 @@ typedef struct dumptype_s {
     int compress;
     float comprate[2]; /* first is full, second is incremental */
     /* flag options */
-    int record:1;
-    int skip_incr:1;
-    int skip_full:1;
-    int no_hold:1;
-    int kencrypt:1;
-    int ignore:1;
-    int index:1;
+    unsigned int record:1;
+    unsigned int skip_incr:1;
+    unsigned int skip_full:1;
+    unsigned int no_hold:1;
+    unsigned int kencrypt:1;
+    unsigned int ignore:1;
+    unsigned int index:1;
 
     /* seen flags */
     int s_comment;
@@ -173,6 +176,9 @@ typedef struct holdingdisk_s {
     void *up;			/* generic user pointer */
 } holdingdisk_t;
 
+extern char *config_name;
+extern char *config_dir;
+
 extern holdingdisk_t *holdingdisks;
 extern int num_holdingdisks;
 
@@ -183,7 +189,9 @@ double getconf_real P((confparm_t parameter));
 char *getconf_str P((confparm_t parameter));
 char *getconf_byname P((char *confname));
 dumptype_t *lookup_dumptype P((char *identifier));
+dumptype_t *read_dumptype P((char *name, FILE *from, char *fname, int *linenum));
 tapetype_t *lookup_tapetype P((char *identifier));
 interface_t *lookup_interface P((char *identifier));
+holdingdisk_t *getconf_holdingdisks();
 
 #endif /* ! CONFFILE_H */
