@@ -25,7 +25,7 @@
  *			   University of Maryland at College Park
  */
 /*
- * $Id$
+ * $Id: diskfile.c,v 1.27.4.7 2001/02/10 04:27:18 jrjackson Exp $
  *
  * read disklist file
  */
@@ -114,6 +114,18 @@ disk_t *disk;
     disk->next = NULL;
 }
 
+void headqueue_disk(list, disk)	/* put disk on head of queue */
+disklist_t *list;
+disk_t *disk;
+{
+    if(list->head == NULL) list->tail = disk;
+    else list->head->prev = disk;
+    disk->next = list->head;
+
+    list->head = disk;
+    disk->prev = NULL;
+}
+
 void insert_disk(list, disk, cmp)	/* insert in sorted order */
 disklist_t *list;
 disk_t *disk;
@@ -138,6 +150,18 @@ int (*cmp) P((disk_t *a, disk_t *b));
     else ptr->prev = disk;
 }
 
+int find_disk(list, disk)
+disklist_t *list;
+disk_t *disk;
+/* check if disk is present in list. Return true if so, false otherwise. */
+{
+disk_t *t;
+
+    for( t = list->head; t && t != disk; t = t->next );
+
+    return t == disk;
+}
+
 void sort_disk(in, out, cmp)	/* sort a whole queue */
 disklist_t *in;
 disklist_t *out;
@@ -191,7 +215,7 @@ char *st;
     char *s = st;
 
     while(*s) {
-	if(islower(*s)) *s = toupper(*s);
+	if(islower((int)*s)) *s = toupper(*s);
 	s++;
     }
     return st;
@@ -206,8 +230,8 @@ static int read_diskline()
     interface_t *netif = 0;
     char *hostname = NULL;
     static char *line = NULL;
-    char *s, *fp;
-    int ch;
+    char *s = NULL, *fp;
+    int ch = '\0', dup = 0;
 
     amfree(line);
     for(; (line = agets(diskf)) != NULL; free(line)) {
@@ -244,36 +268,86 @@ static int read_diskline()
 
     if(host && (disk = lookup_disk(hostname, fp)) != NULL) {
 	parserror("duplicate disk record, previous on line %d", disk->line);
-	return 1;
+	dup = 1;
+    } else {
+	disk = alloc(sizeof(disk_t));
+	malloc_mark(disk);
+	disk->line = line_num;
+	disk->name = stralloc(fp);
+	malloc_mark(disk->name);
+	disk->spindle = -1;
+	disk->up = NULL;
+	disk->inprogress = 0;
     }
 
-    disk = alloc(sizeof(disk_t));
-    malloc_mark(disk);
-    disk->line = line_num;
-    disk->name = stralloc(fp);
-    malloc_mark(disk->name);
-    disk->spindle = -1;
-    disk->up = NULL;
-    disk->inprogress = 0;
-
     skip_whitespace(s, ch);
     if(ch == '\0' || ch == '#') {
 	parserror("disk dumptype expected");
 	if(host == NULL) amfree(hostname);
-	amfree(disk->name);
-	amfree(disk);
+	if(!dup) {
+	    amfree(disk->name);
+	    amfree(disk);
+	}
 	return 1;
     }
     fp = s - 1;
     skip_non_whitespace(s, ch);
     s[-1] = '\0';
-    if((dtype = lookup_dumptype(upcase(fp))) == NULL) {
+    if (fp[0] == '{') {
+	s[-1] = ch;
+	s = fp+2;
+	skip_whitespace(s, ch);
+	if (ch != '\0' && ch != '#') {
+	    parserror("expected line break after `{\', ignoring rest of line");
+	}
+
+	if (strchr(s-1, '}') &&
+	    (strchr(s-1, '#') == NULL ||
+	     strchr(s-1, '}') < strchr(s-1, '#'))) {
+	    if(host == NULL) amfree(hostname);
+	    if(!dup) {
+		amfree(disk->name);
+		amfree(disk);
+	    }
+	    return 1;
+	}
+	amfree(line);
+
+	dtype = read_dumptype(vstralloc("custom(", hostname,
+					":", disk->name, ")", 0),
+			      diskf, diskfname, &line_num);
+
+	line = agets(diskf);
+	/* line_num += 1; */ /* read_dumptype did it already */
+
+	if (dtype == NULL || dup) {
+	    if(host == NULL) amfree(hostname);
+	    if(!dup) {
+	      amfree(disk->name);
+	      amfree(disk);
+	    }
+	    return line != NULL;
+	}
+
+	if (line == NULL)
+	    line = stralloc("");
+	s = line;
+	ch = *s++;
+    } else if((dtype = lookup_dumptype(upcase(fp))) == NULL) {
 	parserror("undefined dumptype `%s'", fp);
 	if(host == NULL) amfree(hostname);
-	amfree(disk->name);
-	amfree(disk);
+	if(!dup) {
+	  amfree(disk->name);
+	  amfree(disk);
+	}
 	return 1;
     }
+
+    if (dup) {
+	if(host == NULL) amfree(hostname);
+	return 1;
+    }
+
     disk->dtype_name	= dtype->name;
     disk->program	= dtype->program;
     disk->exclude	= dtype->exclude;
@@ -341,6 +415,7 @@ static int read_diskline()
 	host->up = NULL;
 	host->inprogress = 0;
 	host->maxdumps = 1;		/* will be overwritten */
+	host->start_t = 0;
     }
 
     host->netif = netif;
@@ -461,6 +536,9 @@ disk_t *dp;
 
 #ifdef TEST
 
+char *config_name = NULL;
+char *config_dir = NULL;
+
 void
 dump_disk(dp)
 disk_t *dp;
@@ -511,6 +589,8 @@ main(argc, argv)
 int argc;
 char *argv[];
 {
+  char *conffile;
+  char *conf_diskfile;
   int result;
   int fd;
   unsigned long malloc_hist_1, malloc_size_1;
@@ -530,15 +610,32 @@ char *argv[];
 
   malloc_size_1 = malloc_inuse(&malloc_hist_1);
 
-  if (argc>1)
-    if (chdir(argv[1])) {
-       perror(argv[1]);
-       return 1;
+  if (argc>1) {
+    config_name = argv[1];
+    if (strchr(config_name, '/') != NULL) {
+      config_dir = stralloc2(argv[1], "/");
+      config_name = strrchr(config_name, '/') + 1;
+    } else {
+      config_dir = vstralloc(CONFIG_DIR, "/", config_name, "/", NULL);
     }
-  if((result = read_conffile(CONFFILE_NAME)) == 0) {
-    result = (read_diskfile(getconf_str(CNF_DISKFILE)) == NULL);
+  } else {
+    config_dir = stralloc("");
   }
-  dump_disklist();
+  conffile = stralloc2(config_dir, CONFFILE_NAME);
+  if((result = read_conffile(conffile)) == 0) {
+    conf_diskfile = getconf_str(CNF_DISKFILE);
+    if (*conf_diskfile == '/') {
+      conf_diskfile = stralloc(conf_diskfile);
+    } else {
+      conf_diskfile = stralloc2(config_dir, conf_diskfile);
+    }
+    if((result = (read_diskfile(conf_diskfile) == NULL)) == 0) {
+      dump_disklist();
+    }
+    amfree(conf_diskfile);
+  }
+  amfree(conffile);
+  amfree(config_dir);
 
   malloc_size_2 = malloc_inuse(&malloc_hist_2);
 
