@@ -25,7 +25,7 @@
  *			   University of Maryland at College Park
  */
 /*
- * $Id$
+ * $Id: diskfile.h,v 1.11.4.3 2000/10/10 21:33:23 martinea Exp $
  *
  * interface for disklist file reading code
  */
@@ -42,6 +42,7 @@ typedef struct host_s {
     int inprogress;			/* # dumps in progress */
     int maxdumps;			/* maximum dumps in parallel */
     interface_t *netif;			/* network interface this host is on */
+    time_t start_t;			/* start dump after this time */
     char *up;				/* generic user pointer */
 } host_t;
 
@@ -91,7 +92,9 @@ host_t *lookup_host P((char *hostname));
 disk_t *lookup_disk P((char *hostname, char *diskname));
 
 void enqueue_disk P((disklist_t *list, disk_t *disk));
+void headqueue_disk P((disklist_t *list, disk_t *disk));
 void insert_disk P((disklist_t *list, disk_t *disk, int (*f)(disk_t *a, disk_t *b)));
+int  find_disk P((disklist_t *list, disk_t *disk));
 void sort_disk P((disklist_t *in, disklist_t *out, int (*f)(disk_t *a, disk_t *b)));
 disk_t *dequeue_disk P((disklist_t *list));
 void remove_disk P((disklist_t *list, disk_t *disk));
