@@ -25,7 +25,7 @@
  *			   University of Maryland at College Park
  */
 /*
- * $Id$
+ * $Id: driverio.c,v 1.35.2.14 1999/11/12 00:16:27 oliva Exp $
  *
  * I/O-related functions for driver program
  */
@@ -43,22 +43,40 @@
 char *cmdstr[] = {
     "BOGUS", "QUIT", "DONE",
     "FILE-DUMP", "PORT-DUMP", "CONTINUE", "ABORT",	/* dumper cmds */
-    "FAILED", "TRY-AGAIN", "NO-ROOM", "ABORT-FINISHED",	/* dumper results */
+    "FAILED", "TRY-AGAIN", "NO-ROOM", "RQ-MORE-DISK",
+    "ABORT-FINISHED",	/* dumper results */
     "FATAL-TRY-AGAIN",
     "START-TAPER", "FILE-WRITE", "PORT-WRITE",		/* taper cmds */
     "PORT", "TAPE-ERROR", "TAPER-OK",			/* taper results */
     NULL
 };
 
-void addfd(fd)
-int fd;
+
+void init_driverio()
+{
+    dumper_t *dumper;
+
+    taper = -1;
+
+    for(dumper = dmptable; dumper < dmptable + MAX_DUMPERS; dumper++) {
+	dumper->outfd = -1;
+    }
+}
+
+
+void addfd(fd, readset, maxfd)
+int    fd;
+fd_set *readset;
+int    *maxfd;
 {
     if(fd < 0 || fd >= FD_SETSIZE) {
 	error("addfd: descriptor %d out of range (0 .. %d)\n",
 	      fd, FD_SETSIZE-1);
     }
-    FD_SET(fd, &readset);
-    if(fd > maxfd) maxfd = fd;
+    if(readset != NULL)
+	FD_SET(fd, readset);
+    if(maxfd != NULL)
+	if(fd > *maxfd) *maxfd = fd;
 }
 
 char *childstr(fd)
@@ -70,7 +88,7 @@ int fd;
 
     if(fd == taper) return "taper";
 
-    for(dumper = dmptable; dumper < dmptable+inparallel; dumper++)
+    for(dumper = dmptable; dumper < dmptable + MAX_DUMPERS; dumper++)
 	if(dumper->outfd == fd) return dumper->name;
 
     ap_snprintf(fd_str, sizeof(fd_str), "%d", fd);
@@ -79,7 +97,8 @@ int fd;
 }
 
 
-void startup_tape_process()
+void startup_tape_process(taper_program)
+char *taper_program;
 {
     int fd[2];
 
@@ -101,17 +120,18 @@ void startup_tape_process()
 	aclose(fd[0]);
 	if(dup2(fd[1], 0) == -1 || dup2(fd[1], 1) == -1)
 	    error("taper dup2: %s", strerror(errno));
-	execle(taper_program, "taper", (char *)0, safe_env());
+	execle(taper_program, "taper", config_name, (char *)0, safe_env());
 	error("exec %s: %s", taper_program, strerror(errno));
     default:	/* parent process */
 	aclose(fd[1]);
 	taper = fd[0];
-	addfd(taper);
+	addfd(taper, &readset, &maxfd);
     }
 }
 
-void startup_dump_process(dumper)
+void startup_dump_process(dumper, dumper_program)
 dumper_t *dumper;
+char *dumper_program;
 {
     int fd[2];
 
@@ -125,13 +145,13 @@ dumper_t *dumper;
 	aclose(fd[0]);
 	if(dup2(fd[1], 0) == -1 || dup2(fd[1], 1) == -1)
 	    error("%s dup2: %s", dumper->name, strerror(errno));
-	execle(dumper_program, "dumper", (char *)0, safe_env());
+	execle(dumper_program, "dumper", config_name, (char *)0, safe_env());
 	error("exec %s (%s): %s", dumper_program,
 	      dumper->name, strerror(errno));
     default:	/* parent process */
 	aclose(fd[1]);
 	dumper->infd = dumper->outfd = fd[0];
-	addfd(dumper->outfd);
+	addfd(dumper->outfd, &readset, &maxfd);
 	dumper->busy = dumper->down = 0;
 	dumper->dp = NULL;
 	fprintf(stderr,"driver: started %s pid %d\n",
@@ -140,7 +160,9 @@ dumper_t *dumper;
     }
 }
 
-void startup_dump_processes()
+void startup_dump_processes(dumper_program, inparallel)
+char *dumper_program;
+int inparallel;
 {
     int i;
     dumper_t *dumper;
@@ -150,13 +172,16 @@ void startup_dump_processes()
 	ap_snprintf(number, sizeof(number), "%d", i);
 	dumper->name = stralloc2("dumper", number);
 
-	startup_dump_process(dumper);
+	startup_dump_process(dumper, dumper_program);
     }
 }
 
-tok_t getresult(fd, show)
+tok_t getresult(fd, show, result_argc, result_argv, max_arg)
 int fd;
 int show;
+int *result_argc;
+char **result_argv;
+int max_arg;
 {
     int arg;
     tok_t t;
@@ -166,9 +191,9 @@ int show;
 	if(errno) {
 	    error("reading result from %s: %s", childstr(fd), strerror(errno));
 	}
-	argc = 0;				/* EOF */
+	*result_argc = 0;				/* EOF */
     } else {
-	argc = split(line, argv, sizeof(argv) / sizeof(argv[0]), " ");
+	*result_argc = split(line, result_argv, max_arg, " ");
     }
     amfree(line);
 
@@ -176,28 +201,28 @@ int show;
 	printf("driver: result time %s from %s:",
 	       walltime_str(curclock()),
 	       childstr(fd));
-	for(arg = 1; arg <= argc; arg++)
-	    printf(" %s", argv[arg]);
+	for(arg = 1; arg <= *result_argc; arg++)
+	    printf(" %s", result_argv[arg]);
 	printf("\n");
 	fflush(stdout);
     }
 
 #ifdef DEBUG
-    printf("argc = %d\n", argc);
-    for(arg = 0; arg < sizeof(argv) / sizeof(argv[0]); arg++)
-	printf("argv[%d] = \"%s\"\n", arg, argv[arg]);
+    printf("argc = %d\n", *result_argc);
+    for(arg = 0; arg < *result_argc; arg++)
+	printf("argv[%d] = \"%s\"\n", arg, result_argv[arg]);
 #endif
 
-    if(argc < 1) return BOGUS;
+    if(*result_argc < 1) return BOGUS;
 
     for(t = BOGUS+1; t < LAST_TOK; t++)
-	if(strcmp(argv[1], cmdstr[t]) == 0) return t;
+	if(strcmp(result_argv[1], cmdstr[t]) == 0) return t;
 
     return BOGUS;
 }
 
 
-void taper_cmd(cmd, /* optional */ ptr, destname, level, datestamp)
+int taper_cmd(cmd, /* optional */ ptr, destname, level, datestamp)
 tok_t cmd;
 void *ptr;
 char *destname;
@@ -250,13 +275,16 @@ char *datestamp;
     fflush(stdout);
     for(l = 0, n = strlen(cmdline); l < n; l += s) {
 	if((s = write(taper, cmdline + l, n - l)) < 0) {
-	    error("writing taper command: %s", strerror(errno));
+	    printf("writing taper command: %s\n", strerror(errno));
+	    fflush(stdout);
+	    return 0;
 	}
     }
     amfree(cmdline);
+    return 1;
 }
 
-void dumper_cmd(dumper, cmd, /* optional */ dp)
+int dumper_cmd(dumper, cmd, /* optional */ dp)
 dumper_t *dumper;
 tok_t cmd;
 disk_t *dp;
@@ -264,14 +292,23 @@ disk_t *dp;
     char *cmdline = NULL;
     char number[NUM_STR_SIZE];
     char chunksize[NUM_STR_SIZE];
+    char use[NUM_STR_SIZE];
     int l, n, s;
     char *o;
+    int activehd=0;
+    assignedhd_t **h=NULL;
+
+    if(dp && sched(dp) && sched(dp)->holdp) {
+	h = sched(dp)->holdp;
+	activehd = sched(dp)->activehd;
+    }
 
     switch(cmd) {
     case FILE_DUMP:
+	holdalloc(h[activehd]->disk)->allocated_dumpers++;
 	ap_snprintf(number, sizeof(number), "%d", sched(dp)->level);
-	ap_snprintf(chunksize, sizeof(chunksize), "%ld",
-		    (long)sched(dp)->holdp->chunksize);
+	ap_snprintf(chunksize, sizeof(chunksize), "%ld", h[0]->disk->chunksize);
+	ap_snprintf(use, sizeof(use), "%ld", h[0]->reserved );
 	o = optionstr(dp);
 	cmdline = vstralloc(cmdstr[cmd],
 			    " ", disk2serial(dp),
@@ -282,6 +319,7 @@ disk_t *dp;
 			    " ", sched(dp)->dumpdate,
 			    " ", chunksize,
 			    " ", dp->program,
+			    " ", use,
 			    " |", o,
 			    "\n", NULL);
 	amfree(o);
@@ -303,8 +341,29 @@ disk_t *dp;
 	break;
     case QUIT:
     case ABORT:
+	if( dp ) {
+	    cmdline = vstralloc(cmdstr[cmd],
+				" ", sched(dp)->destname,
+				"\n", NULL );
+	} else {
+	    cmdline = stralloc2(cmdstr[cmd], "\n");
+	}
+	break;
     case CONTINUE:
-	cmdline = stralloc2(cmdstr[cmd], "\n");
+	if( dp ) {
+	    holdalloc(h[activehd]->disk)->allocated_dumpers++;
+	    ap_snprintf(chunksize, sizeof(chunksize), "%ld", 
+			h[activehd]->disk->chunksize );
+	    ap_snprintf(use, sizeof(use), "%ld", 
+			h[activehd]->reserved - h[activehd]->used );
+	    cmdline = vstralloc(cmdstr[cmd],
+				" ", h[activehd]->destname,
+				" ", chunksize,
+				" ", use,
+				"\n", NULL );
+	} else {
+	    cmdline = stralloc2(cmdstr[cmd], "\n");
+	}
 	break;
     default:
 	assert(0);
@@ -321,11 +380,15 @@ disk_t *dp;
 	fflush(stdout);
 	for(l = 0, n = strlen(cmdline); l < n; l += s) {
 	    if((s = write(dumper->infd, cmdline + l, n - l)) < 0) {
-		error("writing %s command: %s", dumper->name, strerror(errno));
+		printf("writing %s command: %s\n", dumper->name, 
+		       strerror(errno));
+		fflush(stdout);
+		return 0;
 	    }
 	}
     }
     amfree(cmdline);
+    return 1;
 }
 
 #define MAX_SERIAL MAX_DUMPERS+1	/* one for the taper */
@@ -399,34 +462,54 @@ disk_t *dp;
 }
 
 void update_info_dumper(dp, origsize, dumpsize, dumptime)
-disk_t *dp;
-long origsize;
-long dumpsize;
-long dumptime;
+     disk_t *dp;
+     long origsize;
+     long dumpsize;
+     long dumptime;
 {
-    int level;
-    info_t inf;
+    int level, i;
+    info_t info;
     stats_t *infp;
     perf_t *perfp;
-    int rc;
+    char *conf_infofile;
 
     level = sched(dp)->level;
 
-    rc = open_infofile(getconf_str(CNF_INFOFILE));
-    if(rc)
-	error("could not open infofile %s: %s (%d)", getconf_str(CNF_INFOFILE),
-	      strerror(errno), rc);
+    conf_infofile = getconf_str(CNF_INFOFILE);
+    if (*conf_infofile == '/') {
+	conf_infofile = stralloc(conf_infofile);
+    } else {
+	conf_infofile = stralloc2(config_dir, conf_infofile);
+    }
+    if (open_infofile(conf_infofile)) {
+	error("could not open info db \"%s\"", conf_infofile);
+    }
+    amfree(conf_infofile);
 
-    get_info(dp->host->hostname, dp->name, &inf);
+    get_info(dp->host->hostname, dp->name, &info);
 
-    infp = &inf.inf[level];
+    /* Clean up information about this and higher-level dumps.  This
+       assumes that update_info_dumper() is always run before
+       update_info_taper(). */
+    for (i = level; i < DUMP_LEVELS; ++i) {
+      infp = &info.inf[i];
+      infp->size = -1;
+      infp->csize = -1;
+      infp->secs = -1;
+      infp->date = (time_t)-1;
+      infp->label[0] = '\0';
+      infp->filenum = 0;
+    }
+
+    /* now store information about this dump */
+    infp = &info.inf[level];
     infp->size = origsize;
     infp->csize = dumpsize;
     infp->secs = dumptime;
-/*GS    if(dp->record)*/ infp->date = sched(dp)->timestamp;
+    infp->date = sched(dp)->timestamp;
 
-    if(level == 0) perfp = &inf.full;
-    else perfp = &inf.incr;
+    if(level == 0) perfp = &info.full;
+    else perfp = &info.incr;
 
     /* Update the stats, but only if the new values are meaningful */
     if(dp->compress != COMP_NONE && origsize > 0L) {
@@ -440,44 +523,65 @@ long dumptime;
     }
 
     if(getconf_int(CNF_RESERVE)<100) {
-	inf.command = NO_COMMAND;
+	info.command = NO_COMMAND;
     }
 
-    if(put_info(dp->host->hostname, dp->name, &inf))
+    if(level == info.last_level)
+	info.consecutive_runs++;
+    else {
+	info.last_level = level;
+	info.consecutive_runs = 1;
+    }
+    if(put_info(dp->host->hostname, dp->name, &info))
 	error("infofile update failed (%s,%s)\n", dp->host->hostname, dp->name);
 
     close_infofile();
 }
 
-void update_info_taper(dp, label, filenum)
+void update_info_taper(dp, label, filenum, level)
 disk_t *dp;
 char *label;
 int filenum;
+int level;
 {
-    int level;
-    info_t inf;
+    info_t info;
     stats_t *infp;
     int rc;
 
-    level = sched(dp)->level;
-
     rc = open_infofile(getconf_str(CNF_INFOFILE));
     if(rc)
 	error("could not open infofile %s: %s (%d)", getconf_str(CNF_INFOFILE),
 	      strerror(errno), rc);
 
-    get_info(dp->host->hostname, dp->name, &inf);
+    get_info(dp->host->hostname, dp->name, &info);
 
-    infp = &inf.inf[level];
+    infp = &info.inf[level];
     /* XXX - should we record these two if no-record? */
     strncpy(infp->label, label, sizeof(infp->label)-1);
     infp->label[sizeof(infp->label)-1] = '\0';
     infp->filenum = filenum;
 
-    inf.command = NO_COMMAND;
+    info.command = NO_COMMAND;
 
-    if(put_info(dp->host->hostname, dp->name, &inf))
+    if(put_info(dp->host->hostname, dp->name, &info))
 	error("infofile update failed (%s,%s)\n", dp->host->hostname, dp->name);
 
     close_infofile();
 }
+
+/* Free an array of pointers to assignedhd_t after freeing the
+ * assignedhd_t themselves. The array must be NULL-terminated.
+ */
+void free_assignedhd( ahd )
+assignedhd_t **ahd;
+{
+    int i;
+
+    if( !ahd ) { return; }
+
+    for( i = 0; ahd[i]; i++ ) {
+	amfree(ahd[i]->destname);
+	amfree(ahd[i]);
+    }
+    amfree(ahd);
+}
