@@ -25,12 +25,14 @@
  *			   University of Maryland at College Park
  */
 /*
- * $Id$
+ * $Id: driverio.h,v 1.13.2.8 2001/01/24 04:02:32 martinea Exp $
  *
  * driver-related helper functions
  */
 
-#define MAX_DUMPERS 15
+#include "holding.h"
+
+#define MAX_DUMPERS 63
 #define MAX_ARGS 10
 
 #ifndef GLOBAL
@@ -47,6 +49,13 @@ typedef struct dumper_s {
     disk_t *dp;
 } dumper_t;
 
+typedef struct assignedhd_s {
+    holdingdisk_t	*disk;
+    long		used;
+    long		reserved;
+    char		*destname;
+} assignedhd_t;
+
 /* schedule structure */
 
 typedef struct sched_s {
@@ -56,10 +65,12 @@ typedef struct sched_s {
     unsigned long est_size, degr_size, act_size;
     char *dumpdate, *degr_dumpdate;
     int est_kps, degr_kps;
-    char destname[128];				/* file/port name */
+    char *destname;				/* file/port name */
     dumper_t *dumper;
-    holdingdisk_t *holdp;
+    assignedhd_t **holdp;
     time_t timestamp;
+    int activehd;
+    int no_space;
 } sched_t;
 
 #define sched(dp)	((sched_t *) (dp)->up)
@@ -74,48 +85,34 @@ typedef struct holdalloc_s {
 
 #define holdalloc(hp)	((holdalloc_t *) (hp)->up)
 
-
 GLOBAL dumper_t dmptable[MAX_DUMPERS];
-GLOBAL int inparallel, big_dumpers;
-GLOBAL int degraded_mode;
-GLOBAL unsigned long reserved_space;
 
 /* command/result tokens */
 
 typedef enum {
     BOGUS, QUIT, DONE,
     FILE_DUMP, PORT_DUMP, CONTINUE, ABORT,		/* dumper cmds */
-    FAILED, TRYAGAIN, NO_ROOM, ABORT_FINISHED,		/* dumper results */
+    FAILED, TRYAGAIN, NO_ROOM, RQ_MORE_DISK, ABORT_FINISHED,/* dumper results */
     FATAL_TRYAGAIN,
     START_TAPER, FILE_WRITE, PORT_WRITE,		/* taper cmds */
     PORT, TAPE_ERROR, TAPER_OK,				/* taper results */
     LAST_TOK
 } tok_t;
 
-extern char *cmdstr[];
-
-GLOBAL tok_t tok;
-
 GLOBAL int maxfd;
 GLOBAL fd_set readset;
 GLOBAL int taper, taper_busy, taper_pid;
 
-GLOBAL int argc;
-GLOBAL char *argv[MAX_ARGS+1];
-GLOBAL char *datestamp;
-GLOBAL char *taper_program, *dumper_program;
-
-void addfd P((int fd));
-char *childstr P((int fd));
-void startup_tape_process P((void));
-void startup_dump_process P((dumper_t *dumper));
-void startup_dump_processes P((void));
-tok_t getresult P((int fd, int show));
-void taper_cmd P((tok_t cmd, void *ptr, char *destname, int level, char *datestamp));
-char *optionstr P((disk_t *dp));
-void dumper_cmd P((dumper_t *dumper, tok_t cmd, disk_t *dp));
+void init_driverio();
+void startup_tape_process P((char *taper_program));
+void startup_dump_process P((dumper_t *dumper, char *dumper_program));
+void startup_dump_processes P((char *dumper_program, int inparallel));
+tok_t getresult P((int fd, int show, int *result_argc, char **result_argv, int max_arg));
+int taper_cmd P((tok_t cmd, void *ptr, char *destname, int level, char *datestamp));
+int dumper_cmd P((dumper_t *dumper, tok_t cmd, disk_t *dp));
 disk_t *serial2disk P((char *str));
 void free_serial P((char *str));
 char *disk2serial P((disk_t *dp));
 void update_info_dumper P((disk_t *dp, long origsize, long dumpsize, long dumptime));
-void update_info_taper P((disk_t *dp, char *label, int filenum));
+void update_info_taper P((disk_t *dp, char *label, int filenum, int level));
+void free_assignedhd P((assignedhd_t **holdp));
