@@ -23,7 +23,7 @@
  * Authors: the Amanda Development Team.  Its members are listed in a
  * file named AUTHORS, in the root directory of this distribution.
  */
-/* $Id$
+/* $Id: dumper.c,v 1.75.2.16 2001/03/20 00:21:06 jrjackson Exp $
  *
  * requests remote amandad processes to dump filesystems
  */
@@ -38,6 +38,7 @@
 #include "token.h"
 #include "version.h"
 #include "fileheader.h"
+#include "server_util.h"
 
 #ifdef KRB4_SECURITY
 #include "dumper-krb4.c"
@@ -56,13 +57,7 @@
 #endif
 
 #define CONNECT_TIMEOUT	5*60
-#define READ_TIMEOUT	30*60
-#define MAX_ARGS	10
-/* Note: This must be kept in sync with the DATABUF_SIZE defined in
- * client-src/sendbackup-krb4.c, or kerberos encryption won't work...
- *	- Chris Ross (cross@uu.net)  4-Jun-1998
- */
-#define DATABUF_SIZE	TAPE_BLOCK_BYTES
+#define MAX_ARGS	11
 #define MESGBUF_SIZE	4*1024
 
 #define STARTUP_TIMEOUT 60
@@ -79,15 +74,11 @@ char mesgbuf[MESGBUF_SIZE+1];
 char *errstr = NULL;
 char *dataptr;		/* data buffer markers */
 int spaceleft, abort_pending;
-pid_t pid;
-long dumpsize, origsize;
+long dumpsize, origsize, filesize;
 int nb_header_block;
-times_t runtime;
-double dumptime;	/* Time dump took in secs */
 static enum { srvcomp_none, srvcomp_fast, srvcomp_best } srvcompress;
 
-char *errfname = NULL;
-FILE *errf = NULL;
+static FILE *errf = NULL;
 char *filename = NULL;
 string_t cont_filename;
 char *hostname = NULL;
@@ -96,11 +87,14 @@ char *options = NULL;
 char *progname = NULL;
 int level;
 char *dumpdate = NULL;
-long chunksize;
-extern char *datestamp;
+long chunksize, use;
+char *datestamp;
 char *backup_name = NULL;
 char *recover_cmd = NULL;
 char *compress_suffix = NULL;
+char *config_name = NULL;
+char *config_dir = NULL;
+int conf_dtimeout;
 
 dumpfile_t file;
 int filename_seq;
@@ -110,17 +104,15 @@ int datafd = -1;
 int mesgfd = -1;
 int indexfd = -1;
 int amanda_port;
-int compresspid, indexpid, killerr;
 
 /* local functions */
 int main P((int main_argc, char **main_argv));
 static cmd_t getcmd P((void));
 static void putresult P((char *format, ...))
     __attribute__ ((format (printf, 1, 2)));
-static void do_dump P((int mesgfd, int datafd, int indexfd, int outfd));
+int do_dump P((int mesgfd, int datafd, int indexfd, int outfd));
 void check_options P((char *options));
 void service_ports_init P((void));
-static char *construct_datestamp P((void));
 int write_tapeheader P((int outfd, dumpfile_t *type));
 int write_dataptr P((int outf));
 int update_dataptr P((int outf, int size, int split));
@@ -130,7 +122,7 @@ static void add_msg_data P((char *str, int len));
 static void log_msgout P((logtype_t typ));
 void sendbackup_response P((proto_t *p, pkt_t *pkt));
 int startup_dump P((char *hostname, char *disk, int level, char *dumpdate,
-		    char *dumpname, char *options));
+		    char *progname, char *options));
 
 
 void check_options(options)
@@ -171,19 +163,6 @@ void service_ports_init()
 #endif
 }
 
-static char *construct_datestamp()
-{
-    struct tm *tm;
-    time_t timestamp;
-    char datestamp[3*NUM_STR_SIZE];
-
-    timestamp = time((time_t *)NULL);
-    tm = localtime(&timestamp);
-    ap_snprintf(datestamp, sizeof(datestamp),
-		"%04d%02d%02d", tm->tm_year+1900, tm->tm_mon+1, tm->tm_mday);
-    return stralloc(datestamp);
-}
-
 
 int main(main_argc, main_argv)
 int main_argc;
@@ -194,8 +173,10 @@ char **main_argv;
     dgram_t *msg;
     unsigned long malloc_hist_1, malloc_size_1;
     unsigned long malloc_hist_2, malloc_size_2;
+    char *conffile;
     char *q = NULL;
     int fd;
+    char *tmp_filename = NULL;
 
     for(fd = 3; fd < FD_SETSIZE; fd++) {
 	/*
@@ -214,8 +195,28 @@ char **main_argv;
     erroutput_type = (ERR_AMANDALOG|ERR_INTERACTIVE);
     set_logerror(logerror);
 
-    if(read_conffile(CONFFILE_NAME))
-	error("could not read conf file");
+    if (main_argc > 1) {
+	config_name = stralloc(main_argv[1]);
+	config_dir = vstralloc(CONFIG_DIR, "/", config_name, "/", NULL);
+    } else {
+	char my_cwd[STR_SIZE];
+
+	if (getcwd(my_cwd, sizeof(my_cwd)) == NULL) {
+	    error("cannot determine current working directory");
+	}
+	config_dir = stralloc2(my_cwd, "/");
+	if ((config_name = strrchr(my_cwd, '/')) != NULL) {
+	    config_name = stralloc(config_name + 1);
+	}
+    }
+
+    safe_cd();
+
+    conffile = stralloc2(config_dir, CONFFILE_NAME);
+    if(read_conffile(conffile)) {
+	error("could not find config file \"%s\"", conffile);
+    }
+    amfree(conffile);
 
     /* set up dgram port first thing */
 
@@ -247,10 +248,10 @@ char **main_argv;
     signal(SIGCHLD, SIG_IGN);
 
     interactive = isatty(0);
-    pid = getpid();
 
     amfree(datestamp);
-    datestamp = construct_datestamp();
+    datestamp = construct_datestamp(NULL);
+    conf_dtimeout = getconf_int(CNF_DTIMEOUT);
 
     service_ports_init();
     proto_init(msg->socket, time(0), 16);
@@ -264,10 +265,10 @@ char **main_argv;
 	case FILE_DUMP:
 	    /*
 	     * FILE-DUMP handle filename host disk level dumpdate chunksize
-	     *   progname options
+	     *   progname use options
 	     */
-	    if(argc != 10) {
-		error("error [dumper FILE-DUMP argc != 10: %d]", argc);
+	    if(argc != 11) {
+		error("error [dumper FILE-DUMP argc != 11: %d]", argc);
 	    }
 	    handle = newstralloc(handle, argv[2]);
 	    filename = newstralloc(filename, argv[3]);
@@ -278,17 +279,28 @@ char **main_argv;
 	    chunksize = atoi(argv[8]);
 	    chunksize = (chunksize/TAPE_BLOCK_SIZE)*TAPE_BLOCK_SIZE;
 	    progname = newstralloc(progname, argv[9]);
-	    options = newstralloc(options, argv[10]);
+	    use = atoi(argv[10]);
+	    use = (use/TAPE_BLOCK_SIZE)*TAPE_BLOCK_SIZE;
+	    options = newstralloc(options, argv[11]);
 	    cont_filename[0] = '\0';
 
-	    if((outfd = open(filename, O_WRONLY|O_CREAT, 0666)) == -1) {
-		q = squotef("[holding file \"%s\": %s]",
-			    filename, strerror(errno));
-		putresult("FAILED %s %s\n", handle, q);
+	    tmp_filename = newvstralloc(tmp_filename, filename, ".tmp", NULL);
+	    outfd = open(tmp_filename, O_WRONLY|O_CREAT|O_TRUNC, 0600);
+	    if(outfd == -1) {
+		int save_errno = errno;
+		q = squotef("[main holding file \"%s\": %s]",
+			    tmp_filename, strerror(errno));
+		if(save_errno == ENOSPC) {
+		    putresult("NO-ROOM %s %lu\n", handle, use);
+		    putresult("TRY-AGAIN %s %s\n", handle, q);
+		}
+		else {
+		    putresult("FAILED %s %s\n", handle, q);
+		}
 		amfree(q);
 		break;
 	    }
-	    filename_seq = 0;
+	    filename_seq = 1;
 
 	    check_options(options);
 
@@ -314,8 +326,10 @@ char **main_argv;
 	    }
 
 	    abort_pending = 0;
-	    split_size = chunksize;
-	    do_dump(mesgfd, datafd, indexfd, outfd);
+	    split_size = (chunksize>use)?use:chunksize;
+	    use -= split_size;
+	    if(do_dump(mesgfd, datafd, indexfd, outfd)) {
+	    }
 	    aclose(mesgfd);
 	    aclose(datafd);
 	    if (indexfd != -1)
@@ -345,14 +359,14 @@ char **main_argv;
 	    /* connect outf to taper port */
 
 	    outfd = stream_client("localhost", taper_port,
-				  DATABUF_SIZE, DEFAULT_SIZE);
+				  DATABUF_SIZE*2, DEFAULT_SIZE, NULL);
 	    if(outfd == -1) {
 		q = squotef("[taper port open: %s]", strerror(errno));
 		putresult("FAILED %s %s\n", handle, q);
 		amfree(q);
 		break;
 	    }
-	    filename_seq = 0;
+	    filename_seq = 1;
 
 	    check_options(options);
 
@@ -379,7 +393,8 @@ char **main_argv;
 
 	    abort_pending = 0;
 	    split_size = -1;
-	    do_dump(mesgfd, datafd, indexfd, outfd);
+	    if(do_dump(mesgfd, datafd, indexfd, outfd)) {
+	    }
 	    aclose(mesgfd);
 	    aclose(datafd);
 	    if (indexfd != -1)
@@ -409,6 +424,8 @@ char **main_argv;
     amfree(dumpdate);
     amfree(progname);
     amfree(options);
+    amfree(config_dir);
+    amfree(config_name);
 
     malloc_size_2 = malloc_inuse(&malloc_hist_2);
 
@@ -474,12 +491,10 @@ int outf;
  * written if it is full, or the remainder is zeroed if at eof.
  */
 {
-    cmd_t cmd;
     int written;
 
     do {
-	written = write(outf, databuf + spaceleft,
-	sizeof(databuf) - spaceleft);
+	written = write(outf, databuf + spaceleft, sizeof(databuf) - spaceleft);
 	if(written > 0) {
 	    spaceleft += written;
 	    continue;
@@ -487,18 +502,28 @@ int outf;
 	    errstr = squotef("data write: %s", strerror(errno));
 	    return 1;
 	}
-	putresult("NO-ROOM %s\n", handle);
-	cmd = getcmd();
-	if(cmd != CONTINUE && cmd != ABORT) {
-	    error("error [bad command after NO-ROOM: %d]", cmd);
+	/* Modification by Peter Conrad:
+	 * NO-ROOM is informational only. The file will be truncated
+	 * to the last full TAPE_BLOCK. Later, RQ_MORE_DISK will be
+	 * issued to use another holding disk.
+	 */
+	spaceleft = (spaceleft / TAPE_BLOCK_BYTES) * TAPE_BLOCK_BYTES;
+	ftruncate( outf, (filesize*1024) + spaceleft );
+	if( spaceleft > 0 ) {
+	    memmove( databuf, databuf+spaceleft, sizeof(databuf)-spaceleft );
+	    dumpsize += spaceleft/1024;
+	    filesize += spaceleft/1024;
 	}
-	if(cmd == CONTINUE) continue;
-	abort_pending = 1;
-	errstr = "ERROR";
-	return 1;
+	putresult("NO-ROOM %s %lu\n", handle, use+split_size-dumpsize);
+	use = 0; /* force RQ_MORE_DISK */
+	split_size = dumpsize;
+	dataptr = databuf + spaceleft;
+	spaceleft = sizeof(databuf)-spaceleft;
+	return 0;
     } while (spaceleft != sizeof(databuf));
     dataptr = databuf;
     dumpsize += (sizeof(databuf)/1024);
+    filesize += (sizeof(databuf)/1024);
     return 0;
 }
 
@@ -510,6 +535,8 @@ int outf, size, split;
  * written if it is full, or the remainder is zeroed if at eof.
  */
 {
+    int rc=0;
+
     spaceleft -= size;
     dataptr += size;
 
@@ -519,17 +546,19 @@ int outf, size, split;
 	spaceleft = 0;
     }
 
-    if(spaceleft == 0) {	/* buffer is full, write it */
+    while(spaceleft == 0 && !rc) {	/* buffer is full, write it */
 
 	NAUGHTY_BITS;
 
-	if(split && split_size > 0 && dumpsize >= split_size) {
+	while(split && split_size > 0 && dumpsize >= split_size) {
 	    char *new_filename = NULL;
 	    char sequence[10];
-	    int save_outf;
+	    int new_outf;
 	    int save_spaceleft;
 	    char *save_dataptr;
 	    char save_databuf[DATABUF_SIZE];
+	    char *tmp_filename = NULL;
+
 
 	    memcpy(save_databuf, databuf, sizeof(databuf)); 
 	    save_spaceleft = spaceleft;
@@ -537,64 +566,122 @@ int outf, size, split;
     	    spaceleft = sizeof(databuf);
 	    dataptr = databuf;
 
-	    save_outf = outf;
+	    if( use == 0 ) { /* no more space on this disk. request some more */
+	    cmd_t cmd;
 
-	    close(outf);
-
-	    filename_seq++;
-	    ap_snprintf(sequence, sizeof(sequence), "%d", filename_seq);
-	    new_filename = newvstralloc(new_filename,
-					filename,
-					".",
-					sequence,
-					NULL);
-
-	    strncpy(file.cont_filename, new_filename, 
-		    sizeof(file.cont_filename));
-	    file.cont_filename[sizeof(file.cont_filename)-1] = '\0';
-
-	    if((outf = open(cont_filename,O_RDWR)) == -1) {
-		errstr = squotef("holding file \"%s\": %s",
-			    cont_filename, strerror(errno));
-		return 1;
+                putresult("RQ-MORE-DISK %s\n", handle);
+                cmd = getcmd();
+                if(cmd != CONTINUE && cmd != ABORT) {
+                    error("error [bad command after RQ-MORE-DISK: %d]", cmd);
+                }
+                if(cmd == CONTINUE) {
+		    chunksize = atoi(argv[3]);
+		    chunksize = (chunksize/TAPE_BLOCK_SIZE)*TAPE_BLOCK_SIZE;
+                    use = atoi(argv[4]);
+		    if( !strcmp( filename, argv[2] ) ) { /* same disk */
+			split_size += (chunksize-filesize>use)?use:chunksize-filesize;
+			use = (chunksize-filesize>use)?0:use-(chunksize-filesize);
+			if(chunksize>filesize)
+			    new_filename = newstralloc(new_filename, argv[2]);
+		    } else { /* different disk -> use new file */
+			filename = newstralloc(filename, argv[2]);
+		    }
+                } else {
+                    abort_pending = 1;
+                    errstr = newstralloc(errstr, "ERROR");
+                    return 1;
+                }
 	    }
-	    strncpy(file.cont_filename, new_filename, 
-		    sizeof(file.cont_filename));
-	    file.cont_filename[sizeof(file.cont_filename)-1] = '\0';
-	    write_tapeheader(outf, &file);
-	    close(outf);
 
-	    strncpy(cont_filename, new_filename, sizeof(cont_filename));
-	    cont_filename[sizeof(cont_filename)-1] = '\0';
+	    if( !new_filename ) { /* use another file */
+		int save_type, tmp_outf;
 
-	    if((outf = open(new_filename, O_WRONLY|O_CREAT, 0666)) == -1) {
-		errstr = squotef("holding file \"%s\": %s",
-			    filename, strerror(errno));
-		return 1;
-	    }
-	    if(outf != save_outf) {
-		if(dup2(outf,save_outf) == -1) {
-		    errstr = squotef("can't dup2: %s", strerror(errno));
-		    return 1;
+		ap_snprintf(sequence, sizeof(sequence), "%d", filename_seq);
+		new_filename = newvstralloc(new_filename,
+					    filename,
+					    ".",
+					    sequence,
+					    NULL);
+   
+		tmp_filename = newvstralloc(tmp_filename, new_filename, ".tmp", NULL);
+		new_outf = open(tmp_filename, O_WRONLY|O_CREAT|O_TRUNC, 0600);
+		if(new_outf == -1) {
+		    int save_errno = errno;
+		    errstr = squotef("chunk holding file \"%s\": %s",
+		 		     tmp_filename, strerror(save_errno));
+		    if(save_errno == ENOSPC) {
+			putresult("NO-ROOM %s %lu\n", handle, 
+				  use+split_size-dumpsize);
+			use = 0; /* force RQ_MORE_DISK */
+			split_size = dumpsize;
+		    }
+		    else {
+			close(outf);
+			return 1;
+		    }
 		}
-		close(outf);
-		outf = save_outf;
+		else {
+		    save_type = file.type;
+		    file.type = F_CONT_DUMPFILE;
+		    file.cont_filename[0] = '\0';
+		    if(write_tapeheader(new_outf, &file)) {/*failed-disk full?*/
+			close(new_outf);
+			unlink(tmp_filename);
+			putresult("NO-ROOM %s %lu\n", handle, 
+				  use+split_size-dumpsize);
+			use = 0; /* force RQ_MORE_DISK */
+			split_size = dumpsize;
+		    } else { /* everything is fine */
+ 
+			strncpy(file.cont_filename, new_filename, 
+				sizeof(file.cont_filename));
+			file.cont_filename[sizeof(file.cont_filename)-1] = '\0';
+    
+			tmp_filename = newvstralloc(tmp_filename,
+						   cont_filename, ".tmp", NULL);
+			close(outf);
+			if((tmp_outf = open(tmp_filename,O_RDWR)) == -1) {
+			    close(new_outf);
+			    errstr = squotef("rewrite holding file \"%s\": %s",
+					     tmp_filename, strerror(errno));
+			    return 1;
+			}
+			strncpy(file.cont_filename, new_filename, 
+				sizeof(file.cont_filename));
+			file.cont_filename[sizeof(file.cont_filename)-1] = '\0';
+			file.type = save_type;
+			write_tapeheader(tmp_outf, &file);
+			close(tmp_outf);
+			file.type = F_CONT_DUMPFILE;
+			if(dup2(new_outf,outf) == -1) {
+			    close(new_outf);
+ 			    errstr = squotef("can't dup2: %s", strerror(errno));
+			    return 1;
+			}
+			close(new_outf);
+ 
+			strncpy(cont_filename, new_filename, 
+				sizeof(cont_filename));
+			cont_filename[sizeof(cont_filename)-1] = '\0';
+    
+			split_size += (chunksize>use)?use:chunksize;
+			use = (chunksize>use)?0:use-chunksize;
+			spaceleft = sizeof(databuf);
+			dataptr = databuf;
+			dumpsize += TAPE_BLOCK_SIZE;
+			filesize = TAPE_BLOCK_SIZE;
+			nb_header_block++;
+			filename_seq++;
+		    }
+		}
+		memcpy(databuf, save_databuf, sizeof(databuf)); 
 	    }
-	    /*outf = save_outf;*/
-	    split_size += chunksize;
-    	    spaceleft = sizeof(databuf);
-	    dataptr = databuf;
-	    file.type = F_CONT_DUMPFILE;
-	    file.cont_filename[0] = '\0';
-	    write_tapeheader(outf, &file);
-
-	    memcpy(databuf, save_databuf, sizeof(databuf)); 
 	    spaceleft = save_spaceleft;
 	    dataptr = save_dataptr;
 	}
-	return write_dataptr(outf);
+	rc = write_dataptr(outf);
     }
-    return 0;
+    return rc;
 }
 
 
@@ -609,19 +696,21 @@ static void process_dumpeof()
 {
     /* process any partial line in msgbuf? !!! */
     if(msgbuf != NULL) {
-	fprintf(errf,"? dumper: error [partial line in msgbuf: %ld bytes]\n",
-		(long int) strlen(msgbuf));
-	fprintf(errf,"? dumper: error [partial line in msgbuf: \"%s\"]\n",
-		msgbuf);
+	fprintf(errf,"? %s: error [partial line in msgbuf: %ld bytes]\n",
+		get_pname(), (long) strlen(msgbuf));
+	fprintf(errf,"? %s: error [partial line in msgbuf: \"%s\"]\n",
+		get_pname(), msgbuf);
     }
     if(!got_sizeline && dump_result < 2) {
 	/* make a note if there isn't already a failure */
-	fputs("? dumper: strange [missing size line from sendbackup]\n",errf);
+	fprintf(errf,"? %s: strange [missing size line from sendbackup]\n",
+		get_pname());
 	dump_result = max(dump_result, 1);
     }
 
     if(!got_endline && dump_result < 2) {
-	fputs("? dumper: strange [missing end line from sendbackup]\n",errf);
+	fprintf(errf,"? %s: strange [missing end line from sendbackup]\n",
+		get_pname());
 	dump_result = max(dump_result, 1);
     }
 }
@@ -777,9 +866,8 @@ logtype_t typ;
 {
     char *line = NULL;
 
-    if((errf = fopen(errfname, "r")) == NULL)
-	error("opening msg output: %s", strerror(errno));
-
+    fflush(errf);
+    (void) fseek(errf, 0L, SEEK_SET);
     for(; (line = agets(errf)) != NULL; free(line)) {
 	log_add(typ, "%s", line);
     }
@@ -836,72 +924,47 @@ filetype_t type;
 }
 
 /* Send an Amanda dump header to the output file.
-*/
+ * returns true if an error occured, false on success
+ */
 
 int write_tapeheader(outfd, file)
 int outfd;
 dumpfile_t *file;
 {
     char buffer[TAPE_BLOCK_BYTES];
-#if 0
-    char *bufptr;
-    int len;
-    int count;
-#endif
+    int	 written=0;
 
     write_header(buffer, file, sizeof(buffer));
 
-#if 0
-    /*
-     * NB: This chuck of code had to be removed.  As far as I can tell, the
-     * only use for this code would be if buffer were larger than spaceleft,
-     * which it should never be at this point, as no data has previously been
-     * written (via update_dataptr()).
-     *   Additionally, if this code is left in place on a system using
-     * kerberos encryption, it will break things.  update_dataptr() assumes
-     * that any data passed to it is encrypted if the filesystem was supposed
-     * to be encrypted.  As this buffer is generated on the server side, it
-     * is *not* encrypted, and an attempt to decrypt it in update_dataptr()
-     * will screw the whole thing.
-     *   PLEASE DO NOT ADD THIS CODE BACK IN unless you really understand the
-     * kerberos encryption code...
-     *
-     *                    - Chris Ross (cross@uu.net)   4-Jun-1998
-     */
-
-    bufptr = buffer;
-    for (count = sizeof(buffer); count > 0; ) {
-	len = count > spaceleft ? spaceleft : count;
-	memcpy(dataptr, bufptr, len);
-
-	if (update_dataptr(outfd, len, 0)) return 1;
-
-	bufptr += len;
-	count -= len;
-    }
-    nb_header_block++;
-#else
-    write(outfd, buffer, sizeof(buffer));
-#endif
-    return 0;
+    written = write(outfd, buffer, sizeof(buffer));
+    if(written == sizeof(buffer)) return 0;
+    if(written < 0) return written;
+    errno = ENOSPC;
+    return -1;
 }
 
 
-static void do_dump(mesgfd, datafd, indexfd, outfd)
+int do_dump(mesgfd, datafd, indexfd, outfd)
 int mesgfd, datafd, indexfd, outfd;
 {
     int maxfd, nfound, size1, size2, eof1, eof2;
+    int rc;
     fd_set readset, selectset;
     struct timeval timeout;
     int outpipe[2];
     int header_done;	/* flag - header has been written */
-    char *indexfile = NULL;
+    char *indexfile_tmp = NULL;
+    char *indexfile_real = NULL;
     char level_str[NUM_STR_SIZE];
     char kb_str[NUM_STR_SIZE];
     char kps_str[NUM_STR_SIZE];
     char orig_kb_str[NUM_STR_SIZE];
     char *fn;
     char *q;
+    times_t runtime;
+    double dumptime;	/* Time dump took in secs */
+    int compresspid = -1, indexpid = -1, killerr;
+    char *errfname = NULL;
 
 #ifndef DUMPER_SOCKET_BUFFERING
 #define DUMPER_SOCKET_BUFFERING 0
@@ -915,17 +978,17 @@ int mesgfd, datafd, indexfd, outfd;
 #if DUMPER_SOCKET_BUFFERING
     int lowat = DATABUF_SIZE;
     int recbuf = 0;
-    int lowwatset = 0;
     int sizeof_recbuf = sizeof(recbuf);
+    int lowwatset = 0;
+    int lowwatset_count = 0;
 #endif
 
     startclock();
 
     dataptr = databuf;
     spaceleft = sizeof(databuf);
-    dumpsize = origsize = dump_result = 0;
-    dumpsize = TAPE_BLOCK_SIZE;
-    nb_header_block = 1;
+    dumpsize = origsize = filesize = dump_result = 0;
+    nb_header_block = 0;
     got_info_endline = got_sizeline = got_endline = 0;
     header_done = 0;
     amfree(backup_name);
@@ -935,21 +998,25 @@ int mesgfd, datafd, indexfd, outfd;
     ap_snprintf(level_str, sizeof(level_str), "%d", level);
     fn = sanitise_filename(diskname);
     errfname = newvstralloc(errfname,
-			    "/tmp",
+			    AMANDA_TMPDIR,
 			    "/", hostname,
 			    ".", fn,
 			    ".", level_str,
 			    ".errout",
 			    NULL);
     amfree(fn);
-    if((errf = fopen(errfname, "w")) == NULL) {
+    if((errf = fopen(errfname, "w+")) == NULL) {
 	errstr = newvstralloc(errstr,
 			      "errfile open \"", errfname, "\": ",
 			      strerror(errno),
 			      NULL);
 	amfree(errfname);
+	rc = 2;
 	goto failed;
+    } else {
+	unlink(errfname);			/* so it goes away on close */
     }
+    amfree(errfname);
 
     /* insert pipe in the *READ* side, if server-side compression is desired */
     compresspid = -1;
@@ -964,11 +1031,13 @@ int mesgfd, datafd, indexfd, outfd;
 	    aclose(outpipe[1]);
 	    errstr = newstralloc(errstr, "descriptor out of range");
 	    errno = EMFILE;
+	    rc = 2;
 	    goto failed;
 	}
 	switch(compresspid=fork()) {
 	case -1:
 	    errstr = newstralloc2(errstr, "couldn't fork: ", strerror(errno));
+	    rc = 2;
 	    goto failed;
 	default:
 	    aclose(outpipe[1]);
@@ -998,38 +1067,38 @@ int mesgfd, datafd, indexfd, outfd;
     if (indexfd != -1) {
 	int tmpfd;
 
-	indexfile = vstralloc(getconf_str(CNF_INDEXDIR),
-			      "/",
-			      getindexfname(hostname, diskname,
-					    datestamp, level),
-			      ".tmp",
-			      NULL);
+	indexfile_real = getindexfname(hostname, diskname, datestamp, level),
+	indexfile_tmp = stralloc2(indexfile_real, ".tmp");
 
-	if (mkpdir(indexfile, 0755, (uid_t)-1, (gid_t)-1) == -1) {
+	if (mkpdir(indexfile_tmp, 02755, (uid_t)-1, (gid_t)-1) == -1) {
 	   errstr = newvstralloc(errstr,
 				 "err create ",
-				 indexfile,
+				 indexfile_tmp,
 				 ": ",
 				 strerror(errno),
 				 NULL);
-	   amfree(indexfile);
+	   amfree(indexfile_real);
+	   amfree(indexfile_tmp);
+	   rc = 2;
 	   goto failed;
 	}
 
 	switch(indexpid=fork()) {
 	case -1:
 	    errstr = newstralloc2(errstr, "couldn't fork: ", strerror(errno));
+	    rc = 2;
 	    goto failed;
 	default:
 	    aclose(indexfd);
 	    indexfd = -1;			/* redundant */
 	    break;
 	case 0:
-	    if (dup2(indexfd, 0) == -1)
+	    if (dup2(indexfd, 0) == -1) {
 		error("err dup2 in: %s", strerror(errno));
-	    indexfd = open(indexfile, O_WRONLY | O_CREAT | O_TRUNC, 0600);
+	    }
+	    indexfd = open(indexfile_tmp, O_WRONLY | O_CREAT | O_TRUNC, 0600);
 	    if (indexfd == -1)
-		error("err open %s: %s", indexfile, strerror(errno));
+		error("err open %s: %s", indexfile_tmp, strerror(errno));
 	    if (dup2(indexfd,1) == -1)
 		error("err dup2 out: %s", strerror(errno));
 	    for(tmpfd = 3; tmpfd <= FD_SETSIZE; ++tmpfd) {
@@ -1068,14 +1137,14 @@ int mesgfd, datafd, indexfd, outfd;
 	if (setsockopt(datafd, SOL_SOCKET, SO_RCVBUF,
 		       (void *) &recbuf, sizeof_recbuf)) {
 	    const int errornumber = errno;
-	    fprintf(stderr, "dumper: pid %ld setsockopt(SO_RCVBUF): %s\n",
-		    (long) getpid(), strerror(errornumber));
+	    fprintf(stderr, "%s: pid %ld setsockopt(SO_RCVBUF): %s\n",
+		    get_pname(), (long) getpid(), strerror(errornumber));
 	}
 	if (getsockopt(datafd, SOL_SOCKET, SO_RCVBUF,
 		       (void *) &recbuf, (void *)&sizeof_recbuf)) {
 	    const int errornumber = errno;
-	    fprintf(stderr, "dumper: pid %ld getsockopt(SO_RCVBUF): %s\n",
-		    (long) getpid(), strerror(errornumber));
+	    fprintf(stderr, "%s: pid %ld getsockopt(SO_RCVBUF): %s\n",
+		    get_pname(), (long) getpid(), strerror(errornumber));
 	    recbuf = 0;
 	}
 
@@ -1086,6 +1155,8 @@ int mesgfd, datafd, indexfd, outfd;
 	/* if lowwat < ~512, don't bother */
 	if (lowat < EST_PACKET_SIZE)
 	    recbuf = 0;
+	fprintf(stderr, "%s: pid %ld receive size is %d, low water is %d\n",
+		get_pname(), (long) getpid(), recbuf, lowat);
     }
 #endif
 
@@ -1098,14 +1169,15 @@ int mesgfd, datafd, indexfd, outfd;
 			   (void *) &lowat, sizeof(lowat))) {
 		const int errornumber = errno;
 		fprintf(stderr,
-			"dumper: pid %ld setsockopt(SO_RCVLOWAT): %s\n",
-			(long) getpid(), strerror(errornumber));
+			"%s: pid %ld setsockopt(SO_RCVLOWAT): %s\n",
+			get_pname(), (long) getpid(), strerror(errornumber));
 	    }
 	    lowwatset = 1;
+	    lowwatset_count++;
 	}
 #endif
 
-	timeout.tv_sec = READ_TIMEOUT;
+	timeout.tv_sec = conf_dtimeout;
 	timeout.tv_usec = 0;
 	memcpy(&selectset, &readset, sizeof(fd_set));
 
@@ -1121,13 +1193,13 @@ int mesgfd, datafd, indexfd, outfd;
 			   (void *) &zero, sizeof(zero))) {
 		const int errornumber = errno;
 		fprintf(stderr,
-			"dumper: pid %ld setsockopt(SO_RCVLOWAT): %s\n",
-			(long) getpid(), strerror(errornumber));
+			"%s: pid %ld setsockopt(SO_RCVLOWAT): %s\n",
+			get_pname(), (long) getpid(), strerror(errornumber));
 	    }
 	    lowwatset = 0;
 
 	    /* ... try once more */
-	    timeout.tv_sec = READ_TIMEOUT;
+	    timeout.tv_sec = conf_dtimeout;
 	    timeout.tv_usec = 0;
 	    memcpy(&selectset, &readset, sizeof(fd_set));
 	    nfound = select(maxfd, (SELECT_ARG_TYPE *)(&selectset), NULL, NULL, &timeout);
@@ -1136,10 +1208,12 @@ int mesgfd, datafd, indexfd, outfd;
 
 	if(nfound == 0)  {
 	    errstr = newstralloc(errstr, "data timeout");
+	    rc = 2;
 	    goto failed;
 	}
 	if(nfound == -1) {
 	    errstr = newstralloc2(errstr, "select: ", strerror(errno));
+	    rc = 2;
 	    goto failed;
 	}
 
@@ -1150,15 +1224,22 @@ int mesgfd, datafd, indexfd, outfd;
 	    switch(size1) {
 	    case -1:
 		errstr = newstralloc2(errstr, "data read: ", strerror(errno));
+		rc = 2;
 		goto failed;
 	    case 0:
-		if(update_dataptr(outfd, size1, 1)) goto failed;
+		if(update_dataptr(outfd, size1, 1)) {
+		    rc = 2;
+		    goto failed;
+		}
 		eof1 = 1;
 		FD_CLR(datafd, &readset);
 		aclose(datafd);
 		break;
 	    default:
-		if(update_dataptr(outfd, size1, 1)) goto failed;
+		if(update_dataptr(outfd, size1, 1)) {
+		    rc = 2;
+		    goto failed;
+		}
 	    }
 	}
 
@@ -1167,6 +1248,7 @@ int mesgfd, datafd, indexfd, outfd;
 	    switch(size2) {
 	    case -1:
 		errstr = newstralloc2(errstr, "mesg read: ", strerror(errno));
+		rc = 2;
 		goto failed;
 	    case 0:
 		eof2 = 1;
@@ -1182,10 +1264,24 @@ int mesgfd, datafd, indexfd, outfd;
 	    if (got_info_endline && !header_done) { /* time to do the header */
 		make_tapeheader(&file, F_DUMPFILE);
 		if (write_tapeheader(outfd, &file)) {
+		    int save_errno = errno;
 		    errstr = newstralloc2(errstr, "write_tapeheader: ", 
 					  strerror(errno));
+		    if(save_errno == ENOSPC) {
+			putresult("NO-ROOM %s %lu\n", handle, 
+				  use+split_size-dumpsize);
+			use = 0; /* force RQ_MORE_DISK */
+			split_size = dumpsize;
+			rc = 1;
+		    }
+		    else {
+			rc = 2;
+		    }
 		    goto failed;
 		}
+		dumpsize += TAPE_BLOCK_SIZE;
+		filesize += TAPE_BLOCK_SIZE;
+		nb_header_block++;
 		header_done = 1;
 		strncat(cont_filename,filename,sizeof(cont_filename));
 		cont_filename[sizeof(cont_filename)-1] = '\0';
@@ -1196,7 +1292,17 @@ int mesgfd, datafd, indexfd, outfd;
 	}
     } /* end while */
 
-    if(dump_result > 1) goto failed;
+#if DUMPER_SOCKET_BUFFERING
+    if(lowwatset_count > 1) {
+	fprintf(stderr, "%s: pid %ld low water set %d times\n",
+	        get_pname(), (long) getpid(), lowwatset_count);
+    }
+#endif
+
+    if(dump_result > 1) {
+	rc = 2;
+	goto failed;
+    }
 
     runtime = stopclock();
     dumptime = runtime.r.tv_sec + runtime.r.tv_usec/1000000.0;
@@ -1220,8 +1326,6 @@ int mesgfd, datafd, indexfd, outfd;
 	      (long)(dumptime+0.5), q);
     amfree(q);
 
-    afclose(errf);
-
     switch(dump_result) {
     case 0:
 	log_add(L_SUCCESS, "%s %s %s %d [%s]", hostname, diskname, datestamp, level, errstr);
@@ -1237,37 +1341,38 @@ int mesgfd, datafd, indexfd, outfd;
 	break;
     }
 
-    unlink(errfname);
-    amfree(errfname);
+    if(errf) afclose(errf);
 
-    if (indexfile) {
-	char *tmpname = NULL;
-	int len;
-
-	if((len = strlen(indexfile)) < 4) {
-	    errstr = newstralloc2(errstr, "bad indexfile name: ", indexfile);
-	    goto log_failed;
+    if (indexfile_tmp) {
+	waitpid(indexpid,NULL,0);
+	if(rename(indexfile_tmp, indexfile_real) != 0) {
+	    log_add(L_WARNING, "could not rename \"%s\" to \"%s\": %s",
+		    indexfile_tmp, indexfile_real, strerror(errno));
 	}
-	tmpname = stralloc(indexfile);
-	indexfile[len-4] = '\0';
-	unlink(indexfile);
-	rename(tmpname, indexfile);
-	amfree(tmpname);
-	amfree(indexfile);
+	amfree(indexfile_tmp);
+	amfree(indexfile_real);
     }
 
-    return;
+    return 0;
 
  failed:
 
+#if DUMPER_SOCKET_BUFFERING
+    if(lowwatset_count > 1) {
+	fprintf(stderr, "%s: pid %ld low water set %d times\n",
+	        get_pname(), (long) getpid(), lowwatset_count);
+    }
+#endif
+
     if(!abort_pending) {
 	q = squotef("[%s]", errstr);
-	putresult("FAILED %s %s\n", handle, q);
+	if(rc==2)
+	    putresult("FAILED %s %s\n", handle, q);
+	else
+	    putresult("TRY-AGAIN %s %s\n", handle, q);
 	amfree(q);
     }
 
-    if(errf) afclose(errf);
-
     /* kill all child process */
     if(compresspid != -1) {
 	killerr = kill(compresspid,SIGTERM);
@@ -1293,25 +1398,24 @@ int mesgfd, datafd, indexfd, outfd;
 	}
     }
 
- log_failed:
-
     if(!abort_pending) {
 	log_start_multiline();
 	log_add(L_FAIL, "%s %s %d [%s]", hostname, diskname, level, errstr);
-	if (errfname) {
+	if (errf) {
 	    log_msgout(L_FAIL);
 	}
 	log_end_multiline();
     }
-    if (errfname) {
-	unlink(errfname);
-	amfree(errfname);
+
+    if(errf) afclose(errf);
+
+    if (indexfile_tmp) {
+	unlink(indexfile_tmp);
+	amfree(indexfile_tmp);
+	amfree(indexfile_real);
     }
 
-    if (indexfile)
-	unlink(indexfile);
-
-    return;
+    return rc;
 }
 
 /* -------------------- */
@@ -1351,7 +1455,7 @@ pkt_t *pkt;
 	    errstr = newvstralloc(errstr, "nak error:", s - 1, NULL);
 	    if(errstr[strlen(errstr)-1] == '\n' )
 		errstr[strlen(errstr)-1] = '\0';
-	    response_error = 2;
+	    response_error = 1;
 	    return;
 	}
     }
@@ -1478,7 +1582,7 @@ pkt_t *pkt;
     }
 
     datafd = stream_client(hostname, data_port,
-			   DEFAULT_SIZE, DEFAULT_SIZE);
+			   DEFAULT_SIZE, DEFAULT_SIZE, NULL);
     if(datafd == -1) {
 	errstr = newvstralloc(errstr,
 			      "[could not connect to data port: ",
@@ -1490,7 +1594,7 @@ pkt_t *pkt;
 	return;
     }
     mesgfd = stream_client(hostname, mesg_port,
-			   DEFAULT_SIZE, DEFAULT_SIZE);
+			   DEFAULT_SIZE, DEFAULT_SIZE, NULL);
     if(mesgfd == -1) {
 	errstr = newvstralloc(errstr,
 			      "[could not connect to mesg port: ",
@@ -1506,7 +1610,7 @@ pkt_t *pkt;
 
     if (index_port != -1) {
 	indexfd = stream_client(hostname, index_port,
-				DEFAULT_SIZE, DEFAULT_SIZE);
+				DEFAULT_SIZE, DEFAULT_SIZE, NULL);
 	if (indexfd == -1) {
 	    errstr = newvstralloc(errstr,
 				  "[could not connect to index port: ",
@@ -1554,7 +1658,9 @@ pkt_t *pkt;
 
  request_NAK:
 
-/*  fprintf(stderr, "dumper: got strange NAK: %s", pkt->body); */
+#if 0
+    fprintf(stderr, "%s: got strange NAK: %s\n", get_pname(), pkt->body);
+#endif
     errstr = newstralloc(errstr, "[request NAK]");
     response_error = 2;
     amfree(optionstr);
@@ -1576,8 +1682,8 @@ pkt_t *pkt;
     return;
 }
 
-int startup_dump(hostname, disk, level, dumpdate, dumpname, options)
-char *hostname, *disk, *dumpdate, *dumpname, *options;
+int startup_dump(hostname, disk, level, dumpdate, progname, options)
+char *hostname, *disk, *dumpdate, *progname, *options;
 int level;
 {
     char level_string[NUM_STR_SIZE];
