@@ -25,11 +25,12 @@
  *			   University of Maryland at College Park
  */
 /*
- * $Id$
+ * $Id: find.c,v 1.6.2.4 1999/09/08 23:28:11 jrj Exp $
  *
  * controlling process for the Amanda backup system
  */
 #include "amanda.h"
+#include "conffile.h"
 #include "tapefile.h"
 #include "logfile.h"
 #include "holding.h"
@@ -53,7 +54,7 @@ char *lfind_hostname;
 int lfind_ndisks;
 char **lfind_diskstrs;
 {
-    char *conflogdir, *logfile = NULL;
+    char *conf_logdir, *logfile = NULL;
     int tape, maxtape, seq, logs;
     tape_t *tp;
     find_result_t *output_find = NULL;
@@ -66,7 +67,12 @@ char **lfind_diskstrs;
     else
 	find_nhosts=1;
 
-    conflogdir = getconf_str(CNF_LOGDIR);
+    conf_logdir = getconf_str(CNF_LOGDIR);
+    if (*conf_logdir == '/') {
+	conf_logdir = stralloc(conf_logdir);
+    } else {
+	conf_logdir = stralloc2(config_dir, conf_logdir);
+    }
     maxtape = lookup_nb_tape();
 
     for(tape = 1; tape <= maxtape; tape++) {
@@ -87,7 +93,7 @@ char **lfind_diskstrs;
 
 	    ap_snprintf(seq_str, sizeof(seq_str), "%d", seq);
 	    logfile = newvstralloc(logfile,
-			conflogdir, "/log.", ds_str, ".", seq_str, NULL);
+			conf_logdir, "/log.", ds_str, ".", seq_str, NULL);
 	    if(access(logfile, R_OK) != 0) break;
 	    logs += search_logfile(&output_find, tp->label, tp->datestamp, seq, logfile);
 	}
@@ -95,14 +101,14 @@ char **lfind_diskstrs;
 	/* search old-style amflush log, if any */
 
 	logfile = newvstralloc(logfile,
-			       conflogdir, "/log.", ds_str, ".amflush", NULL);
+			       conf_logdir, "/log.", ds_str, ".amflush", NULL);
 	if(access(logfile,R_OK) == 0) {
 	    logs += search_logfile(&output_find, tp->label, tp->datestamp, 1000, logfile);
 	}
 
 	/* search old-style main log, if any */
 
-	logfile = newvstralloc(logfile, conflogdir, "/log.", ds_str, NULL);
+	logfile = newvstralloc(logfile, conf_logdir, "/log.", ds_str, NULL);
 	if(access(logfile,R_OK) == 0) {
 	    logs += search_logfile(&output_find, tp->label, tp->datestamp, -1, logfile);
 	}
@@ -116,11 +122,89 @@ char **lfind_diskstrs;
     return(output_find);
 }
 
+char **find_log()
+{
+    char *conf_logdir, *logfile = NULL;
+    int tape, maxtape, seq, logs;
+    tape_t *tp;
+    char **output_find_log = NULL;
+    char **current_log;
+
+    conf_logdir = getconf_str(CNF_LOGDIR);
+    if (*conf_logdir == '/') {
+	conf_logdir = stralloc(conf_logdir);
+    } else {
+	conf_logdir = stralloc2(config_dir, conf_logdir);
+    }
+    maxtape = lookup_nb_tape();
+
+    output_find_log = alloc((maxtape*5+10) * sizeof(char *));
+    current_log = output_find_log;
+
+    for(tape = 1; tape <= maxtape; tape++) {
+	char ds_str[NUM_STR_SIZE];
+
+	tp = lookup_tapepos(tape);
+	if(tp == NULL) continue;
+	ap_snprintf(ds_str, sizeof(ds_str), "%d", tp->datestamp);
+
+	/* search log files */
+
+	logs = 0;
+
+	/* new-style log.<date>.<seq> */
+
+	for(seq = 0; 1; seq++) {
+	    char seq_str[NUM_STR_SIZE];
+
+	    ap_snprintf(seq_str, sizeof(seq_str), "%d", seq);
+	    logfile = newvstralloc(logfile,
+			conf_logdir, "/log.", ds_str, ".", seq_str, NULL);
+	    if(access(logfile, R_OK) != 0) break;
+	    if( search_logfile(NULL, tp->label, tp->datestamp, seq, logfile)) {
+		*current_log = vstralloc("log.", ds_str, ".", seq_str, NULL);
+		current_log++;
+		logs++;
+		break;
+	    }
+	}
+
+	/* search old-style amflush log, if any */
+
+	logfile = newvstralloc(logfile,
+			       conf_logdir, "/log.", ds_str, ".amflush", NULL);
+	if(access(logfile,R_OK) == 0) {
+	    if( search_logfile(NULL, tp->label, tp->datestamp, 1000, logfile)) {
+		*current_log = vstralloc("log.", ds_str, ".amflush", NULL);
+		current_log++;
+		logs++;
+	    }
+	}
+
+	/* search old-style main log, if any */
+
+	logfile = newvstralloc(logfile, conf_logdir, "/log.", ds_str, NULL);
+	if(access(logfile,R_OK) == 0) {
+	    if(search_logfile(NULL, tp->label, tp->datestamp, -1, logfile)) {
+		*current_log = vstralloc("log.", ds_str, NULL);
+		current_log++;
+		logs++;
+	    }
+	}
+	if(logs == 0)
+	    printf("Warning: no log files found for tape %s written %s\n",
+		   tp->label, find_nicedate(tp->datestamp));
+    }
+    amfree(logfile);
+    *current_log = NULL;
+    return(output_find_log);
+}
+
 void search_holding_disk(output_find)
 find_result_t **output_find;
 {
     holdingdisk_t *hdisk;
-    struct dirname *dir;
+    holding_t     *holding_list, *dir;
     char *sdirname = NULL;
     char *destname = NULL;
     char *hostname = NULL;
@@ -130,11 +214,10 @@ find_result_t **output_find;
     int level;
     disk_t *dp;
 
-    for(hdisk = holdingdisks; hdisk != NULL; hdisk = hdisk->next)
-	scan_holdingdisk(hdisk->diskdir,0);
+    holding_list = pick_all_datestamp();
 
-    for(hdisk = holdingdisks; hdisk != NULL; hdisk = hdisk->next) {
-	for(dir = dir_list; dir != NULL; dir = dir->next) {
+    for(hdisk = getconf_holdingdisks(); hdisk != NULL; hdisk = hdisk->next) {
+	for(dir = holding_list; dir != NULL; dir = dir->next) {
 	    sdirname = newvstralloc(sdirname,
 				    hdisk->diskdir, "/", dir->name,
 				    NULL);
@@ -142,16 +225,16 @@ find_result_t **output_find;
 	        continue;
 	    }
 
-	    chdir(sdirname);
 	    while((entry = readdir(workdir)) != NULL) {
-		if(strcmp(entry->d_name, ".") == 0
-			  || strcmp(entry->d_name, "..") == 0)
-		    continue;
-		if(is_emptyfile(entry->d_name))
+		if(is_dot_or_dotdot(entry->d_name)) {
 		    continue;
+		}
 		destname = newvstralloc(destname,
 					sdirname, "/", entry->d_name,
 					NULL);
+		if(is_emptyfile(destname)) {
+		    continue;
+		}
 		amfree(hostname);
 		amfree(diskname);
 		if(get_amanda_names(destname, &hostname, &diskname, &level) != F_DUMPFILE) {
@@ -454,6 +537,12 @@ char **label;
     return 1;
 }
 
+/* if output_find is NULL					*/
+/*	return 1 if this is the logfile for this label		*/
+/*	return 0 if this is not the logfile for this label	*/
+/* else								*/
+/*	add to output_find all the dump for this label		*/
+/*	return the number of dump added.			*/
 int search_logfile(output_find, label, datestamp, datestamp_aux, logfile)
 find_result_t **output_find;
 char *label, *logfile;
@@ -487,6 +576,14 @@ int datestamp, datestamp_aux;
 	}
     }
 
+    if(output_find == NULL) {
+	afclose(logf);
+        if(tapematch == 0)
+	    return 0;
+	else
+	    return 1;
+    }
+
     if(tapematch == 0) {
 	afclose(logf);
 	return 0;
