@@ -24,7 +24,7 @@
  * file named AUTHORS, in the root directory of this distribution.
  */
 /*
- * $Id$
+ * $Id: holding.h,v 1.11.2.7 1999/02/26 22:07:27 martinea Exp $
  *
  */
 
@@ -35,25 +35,26 @@
 #include "diskfile.h"
 #include "fileheader.h"
 
-extern struct dirname {
-    struct dirname *next;
+typedef struct holding_s {
+    struct holding_s *next;
     char *name;
-} *dir_list;
+} holding_t;
 
 /* local functions */
 int is_dir P((char *fname));
 int is_emptyfile P((char *fname));
 int is_datestr P((char *fname));
 int non_empty P((char *fname));
-struct dirname *insert_dirname P((char *name));
-char get_letter_from_user P((void));
-/*int select_dir P((void));*/
-void scan_holdingdisk P((char *diskdir,int verbose));
-char **pick_datestamp P((void));
+void free_holding_list P(( holding_t *holding_list));
+holding_t *pick_datestamp P((void));
+holding_t *pick_all_datestamp P((void));
 filetype_t get_amanda_names P((char *fname,
 			       char **hostname,
 			       char **diskname,
 			       int *level));
-int unlink_holding_files P((char *filename));
+void get_dumpfile P((char *fname, dumpfile_t *file));
+long size_holding_files P((char *holding_file));
+int unlink_holding_files P((char *holding_file));
+int rename_tmp_holding P((char *holding_file, int complete));
 
 #endif /* HOLDING_H */
