@@ -25,7 +25,7 @@
  *			   University of Maryland at College Park
  */
 /*
- * $Id$
+ * $Id: infofile.h,v 1.7.4.4 1998/12/03 03:02:45 martinea Exp $
  *
  * interface for current info file reading code
  */
@@ -58,26 +58,29 @@ typedef struct perf_s {
 } perf_t;
 
 typedef struct info_s {
-    int  command;		/* command word */
+    unsigned int  command;		/* command word */
 #	define NO_COMMAND	0	/* no outstanding commands */
-#	define PLANNER_FORCE	1	/* force level 0 tonight */
+#	define FORCE_FULL	1	/* force level 0 at next run */
+#	define FORCE_BUMP	2	/* force bump at next run */
+#	define FORCE_NO_BUMP	4	/* force no-bump at next run */
     perf_t  full;
     perf_t  incr;
     stats_t inf[DUMP_LEVELS];
+    int last_level, consecutive_runs;
 } info_t;
 
 
 int open_infofile P((char *infofile));
 void close_infofile P((void));
 
-char *get_dumpdate P((info_t *record, int level));
+char *get_dumpdate P((info_t *info, int level));
 double perf_average P((float *array, double def));
-int get_info P((char *hostname, char *diskname, info_t *record));
+int get_info P((char *hostname, char *diskname, info_t *info));
 int get_firstkey P((char *hostname, int hostname_size,
 		    char *diskname, int diskname_size));
 int get_nextkey  P((char *hostname, int hostname_size,
 		    char *diskname, int diskname_size));
-int put_info P((char *hostname, char *diskname, info_t *record));
+int put_info P((char *hostname, char *diskname, info_t *info));
 int del_info P((char *hostname, char *diskname));
 
 #endif /* ! INFOFILE_H */
