@@ -24,7 +24,7 @@
  * file named AUTHORS, in the root directory of this distribution.
  */
 /*
- * $Id$
+ * $Id: planner.c,v 1.76.2.19 2001/03/20 00:21:06 jrjackson Exp $
  *
  * backup schedule planner for the Amanda backup system.
  */
@@ -38,6 +38,7 @@
 #include "dgram.h"
 #include "protocol.h"
 #include "version.h"
+#include "server_util.h"
 
 #define MAX_LEVELS		    3	/* max# of estimates per filesys */
 
@@ -47,9 +48,9 @@
 #define DEFAULT_DUMPRATE	 30.0	/* K/s */
 
 /* configuration file stuff */
-char *conf_diskfile;
-char *conf_tapelist;
-char *conf_infofile;
+char *config_name = NULL;
+char *config_dir = NULL;
+
 char *conf_tapetype;
 int conf_runtapes;
 int conf_dumpcycle;
@@ -83,11 +84,11 @@ typedef struct est_s {
 #define est(dp)	((est_t *)(dp)->up)
 
 disklist_t startq, waitq, estq, failq, schedq;
-long total_size, initial_size;
+long total_size;
 double total_lev0, balanced_size, balance_threshold;
 unsigned long tape_length, tape_mark;
-int result_port, result_socket, amanda_port;
-int total_waiting, max_disks;
+int result_port, amanda_port;
+int max_disks;
 
 #ifdef KRB4_SECURITY
 int kamanda_port;
@@ -126,7 +127,6 @@ bilist_t biq;			/* The BI queue itself */
  *
  */
 
-static char *construct_datestamp P((void));  /* subroutines */
 static void setup_estimate P((disk_t *dp));
 static void get_estimates P((void));
 static void analyze_estimate P((disk_t *dp));
@@ -145,7 +145,12 @@ char **argv;
     char *datestamp = NULL, **vp;
     unsigned long malloc_hist_1, malloc_size_1;
     unsigned long malloc_hist_2, malloc_size_2;
+    long initial_size;
     int fd;
+    char *conffile;
+    char *conf_diskfile;
+    char *conf_tapelist;
+    char *conf_infofile;
 
     for(fd = 3; fd < FD_SETSIZE; fd++) {
 	/*
@@ -157,6 +162,23 @@ char **argv;
 	close(fd);
     }
 
+    if (argc > 1) {
+	config_name = stralloc(argv[1]);
+	config_dir = vstralloc(CONFIG_DIR, "/", config_name, "/", NULL);
+    } else {
+	char my_cwd[STR_SIZE];
+
+	if (getcwd(my_cwd, sizeof(my_cwd)) == NULL) {
+	    error("cannot determine current working directory");
+	}
+	config_dir = stralloc2(my_cwd, "/");
+	if ((config_name = strrchr(my_cwd, '/')) != NULL) {
+	    config_name = stralloc(config_name + 1);
+	}
+    }
+
+    safe_cd();
+
     set_pname("planner");
 
     malloc_size_1 = malloc_inuse(&malloc_hist_1);
@@ -183,8 +205,9 @@ char **argv;
 
     msg = dgram_alloc();
 
-    if(dgram_bind(msg, &result_port) == -1)
+    if(dgram_bind(msg, &result_port) == -1) {
 	error("could not bind result datagram port: %s", strerror(errno));
+    }
 
     if(geteuid() == 0) {
 	/* set both real and effective uid's to real uid, likewise for gid */
@@ -198,8 +221,9 @@ char **argv;
      * are a valid user.
      */
 
-    if(getpwuid(getuid()) == NULL)
+    if(getpwuid(getuid()) == NULL) {
 	error("can't get login name for my uid %ld", (long)getuid());
+    }
 
     /*
      * 2. Read in Configuration Information
@@ -209,17 +233,45 @@ char **argv;
 
     fprintf(stderr,"READING CONF FILES...\n");
 
-    if(read_conffile(CONFFILE_NAME))
-	error("could not find \"%s\" in this directory.\n", CONFFILE_NAME);
-
+    conffile = stralloc2(config_dir, CONFFILE_NAME);
+    if(read_conffile(conffile)) {
+	error("could not find config file \"%s\"", conffile);
+    }
+    amfree(conffile);
     conf_diskfile = getconf_str(CNF_DISKFILE);
+    if (*conf_diskfile == '/') {
+	conf_diskfile = stralloc(conf_diskfile);
+    } else {
+	conf_diskfile = stralloc2(config_dir, conf_diskfile);
+    }
+    if((origqp = read_diskfile(conf_diskfile)) == NULL) {
+	error("could not load disklist \"%s\"", conf_diskfile);
+    }
+    amfree(conf_diskfile);
     conf_tapelist = getconf_str(CNF_TAPELIST);
+    if (*conf_tapelist == '/') {
+	conf_tapelist = stralloc(conf_tapelist);
+    } else {
+	conf_tapelist = stralloc2(config_dir, conf_tapelist);
+    }
+    if(read_tapelist(conf_tapelist)) {
+	error("could not load tapelist \"%s\"", conf_tapelist);
+    }
     conf_infofile = getconf_str(CNF_INFOFILE);
+    if (*conf_infofile == '/') {
+	conf_infofile = stralloc(conf_infofile);
+    } else {
+	conf_infofile = stralloc2(config_dir, conf_infofile);
+    }
+    if(open_infofile(conf_infofile)) {
+	error("could not open info db \"%s\"", conf_infofile);
+    }
+    amfree(conf_infofile);
+
     conf_tapetype = getconf_str(CNF_TAPETYPE);
     conf_runtapes = getconf_int(CNF_RUNTAPES);
     conf_dumpcycle = getconf_int(CNF_DUMPCYCLE);
-    conf_runspercycle = !getconf_seen(CNF_RUNSPERCYCLE) ? 0 :
-			getconf_int(CNF_RUNSPERCYCLE);
+    conf_runspercycle = getconf_int(CNF_RUNSPERCYCLE);
     conf_tapecycle = getconf_int(CNF_TAPECYCLE);
     conf_bumpdays = getconf_int(CNF_BUMPDAYS);
     conf_bumpsize = getconf_int(CNF_BUMPSIZE);
@@ -228,19 +280,10 @@ char **argv;
     conf_reserve  = getconf_int(CNF_RESERVE);
 
     amfree(datestamp);
-    datestamp = construct_datestamp();
+    today = time(0);
+    datestamp = construct_datestamp(NULL);
     log_add(L_START, "date %s", datestamp);
 
-    if((origqp = read_diskfile(conf_diskfile)) == NULL)
-	error("could not load \"%s\"\n", conf_diskfile);
-
-    if(read_tapelist(conf_tapelist))
-	error("could not load \"%s\"\n", conf_tapelist);
-
-    if(open_infofile(conf_infofile))
-	error("could not open info db \"%s\"\n", conf_infofile);
-
-
     /* some initializations */
 
     if(conf_runspercycle == 0) {
@@ -426,6 +469,8 @@ char **argv;
 
     amfree(msg);
     amfree(datestamp);
+    amfree(config_dir);
+    amfree(config_name);
 
     malloc_size_2 = malloc_inuse(&malloc_hist_2);
 
@@ -436,18 +481,6 @@ char **argv;
     return 0;
 }
 
-static char *construct_datestamp()
-{
-    struct tm *tm;
-    char datestamp[3*NUM_STR_SIZE];
-
-    today = time((time_t *)NULL);
-    tm = localtime(&today);
-    ap_snprintf(datestamp, sizeof(datestamp),
-		"%04d%02d%02d", tm->tm_year+1900, tm->tm_mon+1, tm->tm_mday);
-    return stralloc(datestamp);
-}
-
 
 
 /*
@@ -456,19 +489,19 @@ static char *construct_datestamp()
  *
  */
 
-static int last_level P((info_t *ip));		  /* subroutines */
+static int last_level P((info_t *info));		  /* subroutines */
 static long est_size P((disk_t *dp, int level));
 static long est_tape_size P((disk_t *dp, int level));
-static int next_level0 P((disk_t *dp, info_t *ip));
-static int runs_at P((info_t *ip, int lev));
+static int next_level0 P((disk_t *dp, info_t *info));
+static int runs_at P((info_t *info, int lev));
 static long bump_thresh P((int level));
 static int when_overwrite P((char *label));
 
-static void askfor(ep, seq, lev, inf)
+static void askfor(ep, seq, lev, info)
 est_t *ep;	/* esimate data block */
 int seq;	/* sequence number of request */
 int lev;	/* dump level being requested */
-info_t *inf;	/* info block for disk */
+info_t *info;	/* info block for disk */
 {
     stats_t *stat;
 
@@ -490,21 +523,22 @@ info_t *inf;	/* info block for disk */
 
     ep->level[seq] = lev;
 
-    ep->dumpdate[seq] = stralloc(get_dumpdate(inf,lev));
+    ep->dumpdate[seq] = stralloc(get_dumpdate(info,lev));
     malloc_mark(ep->dumpdate[seq]);
 
-    stat = &inf->inf[lev];
+    stat = &info->inf[lev];
     if(stat->date == EPOCH) ep->est_size[seq] = -1;
     else ep->est_size[seq] = stat->size;
 
     return;
 }
 
-static void setup_estimate(dp)
-disk_t *dp;
+static void
+setup_estimate(dp)
+     disk_t *dp;
 {
     est_t *ep;
-    info_t inf;
+    info_t info;
     int i;
 
     assert(dp && dp->host);
@@ -512,7 +546,7 @@ disk_t *dp;
 
     /* get current information about disk */
 
-    if(get_info(dp->host->hostname, dp->name, &inf)) {
+    if(get_info(dp->host->hostname, dp->name, &info)) {
 	/* no record for this disk, make a note of it */
 	log_add(L_INFO, "Adding new disk %s:%s.", dp->host->hostname, dp->name);
     }
@@ -528,7 +562,7 @@ disk_t *dp;
 
     /* calculated fields */
 
-    if(inf.command == PLANNER_FORCE) {
+    if(info.command & FORCE_FULL) {
 	/* force a level 0, kind of like a new disk */
 	if(dp->strategy == DS_NOFULL) {
 	    /*
@@ -547,13 +581,13 @@ disk_t *dp;
 		    dp->host->hostname, dp->name);
 
 	    /* clear force command */
-	    if(inf.command == PLANNER_FORCE)
-		inf.command = NO_COMMAND;
-	    if(put_info(dp->host->hostname, dp->name, &inf))
+	    if(info.command & FORCE_FULL)
+		info.command ^= FORCE_FULL;
+	    if(put_info(dp->host->hostname, dp->name, &info))
 		error("could not put info record for %s:%s: %s",
 		      dp->host->hostname, dp->name, strerror(errno));
-	    ep->last_level = last_level(&inf);
-	    ep->next_level0 = next_level0(dp, &inf);
+	    ep->last_level = last_level(&info);
+	    ep->next_level0 = next_level0(dp, &info);
 	}
 	else {
 	    ep->last_level = -1;
@@ -565,11 +599,11 @@ disk_t *dp;
     else if(dp->strategy == DS_NOFULL) {
 	/* force estimate of level 1 */
 	ep->last_level = 1;
-	ep->next_level0 = next_level0(dp, &inf);
+	ep->next_level0 = next_level0(dp, &info);
     }
     else {
-	ep->last_level = last_level(&inf);
-	ep->next_level0 = next_level0(dp, &inf);
+	ep->last_level = last_level(&info);
+	ep->next_level0 = next_level0(dp, &info);
     }
 
     /* adjust priority levels */
@@ -581,21 +615,21 @@ disk_t *dp;
 	ep->dump_priority -= ep->next_level0;
 	/* warn if dump will be overwritten */
 	if(ep->last_level > -1) {
-	    int overwrite_runs = when_overwrite(inf.inf[0].label);
+	    int overwrite_runs = when_overwrite(info.inf[0].label);
 	    if(overwrite_runs == 0) {
 		log_add(L_WARNING,
 		  "Last full dump of %s:%s on tape %s overwritten on this run.",
-		        dp->host->hostname, dp->name, inf.inf[0].label);
+		        dp->host->hostname, dp->name, info.inf[0].label);
 	    }
 	    else if(overwrite_runs < RUNS_REDZONE) {
 		log_add(L_WARNING,
 		  "Last full dump of %s:%s on tape %s overwritten in %d run%s.",
-		        dp->host->hostname, dp->name, inf.inf[0].label,
+		        dp->host->hostname, dp->name, info.inf[0].label,
 		    overwrite_runs, overwrite_runs == 1? "" : "s");
 	    }
 	}
     }
-    else if(inf.command == PLANNER_FORCE)
+    else if(info.command & FORCE_FULL)
 	ep->dump_priority += 1;
     /* else XXX bump up the priority of incrementals that failed last night */
 
@@ -604,12 +638,12 @@ disk_t *dp;
     if(dp->skip_full) {
 	if(ep->next_level0 <= 0) {
 	    /* update the date field */
-	    inf.inf[0].date = today;
-	    if(inf.command == PLANNER_FORCE)
-		inf.command = NO_COMMAND;
+	    info.inf[0].date = today;
+	    if(info.command & FORCE_FULL)
+		info.command ^= FORCE_FULL;
 	    ep->next_level0 += conf_dumpcycle;
 	    ep->last_level = 0;
-	    if(put_info(dp->host->hostname, dp->name, &inf))
+	    if(put_info(dp->host->hostname, dp->name, &info))
 		error("could not put info record for %s:%s: %s",
 		      dp->host->hostname, dp->name, strerror(errno));
 	    log_add(L_INFO, "Skipping full dump of %s:%s today.",
@@ -617,9 +651,9 @@ disk_t *dp;
 	    fprintf(stderr,"%s:%s lev 0 skipped due to skip-full flag\n",
 		    dp->host->hostname, dp->name);
 	    /* don't enqueue the disk */
-	    askfor(ep, 0, -1, &inf);
-	    askfor(ep, 1, -1, &inf);
-	    askfor(ep, 2, -1, &inf);
+	    askfor(ep, 0, -1, &info);
+	    askfor(ep, 1, -1, &info);
+	    askfor(ep, 2, -1, &info);
 	    fprintf(stderr, "planner: SKIPPED %s %s 0 [skip-full]\n",
 		    dp->host->hostname, dp->name);
 	    log_add(L_SUCCESS, "%s %s 0 [skipped: skip-full]",
@@ -633,7 +667,7 @@ disk_t *dp;
 	}
 
 	if(ep->next_level0 == 1) {
-	    log_add(L_WARNING, "Skipping full dump of %s:%s tommorrow.",
+	    log_add(L_WARNING, "Skipping full dump of %s:%s tomorrow.",
 		    dp->host->hostname, dp->name);
 	}
     }
@@ -644,9 +678,9 @@ disk_t *dp;
 	fprintf(stderr,"%s:%s lev 1 skipped due to skip-incr flag\n",
 		dp->host->hostname, dp->name);
 	/* don't enqueue the disk */
-	askfor(ep, 0, -1, &inf);
-	askfor(ep, 1, -1, &inf);
-	askfor(ep, 2, -1, &inf);
+	askfor(ep, 0, -1, &info);
+	askfor(ep, 1, -1, &info);
+	askfor(ep, 2, -1, &info);
 
 	fprintf(stderr, "planner: SKIPPED %s %s 1 [skip-incr]\n",
 		dp->host->hostname, dp->name);
@@ -657,7 +691,8 @@ disk_t *dp;
     }
 
     if( ep->last_level == -1 && ep->next_level0 > 0 && 
-	dp->strategy != DS_NOFULL && conf_reserve == 100) {
+	dp->strategy != DS_NOFULL && dp->strategy != DS_INCRONLY &&
+	conf_reserve == 100) {
 	log_add(L_WARNING,
 	     "%s:%s mismatch: no tapelist record, but curinfo next_level0: %d.",
 	        dp->host->hostname, dp->name, ep->next_level0);
@@ -665,26 +700,46 @@ disk_t *dp;
     }
 
     if(ep->last_level == 0) ep->level_days = 0;
-    else ep->level_days = runs_at(&inf, ep->last_level);
-    ep->last_lev0size = inf.inf[0].csize;
+    else ep->level_days = runs_at(&info, ep->last_level);
+    ep->last_lev0size = info.inf[0].csize;
 
-    ep->fullrate = perf_average(inf.full.rate, 0.0);
-    ep->incrrate = perf_average(inf.incr.rate, 0.0);
+    ep->fullrate = perf_average(info.full.rate, 0.0);
+    ep->incrrate = perf_average(info.incr.rate, 0.0);
 
-    ep->fullcomp = perf_average(inf.full.comp, dp->comprate[0]);
-    ep->incrcomp = perf_average(inf.incr.comp, dp->comprate[1]);
+    ep->fullcomp = perf_average(info.full.comp, dp->comprate[0]);
+    ep->incrcomp = perf_average(info.incr.comp, dp->comprate[1]);
 
     /* determine which estimates to get */
 
     i = 0;
 
-    if(!(dp->skip_full || dp->strategy == DS_NOFULL))
-	askfor(ep, i++, 0, &inf);
+    if(!dp->skip_full &&
+       (!(info.command & FORCE_BUMP) || dp->skip_incr || ep->last_level == -1)){
+	if(info.command & FORCE_BUMP && ep->last_level == -1) {
+	    log_add(L_INFO,
+		  "Remove force-bump command of %s:%s because it's a new disk.",
+		    dp->host->hostname, dp->name);
+	}
+	switch (dp->strategy) {
+	case DS_STANDARD: 
+	case DS_NOINC:
+	    askfor(ep, i++, 0, &info);
+	    break;
+
+	case DS_NOFULL:
+	    break;
+
+	case DS_INCRONLY:
+	    if (info.command & FORCE_FULL)
+		askfor(ep, i++, 0, &info);
+	    break;
+	}
+    }
 
     if(!dp->skip_incr) {
 	if(ep->last_level == -1) {		/* a new disk */
-	    if(dp->strategy == DS_NOFULL) {
-		askfor(ep, i++, 1, &inf);
+	    if(dp->strategy == DS_NOFULL || dp->strategy == DS_INCRONLY) {
+		askfor(ep, i++, 1, &info);
 	    } else {
 		assert(!dp->skip_full);		/* should be handled above */
 	    }
@@ -692,10 +747,24 @@ disk_t *dp;
 	    int curr_level;
 
 	    curr_level = ep->last_level;
-	    if(curr_level == 0)
-		askfor(ep, i++, 1, &inf);
+
+	    if(info.command & FORCE_NO_BUMP) {
+		if(curr_level > 0) { /* level 0 already asked for */
+		    askfor(ep, i++, curr_level, &info);
+		}
+		log_add(L_INFO,"Preventing bump of %s:%s as directed.",
+			dp->host->hostname, dp->name);
+	    }
+	    else if(info.command & FORCE_BUMP) {
+		askfor(ep, i++, curr_level+1, &info);
+		log_add(L_INFO,"Bumping of %s:%s at level %d as directed.",
+			dp->host->hostname, dp->name, curr_level+1);
+	    }
+	    else if(curr_level == 0) {
+		askfor(ep, i++, 1, &info);
+	    }
 	    else {
-		askfor(ep, i++, curr_level, &inf);
+		askfor(ep, i++, curr_level, &info);
 		/*
 		 * If last time we dumped less than the threshold, then this
 		 * time we will too, OR the extra size will be charged to both
@@ -703,23 +772,24 @@ disk_t *dp;
 		 * if we haven't been at this level 2 days, or the dump failed
 		 * last night, we can't bump.
 		 */
-		if((inf.inf[curr_level].size == 0 || /* no data, try it anyway */
-		    (((inf.inf[curr_level].size > bump_thresh(curr_level)))
+		if((info.inf[curr_level].size == 0 || /* no data, try it anyway */
+		    (((info.inf[curr_level].size > bump_thresh(curr_level)))
 		     && ep->level_days >= conf_bumpdays))) {
-		    askfor(ep, i++, curr_level+1, &inf);
+		    askfor(ep, i++, curr_level+1, &info);
 		}
 	    } 
 	}
     }
 
     while(i < MAX_LEVELS) 	/* mark end of estimates */
-	askfor(ep, i++, -1, &inf);
+	askfor(ep, i++, -1, &info);
 
     /* debug output */
 
     fprintf(stderr, "setup_estimate: %s:%s: command %d, options:",
-	    dp->host->hostname, dp->name, inf.command);
+	    dp->host->hostname, dp->name, info.command);
     if(dp->strategy == DS_NOFULL) fputs(" no-full", stderr);
+    if(dp->strategy == DS_INCRONLY) fputs(" incr-only", stderr);
     if(dp->skip_full) fputs(" skip-full", stderr);
     if(dp->skip_incr) fputs(" skip-incr", stderr);
     fprintf(stderr, "\n    last_level %d next_level0 %d level_days %d\n",
@@ -740,6 +810,8 @@ char *label;
 
     if((tp = lookup_tapelabel(label)) == NULL)
 	return 1;	/* "shouldn't happen", but trigger warning message */
+    else if(!reusable_tape(tp))
+	return 1024;
     else if(lookup_nb_tape() > conf_tapecycle)
 	return (lookup_nb_tape() - tp->position) / conf_runtapes;
     else
@@ -794,29 +866,35 @@ int level;
 
 
 /* what was the level of the last successful dump to tape? */
-static int last_level(ip)
-info_t *ip;
+static int last_level(info)
+info_t *info;
 {
     int min_pos, min_level, i;
     time_t lev0_date, last_date;
     tape_t *tp;
 
+    if(info->last_level != -1)
+	return info->last_level;
+
+    /* to keep compatibility with old infofile */
     min_pos = 1000000000;
     min_level = -1;
     lev0_date = EPOCH;
     last_date = EPOCH;
     for(i = 0; i < 9; i++) {
 	if(conf_reserve < 100) {
-	    if(i == 0) lev0_date = ip->inf[0].date;
-	    else if(ip->inf[i].date < lev0_date) continue;
-	    if(ip->inf[i].date > last_date)
+	    if(i == 0) lev0_date = info->inf[0].date;
+	    else if(info->inf[i].date < lev0_date) continue;
+	    if(info->inf[i].date > last_date) {
+		last_date = info->inf[i].date;
 		min_level = i;
+	    }
 	}
 	else {
-	    if((tp = lookup_tapelabel(ip->inf[i].label)) == NULL) continue;
+	    if((tp = lookup_tapelabel(info->inf[i].label)) == NULL) continue;
 	    /* cull any entries from previous cycles */
-	    if(i == 0) lev0_date = ip->inf[0].date;
-	    else if(ip->inf[i].date < lev0_date) continue;
+	    if(i == 0) lev0_date = info->inf[0].date;
+	    else if(info->inf[i].date < lev0_date) continue;
 
 	    if(tp->position < min_pos) {
 		min_pos = tp->position;
@@ -824,39 +902,50 @@ info_t *ip;
 	    }
 	}
     }
+    info->last_level = i;
     return min_level;
 }
 
-/* when is next level 0 due? 0 = today, 1 = tommorrow, etc*/
-static int next_level0(dp, ip)
-disk_t *dp;
-info_t *ip;
+/* when is next level 0 due? 0 = today, 1 = tomorrow, etc*/
+static int
+next_level0(dp, info)
+     disk_t *dp;
+     info_t *info;
 {
-    if(dp->strategy == DS_NOFULL)
+    if(dp->strategy == DS_NOFULL || dp->strategy == DS_INCRONLY)
 	return 1;		/* fake it */
-    else if(ip->inf[0].date < (time_t)0)
+    else if (dp->strategy == DS_NOINC)
+	return 0;
+    else if(info->inf[0].date < (time_t)0)
 	return -days_diff(EPOCH, today);	/* new disk */
     else
-	return dp->dumpcycle - days_diff(ip->inf[0].date, today);
+	return dp->dumpcycle - days_diff(info->inf[0].date, today);
 }
 
 /* how many runs at current level? */
-static int runs_at(ip, lev)
-info_t *ip;
+static int runs_at(info, lev)
+info_t *info;
 int lev;
 {
     tape_t *cur_tape, *old_tape;
-    int last;
+    int last, nb_runs;
 
-    last = last_level(ip);
+    last = last_level(info);
     if(lev != last) return 0;
     if(lev == 0) return 1;
 
-    cur_tape = lookup_tapelabel(ip->inf[lev].label);
-    old_tape = lookup_tapelabel(ip->inf[lev-1].label);
+    if(info->consecutive_runs != -1)
+	return info->consecutive_runs;
+
+    /* to keep compatibility with old infofile */
+    cur_tape = lookup_tapelabel(info->inf[lev].label);
+    old_tape = lookup_tapelabel(info->inf[lev-1].label);
     if(cur_tape == NULL || old_tape == NULL) return 0;
 
-    return (old_tape->position - cur_tape->position) / conf_runtapes;
+    nb_runs = (old_tape->position - cur_tape->position) / conf_runtapes;
+    info->consecutive_runs = nb_runs;
+
+    return nb_runs;
 }
 
 
@@ -1028,7 +1117,7 @@ pkt_t *pkt;
     disk_t *dp;
     host_t *hostp;
     char *resp;
-    char *msgdisk=NULL, *msgdisk_undo=NULL, msgdisk_undo_ch;
+    char *msgdisk=NULL, *msgdisk_undo=NULL, msgdisk_undo_ch = '\0';
     char *remoterr, *errbuf = NULL;
     char *s;
     int ch;
@@ -1235,6 +1324,8 @@ static void analyze_estimate(dp)
 disk_t *dp;
 {
     est_t *ep;
+    info_t info;
+    int have_info = 0;
 
     ep = est(dp);
 
@@ -1243,22 +1334,37 @@ disk_t *dp;
     fprintf(stderr, "next_level0 %d last_level %d ",
 	    ep->next_level0, ep->last_level);
 
-    if(ep->next_level0 <= 0) {
-	fprintf(stderr,"(due for level 0) ");
+    if(get_info(dp->host->hostname, dp->name, &info) == 0) {
+	have_info = 1;
+    }
+
+    if(ep->next_level0 <= 0
+       || (have_info && ep->last_level == 0 && (info.command & FORCE_NO_BUMP))) {
+	if(ep->next_level0 <= 0) {
+	    fprintf(stderr,"(due for level 0) ");
+	}
 	ep->dump_level = 0;
 	ep->dump_size = est_tape_size(dp, 0);
-	total_lev0 += (double) ep->dump_size;
-	if(ep->last_level == -1 || dp->skip_incr) {
-	    fprintf(stderr,"(%s disk, can't switch to degraded mode)\n",
-		    dp->skip_incr? "skip-incr":"new");
-	    ep->degr_level = -1;
-	    ep->degr_size = -1;
+	if(ep->dump_size <= 0) {
+	    fprintf(stderr,
+		    "(no estimate for level 0, picking an incr level)\n");
+	    ep->dump_level = pick_inclevel(dp);
+	    ep->dump_size = est_tape_size(dp, ep->dump_level);
 	}
 	else {
-	    /* fill in degraded mode info */
-	    fprintf(stderr,"(picking inclevel for degraded mode)\n");
-	    ep->degr_level = pick_inclevel(dp);
-	    ep->degr_size = est_tape_size(dp, ep->degr_level);
+	    total_lev0 += (double) ep->dump_size;
+	    if(ep->last_level == -1 || dp->skip_incr) {
+		fprintf(stderr,"(%s disk, can't switch to degraded mode)\n",
+			dp->skip_incr? "skip-incr":"new");
+		ep->degr_level = -1;
+		ep->degr_size = -1;
+	    }
+	    else {
+		/* fill in degraded mode info */
+		fprintf(stderr,"(picking inclevel for degraded mode)\n");
+		ep->degr_level = pick_inclevel(dp);
+		ep->degr_size = est_tape_size(dp, ep->degr_level);
+	    }
 	}
     }
     else {
@@ -1275,7 +1381,8 @@ disk_t *dp;
     total_size += TAPE_BLOCK_SIZE + ep->dump_size + tape_mark;
 
     /* update the balanced size */
-    if(!(dp->skip_full || dp->strategy == DS_NOFULL)) {
+    if(!(dp->skip_full || dp->strategy == DS_NOFULL || 
+	 dp->strategy == DS_INCRONLY)) {
 	long lev0size;
 
 	lev0size = est_tape_size(dp, 0);
@@ -1372,8 +1479,11 @@ disk_t *dp;
 
     /* if we didn't get an estimate, we can't do an inc */
     if(base_size == -1) {
+	base_size = est_size(dp, base_level+1);
+	if(base_size > 0) /* FORCE_BUMP */
+	    return base_level+1;
 	fprintf(stderr,"   picklev: no estimate for level %d, so no incs\n", base_level);
-	return -1;
+	return base_level;
     }
 
     thresh = bump_thresh(base_level);
@@ -1732,16 +1842,16 @@ static int promote_highest_priority_incremental P((void))
 		check_days, (check_days == 1) ? "" : "s");
 
 	for(dp = schedq.head; dp != NULL; dp = dp->next) {
-	    if(dp->skip_full || dp->strategy == DS_NOFULL) {
-		fprintf(stderr,
-	"    promote: can't move %s:%s: no full dumps allowed.\n",
-			dp->host->hostname, dp->name);
-		continue;
-	    }
-
 	    if(est(dp)->next_level0 != check_days)
 		continue; /* totals continue here too */
 
+	    if(est_size(dp,0) <= 0) {
+		fprintf(stderr,
+		    "    promote: can't move %s:%s: no full dumps allowed.\n",
+		    dp->host->hostname, dp->name);
+		continue;
+	    }
+
 	    new_size = est_tape_size(dp, 0);
 	    new_total = total_size - est(dp)->dump_size + new_size;
 	    new_lev0 = total_lev0 + new_size;
@@ -1791,7 +1901,7 @@ static int promote_hills P((void))
     } *sp = NULL;
     int tapecycle;
     int days;
-    int hill_days;
+    int hill_days = 0;
     long hill_size;
     long new_size;
     long new_total;
@@ -1811,7 +1921,8 @@ static int promote_hills P((void))
 
     for(dp = schedq.head; dp != NULL; dp = dp->next) {
 	days = est(dp)->next_level0;   /* This is > 0 by definition */
-	if(days<tapecycle && !dp->skip_full && dp->strategy != DS_NOFULL) {
+	if(days<tapecycle && !dp->skip_full && dp->strategy != DS_NOFULL &&
+	   dp->strategy != DS_INCRONLY) {
 	    sp[days].disks++;
 	    sp[days].size += est(dp)->last_lev0size;
 	}
@@ -1834,7 +1945,8 @@ static int promote_hills P((void))
 	for(dp = schedq.head; dp != NULL; dp = dp->next) {
 	    if(est(dp)->next_level0 != hill_days ||
 	       dp->skip_full ||
-	       dp->strategy == DS_NOFULL)
+	       dp->strategy == DS_NOFULL ||
+	       dp->strategy == DS_INCRONLY)
 		continue;
 	    new_size = est_tape_size(dp, 0);
 	    new_total = total_size - est(dp)->dump_size + new_size;
@@ -1877,10 +1989,10 @@ static int promote_hills P((void))
  * XXX - memory leak - we shouldn't just throw away *dp
  */
 static void output_scheduleline(dp)
-disk_t *dp;
+    disk_t *dp;
 {
     est_t *ep;
-    long dump_time, degr_time;
+    long dump_time = 0, degr_time = 0;
     char *schedline = NULL, *degr_str = NULL;
     char dump_priority_str[NUM_STR_SIZE];
     char dump_level_str[NUM_STR_SIZE];
@@ -1897,9 +2009,9 @@ disk_t *dp;
     if(ep->dump_size == -1) {
 	/* no estimate, fail the disk */
 	fprintf(stderr,
-		"planner: FAILED %s %s %d [no estimate or historical data]\n",
+		"planner: FAILED %s %s %d [no estimate]\n",
 		dp->host->hostname, dp->name, ep->dump_level);
-	log_add(L_FAIL, "%s %s %d [no estimate or historical data]",
+	log_add(L_FAIL, "%s %s %d [no estimate]",
 	        dp->host->hostname, dp->name, ep->dump_level);
 	return;
     }
@@ -1917,7 +2029,7 @@ disk_t *dp;
     if(ep->dump_level == 0) {
 	dump_time = ep->dump_size / fix_rate(ep->fullrate);
 
-	if(ep->degr_level != -1) {
+	if(ep->degr_size != -1) {
 	    degr_time = ep->degr_size / fix_rate(ep->incrrate);
 	}
     }
@@ -1925,7 +2037,7 @@ disk_t *dp;
 	dump_time = ep->dump_size / fix_rate(ep->incrrate);
     }
 
-    if(ep->dump_level == 0 && ep->degr_level != -1) {
+    if(ep->dump_level == 0 && ep->degr_size != -1) {
 	ap_snprintf(degr_level_str, sizeof(degr_level_str),
 		    "%d", ep->degr_level);
 	ap_snprintf(degr_size_str, sizeof(degr_size_str),
