@@ -24,7 +24,7 @@
  * file named AUTHORS, in the root directory of this distribution.
  */
 /*
- * $Id$
+ * $Id: tapefile.c,v 1.15.2.6 1999/09/08 23:28:29 jrj Exp $
  *
  * routines to read and write the amanda active tape list
  */
@@ -138,7 +138,7 @@ int datestamp;
 int lookup_nb_tape()
 {
     tape_t *tp;
-    int pos;
+    int pos=0;
 
     for(tp = tape_list; tp != NULL; tp = tp->next) {
 	pos=tp->position;
@@ -147,12 +147,13 @@ int lookup_nb_tape()
 }
 
 tape_t *lookup_last_reusable_tape(skip)
-int skip;
+     int skip;
 {
     tape_t *tp, **tpsave;
     int count=0;
     int s;
     int tapecycle = getconf_int(CNF_TAPECYCLE);
+    char *labelstr = getconf_str (CNF_LABELSTR);
 
     /*
      * The idea here is we keep the last "several" reusable tapes we
@@ -165,7 +166,7 @@ int skip;
 	tpsave[s] = NULL;
     }
     for(tp = tape_list; tp != NULL; tp = tp->next) {
-	if(tp->reuse == 1) {
+	if(tp->reuse == 1 && match (labelstr, tp->label)) {
 	    count++;
 	    for(s = skip; s > 0; s--) {
 	        tpsave[s] = tpsave[s - 1];
@@ -182,12 +183,13 @@ int skip;
 }
 
 int reusable_tape(tp)
-tape_t *tp;
+    tape_t *tp;
 {
     int count = 0;
 
     if(tp == NULL) return 0;
     if(tp->reuse == 0) return 0;
+    if(tp->datestamp == 0) return 1;
     while(tp != NULL) {
 	if(tp->reuse == 1) count++;
 	tp = tp->prev;
@@ -294,7 +296,6 @@ char *line;
     int ch;
 
     tp = (tape_t *) alloc(sizeof(tape_t));
-    malloc_mark(tp);
 
     tp->prev = NULL;
     tp->next = NULL;
@@ -314,7 +315,6 @@ char *line;
     skip_non_whitespace(s, ch);
     s[-1] = '\0';
     tp->label = stralloc(s1);
-    malloc_mark(tp->label);
     skip_whitespace(s, ch);
     tp->reuse = 1;
 #define sc "reuse"
