@@ -1,6 +1,6 @@
 /*
  * Amanda, The Advanced Maryland Automatic Network Disk Archiver
- * Copyright (c) 1991-1998 University of Maryland at College Park
+ * Copyright (c) 1991-1998, 2000 University of Maryland at College Park
  * All Rights Reserved.
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
@@ -23,7 +23,7 @@
  * Authors: the Amanda Development Team.  Its members are listed in a
  * file named AUTHORS, in the root directory of this distribution.
  */
-/* $Id$
+/* $Id: taper.c,v 1.47.2.16 2001/03/09 19:28:24 jrjackson Exp $
  *
  * moves files from holding disk to tape, or from a socket to tape
  */
@@ -65,11 +65,16 @@ static vtbl_lbls vtbl_entry[MAX_VOLUMES];
  * XXX label is being read twice?
  */
 #define MAX_ARGS	7
-#define NBUFS		20
-#define CONNECT_TIMEOUT 2*60
-#define DATABUF_SIZE	TAPE_BLOCK_BYTES
 
-#define THRESHOLD	(NBUFS-1)
+/* NBUFS replaced by conf_tapebufs */
+/* #define NBUFS		20 */
+int conf_tapebufs;
+
+/* This is now the number of empties, not full bufs */
+#define THRESHOLD	1
+
+#define CONNECT_TIMEOUT 2*60
+
 
 
 typedef enum { BOGUS, START_TAPER, FILE_WRITE, PORT_WRITE, QUIT } cmd_t;
@@ -83,8 +88,8 @@ typedef struct buffer_s {
     char buffer[TAPE_BLOCK_BYTES];
 } buffer_t;
 
-#define nextbuf(p)    ((p) == buftable+NBUFS-1? buftable : (p)+1)
-#define prevbuf(p)    ((p) == buftable? buftable+NBUFS-1 : (p)-1)
+#define nextbuf(p)    ((p) == buftable+conf_tapebufs-1? buftable : (p)+1)
+#define prevbuf(p)    ((p) == buftable? buftable+conf_tapebufs-1 : (p)-1)
 
 /* major modules */
 int main P((int main_argc, char **main_argv));
@@ -138,10 +143,19 @@ char *taper_datestamp = NULL;
 char *label = NULL;
 int filenum;
 char *errstr = NULL;
-int tape_fd;
+int tape_fd = -1;
 char *tapedev = NULL;
 static unsigned long malloc_hist_1, malloc_size_1;
 static unsigned long malloc_hist_2, malloc_size_2;
+char *config_name = NULL;
+char *config_dir = NULL;
+
+int runtapes, cur_tape, have_changer, tapedays;
+char *labelstr, *conf_tapelist;
+#ifdef HAVE_LIBVTBLC
+char *rawtapedev;
+int first_seg, last_seg;
+#endif /* HAVE_LIBVTBLC */
 
 /*
  * ========================================================================
@@ -153,6 +167,7 @@ int main_argc;
 char **main_argv;
 {
     int p2c[2], c2p[2];		/* parent-to-child, child-to-parent pipes */
+    char *conffile;
     int fd;
 
     for(fd = 3; fd < FD_SETSIZE; fd++) {
@@ -169,6 +184,29 @@ char **main_argv;
 
     malloc_size_1 = malloc_inuse(&malloc_hist_1);
 
+    fprintf(stderr, "%s: pid %ld executable %s version %s\n",
+	    get_pname(), (long) getpid(), main_argv[0], version());
+    fflush(stderr);
+
+    if (main_argc > 1 && main_argv[1][0] != '-') {
+	config_name = stralloc(main_argv[1]);
+	config_dir = vstralloc(CONFIG_DIR, "/", main_argv[1], "/", NULL);
+	main_argc--;
+	main_argv++;
+    } else {
+	char my_cwd[STR_SIZE];
+
+	if (getcwd(my_cwd, sizeof(my_cwd)) == NULL) {
+	    error("cannot determine current working directory");
+	}
+	config_dir = stralloc2(my_cwd, "/");
+	if ((config_name = strrchr(my_cwd, '/')) != NULL) {
+	    config_name = stralloc(config_name + 1);
+	}
+    }
+
+    safe_cd();
+
     /* print prompts and debug messages if running interactive */
 
     interactive = (main_argc > 1 && strcmp(main_argv[1],"-t") == 0);
@@ -179,12 +217,31 @@ char **main_argv;
 	set_logerror(logerror);
     }
 
-    fprintf(stderr, "%s: pid %ld executable %s version %s\n",
-	    get_pname(), (long) getpid(), main_argv[0], version());
-    fflush(stderr);
+    conffile = stralloc2(config_dir, CONFFILE_NAME);
+    if(read_conffile(conffile)) {
+	error("could not find config file \"%s\"", conffile);
+    }
+    amfree(conffile);
 
-    if(read_conffile(CONFFILE_NAME))
-	error("parse error in %s", CONFFILE_NAME);
+    conf_tapelist = getconf_str(CNF_TAPELIST);
+    if (*conf_tapelist == '/') {
+	conf_tapelist = stralloc(conf_tapelist);
+    } else {
+	conf_tapelist = stralloc2(config_dir, conf_tapelist);
+    }
+    if(read_tapelist(conf_tapelist)) {
+	error("could not load tapelist \"%s\"", conf_tapelist);
+    }
+
+    tapedev	= getconf_str(CNF_TAPEDEV);
+#ifdef HAVE_LIBVTBLC
+    rawtapedev = getconf_str(CNF_RAWTAPEDEV);
+#endif /* HAVE_LIBVTBLC */
+    tapedays	= getconf_int(CNF_TAPECYCLE);
+    labelstr	= getconf_str(CNF_LABELSTR);
+
+    runtapes	= getconf_int(CNF_RUNTAPES);
+    cur_tape	= 0;
 
     if(interactive) {
 	fprintf(stderr,"taper: running in interactive test mode\n");
@@ -245,11 +302,12 @@ int rdpipe, wrpipe;
     int argc;
     char **argv;
     char *handle = NULL, *hostname = NULL, *diskname = NULL, *result;
-    char *datestamp;
+    char *datestamp = NULL;
     char tok;
     char *q;
     int level, fd, data_port, data_socket, wpid;
     struct stat stat_file;
+    int tape_started;
 
     procname = "reader";
     syncpipe_init(rdpipe, wrpipe);
@@ -268,6 +326,7 @@ int rdpipe, wrpipe;
 
     taper_datestamp = newstralloc(taper_datestamp, argv[2]);
 
+    tape_started = 0;
     syncpipe_put('S');
     syncpipe_putstr(taper_datestamp);
 
@@ -277,6 +336,7 @@ int rdpipe, wrpipe;
     switch(tok) {
     case 'S':
 	putresult("TAPER-OK\n");
+	tape_started = 1;
 	/* start is logged in writer */
 	break;
     case 'E':
@@ -287,7 +347,8 @@ int rdpipe, wrpipe;
 	amfree(q);
 	log_add(L_ERROR,"no-tape [%s]", result);
 	amfree(result);
-	exit(1);
+	syncpipe_put('e');			/* ACK error */
+	break;
     default:
 	error("expected 'S' or 'E' for START-TAPER, got '%c'", tok);
     }
@@ -297,6 +358,9 @@ int rdpipe, wrpipe;
     while(1) {
 	startclock();
 	cmd = getcmd(&argc, &argv);
+	if(cmd != QUIT && !tape_started) {
+	    error("error [file_reader_side cmd %d without tape ready]", cmd);
+	}
 	total_wait = timesadd(total_wait, stopclock());
 
 	switch(cmd) {
@@ -315,8 +379,25 @@ int rdpipe, wrpipe;
 	    datestamp = stralloc(argv[6]);
 
 	    data_port = 0;
-/*	    data_socket = stream_server(&data_port);	*/
-	    data_socket = stream_server(&data_port, DEFAULT_SIZE, DATABUF_SIZE);	
+	    data_socket = stream_server(&data_port,
+					DEFAULT_SIZE,
+					DATABUF_SIZE*2);	
+	    if(data_socket < 0) {
+		char *m;
+
+		m = vstralloc("[port create failure: ",
+			      strerror(errno),
+			      "]",
+			      NULL);
+		q = squote(m);
+		putresult("TAPE-ERROR %s %s\n", handle, q);
+		amfree(q);
+		amfree(m);
+		amfree(handle);
+		amfree(hostname);
+		amfree(diskname);
+		break;
+	    }
 	    putresult("PORT %d\n", data_port);
 
 	    if((fd = stream_accept(data_socket, CONNECT_TIMEOUT,
@@ -398,6 +479,8 @@ int rdpipe, wrpipe;
 	    amfree(errstr);
 	    amfree(changer_resultstr);
 	    amfree(tapedev);
+	    amfree(config_dir);
+	    amfree(config_name);
 
 	    malloc_size_2 = malloc_inuse(&malloc_hist_2);
 
@@ -425,9 +508,9 @@ char *str1;
     long v;
 
     fprintf(stderr, "%s: state", str1);
-    for(i = j = 0; i < NBUFS; i = j+1) {
+    for(i = j = 0; i < conf_tapebufs; i = j+1) {
 	v = buftable[i].status;
-	for(j = i; j < NBUFS && buftable[j].status == v; j++);
+	for(j = i; j < conf_tapebufs && buftable[j].status == v; j++);
 	j--;
 	if(i == j) fprintf(stderr, " %d:", i);
 	else fprintf(stderr, " %d-%d:", i, j);
@@ -481,6 +564,7 @@ void read_file(fd, handle, hostname, diskname, datestamp, level, port_flag)
     int rc, err, opening, closing, bufnum;
     long filesize;
     times_t runtime;
+    char *strclosing = NULL;
     char *str;
     int header_read = 0;
     dumpfile_t file;
@@ -505,7 +589,7 @@ void read_file(fd, handle, hostname, diskname, datestamp, level, port_flag)
 	fflush(stderr);
     }
 
-    for(bp = buftable; bp < buftable + NBUFS; bp++) {
+    for(bp = buftable; bp < buftable + conf_tapebufs; bp++) {
 	bp->status = EMPTY;
 	if(interactive || bufdebug) dumpstatus(bp);
     }
@@ -588,6 +672,7 @@ void read_file(fd, handle, hostname, diskname, datestamp, level, port_flag)
 				       sizeof(bp->buffer))) < 0) {
 		err = (rc < 0)? errno : 0;
 		closing = 1;
+		strclosing = newvstralloc(strclosing,"Can't read data: ",NULL);
 		syncpipe_put('C');
 	    }
 	    else {
@@ -604,14 +689,15 @@ void read_file(fd, handle, hostname, diskname, datestamp, level, port_flag)
 		    else if((fd = open(file.cont_filename,O_RDONLY)) == -1) {
 			err = errno;
 			closing = 1;
+			strclosing = newvstralloc(strclosing,"can't open: ",file.cont_filename,NULL);
 			syncpipe_put('C');
 			rc = 0;
 		    }
 		    else if((fd != save_fd) && dup2(fd, save_fd) == -1) {
 			err = errno;
 			closing = 1;
+			strclosing = newvstralloc(strclosing,"can't dup2: ",file.cont_filename,NULL);
 			syncpipe_put('C');
-			fprintf(stderr,"can't dup2\n");
 		    }
 		    else {
 			if(fd != save_fd) {
@@ -622,6 +708,7 @@ void read_file(fd, handle, hostname, diskname, datestamp, level, port_flag)
 						   sizeof(bp->buffer))) <= 0) {
 			    err = (rc < 0)? errno : 0;
 			    closing = 1;
+			    strclosing = newvstralloc(strclosing,"Can't read header: ",file.cont_filename,NULL);
 			    syncpipe_put('C');
 			}
 			else {
@@ -631,6 +718,8 @@ void read_file(fd, handle, hostname, diskname, datestamp, level, port_flag)
 						   sizeof(bp->buffer))) <= 0) {
 				err = (rc < 0)? errno : 0;
 				closing = 1;
+				if(rc<0)
+			    	    strclosing = newvstralloc(strclosing,"Can't read data: ",NULL);
 				syncpipe_put('C');
 			    }
 			}
@@ -683,13 +772,10 @@ void read_file(fd, handle, hostname, diskname, datestamp, level, port_flag)
 		log_add(L_FAIL, "%s %s %d [out of tape]",
 			hostname, diskname, level);
 		log_add(L_ERROR,"no-tape [%s]", errstr);
-		amfree(q);
-		exit(1);
 	    }
 	    amfree(q);
 	    return;
 
-
 	case 'C':
 	    assert(!opening);
 	    assert(closing);
@@ -707,9 +793,16 @@ void read_file(fd, handle, hostname, diskname, datestamp, level, port_flag)
 	    aclose(fd);
 	    runtime = stopclock();
 	    if(err) {
-		errstr = newvstralloc(errstr,
-				      "[input: ", strerror(err), "]",
-				      NULL);
+		if(strclosing) {
+		    errstr = newvstralloc(errstr,
+				          "[input: ", strclosing, ": ",
+					  strerror(err), "]", NULL);
+		    amfree(strclosing);
+		}
+		else
+		    errstr = newvstralloc(errstr,
+				          "[input: ", strerror(err), "]",
+				          NULL);
 		q = squote(errstr);
 		putresult("TAPE-ERROR %s %s\n", handle, q);
 		amfree(q);
@@ -886,25 +979,39 @@ int getp, putp;
 	startclock();
 	tok = syncpipe_get();
 	idlewait = timesadd(idlewait, stopclock());
+	if(tok != 'S' && tok != 'Q' && !tape_started) {
+	    error("writer: token '%c' before start", tok);
+	}
 
 	switch(tok) {
 	case 'S':		/* start-tape */
-	    assert(!tape_started);
-	    tape_started = 1;
+	    if(tape_started) {
+		error("writer: multiple start requests");
+	    }
 	    str = syncpipe_getstr();
 	    if(!first_tape(str ? str : "bad-datestamp")) {
+		if(tape_fd >= 0) {
+		    tapefd_close(tape_fd);
+		    tape_fd = -1;
+		}
 		syncpipe_put('E');
 		syncpipe_putstr(errstr);
-		exit(1);
-	    }
-	    else
+		/* wait for reader to acknowledge error */
+		do {
+		    tok = syncpipe_get();
+		    if(tok != 'e') {
+			error("writer: got '%c' unexpectedly after error", tok);
+		    }
+		} while(tok != 'e');
+	    } else {
 		syncpipe_put('S');
+		tape_started = 1;
+	    }
 	    amfree(str);
 
 	    break;
 
 	case 'O':		/* open-output */
-	    assert(tape_started);
 	    write_file();
 	    break;
 
@@ -934,6 +1041,8 @@ int getp, putp;
 	    amfree(errstr);
 	    amfree(changer_resultstr);
 	    amfree(tapedev);
+	    amfree(config_dir);
+	    amfree(config_name);
 
 	    malloc_size_2 = malloc_inuse(&malloc_hist_2);
 
@@ -942,6 +1051,7 @@ int getp, putp;
 	    }
 
 	    exit(0);
+
 	default:
 	    assert(0);
 	}
@@ -972,7 +1082,7 @@ void write_file()
      * Tell the reader that the tape is open, and give it all the buffers.
      */
     syncpipe_put('O');
-    for(i = 0; i < NBUFS; i++) {
+    for(i = 0; i < conf_tapebufs; i++) {
 	if(bufdebug) {
 	    fprintf(stderr, "taper: w: put R%d\n", i);
 	    fflush(stderr);
@@ -1008,7 +1118,7 @@ void write_file()
 
 	if(interactive) fputs("[WS]", stderr);
 	startclock();
-	while(full_buffers < THRESHOLD) {
+	while(full_buffers < conf_tapebufs - THRESHOLD) {
 	    tok = syncpipe_get();
 	    if(tok != 'W') break;
 	    bufnum = syncpipe_getint();
@@ -1134,6 +1244,16 @@ void write_file()
     return;
 }
 
+/*
+ * Define the following to test tape error handling.  An EIO error will
+ * be simulated after the number of blocks in block_counter.  This works
+ * especially well with tapedev set to /dev/null.
+ */
+#undef FAKE_TAPE_ERROR
+#if defined(FAKE_TAPE_ERROR)
+static int block_counter = 10;
+#endif
+
 int write_buffer(bp)
 buffer_t *bp;
 {
@@ -1146,8 +1266,27 @@ buffer_t *bp;
 
     startclock();
     rc = tapefd_write(tape_fd, bp->buffer, sizeof(bp->buffer));
+#if defined(FAKE_TAPE_ERROR)
+    if(--block_counter <= 0) {
+	rc = -1;
+	errno = EIO;
+    }
+#endif
     if(rc == sizeof(bp->buffer)) {
-	tapefd_resetofs(tape_fd);
+#if defined(NEED_RESETOFS)
+	static double tape_used_modulus_2gb = 0;
+
+	/*
+	 * If the next write will go over the 2 GByte boundary, reset
+	 * the kernel concept of where we are to make sure it does not
+	 * go silly on us.
+	 */
+	tape_used_modulus_2gb += (double)rc;
+	if(tape_used_modulus_2gb + (double)rc > (double)0x7fffffff) {
+	    tape_used_modulus_2gb = 0;
+	    tapefd_resetofs(tape_fd);
+	}
+#endif
 	wrwait = timesadd(wrwait, stopclock());
 	total_writes += 1;
 	total_tape_used += (double)rc;
@@ -1253,9 +1392,11 @@ buffer_t *attach_buffers()
 {
     buffer_t *result;
 
-    shmid = shmget(IPC_PRIVATE, sizeof(buffer_t)*NBUFS, IPC_CREAT|0700);
+    conf_tapebufs = getconf_int(CNF_TAPEBUFS);
+    shmid = shmget(IPC_PRIVATE, sizeof(buffer_t)*conf_tapebufs, IPC_CREAT|0700);
+    
     if(shmid == -1)
-	error("shmget: %s", strerror(errno));
+	error("shmget: (%d tapebufs) %s", sizeof(buffer_t)*conf_tapebufs,strerror(errno));
 
     result = (buffer_t *)shmat(shmid, (SHM_ARG_TYPE *)NULL, 0);
 
@@ -1269,10 +1410,10 @@ buffer_t *attach_buffers()
 
 
 void detach_buffers(bufp)
-buffer_t *bufp;
+     buffer_t *bufp;
 {
     if(shmdt((SHM_ARG_TYPE *)bufp) == -1)
-	error("shmdr: %s", strerror(errno));
+	error("shmdt: %s", strerror(errno));
 }
 
 void destroy_buffers()
@@ -1304,6 +1445,7 @@ buffer_t *attach_buffers()
 {
     buffer_t *shmbuf;
 
+    conf_tapebufs = getconf_int(CNF_TAPEBUFS);
 #ifdef ZERO_FILE
     shmfd = open(ZERO_FILE, O_RDWR);
     if(shmfd == -1)
@@ -1312,7 +1454,7 @@ buffer_t *attach_buffers()
 #endif
 
     shmbuf = (buffer_t *) mmap((void *) 0,
-			       sizeof(buffer_t)*NBUFS,
+			       sizeof(buffer_t)*conf_tapebufs,
 			       PROT_READ|PROT_WRITE,
 			       MAP_ANON|MAP_SHARED,
 			       shmfd, 0);
@@ -1325,7 +1467,7 @@ buffer_t *attach_buffers()
 void detach_buffers(bufp)
 buffer_t *bufp;
 {
-    if(munmap((void *)bufp, sizeof(buffer_t)*NBUFS) == -1)
+    if(munmap((void *)bufp, sizeof(buffer_t)*conf_tapebufs) == -1)
 	error("detach_buffers: munmap: %s", strerror(errno));
 
     if(shmfd != -1) aclose(shmfd);
@@ -1471,13 +1613,6 @@ char *item;
  *
  */
 
-int runtapes, cur_tape, have_changer, tapedays;
-char *labelstr, *tapefilename;
-#ifdef HAVE_LIBVTBLC
-char *rawtapedev;
-int first_seg, last_seg;
-#endif /* HAVE_LIBVTBLC */
-
 /* local functions */
 int scan_init P((int rc, int ns, int bk));
 int taperscan_slot P((int rc, char *slotstr, char *device));
@@ -1486,10 +1621,11 @@ int label_tape P((void));
 
 int label_tape()
 {
-    char *oldtapefilename = NULL;
+    char *conf_tapelist_old = NULL;
     char *olddatestamp = NULL;
     char *result;
     tape_t *tp;
+    static int first_call = 1;
 
     if(have_changer) {
 	amfree(tapedev);
@@ -1516,6 +1652,7 @@ int label_tape()
 	    return 0;
 	} 
 	tapefd_close(tape_fd);
+	tape_fd = -1;
     }
     else
 #endif /* !HAVE_LINUX_ZFTAPE_H */
@@ -1530,20 +1667,22 @@ int label_tape()
     amfree(olddatestamp);
 
     /* check against tape list */
-    tp = lookup_tapelabel(label);
-    if(tp != NULL && !reusable_tape(tp)) {
-	errstr = newvstralloc(errstr,
-			      "cannot overwrite active tape ", label,
-			      NULL);
-	return 0;
-    }
+    if (strcmp(tapedev, "/dev/null") != 0) {
+	tp = lookup_tapelabel(label);
+	if(tp != NULL && !reusable_tape(tp)) {
+	    errstr = newvstralloc(errstr,
+			          "cannot overwrite active tape ", label,
+			          NULL);
+	    return 0;
+	}
 
-    if(!match(labelstr, label)) {
-	errstr = newvstralloc(errstr,
-			      "label ", label,
-			      " doesn\'t match labelstr \"", labelstr, "\"",
-			      NULL);
-	return 0;
+	if(!match(labelstr, label)) {
+	    errstr = newvstralloc(errstr,
+			          "label ", label,
+			          " doesn\'t match labelstr \"", labelstr, "\"",
+			          NULL);
+	    return 0;
+	}
     }
 
     if((tape_fd = tape_open(tapedev, O_WRONLY)) == -1) {
@@ -1577,24 +1716,36 @@ int label_tape()
     /* write tape list */
 
     /* XXX add cur_tape number to tape list structure */
-    remove_tapelabel(label);
-    add_tapelabel(atoi(taper_datestamp), label);
+    if (strcmp(tapedev, "/dev/null") != 0) {
+	remove_tapelabel(label);
+	add_tapelabel(atoi(taper_datestamp), label);
 
-    if(cur_tape == 0) {
-	oldtapefilename = stralloc2(tapefilename, ".yesterday");
-    } else {
-	char cur_str[NUM_STR_SIZE];
+	if(cur_tape == 0) {
+	    conf_tapelist_old = stralloc2(conf_tapelist, ".yesterday");
+	} else {
+	    char cur_str[NUM_STR_SIZE];
 
-	ap_snprintf(cur_str, sizeof(cur_str), "%d", cur_tape - 1);
-	oldtapefilename = vstralloc(tapefilename, ".today.", cur_str, NULL);
+	    ap_snprintf(cur_str, sizeof(cur_str), "%d", cur_tape - 1);
+	    conf_tapelist_old = vstralloc(conf_tapelist,
+					".today.", cur_str, NULL);
+	}
+	if(rename(conf_tapelist, conf_tapelist_old) != 0) {
+	    if(errno != ENOENT)
+		error("could not rename \"%s\" to \"%s\": %s",
+		      conf_tapelist, conf_tapelist_old, strerror(errno));
+	}
+	amfree(conf_tapelist_old);
+	if(write_tapelist(conf_tapelist)) {
+	    error("could not write tapelist: %s", strerror(errno));
+	}
     }
-    rename(tapefilename, oldtapefilename);
-    amfree(oldtapefilename);
-    if(write_tapelist(tapefilename))
-	error("couldn't write tapelist: %s", strerror(errno));
 
     log_add(L_START, "datestamp %s label %s tape %d",
 	    taper_datestamp, label, cur_tape);
+    if (first_call && strcmp(tapedev, "/dev/null") == 0) {
+	first_call = 0;
+	log_add(L_WARNING, "tapedev is %s, dumps will be thrown away", tapedev);
+    }
 
     total_tape_used=0.0;
     total_tape_fm = 0;
@@ -1605,23 +1756,10 @@ int label_tape()
 int first_tape(new_datestamp)
 char *new_datestamp;
 {
-
-    tapefilename = getconf_str(CNF_TAPELIST);
-
-    tapedev	= getconf_str(CNF_TAPEDEV);
-#ifdef HAVE_LIBVTBLC
-    rawtapedev = getconf_str(CNF_RAWTAPEDEV);
-#endif /* HAVE_LIBVTBLC */
-    tapedays	= getconf_int(CNF_TAPECYCLE);
-    labelstr	= getconf_str(CNF_LABELSTR);
-
-    runtapes	= getconf_int(CNF_RUNTAPES);
-    cur_tape	= 0;
-
-    if(read_tapelist(tapefilename))
-	error("parse error in %s", tapefilename);
-
-    have_changer = changer_init();
+    if((have_changer = changer_init()) < 0) {
+	error("changer initialization failed: %s", strerror(errno));
+    }
+    changer_debug = 1;
 
     taper_datestamp = newstralloc(taper_datestamp, new_datestamp);
 
@@ -1653,9 +1791,9 @@ int end_tape(writerror)
 int writerror;
 {
     char *result;
+    int rc = 0;
 
-    if (label != NULL) {
-	/* label would be null after entering degraded mode */
+    if(tape_fd >= 0) {
 	log_add(L_INFO, "tape %s kb %ld fm %d %s", 
 		label,
 		(long) ((total_tape_used+1023.0) / 1024.0),
@@ -1668,35 +1806,37 @@ int writerror;
 		(long) ((total_tape_used+1023.0) / 1024.0),
 		total_tape_fm);
 	fflush(stderr);
-    }
+	if(! writerror) {
+	    if(! write_filemark()) {
+		rc = 1;
+		goto common_exit;
+	    }
 
-    if(!writerror) {
-	if(!write_filemark()) {
-	    tapefd_close(tape_fd);
-	    goto tape_error;
-	}
-
-	if((result = tapefd_wrendmark(tape_fd, taper_datestamp)) != NULL) {
-	    tapefd_close(tape_fd);
-	    errstr = newstralloc(errstr, result);
-	    goto tape_error;
+	    if((result = tapefd_wrendmark(tape_fd, taper_datestamp)) != NULL) {
+		errstr = newstralloc(errstr, result);
+		rc = 1;
+		goto common_exit;
+	    }
 	}
     }
 
 #ifdef HAVE_LINUX_ZFTAPE_H
-    if (is_zftape(tapedev) == 1){
+    if (tape_fd >= 0 && is_zftape(tapedev) == 1) {
 	/* rewind the tape */
 
 	if(tapefd_rewind(tape_fd) == -1 ) {
 	    errstr = newstralloc2(errstr, "rewinding tape: ", strerror(errno));
-	    goto tape_error;
+	    rc = 1;
+	    goto common_exit;
 	}
 	/* close the tape */
 
 	if(tapefd_close(tape_fd) == -1) {
 	    errstr = newstralloc2(errstr, "closing tape: ", strerror(errno));
-	    goto tape_error;
+	    rc = 1;
+	    goto common_exit;
 	}
+	tape_fd = -1;
 
 #ifdef HAVE_LIBVTBLC
 	/* update volume table */
@@ -1712,7 +1852,8 @@ int writerror;
 				      "updating volume table: ", 
 				      strerror(errno));
 	    }
-	    goto tape_error;
+	    rc = 1;
+	    goto common_exit;
 	}
 	/* read volume table */
 	if ((num_volumes = read_vtbl(tape_fd, volumes, vtbl_buffer,
@@ -1720,7 +1861,8 @@ int writerror;
 	    errstr = newstralloc2(errstr,
 				  "reading volume table: ", 
 				  strerror(errno));
-	    goto tape_error;
+	    rc = 1;
+	    goto common_exit;
 	}
 	/* set volume label and date for first entry */
 	vtbl_no = 0;
@@ -1728,14 +1870,16 @@ int writerror;
 	    errstr = newstralloc2(errstr,
 				  "setting label for entry 1: ",
 				  strerror(errno));
-	    goto tape_error;
+	    rc = 1;
+	    goto common_exit;
 	}
 	/* date of start writing this tape */
 	if (set_date(start_datestr, volumes, num_volumes, vtbl_no)){
 	    errstr = newstralloc2(errstr,
 				  "setting date for entry 1: ", 
 				  strerror(errno));
-	    goto tape_error;
+	    rc = 1;
+	    goto common_exit;
 	}
 	/* set volume labels and dates for backup files */
 	for (vtbl_no = 1; vtbl_no <= num_volumes - 2; vtbl_no++){ 
@@ -1749,14 +1893,16 @@ int writerror;
 		errstr = newstralloc2(errstr,
 				      "setting label for entry i: ", 
 				      strerror(errno));
-		goto tape_error;
+		rc = 1;
+		goto common_exit;
 	    }
 	    if(set_date(vtbl_entry[vtbl_no].date, 
 			volumes, num_volumes, vtbl_no)){
 		errstr = newstralloc2(errstr,
 				      "setting date for entry i: ",
 				      strerror(errno));
-		goto tape_error;
+		rc = 1;
+		goto common_exit;
 	    }
 	}
 	/* set volume label and date for last entry */
@@ -1765,14 +1911,16 @@ int writerror;
 	    errstr = newstralloc2(errstr,
 				  "setting label for last entry: ", 
 				  strerror(errno));
-	    goto tape_error;
+	    rc = 1;
+	    goto common_exit;
 	}
 	datestr = NULL; /* take current time */ 
 	if (set_date(datestr, volumes, num_volumes, vtbl_no)){
 	    errstr = newstralloc2(errstr,
 				  "setting date for last entry 1: ", 
 				  strerror(errno));
-	    goto tape_error;
+	    rc = 1;
+	    goto common_exit;
 	}
 	/* write volume table back */
 	if (write_vtbl(tape_fd, volumes, vtbl_buffer, num_volumes, first_seg,
@@ -1780,33 +1928,40 @@ int writerror;
 	    errstr = newstralloc2(errstr,
 				  "writing volume table: ", 
 				  strerror(errno));
-	    goto tape_error;
+	    rc = 1;
+	    goto common_exit;
 	}  
-	close(tape_fd);
 
 	fprintf(stderr, "taper: updating volume table: done.\n");
 	fflush(stderr);
 #endif /* HAVE_LIBVTBLC */
     }
-    else
 #endif /* !HAVE_LINUX_ZFTAPE_H */
-    /* write the final filemarks */
 
-    if(tapefd_close(tape_fd) == -1) {
+    /* close the tape and let the OS write the final filemarks */
+
+common_exit:
+
+    if(tape_fd >= 0 && tapefd_close(tape_fd) == -1 && ! writerror) {
 	errstr = newstralloc2(errstr, "closing tape: ", strerror(errno));
-	goto tape_error;
+	rc = 1;
     }
+    tape_fd = -1;
+    amfree(label);
 
-    return 1;
-
-tape_error:
-    /* global "errstr" should get passed back to reader-side */
-    return 0;
+    return rc;
 }
 
 
 int write_filemark()
 {
+#if defined(FAKE_TAPE_ERROR)
+    if(--block_counter <= 0) {
+	errno = EIO;
+	errstr = newstralloc2(errstr, "writing filemark: ", strerror(errno));
+	return 0;
+    }
+#endif
     if(tapefd_weof(tape_fd, 1) == -1) {
 	errstr = newstralloc2(errstr, "writing filemark: ", strerror(errno));
 	return 0;
@@ -1928,7 +2083,10 @@ char *taper_scan()
     found = 0;
     got_match = 0;
 
-    changer_scan(scan_init, taperscan_slot);
+    if (searchlabel != NULL)
+      changer_find(scan_init, taperscan_slot,searchlabel);
+    else
+      changer_scan(scan_init, taperscan_slot);
 
     if(found == 2)
 	searchlabel = first_match_label;
