@@ -25,7 +25,7 @@
  *			   University of Maryland at College Park
  */
 /*
- * $Id$
+ * $Id: tapeio.c,v 1.20.4.7 2000/12/13 18:33:31 jrjackson Exp $
  *
  * implements tape I/O functions
  */
@@ -40,6 +40,36 @@
 
 static char *errstr = NULL;
 
+static int no_op_tapefd = -1;
+
+#if defined(HAVE_BROKEN_FSF)
+/*
+ * tapefd_fsf_broken -- handle systems that have a broken fsf operation
+ * and cannot do an fsf operation unless they are positioned at a tape
+ * mark (or BOT).  This shows up in amrestore as I/O errors when skipping.
+ */
+
+static int
+tapefd_fsf_broken(tapefd, count)
+int tapefd;
+int count;
+{
+    char buffer[TAPE_BLOCK_BYTES];
+    int len = 0;
+
+    if(tapefd == no_op_tapefd) {
+	return 0;
+    }
+    while(--count >= 0) {
+	while((len = tapefd_read(tapefd, buffer, sizeof(buffer))) > 0) {}
+	if(len < 0) {
+	    break;
+	}
+    }
+    return len;
+}
+#endif
+
 #ifdef UWARE_TAPEIO 
 
 #include <sys/tape.h>
@@ -48,7 +78,7 @@ int tapefd_rewind(tapefd)
 int tapefd;
 {
     int st;
-    return ioctl(tapefd, T_RWD, &st);
+    return (tapefd == no_op_tapefd) ? 0 : ioctl(tapefd, T_RWD, &st);
 }
 
 int tapefd_fsf(tapefd, count)
@@ -57,15 +87,19 @@ int tapefd, count;
  * fast-forwards the tape device count files.
  */
 {
+#if defined(HAVE_BROKEN_FSF)
+    return tapefd_fsf_broken(tapefd, count);
+#else
     int st;
     int c;
     int status;
 
     for ( c = count; c ; c--)
-        if (status = ioctl(tapefd, T_SFF, &st))
+        if (status = (tapefd == no_op_tapefd) ? 0 : ioctl(tapefd, T_SFF, &st))
             break;
 
     return status;
+#endif
 }
 
 int tapefd_weof(tapefd, count)
@@ -79,7 +113,7 @@ int tapefd, count;
     int status;
 
     for ( c = count; c ; c--)
-        if (status = ioctl(tapefd, T_WRFILEM, &st))
+        if (status = (tapefd == no_op_tapefd) ? 0 : ioctl(tapefd, T_WRFILEM, &st))
             break;
 
     return status;
@@ -98,7 +132,7 @@ int tapefd;
     st.st_op = STREW;
     st.st_count = 1;
 
-    return ioctl(tapefd, STIOCTOP, &st);
+    return (tapefd == no_op_tapefd) ? 0 : ioctl(tapefd, STIOCTOP, &st);
 }
 
 int tapefd_fsf(tapefd, count)
@@ -107,12 +141,16 @@ int tapefd, count;
  * fast-forwards the tape device count files.
  */
 {
+#if defined(HAVE_BROKEN_FSF)
+    return tapefd_fsf_broken(tapefd, count);
+#else
     struct stop st;
 
     st.st_op = STFSF;
     st.st_count = count;
 
-    return ioctl(tapefd, STIOCTOP, &st);
+    return (tapefd == no_op_tapefd) ? 0 : ioctl(tapefd, STIOCTOP, &st);
+#endif
 }
 
 int tapefd_weof(tapefd, count)
@@ -126,7 +164,7 @@ int tapefd, count;
     st.st_op = STWEOF;
     st.st_count = count;
 
-    return ioctl(tapefd, STIOCTOP, &st);
+    return (tapefd == no_op_tapefd) ? 0 : ioctl(tapefd, STIOCTOP, &st);
 }
 
 #else /* AIX_TAPEIO */
@@ -138,7 +176,7 @@ int tapefd_rewind(tapefd)
 int tapefd;
 {
     int st;
-    return ioctl(tapefd, MT_REWIND, &st);
+    return (tapefd == no_op_tapefd) ? 0 : ioctl(tapefd, MT_REWIND, &st);
 }
 
 int tapefd_fsf(tapefd, count)
@@ -147,15 +185,19 @@ int tapefd, count;
  * fast-forwards the tape device count files.
  */
 {
+#if defined(HAVE_BROKEN_FSF)
+    return tapefd_fsf_broken(tapefd, count);
+#else
     int st;
     int c;
     int status;
 
     for ( c = count; c ; c--)
-	if (status = ioctl(tapefd, MT_RFM, &st))
+	if (status = (tapefd == no_op_tapefd) ? 0 : ioctl(tapefd, MT_RFM, &st))
 	    break;
 
     return status;
+#endif
 }
 
 int tapefd_weof(tapefd, count)
@@ -169,7 +211,7 @@ int tapefd, count;
     int status;
 
     for ( c = count; c ; c--)
-	if (status = ioctl(tapefd, MT_WFM, &st))
+	if (status = (tapefd == no_op_tapefd) ? 0 : ioctl(tapefd, MT_WFM, &st))
 	    break;
 
     return status;
@@ -193,7 +235,7 @@ int tapefd;
     /* EXB-8200 drive on FreeBSD can fail to rewind, but retrying
      * won't hurt, and it will usually even work! */
     for(cnt = 0; cnt < 10; ++cnt) {
-	rc = ioctl(tapefd, MTIOCTOP, &mt);
+	rc = (tapefd == no_op_tapefd) ? 0 : ioctl(tapefd, MTIOCTOP, &mt);
 	if (rc == 0)
 	    break;
 	sleep(3);
@@ -207,12 +249,16 @@ int tapefd, count;
  * fast-forwards the tape device count files.
  */
 {
+#if defined(HAVE_BROKEN_FSF)
+    return tapefd_fsf_broken(tapefd, count);
+#else
     struct mtop mt;
 
     mt.mt_op = MTFSF;
     mt.mt_count = count;
 
-    return ioctl(tapefd, MTIOCTOP, &mt);
+    return (tapefd == no_op_tapefd) ? 0 : ioctl(tapefd, MTIOCTOP, &mt);
+#endif
 }
 
 int tapefd_weof(tapefd, count)
@@ -226,7 +272,7 @@ int tapefd, count;
     mt.mt_op = MTWEOF;
     mt.mt_count = count;
 
-    return ioctl(tapefd, MTIOCTOP, &mt);
+    return (tapefd == no_op_tapefd) ? 0 : ioctl(tapefd, MTIOCTOP, &mt);
 }
 
 
@@ -239,8 +285,8 @@ int tapefd, count;
 
 
 int tape_open(filename, mode)
-char *filename;
-int mode;
+     char *filename;
+     int mode;
 {
 #ifdef HAVE_LINUX_ZFTAPE_H
     struct mtop mt;
@@ -250,24 +296,47 @@ int mode;
 	mode = O_RDONLY;
     else
 	mode = O_RDWR;
+#if 0
+    /* Since we're no longer using a special name for no-tape, we no
+       longer need this */
+    if (strcmp(filename, "/dev/null") == 0) {
+	filename = "/dev/null";
+    }
+#endif
     do {
 	ret = open(filename, mode);
-	/* if tape open fails with errno==EAGAIN, it is worth retrying
-	 * a few seconds later.  */
-	if (ret >= 0 || errno != EAGAIN)
+	/* if tape open fails with errno==EAGAIN, EBUSY or EINTR, it
+	 * is worth retrying a few seconds later.  */
+	if (ret >= 0 ||
+	    (1
+#ifdef EAGAIN
+	     && errno != EAGAIN
+#endif
+#ifdef EBUSY
+	     && errno != EBUSY
+#endif
+#ifdef EINTR
+	     && errno != EINTR
+#endif
+	     ))
 	    break;
 	sleep(delay);
 	timeout -= delay;
 	if (delay < 16)
 	    delay *= 2;
     } while (timeout > 0);
+    if (strcmp(filename, "/dev/null") == 0) {
+	no_op_tapefd = ret;
+    } else {
+	no_op_tapefd = -1;
+    }
 #ifdef HAVE_LINUX_ZFTAPE_H
     /* 
      * switch the block size for the zftape driver (3.04d) 
      * (its default is 10kb and not TAPE_BLOCK_BYTES=32kb) 
      *        A. Gebhardt <albrecht.gebhardt@uni-klu.ac.at>
      */
-    if (ret >= 0 && is_zftape(filename) == 1)
+    if (no_op_tapefd < 0 && ret >= 0 && is_zftape(filename) == 1)
 	{
 	    mt.mt_op = MTSETBLK;
 	    mt.mt_count = TAPE_BLOCK_BYTES;
@@ -294,6 +363,9 @@ void *buffer;
 int tapefd_close(tapefd)
 int tapefd;
 {
+    if (tapefd == no_op_tapefd) {
+	no_op_tapefd = -1;
+    }
     return close(tapefd);
 }
 
@@ -380,10 +452,15 @@ char **datestamp, **label;
     if(rc == sizeof(buffer)) rc--;
     buffer[rc] = '\0';
 
-    parse_file_header(buffer, &file, sizeof(buffer));
-    if(file.type != F_TAPESTART) {
-	errstr = newstralloc(errstr, "not an amanda tape");
-	return errstr;
+    if (tapefd == no_op_tapefd) {
+	strcpy(file.datestamp, "X");
+	strcpy(file.name, "/dev/null");
+    } else {
+	parse_file_header(buffer, &file, sizeof(buffer));
+	if(file.type != F_TAPESTART) {
+	    errstr = newstralloc(errstr, "not an amanda tape");
+	    return errstr;
+	}
     }
     *datestamp = stralloc(file.datestamp);
     *label = stralloc(file.name);
@@ -494,7 +571,7 @@ char *datestamp;
 
 
 char *tape_wrendmark(devname, datestamp)
-char *devname, *datestamp;
+    char *devname, *datestamp;
 {
     int fd;
 
@@ -510,6 +587,7 @@ char *devname, *datestamp;
 	return errstr;
     }
 
+    tapefd_close(fd);
     return NULL;
 }
 
